@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -19,15 +19,24 @@
 #include <retro_assert.h>
 
 #include <lists/string_list.h>
-#include <conversion/float_to_s16.h>
-#include <conversion/s16_to_float.h>
+#include <audio/conversion/float_to_s16.h>
+#include <audio/conversion/s16_to_float.h>
+#include <audio/audio_resampler.h>
+#include <audio/dsp_filter.h>
+#include <file/file_path.h>
+#include <lists/dir_list.h>
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
 
 #include "audio_driver.h"
-#include "audio_resampler_driver.h"
-#include "../record/record_driver.h"
 #include "audio_thread_wrapper.h"
+#include "../record/record_driver.h"
+#include "../frontend/frontend_driver.h"
 
 #include "../command.h"
+#include "../driver.h"
 #include "../configuration.h"
 #include "../retroarch.h"
 #include "../runloop.h"
@@ -35,61 +44,7 @@
 #include "../verbosity.h"
 #include "../list_special.h"
 
-#ifndef AUDIO_BUFFER_FREE_SAMPLES_COUNT
 #define AUDIO_BUFFER_FREE_SAMPLES_COUNT (8 * 1024)
-#endif
-
-struct audio_driver_input_data
-{
-   float *data;
-
-   size_t data_ptr;
-   struct
-   {
-      size_t size;
-      size_t nonblock_size;
-      size_t block_size;
-   } chunk;
-
-
-   struct
-   {
-      float input;
-      bool  control; 
-      struct
-      {
-         double original;
-         double current;
-      } source_ratio;
-   } audio_rate;
-
-   bool use_float;
-
-   struct
-   {
-      int16_t *buf;
-      size_t ptr;
-      size_t size;
-   } rewind;
-
-   rarch_dsp_filter_t *dsp;
-
-   size_t driver_buffer_size;
-
-   float volume_gain;
-
-   struct
-   {
-      float *buf;
-      int16_t *conv_buf;
-   } output_samples;
-
-   struct
-   {
-      unsigned buf[AUDIO_BUFFER_FREE_SAMPLES_COUNT];
-      uint64_t count;
-   } free_samples;
-};
 
 static const audio_driver_t *audio_drivers[] = {
 #ifdef HAVE_ALSA
@@ -140,6 +95,9 @@ static const audio_driver_t *audio_drivers[] = {
 #ifdef GEKKO
    &audio_gx,
 #endif
+#ifdef WIIU
+   &audio_ax,
+#endif
 #ifdef EMSCRIPTEN
    &audio_rwebaudio,
 #endif
@@ -154,16 +112,43 @@ static const audio_driver_t *audio_drivers[] = {
    NULL,
 };
 
-static struct audio_driver_input_data audio_driver_data;
-static struct retro_audio_callback audio_callback;
-static struct string_list *audio_driver_devices_list   = NULL;
-static struct retro_perf_counter resampler_proc        = {0};
-static const rarch_resampler_t *audio_driver_resampler = NULL;
-static void *audio_driver_resampler_data               = NULL;
-static bool audio_driver_active                        = false;
-static bool audio_driver_data_own                      = false;
-static const audio_driver_t *current_audio             = NULL;
-static void *audio_driver_context_audio_data           = NULL;
+static size_t audio_driver_chunk_size                    = 0;
+static size_t audio_driver_chunk_nonblock_size           = 0;
+static size_t audio_driver_chunk_block_size              = 0;
+
+static size_t audio_driver_rewind_ptr                    = 0;
+static size_t audio_driver_rewind_size                   = 0;
+static int16_t *audio_driver_rewind_buf                  = NULL;
+
+static float *audio_driver_input_data                    = NULL;
+
+static unsigned audio_driver_free_samples_buf[AUDIO_BUFFER_FREE_SAMPLES_COUNT];
+static uint64_t audio_driver_free_samples_count          = 0;
+
+static float   *audio_driver_output_samples_buf          = NULL;
+static int16_t *audio_driver_output_samples_conv_buf     = NULL;
+
+static float audio_driver_volume_gain                    = 0.0f;
+
+static size_t audio_driver_buffer_size                   = 0;
+static size_t audio_driver_data_ptr                      = 0;
+
+static bool  audio_driver_control                        = false; 
+static float audio_driver_input                          = 0.0f;
+static double audio_source_ratio_original                = 0.0f;
+static double audio_source_ratio_current                 = 0.0f;
+static struct retro_audio_callback audio_callback        = {0};
+
+static retro_dsp_filter_t *audio_driver_dsp              = NULL;
+static struct string_list *audio_driver_devices_list     = NULL;
+static const retro_resampler_t *audio_driver_resampler   = NULL;
+static void *audio_driver_resampler_data                 = NULL;
+static const audio_driver_t *current_audio               = NULL;
+static void *audio_driver_context_audio_data             = NULL;
+
+static bool audio_driver_use_float                       = false;
+static bool audio_driver_active                          = false;
+static bool audio_driver_data_own                        = false;
 
 /**
  * compute_audio_buffer_statistics:
@@ -180,35 +165,35 @@ static void compute_audio_buffer_statistics(void)
    unsigned low_water_count      = 0;
    unsigned high_water_count     = 0;
    unsigned samples              = MIN(
-         audio_driver_data.free_samples.count,
+         audio_driver_free_samples_count,
          AUDIO_BUFFER_FREE_SAMPLES_COUNT);
 
    if (samples < 3)
       return;
 
    for (i = 1; i < samples; i++)
-      accum += audio_driver_data.free_samples.buf[i];
+      accum += audio_driver_free_samples_buf[i];
 
    avg = accum / (samples - 1);
 
    for (i = 1; i < samples; i++)
    {
-      int diff = avg - audio_driver_data.free_samples.buf[i];
+      int diff = avg - audio_driver_free_samples_buf[i];
       accum_var += diff * diff;
    }
 
    stddev          = (unsigned)sqrt((double)accum_var / (samples - 2));
-   avg_filled      = 1.0f - (float)avg / audio_driver_data.driver_buffer_size;
-   deviation       = (float)stddev / audio_driver_data.driver_buffer_size;
+   avg_filled      = 1.0f - (float)avg / audio_driver_buffer_size;
+   deviation       = (float)stddev / audio_driver_buffer_size;
 
-   low_water_size  = audio_driver_data.driver_buffer_size * 3 / 4;
-   high_water_size = audio_driver_data.driver_buffer_size / 4;
+   low_water_size  = audio_driver_buffer_size * 3 / 4;
+   high_water_size = audio_driver_buffer_size / 4;
 
    for (i = 1; i < samples; i++)
    {
-      if (audio_driver_data.free_samples.buf[i] >= low_water_size)
+      if (audio_driver_free_samples_buf[i] >= low_water_size)
          low_water_count++;
-      else if (audio_driver_data.free_samples.buf[i] <= high_water_size)
+      else if (audio_driver_free_samples_buf[i] <= high_water_size)
          high_water_count++;
    }
 
@@ -261,7 +246,7 @@ const char *config_get_audio_driver_options(void)
    return char_list_new_special(STRING_LIST_AUDIO_DRIVERS, NULL);
 }
 
-static bool uninit_audio(void)
+static bool audio_driver_deinit_internal(void)
 {
    settings_t *settings = config_get_ptr();
 
@@ -272,31 +257,33 @@ static bool uninit_audio(void)
       audio_driver_context_audio_data = NULL;
    }
 
-   if (audio_driver_data.output_samples.conv_buf)
-      free(audio_driver_data.output_samples.conv_buf);
-   audio_driver_data.output_samples.conv_buf = NULL;
-   audio_driver_data.data_ptr                = 0;
+   if (audio_driver_output_samples_conv_buf)
+      free(audio_driver_output_samples_conv_buf);
+   audio_driver_output_samples_conv_buf = NULL;
 
-   if (audio_driver_data.rewind.buf)
-      free(audio_driver_data.rewind.buf);
-   audio_driver_data.rewind.buf   = NULL;
-   audio_driver_data.rewind.size  = 0;
+   audio_driver_data_ptr                = 0;
 
-   if (!settings->audio.enable)
+   if (audio_driver_rewind_buf)
+      free(audio_driver_rewind_buf);
+   audio_driver_rewind_buf   = NULL;
+
+   audio_driver_rewind_size  = 0;
+
+   if (settings && !settings->audio.enable)
    {
-      audio_driver_unset_active();
+      audio_driver_active = false;
       return false;
    }
 
    audio_driver_deinit_resampler();
 
-   if (audio_driver_data.data)
-      free(audio_driver_data.data);
-   audio_driver_data.data = NULL;
+   if (audio_driver_input_data)
+      free(audio_driver_input_data);
+   audio_driver_input_data = NULL;
 
-   if (audio_driver_data.output_samples.buf)
-      free(audio_driver_data.output_samples.buf);
-   audio_driver_data.output_samples.buf = NULL;
+   if (audio_driver_output_samples_buf)
+      free(audio_driver_output_samples_buf);
+   audio_driver_output_samples_buf = NULL;
 
    command_event(CMD_EVENT_DSP_FILTER_DEINIT, NULL);
 
@@ -307,8 +294,14 @@ static bool uninit_audio(void)
 
 static bool audio_driver_init_internal(bool audio_cb_inited)
 {
-   size_t outsamples_max, max_bufsamples = AUDIO_CHUNK_SIZE_NONBLOCKING * 2;
-   settings_t *settings = config_get_ptr();
+   unsigned new_rate     = 0;
+   float   *aud_inp_data = NULL;
+   float *samples_buf    = NULL;
+   int16_t *conv_buf     = NULL;
+   int16_t *rewind_buf   = NULL;
+   size_t outsamples_max = AUDIO_CHUNK_SIZE_NONBLOCKING * 2;
+   size_t max_bufsamples = AUDIO_CHUNK_SIZE_NONBLOCKING * 2;
+   settings_t *settings  = config_get_ptr();
 
    convert_s16_to_float_init_simd();
    convert_float_to_s16_init_simd();
@@ -317,32 +310,33 @@ static bool audio_driver_init_internal(bool audio_cb_inited)
    outsamples_max = max_bufsamples * AUDIO_MAX_RATIO * 
       settings->slowmotion_ratio;
 
-   audio_driver_data.output_samples.conv_buf =
-      (int16_t*)malloc(outsamples_max * sizeof(int16_t));
+   conv_buf = (int16_t*)malloc(outsamples_max 
+         * sizeof(int16_t));
    /* Used for recording even if audio isn't enabled. */
-   retro_assert(audio_driver_data.output_samples.conv_buf != NULL);
+   retro_assert(conv_buf != NULL);
 
-   if (!audio_driver_data.output_samples.conv_buf)
+   if (!conv_buf)
       goto error;
 
-   audio_driver_data.chunk.block_size    = AUDIO_CHUNK_SIZE_BLOCKING;
-   audio_driver_data.chunk.nonblock_size = AUDIO_CHUNK_SIZE_NONBLOCKING;
-   audio_driver_data.chunk.size          = audio_driver_data.chunk.block_size;
+   audio_driver_output_samples_conv_buf = conv_buf;
+   audio_driver_chunk_block_size        = AUDIO_CHUNK_SIZE_BLOCKING;
+   audio_driver_chunk_nonblock_size     = AUDIO_CHUNK_SIZE_NONBLOCKING;
+   audio_driver_chunk_size              = audio_driver_chunk_block_size;
 
    /* Needs to be able to hold full content of a full max_bufsamples
     * in addition to its own. */
-   audio_driver_data.rewind.buf = (int16_t*)malloc
-      (max_bufsamples * sizeof(int16_t));
-   retro_assert(audio_driver_data.rewind.buf != NULL);
+   rewind_buf = (int16_t*)malloc(max_bufsamples * sizeof(int16_t));
+   retro_assert(rewind_buf != NULL);
 
-   if (!audio_driver_data.rewind.buf)
+   if (!rewind_buf)
       goto error;
 
-   audio_driver_data.rewind.size             = max_bufsamples;
+   audio_driver_rewind_buf              = rewind_buf;
+   audio_driver_rewind_size             = max_bufsamples;
 
    if (!settings->audio.enable)
    {
-      audio_driver_unset_active();
+      audio_driver_active = false;
       return false;
    }
 
@@ -355,7 +349,9 @@ static bool audio_driver_init_internal(bool audio_cb_inited)
                &current_audio,
                &audio_driver_context_audio_data,
                *settings->audio.device ? settings->audio.device : NULL,
-               settings->audio.out_rate, settings->audio.latency,
+               settings->audio.out_rate, &new_rate, 
+               settings->audio.latency,
+               settings->audio.block_frames,
                current_audio))
       {
          RARCH_ERR("Cannot open threaded audio driver ... Exiting ...\n");
@@ -368,68 +364,78 @@ static bool audio_driver_init_internal(bool audio_cb_inited)
       audio_driver_context_audio_data = 
          current_audio->init(*settings->audio.device ?
                settings->audio.device : NULL,
-               settings->audio.out_rate, settings->audio.latency);
+               settings->audio.out_rate, settings->audio.latency,
+               settings->audio.block_frames,
+               &new_rate);
    }
 
+   if (new_rate != 0)
+      settings->audio.out_rate = new_rate;
+
    if (!audio_driver_context_audio_data)
    {
       RARCH_ERR("Failed to initialize audio driver. Will continue without audio.\n");
-      audio_driver_unset_active();
+      audio_driver_active = false;
    }
 
-   audio_driver_data.use_float = false;
-   if (     audio_driver_is_active() 
+   audio_driver_use_float = false;
+   if (     audio_driver_active 
          && current_audio->use_float(audio_driver_context_audio_data))
-      audio_driver_data.use_float = true;
+      audio_driver_use_float = true;
 
-   if (!settings->audio.sync && audio_driver_is_active())
+   if (!settings->audio.sync && audio_driver_active)
    {
       command_event(CMD_EVENT_AUDIO_SET_NONBLOCKING_STATE, NULL);
-      audio_driver_data.chunk.size = audio_driver_data.chunk.nonblock_size;
+      audio_driver_chunk_size = audio_driver_chunk_nonblock_size;
    }
 
-   if (audio_driver_data.audio_rate.input <= 0.0f)
+   if (audio_driver_input <= 0.0f)
    {
       /* Should never happen. */
       RARCH_WARN("Input rate is invalid (%.3f Hz). Using output rate (%u Hz).\n",
-            audio_driver_data.audio_rate.input, settings->audio.out_rate);
-      audio_driver_data.audio_rate.input = settings->audio.out_rate;
+            audio_driver_input, settings->audio.out_rate);
+      audio_driver_input = settings->audio.out_rate;
    }
 
-   audio_driver_data.audio_rate.source_ratio.original   = 
-      audio_driver_data.audio_rate.source_ratio.current =
-      (double)settings->audio.out_rate / audio_driver_data.audio_rate.input;
+   audio_source_ratio_original   = audio_source_ratio_current =
+      (double)settings->audio.out_rate / audio_driver_input;
 
-   if (!audio_driver_init_resampler())
+   if (!retro_resampler_realloc(
+            &audio_driver_resampler_data,
+            &audio_driver_resampler,
+            settings->audio.resampler,
+            audio_source_ratio_original))
    {
       RARCH_ERR("Failed to initialize resampler \"%s\".\n",
             settings->audio.resampler);
-      audio_driver_unset_active();
+      audio_driver_active = false;
    }
 
-   audio_driver_data.data = (float*)
-      malloc(max_bufsamples * sizeof(float));
-   retro_assert(audio_driver_data.data != NULL);
+   aud_inp_data = (float*)malloc(max_bufsamples * sizeof(float));
+   retro_assert(aud_inp_data != NULL);
 
-   if (!audio_driver_data.data)
+   if (!aud_inp_data)
       goto error;
 
-   audio_driver_data.data_ptr = 0;
+   audio_driver_input_data = aud_inp_data;
+   audio_driver_data_ptr   = 0;
 
    retro_assert(settings->audio.out_rate <
-         audio_driver_data.audio_rate.input * AUDIO_MAX_RATIO);
+         audio_driver_input * AUDIO_MAX_RATIO);
 
-   audio_driver_data.output_samples.buf = (float*)
-      malloc(outsamples_max * sizeof(float));
-   retro_assert(audio_driver_data.output_samples.buf != NULL);
+   samples_buf = (float*)malloc(outsamples_max * sizeof(float));
 
-   if (!audio_driver_data.output_samples.buf)
+   retro_assert(samples_buf != NULL);
+
+   if (!samples_buf)
       goto error;
 
-   audio_driver_data.audio_rate.control = false;
+   audio_driver_output_samples_buf = samples_buf;
+   audio_driver_control            = false;
+
    if (
          !audio_cb_inited
-         && audio_driver_is_active() 
+         && audio_driver_active 
          && settings->audio.rate_control
          )
    {
@@ -437,25 +443,32 @@ static bool audio_driver_init_internal(bool audio_cb_inited)
        * and buffer_size to be implemented. */
       if (current_audio->buffer_size)
       {
-         audio_driver_data.driver_buffer_size = 
+         audio_driver_buffer_size = 
             current_audio->buffer_size(audio_driver_context_audio_data);
-         audio_driver_data.audio_rate.control = true;
+         audio_driver_control = true;
       }
       else
          RARCH_WARN("Audio rate control was desired, but driver does not support needed features.\n");
    }
 
+   /* If we start muted, stop the audio driver, 
+    * so subsequent unmute works. */
+   if (     !audio_cb_inited 
+         &&  audio_driver_active 
+         &&  settings->audio.mute_enable)
+      audio_driver_stop();
+
    command_event(CMD_EVENT_DSP_FILTER_INIT, NULL);
 
-   audio_driver_data.free_samples.count = 0;
+   audio_driver_free_samples_count = 0;
 
    /* Threaded driver is initially stopped. */
    if (
-         audio_driver_is_active()
+         audio_driver_active
          && !settings->audio.mute_enable
          && audio_cb_inited
          )
-      audio_driver_start();
+      audio_driver_start(false);
 
    return true;
 
@@ -463,52 +476,19 @@ error:
    return audio_driver_deinit();
 }
 
-/*
- * audio_driver_readjust_input_rate:
- *
- * Readjust the audio input rate.
- */
-static void audio_driver_readjust_input_rate(void)
-{
-   settings_t *settings = config_get_ptr();
-   unsigned write_idx   = audio_driver_data.free_samples.count++ &
-      (AUDIO_BUFFER_FREE_SAMPLES_COUNT - 1);
-   int      half_size   = audio_driver_data.driver_buffer_size / 2;
-   int      avail       = 
-      current_audio->write_avail(audio_driver_context_audio_data);
-   int      delta_mid   = avail - half_size;
-   double   direction   = (double)delta_mid / half_size;
-   double   adjust      = 1.0 + settings->audio.rate_control_delta * direction;
-
-#if 0
-   RARCH_LOG_OUTPUT("Audio buffer is %u%% full\n",
-         (unsigned)(100 - (avail * 100) / audio_driver_data.driver_buffer_size));
-#endif
-
-   audio_driver_data.free_samples.buf[write_idx] = avail;
-   audio_driver_data.audio_rate.source_ratio.current = 
-      audio_driver_data.audio_rate.source_ratio.original * adjust;
-
-#if 0
-   RARCH_LOG_OUTPUT("New rate: %lf, Orig rate: %lf\n",
-         audio_driver_data.audio_rate.source_ratio.current,
-         audio_driver_data.audio_rate.source_ratio.original);
-#endif
-}
-
 void audio_driver_set_nonblocking_state(bool enable)
 {
    settings_t *settings = config_get_ptr();
    if (
-         audio_driver_is_active()
+         audio_driver_active
          && audio_driver_context_audio_data
       )
       current_audio->set_nonblock_state(audio_driver_context_audio_data,
             settings->audio.sync ? enable : true);
 
-   audio_driver_data.chunk.size = enable ? 
-      audio_driver_data.chunk.nonblock_size : 
-      audio_driver_data.chunk.block_size;
+   audio_driver_chunk_size = enable ? 
+      audio_driver_chunk_nonblock_size : 
+      audio_driver_chunk_block_size;
 }
 
 /**
@@ -524,43 +504,62 @@ void audio_driver_set_nonblocking_state(bool enable)
  **/
 static bool audio_driver_flush(const int16_t *data, size_t samples)
 {
-   static struct retro_perf_counter audio_convert_s16 = {0};
-   static struct retro_perf_counter audio_convert_float = {0};
-   static struct retro_perf_counter audio_dsp         = {0};
-   struct resampler_data src_data              = {0};
-   struct rarch_dsp_data dsp_data              = {0};
-   const void *output_data                     = NULL;
-   unsigned output_frames                      = 0;
-   size_t   output_size                        = sizeof(float);
-   settings_t *settings                        = config_get_ptr();
+   struct resampler_data src_data;
+   bool is_perfcnt_enable                               = false;
+   bool is_paused                                       = false;
+   bool is_idle                                         = false;
+   bool is_slowmotion                                   = false;
+   static struct retro_perf_counter resampler_proc      = {0};
+   static struct retro_perf_counter audio_convert_s16   = {0};
+   const void *output_data                              = NULL;
+   unsigned output_frames                               = 0;
+   size_t   output_size                                 = sizeof(float);
+   settings_t *settings                                 = config_get_ptr();
 
-   recording_push_audio(data, samples);
+   src_data.data_in                                     = NULL;
+   src_data.data_out                                    = NULL;
+   src_data.input_frames                                = 0;
+   src_data.output_frames                               = 0;
+   src_data.ratio                                       = 0.0f;
 
-   if (runloop_ctl(RUNLOOP_CTL_IS_PAUSED, NULL) || settings->audio.mute_enable)
+   if (recording_data)
+      recording_push_audio(data, samples);
+
+   runloop_get_status(&is_paused, &is_idle, &is_slowmotion,
+         &is_perfcnt_enable);
+
+   if (is_paused || settings->audio.mute_enable)
       return true;
-   if (!audio_driver_is_active())
-      return false;
-   if (!audio_driver_data.data)
+   if (!audio_driver_active || !audio_driver_input_data)
       return false;
 
-   performance_counter_init(&audio_convert_s16, "audio_convert_s16");
-   performance_counter_start(&audio_convert_s16);
-   convert_s16_to_float(audio_driver_data.data, data, samples,
-         audio_driver_data.volume_gain);
-   performance_counter_stop(&audio_convert_s16);
+   performance_counter_init(audio_convert_s16, "audio_convert_s16");
+   performance_counter_start_plus(is_perfcnt_enable, audio_convert_s16);
+   convert_s16_to_float(audio_driver_input_data, data, samples,
+         audio_driver_volume_gain);
+   performance_counter_stop_plus(is_perfcnt_enable, audio_convert_s16);
 
-   src_data.data_in               = audio_driver_data.data;
+   src_data.data_in               = audio_driver_input_data;
    src_data.input_frames          = samples >> 1;
 
-   dsp_data.input                 = audio_driver_data.data;
-   dsp_data.input_frames          = samples >> 1;
 
-   if (audio_driver_data.dsp)
+   if (audio_driver_dsp)
    {
-      performance_counter_init(&audio_dsp, "audio_dsp");
-      performance_counter_start(&audio_dsp);
-      rarch_dsp_filter_process(audio_driver_data.dsp, &dsp_data);
-      performance_counter_stop(&audio_dsp);
+      static struct retro_perf_counter audio_dsp           = {0};
+      struct retro_dsp_data dsp_data;
+
+      dsp_data.input                 = NULL;
+      dsp_data.input_frames          = 0;
+      dsp_data.output                = NULL;
+      dsp_data.output_frames         = 0;
+
+      dsp_data.input                 = audio_driver_input_data;
+      dsp_data.input_frames          = samples >> 1;
+
+      performance_counter_init(audio_dsp, "audio_dsp");
+      performance_counter_start_plus(is_perfcnt_enable, audio_dsp);
+      retro_dsp_filter_process(audio_driver_dsp, &dsp_data);
+      performance_counter_stop_plus(is_perfcnt_enable, audio_dsp);
 
       if (dsp_data.output)
       {
@@ -569,37 +568,69 @@ static bool audio_driver_flush(const int16_t *data, size_t samples)
       }
    }
 
-   src_data.data_out = audio_driver_data.output_samples.buf;
+   src_data.data_out = audio_driver_output_samples_buf;
 
-   if (audio_driver_data.audio_rate.control)
-      audio_driver_readjust_input_rate();
+   if (audio_driver_control)
+   {
+      /* Readjust the audio input rate. */
+      unsigned write_idx   = audio_driver_free_samples_count++ &
+         (AUDIO_BUFFER_FREE_SAMPLES_COUNT - 1);
+      int      half_size   = audio_driver_buffer_size / 2;
+      int      avail       = 
+         current_audio->write_avail(audio_driver_context_audio_data);
+      int      delta_mid   = avail - half_size;
+      double   direction   = (double)delta_mid / half_size;
+      double   adjust      = 1.0 + settings->audio.rate_control_delta * direction;
 
-   src_data.ratio = audio_driver_data.audio_rate.source_ratio.current;
+#if 0
+      RARCH_LOG_OUTPUT("Audio buffer is %u%% full\n",
+            (unsigned)(100 - (avail * 100) / audio_driver_buffer_size));
+#endif
 
-   if (runloop_ctl(RUNLOOP_CTL_IS_SLOWMOTION, NULL))
+      audio_driver_free_samples_buf[write_idx] = avail;
+      audio_source_ratio_current   = 
+         audio_source_ratio_original * adjust;
+
+#if 0
+      RARCH_LOG_OUTPUT("New rate: %lf, Orig rate: %lf\n",
+            audio_source_ratio_current,
+            audio_source_ratio_original);
+#endif
+   }
+
+   src_data.ratio = audio_source_ratio_current;
+
+   if (is_slowmotion)
       src_data.ratio *= settings->slowmotion_ratio;
 
-   audio_driver_process_resampler(&src_data);
+   performance_counter_init(resampler_proc, "resampler_proc");
+   performance_counter_start_plus(is_perfcnt_enable, resampler_proc);
 
-   output_data   = audio_driver_data.output_samples.buf;
+   audio_driver_resampler->process(audio_driver_resampler_data, &src_data);
+   performance_counter_stop_plus(is_perfcnt_enable, resampler_proc);
+
+   output_data   = audio_driver_output_samples_buf;
    output_frames = src_data.output_frames;
 
-   if (!audio_driver_data.use_float)
+   if (!audio_driver_use_float)
    {
-      performance_counter_init(&audio_convert_float, "audio_convert_float");
-      performance_counter_start(&audio_convert_float);
-      convert_float_to_s16(audio_driver_data.output_samples.conv_buf,
-            (const float*)output_data, output_frames * 2);
-      performance_counter_stop(&audio_convert_float);
+      static struct retro_perf_counter audio_convert_float = {0};
 
-      output_data = audio_driver_data.output_samples.conv_buf;
+      performance_counter_init(audio_convert_float, "audio_convert_float");
+      performance_counter_start_plus(is_perfcnt_enable, audio_convert_float);
+      convert_float_to_s16(audio_driver_output_samples_conv_buf,
+            (const float*)output_data, output_frames * 2);
+      performance_counter_stop_plus(is_perfcnt_enable, audio_convert_float);
+
+      output_data = audio_driver_output_samples_conv_buf;
       output_size = sizeof(int16_t);
    }
 
    if (current_audio->write(audio_driver_context_audio_data,
-            output_data, output_frames * output_size * 2) < 0)
+            output_data, output_frames * output_size * 2,
+            is_perfcnt_enable) < 0)
    {
-      audio_driver_unset_active();
+      audio_driver_active = false;
       return false;
    }
 
@@ -615,16 +646,16 @@ static bool audio_driver_flush(const int16_t *data, size_t samples)
  **/
 void audio_driver_sample(int16_t left, int16_t right)
 {
-   audio_driver_data.output_samples.conv_buf[audio_driver_data.data_ptr++] = left;
-   audio_driver_data.output_samples.conv_buf[audio_driver_data.data_ptr++] = right;
+   audio_driver_output_samples_conv_buf[audio_driver_data_ptr++] = left;
+   audio_driver_output_samples_conv_buf[audio_driver_data_ptr++] = right;
 
-   if (audio_driver_data.data_ptr < audio_driver_data.chunk.size)
+   if (audio_driver_data_ptr < audio_driver_chunk_size)
       return;
 
-   audio_driver_flush(audio_driver_data.output_samples.conv_buf, 
-         audio_driver_data.data_ptr);
+   audio_driver_flush(audio_driver_output_samples_conv_buf, 
+         audio_driver_data_ptr);
 
-   audio_driver_data.data_ptr = 0;
+   audio_driver_data_ptr = 0;
 }
 
 /**
@@ -658,8 +689,8 @@ size_t audio_driver_sample_batch(const int16_t *data, size_t frames)
  **/
 void audio_driver_sample_rewind(int16_t left, int16_t right)
 {
-   audio_driver_data.rewind.buf[--audio_driver_data.rewind.ptr] = right;
-   audio_driver_data.rewind.buf[--audio_driver_data.rewind.ptr] = left;
+   audio_driver_rewind_buf[--audio_driver_rewind_ptr] = right;
+   audio_driver_rewind_buf[--audio_driver_rewind_ptr] = left;
 }
 
 /**
@@ -681,35 +712,55 @@ size_t audio_driver_sample_batch_rewind(const int16_t *data, size_t frames)
    size_t samples   = frames << 1;
 
    for (i = 0; i < samples; i++)
-      audio_driver_data.rewind.buf[--audio_driver_data.rewind.ptr] = data[i];
+      audio_driver_rewind_buf[--audio_driver_rewind_ptr] = data[i];
 
    return frames;
 }
 
 void audio_driver_set_volume_gain(float gain)
 {
-   audio_driver_data.volume_gain = gain;
+   audio_driver_volume_gain = gain;
 }
 
 void audio_driver_dsp_filter_free(void)
 {
-   if (audio_driver_data.dsp)
-      rarch_dsp_filter_free(audio_driver_data.dsp);
-   audio_driver_data.dsp = NULL;
+   if (audio_driver_dsp)
+      retro_dsp_filter_free(audio_driver_dsp);
+   audio_driver_dsp = NULL;
 }
 
 void audio_driver_dsp_filter_init(const char *device)
 {
-   audio_driver_data.dsp = rarch_dsp_filter_new(
-         device, audio_driver_data.audio_rate.input);
+#if defined(HAVE_DYLIB) && !defined(HAVE_FILTERS_BUILTIN)
+   char basedir[PATH_MAX_LENGTH];
+   char ext_name[PATH_MAX_LENGTH];
+#endif
+   struct string_list *plugs     = NULL;
+#if defined(HAVE_DYLIB) && !defined(HAVE_FILTERS_BUILTIN)
+   fill_pathname_basedir(basedir, device, sizeof(basedir));
 
-   if (!audio_driver_data.dsp)
+   if (!frontend_driver_get_core_extension(ext_name, sizeof(ext_name)))
+      goto error;
+
+   plugs = dir_list_new(basedir, ext_name, false, true, false, false);
+   if (!plugs)
+      goto error;
+#endif
+   audio_driver_dsp = retro_dsp_filter_new(
+         device, plugs, audio_driver_input);
+   if (!audio_driver_dsp)
+      goto error;
+
+   return;
+
+error:
+   if (!audio_driver_dsp)
       RARCH_ERR("[DSP]: Failed to initialize DSP filter \"%s\".\n", device);
 }
 
 void audio_driver_set_buffer_size(size_t bufsize)
 {
-   audio_driver_data.driver_buffer_size = bufsize;
+   audio_driver_buffer_size = bufsize;
 }
 
 void audio_driver_monitor_adjust_system_rates(void)
@@ -725,14 +776,14 @@ void audio_driver_monitor_adjust_system_rates(void)
    if (!info || info->sample_rate <= 0.0)
       return;
 
-   timing_skew        = fabs(1.0f - info->fps / settings->video.refresh_rate);
-   audio_driver_data.audio_rate.input = info->sample_rate;
+   timing_skew             = fabs(1.0f - info->fps / settings->video.refresh_rate);
+   audio_driver_input      = info->sample_rate;
 
    if (timing_skew <= settings->audio.max_timing_skew)
-      audio_driver_data.audio_rate.input *= (settings->video.refresh_rate / info->fps);
+      audio_driver_input *= (settings->video.refresh_rate / info->fps);
 
    RARCH_LOG("Set audio input rate to: %.2f Hz.\n",
-         audio_driver_data.audio_rate.input);
+         audio_driver_input);
 }
 
 void audio_driver_setup_rewind(void)
@@ -740,18 +791,18 @@ void audio_driver_setup_rewind(void)
    unsigned i;
 
    /* Push audio ready to be played. */
-   audio_driver_data.rewind.ptr = audio_driver_data.rewind.size;
+   audio_driver_rewind_ptr = audio_driver_rewind_size;
 
-   for (i = 0; i < audio_driver_data.data_ptr; i += 2)
+   for (i = 0; i < audio_driver_data_ptr; i += 2)
    {
-      audio_driver_data.rewind.buf[--audio_driver_data.rewind.ptr] =
-         audio_driver_data.output_samples.conv_buf[i + 1];
+      audio_driver_rewind_buf[--audio_driver_rewind_ptr] =
+         audio_driver_output_samples_conv_buf[i + 1];
 
-      audio_driver_data.rewind.buf[--audio_driver_data.rewind.ptr] =
-         audio_driver_data.output_samples.conv_buf[i + 0];
+      audio_driver_rewind_buf[--audio_driver_rewind_ptr] =
+         audio_driver_output_samples_conv_buf[i + 0];
    }
 
-   audio_driver_data.data_ptr = 0;
+   audio_driver_data_ptr = 0;
 }
 
 bool audio_driver_find_driver(void)
@@ -790,8 +841,10 @@ bool audio_driver_find_driver(void)
 
 void audio_driver_deinit_resampler(void)
 {
-   rarch_resampler_freep(&audio_driver_resampler,
-         &audio_driver_resampler_data);
+   if (audio_driver_resampler && audio_driver_resampler_data)
+      audio_driver_resampler->free(audio_driver_resampler_data);
+   audio_driver_resampler      = NULL;
+   audio_driver_resampler_data = NULL;
 }
 
 bool audio_driver_free_devices_list(void)
@@ -831,29 +884,11 @@ bool audio_driver_get_devices_list(void **data)
    return true;
 }
 
-bool audio_driver_init_resampler(void)
-{
-   settings_t *settings = config_get_ptr();
-   return rarch_resampler_realloc(
-         &audio_driver_resampler_data,
-         &audio_driver_resampler,
-         settings->audio.resampler,
-         audio_driver_data.audio_rate.source_ratio.original);
-}
-
-void audio_driver_process_resampler(struct resampler_data *data)
-{
-   performance_counter_init(&resampler_proc, "resampler_proc");
-   performance_counter_start(&resampler_proc);
-   rarch_resampler_process(audio_driver_resampler, 
-         audio_driver_resampler_data, data);
-   performance_counter_stop(&resampler_proc);
-}
 
 bool audio_driver_deinit(void)
 {
    audio_driver_free_devices_list();
-   if (!uninit_audio())
+   if (!audio_driver_deinit_internal())
       return false;
    return true;
 }
@@ -861,17 +896,7 @@ bool audio_driver_deinit(void)
 bool audio_driver_set_callback(const void *data)
 {
    const struct retro_audio_callback *cb = (const struct retro_audio_callback*)data;
-#ifdef HAVE_NETPLAY
-   global_t *global = global_get_ptr();
-#endif
 
-   if (recording_driver_get_data_ptr()) /* A/V sync is a must. */
-      return false;
-
-#ifdef HAVE_NETPLAY
-   if (global->netplay.enable)
-      return false;
-#endif
    if (cb)
       audio_callback = *cb;
 
@@ -880,7 +905,7 @@ bool audio_driver_set_callback(const void *data)
 
 bool audio_driver_enable_callback(void)
 {
-   if (!audio_driver_has_callback())
+   if (!audio_callback.callback)
       return false; 
    if (audio_callback.set_state)
       audio_callback.set_state(true);
@@ -889,7 +914,7 @@ bool audio_driver_enable_callback(void)
 
 bool audio_driver_disable_callback(void)
 {
-   if (!audio_driver_has_callback())
+   if (!audio_callback.callback)
       return false;
 
    if (audio_callback.set_state)
@@ -902,15 +927,15 @@ void audio_driver_monitor_set_rate(void)
 {
    settings_t *settings = config_get_ptr();
    double new_src_ratio = (double)settings->audio.out_rate / 
-      audio_driver_data.audio_rate.input;
+      audio_driver_input;
 
-   audio_driver_data.audio_rate.source_ratio.original = new_src_ratio;
-   audio_driver_data.audio_rate.source_ratio.current  = new_src_ratio;
+   audio_source_ratio_original = new_src_ratio;
+   audio_source_ratio_current  = new_src_ratio;
 }
 
 bool audio_driver_callback(void)
 {
-   if (!audio_driver_has_callback())
+   if (!audio_callback.callback)
       return false;
 
    if (audio_callback.callback)
@@ -929,7 +954,7 @@ bool audio_driver_toggle_mute(void)
    settings_t *settings = config_get_ptr();
    if (!audio_driver_context_audio_data)
       return false;
-   if (!audio_driver_is_active())
+   if (!audio_driver_active)
       return false;
 
    settings->audio.mute_enable = !settings->audio.mute_enable;
@@ -938,18 +963,18 @@ bool audio_driver_toggle_mute(void)
       command_event(CMD_EVENT_AUDIO_STOP, NULL);
    else if (!command_event(CMD_EVENT_AUDIO_START, NULL))
    {
-      audio_driver_unset_active();
+      audio_driver_active = false;
       return false;
    }
    return true;
 }
 
-bool audio_driver_start(void)
+bool audio_driver_start(bool is_shutdown)
 {
    if (!current_audio || !current_audio->start 
          || !audio_driver_context_audio_data)
       return false;
-   return current_audio->start(audio_driver_context_audio_data);
+   return current_audio->start(audio_driver_context_audio_data, is_shutdown);
 }
 
 bool audio_driver_stop(void)
@@ -968,18 +993,19 @@ void audio_driver_unset_callback(void)
 
 bool audio_driver_alive(void)
 {
-   if (!current_audio || !current_audio->alive 
-         || !audio_driver_context_audio_data)
-      return false;
-   return current_audio->alive(audio_driver_context_audio_data);
+   if (     current_audio 
+         && current_audio->alive 
+         && audio_driver_context_audio_data)
+      return current_audio->alive(audio_driver_context_audio_data);
+   return false;
 }
 
 void audio_driver_frame_is_reverse(void)
 {
    /* We just rewound. Flush rewind audio buffer. */
    audio_driver_flush(
-         audio_driver_data.rewind.buf + audio_driver_data.rewind.ptr,
-         audio_driver_data.rewind.size - audio_driver_data.rewind.ptr);
+         audio_driver_rewind_buf + audio_driver_rewind_ptr,
+         audio_driver_rewind_size - audio_driver_rewind_ptr);
 }
 
 void audio_driver_destroy_data(void)
@@ -1012,11 +1038,6 @@ void audio_driver_unset_active(void)
    audio_driver_active = false;
 }
 
-bool audio_driver_is_active(void)
-{
-   return audio_driver_active;
-}
-
 void audio_driver_destroy(void)
 {
    audio_driver_active   = false;
