@@ -1,5 +1,5 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -13,23 +13,27 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <compat/strl.h>
 #include <file/file_path.h>
 #include <string/stdstring.h>
 #include <lists/string_list.h>
 
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
 #include "../menu_driver.h"
 #include "../menu_cbs.h"
 #include "../../msg_hash.h"
 
-#ifdef HAVE_LIBRETRODB
 #include "../../database_info.h"
-#endif
 
 #include "../../cores/internal_cores.h"
 
+#include "../../configuration.h"
+#include "../../core.h"
 #include "../../core_info.h"
-#include "../../general.h"
-#include "../../system.h"
+#include "../../runloop.h"
 #include "../../verbosity.h"
 
 #ifndef BIND_ACTION_DEFERRED_PUSH
@@ -69,16 +73,16 @@ static int deferred_push_network_information(menu_displaylist_info_t *info)
    return deferred_push_dlist(info, DISPLAYLIST_NETWORK_INFO);
 }
 
-static int deferred_push_debug_information(menu_displaylist_info_t *info)
-{
-   return deferred_push_dlist(info, DISPLAYLIST_DEBUG_INFO);
-}
-
 static int deferred_push_achievement_list(menu_displaylist_info_t *info)
 {
    return deferred_push_dlist(info, DISPLAYLIST_ACHIEVEMENT_LIST);
 }
 
+static int deferred_push_achievement_list_hardcore(menu_displaylist_info_t *info)
+{
+   return deferred_push_dlist(info, DISPLAYLIST_ACHIEVEMENT_LIST_HARDCORE);
+}
+
 static int deferred_push_rdb_collection(menu_displaylist_info_t *info)
 {
    return deferred_push_dlist(info, DISPLAYLIST_PLAYLIST_COLLECTION);
@@ -144,11 +148,21 @@ static int deferred_push_onscreen_display_settings_list(menu_displaylist_info_t
    return deferred_push_dlist(info, DISPLAYLIST_ONSCREEN_DISPLAY_SETTINGS_LIST);
 }
 
+static int deferred_push_onscreen_notifications_settings_list(menu_displaylist_info_t *info)
+{
+   return deferred_push_dlist(info, DISPLAYLIST_ONSCREEN_NOTIFICATIONS_SETTINGS_LIST);
+}
+
 static int deferred_push_onscreen_overlay_settings_list(menu_displaylist_info_t *info)
 {
    return deferred_push_dlist(info, DISPLAYLIST_ONSCREEN_OVERLAY_SETTINGS_LIST);
 }
 
+static int deferred_push_menu_file_browser_settings_list(menu_displaylist_info_t *info)
+{
+   return deferred_push_dlist(info, DISPLAYLIST_MENU_FILE_BROWSER_SETTINGS_LIST);
+}
+
 static int deferred_push_menu_settings_list(menu_displaylist_info_t *info)
 {
    return deferred_push_dlist(info, DISPLAYLIST_MENU_SETTINGS_LIST);
@@ -169,11 +183,26 @@ static int deferred_push_updater_settings_list(menu_displaylist_info_t *info)
    return deferred_push_dlist(info, DISPLAYLIST_UPDATER_SETTINGS_LIST);
 }
 
+static int deferred_push_wifi_settings_list(menu_displaylist_info_t *info)
+{
+   return deferred_push_dlist(info, DISPLAYLIST_WIFI_SETTINGS_LIST);
+}
+
 static int deferred_push_network_settings_list(menu_displaylist_info_t *info)
 {
    return deferred_push_dlist(info, DISPLAYLIST_NETWORK_SETTINGS_LIST);
 }
 
+static int deferred_push_netplay_lan_scan_settings_list(menu_displaylist_info_t *info)
+{
+   return deferred_push_dlist(info, DISPLAYLIST_NETPLAY_LAN_SCAN_SETTINGS_LIST);
+}
+
+static int deferred_push_lakka_services_list(menu_displaylist_info_t *info)
+{
+   return deferred_push_dlist(info, DISPLAYLIST_LAKKA_SERVICES_LIST);
+}
+
 static int deferred_push_user_settings_list(menu_displaylist_info_t *info)
 {
    return deferred_push_dlist(info, DISPLAYLIST_USER_SETTINGS_LIST);
@@ -199,6 +228,11 @@ static int deferred_push_input_settings_list(menu_displaylist_info_t *info)
    return deferred_push_dlist(info, DISPLAYLIST_INPUT_SETTINGS_LIST);
 }
 
+static int deferred_push_recording_settings_list(menu_displaylist_info_t *info)
+{
+   return deferred_push_dlist(info, DISPLAYLIST_RECORDING_SETTINGS_LIST);
+}
+
 static int deferred_push_playlist_settings_list(menu_displaylist_info_t *info)
 {
    return deferred_push_dlist(info, DISPLAYLIST_PLAYLIST_SETTINGS_LIST);
@@ -276,6 +310,11 @@ static int deferred_push_options(menu_displaylist_info_t *info)
    return deferred_push_dlist(info, DISPLAYLIST_OPTIONS);
 }
 
+static int deferred_push_netplay(menu_displaylist_info_t *info)
+{
+   return deferred_push_dlist(info, DISPLAYLIST_NETPLAY);
+}
+
 static int deferred_push_content_settings(menu_displaylist_info_t *info)
 {
    return deferred_push_dlist(info, DISPLAYLIST_CONTENT_SETTINGS);
@@ -286,6 +325,11 @@ static int deferred_push_add_content_list(menu_displaylist_info_t *info)
    return deferred_push_dlist(info, DISPLAYLIST_ADD_CONTENT_LIST);
 }
 
+static int deferred_push_configurations_list(menu_displaylist_info_t *info)
+{
+   return deferred_push_dlist(info, DISPLAYLIST_CONFIGURATIONS_LIST);
+}
+
 static int deferred_push_load_content_list(menu_displaylist_info_t *info)
 {
    return deferred_push_dlist(info, DISPLAYLIST_LOAD_CONTENT_LIST);
@@ -377,7 +421,7 @@ static int deferred_archive_action(menu_displaylist_info_t *info)
 static int deferred_push_cursor_manager_list_deferred(
       menu_displaylist_info_t *info)
 {
-   char rdb_path[PATH_MAX_LENGTH] = {0};
+   char rdb_path[PATH_MAX_LENGTH];
    int ret                        = -1;
    char *query                    = NULL;
    char *rdb                      = NULL;
@@ -393,6 +437,8 @@ static int deferred_push_cursor_manager_list_deferred(
    if (!config_get_string(conf, "rdb", &rdb))
       goto end;
 
+   rdb_path[0] = '\0';
+
    fill_pathname_join(rdb_path, settings->path.content_database,
          rdb, sizeof(rdb_path));
 
@@ -410,14 +456,141 @@ end:
    return ret;
 }
 
+
+#ifdef HAVE_LIBRETRODB
+static int deferred_push_cursor_manager_list_generic(
+      menu_displaylist_info_t *info, enum database_query_type type)
+{
+   int ret                       = -1;
+   char query[PATH_MAX_LENGTH];
+   struct string_list *str_list  = string_split(info->path, "|"); 
+
+   query[0] = '\0';
+
+   database_info_build_query_enum(query, sizeof(query), type, str_list->elems[0].data);
+
+   if (string_is_empty(query))
+      goto end;
+
+   strlcpy(info->path,   str_list->elems[1].data, sizeof(info->path));
+   strlcpy(info->path_b, str_list->elems[0].data, sizeof(info->path_b));
+   strlcpy(info->path_c, query, sizeof(info->path_c));
+
+   ret = deferred_push_dlist(info, DISPLAYLIST_DATABASE_QUERY);
+
+end:
+   string_list_free(str_list);
+   return ret;
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_max_users(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_MAX_USERS);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_famitsu_magazine_rating(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_FAMITSU_MAGAZINE_RATING);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_edge_magazine_rating(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_EDGE_MAGAZINE_RATING);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_edge_magazine_issue(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_EDGE_MAGAZINE_ISSUE);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_elspa_rating(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_ELSPA_RATING);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_cero_rating(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_CERO_RATING);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_pegi_rating(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_PEGI_RATING);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_bbfc_rating(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_BBFC_RATING);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_esrb_rating(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_ESRB_RATING);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_enhancement_hw(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_ENHANCEMENT_HW);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_franchise(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_FRANCHISE);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_publisher(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_PUBLISHER);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_developer(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_DEVELOPER);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_origin(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_ORIGIN);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_releasemonth(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_RELEASEDATE_MONTH);
+}
+
+static int deferred_push_cursor_manager_list_deferred_query_rdb_entry_releaseyear(
+      menu_displaylist_info_t *info)
+{
+   return deferred_push_cursor_manager_list_generic(info, DATABASE_QUERY_ENTRY_RELEASEDATE_YEAR);
+}
+#endif
+
+#if 0
 static int deferred_push_cursor_manager_list_deferred_query_subsearch(
       menu_displaylist_info_t *info)
 {
    int ret                       = -1;
 #ifdef HAVE_LIBRETRODB
-   char query[PATH_MAX_LENGTH]   = {0};
+   char query[PATH_MAX_LENGTH];
    struct string_list *str_list  = string_split(info->path, "|"); 
 
+   query[0] = '\0';
+
    database_info_build_query(query, sizeof(query),
          info->label, str_list->elems[0].data);
 
@@ -435,7 +608,7 @@ end:
 #endif
    return ret;
 }
-
+#endif
 
 static int general_push(menu_displaylist_info_t *info,
       unsigned id, enum menu_displaylist_ctl_state state)
@@ -483,17 +656,17 @@ static int general_push(menu_displaylist_info_t *info,
    switch (id)
    {
       case PUSH_ARCHIVE_OPEN_DETECT_CORE:
-  
-         if (!string_is_empty(list->all_ext))
+         if (list && !string_is_empty(list->all_ext))
             strlcpy(info->exts, list->all_ext, sizeof(info->exts));
          else if (system_menu->valid_extensions)
          {
-            if (*system_menu->valid_extensions)
+            if (!string_is_empty(system_menu->valid_extensions))
                strlcpy(info->exts, system_menu->valid_extensions,
                      sizeof(info->exts));
          }
          else
             strlcpy(info->exts, system->valid_extensions, sizeof(info->exts));
+
          break;
       case PUSH_ARCHIVE_OPEN:
          if (system_menu->valid_extensions)
@@ -522,8 +695,14 @@ static int general_push(menu_displaylist_info_t *info,
          }
          break;
       case PUSH_DETECT_CORE_LIST:
-         if (!string_is_empty(list->all_ext))
+         if (list && !string_is_empty(list->all_ext))
             strlcpy(info->exts, list->all_ext, sizeof(info->exts));
+         else if (system_menu->valid_extensions)
+         {
+            if (!string_is_empty(system_menu->valid_extensions))
+               strlcpy(info->exts, system_menu->valid_extensions,
+                     sizeof(info->exts));
+         }
          break;
    }
 
@@ -598,6 +777,16 @@ static int deferred_push_content_collection_list(menu_displaylist_info_t *info)
    return deferred_push_dlist(info, DISPLAYLIST_DATABASE_PLAYLISTS);
 }
 
+static int deferred_push_browse_url_list(menu_displaylist_info_t *info)
+{
+   return deferred_push_dlist(info, DISPLAYLIST_BROWSE_URL_LIST);
+}
+
+static int deferred_push_browse_url_start(menu_displaylist_info_t *info)
+{
+   return deferred_push_dlist(info, DISPLAYLIST_BROWSE_URL_START);
+}
+
 static int deferred_push_core_list(menu_displaylist_info_t *info)
 {
    return deferred_push_dlist(info, DISPLAYLIST_CORES);
@@ -653,11 +842,6 @@ static int deferred_push_input_overlay(menu_displaylist_info_t *info)
    return deferred_push_dlist(info, DISPLAYLIST_OVERLAYS);
 }
 
-static int deferred_push_input_osk_overlay(menu_displaylist_info_t *info)
-{
-   return deferred_push_dlist(info, DISPLAYLIST_OVERLAYS);
-}
-
 static int deferred_push_video_font_path(menu_displaylist_info_t *info)
 {
    return deferred_push_dlist(info, DISPLAYLIST_FONTS);
@@ -677,6 +861,16 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
       menu_file_list_cbs_t *cbs, 
       const char *label, uint32_t label_hash)
 {
+   if (string_is_equal(label, msg_hash_to_str(MENU_ENUM_LABEL_DEFERRED_BROWSE_URL_LIST)))
+   {
+      BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_browse_url_list);
+      return 0;
+   }
+   if (string_is_equal(label, msg_hash_to_str(MENU_ENUM_LABEL_DEFERRED_BROWSE_URL_START)))
+   {
+      BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_browse_url_start);
+      return 0;
+   }
    if (string_is_equal(label, msg_hash_to_str(MENU_ENUM_LABEL_DEFERRED_CORE_SETTINGS_LIST)))
    {
       BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_core_settings_list);
@@ -712,11 +906,21 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
       BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_onscreen_display_settings_list);
       return 0;
    }
+   if (string_is_equal(label, msg_hash_to_str(MENU_ENUM_LABEL_DEFERRED_ONSCREEN_NOTIFICATIONS_SETTINGS_LIST)))
+   {
+      BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_onscreen_notifications_settings_list);
+      return 0;
+   }
    if (string_is_equal(label, msg_hash_to_str(MENU_ENUM_LABEL_DEFERRED_ONSCREEN_OVERLAY_SETTINGS_LIST)))
    {
       BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_onscreen_overlay_settings_list);
       return 0;
    }
+   if (string_is_equal(label, msg_hash_to_str(MENU_ENUM_LABEL_DEFERRED_MENU_FILE_BROWSER_SETTINGS_LIST)))
+   {
+      BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_menu_file_browser_settings_list);
+      return 0;
+   }
    if (string_is_equal(label, msg_hash_to_str(MENU_ENUM_LABEL_DEFERRED_MENU_SETTINGS_LIST)))
    {
       BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_menu_settings_list);
@@ -746,6 +950,24 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
       return 0;
    }
 
+   if (string_is_equal(label, msg_hash_to_str(MENU_ENUM_LABEL_DEFERRED_WIFI_SETTINGS_LIST)))
+   {
+      BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_wifi_settings_list);
+      return 0;
+   }
+
+   if (string_is_equal(label, msg_hash_to_str(MENU_ENUM_LABEL_DEFERRED_NETPLAY_LAN_SCAN_SETTINGS_LIST)))
+   {
+      BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_netplay_lan_scan_settings_list);
+      return 0;
+   }
+
+   if (string_is_equal(label, msg_hash_to_str(MENU_ENUM_LABEL_DEFERRED_LAKKA_SERVICES_LIST)))
+   {
+      BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_lakka_services_list);
+      return 0;
+   }
+
    if (string_is_equal(label, msg_hash_to_str(MENU_ENUM_LABEL_DEFERRED_USER_SETTINGS_LIST)))
    {
       BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_user_settings_list);
@@ -810,6 +1032,9 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
             case MENU_ENUM_LABEL_DEFERRED_PLAYLIST_SETTINGS_LIST:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_playlist_settings_list);
                break;
+            case MENU_ENUM_LABEL_DEFERRED_RECORDING_SETTINGS_LIST:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_recording_settings_list);
+               break;
             case MENU_ENUM_LABEL_DEFERRED_INPUT_HOTKEY_BINDS_LIST:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_input_hotkey_binds_list);
                break;
@@ -882,12 +1107,18 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
             case MENU_ENUM_LABEL_ONLINE_UPDATER:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_options);
                break;
+            case MENU_ENUM_LABEL_NETPLAY:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_netplay);
+               break;
             case MENU_ENUM_LABEL_CONTENT_SETTINGS:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_content_settings);
                break;
             case MENU_ENUM_LABEL_ADD_CONTENT_LIST:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_add_content_list);
                break;
+            case MENU_ENUM_LABEL_CONFIGURATIONS_LIST:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_configurations_list);
+               break;
             case MENU_ENUM_LABEL_LOAD_CONTENT_LIST:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_load_content_list);
                break;
@@ -915,24 +1146,56 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
             case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred);
                break;
+#ifdef HAVE_LIBRETRODB
             case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_PUBLISHER:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_DEVELOPER:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ORIGIN:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_FRANCHISE:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ENHANCEMENT_HW:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ESRB_RATING:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_BBFC_RATING:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ELSPA_RATING:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_PEGI_RATING:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_CERO_RATING:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_EDGE_MAGAZINE_RATING:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_EDGE_MAGAZINE_ISSUE:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_FAMITSU_MAGAZINE_RATING:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_MAX_USERS:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_RELEASEMONTH:
-            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_RELEASEYEAR:
-               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_subsearch);
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_publisher);
                break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_DEVELOPER:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_developer);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ORIGIN:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_origin);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_FRANCHISE:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_franchise);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ENHANCEMENT_HW:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_enhancement_hw);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ESRB_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_esrb_rating);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_BBFC_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_bbfc_rating);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ELSPA_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_elspa_rating);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_PEGI_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_pegi_rating);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_CERO_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_cero_rating);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_EDGE_MAGAZINE_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_edge_magazine_rating);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_EDGE_MAGAZINE_ISSUE:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_edge_magazine_issue);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_FAMITSU_MAGAZINE_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_famitsu_magazine_rating);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_MAX_USERS:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_max_users);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_RELEASEMONTH:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_releasemonth);
+               break;
+            case MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_RELEASEYEAR:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_releaseyear);
+               break;
+#endif
             case MENU_ENUM_LABEL_CORE_INFORMATION:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_core_information);
                break;
@@ -942,12 +1205,12 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
             case MENU_ENUM_LABEL_NETWORK_INFORMATION:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_network_information);
                break;
-            case MENU_ENUM_LABEL_DEBUG_INFORMATION:
-               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_debug_information);
-               break;
             case MENU_ENUM_LABEL_ACHIEVEMENT_LIST:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_achievement_list);
                break;
+            case MENU_ENUM_LABEL_ACHIEVEMENT_LIST_HARDCORE:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_achievement_list_hardcore);
+               break;
             case MENU_ENUM_LABEL_CORE_COUNTERS:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_core_counters);
                break;
@@ -999,9 +1262,6 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
             case MENU_ENUM_LABEL_INPUT_OVERLAY:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_input_overlay);
                break;
-            case MENU_ENUM_LABEL_INPUT_OSK_OVERLAY:
-               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_input_osk_overlay);
-               break;
             case MENU_ENUM_LABEL_VIDEO_FONT_PATH:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_video_font_path);
                break;
@@ -1042,7 +1302,7 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_core_settings_list);
                break;
             case MENU_ENUM_LABEL_DOWNLOADED_FILE_DETECT_CORE_LIST:
-            case MENU_ENUM_LABEL_DETECT_CORE_LIST:
+            case MENU_ENUM_LABEL_FAVORITES:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_detect_core_list);
                break;
             default:
@@ -1053,6 +1313,9 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
       {
          switch (label_hash)
          {
+            case MENU_LABEL_DEFERRED_CONFIGURATIONS_LIST:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_configurations_list);
+               break;
             case MENU_LABEL_DEFERRED_USER_BINDS_LIST:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_user_binds_list);
                break;
@@ -1074,6 +1337,9 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
             case MENU_LABEL_DEFERRED_PLAYLIST_SETTINGS_LIST:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_playlist_settings_list);
                break;
+            case MENU_LABEL_DEFERRED_RECORDING_SETTINGS_LIST:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_recording_settings_list);
+               break;
             case MENU_LABEL_DEFERRED_INPUT_HOTKEY_BINDS_LIST:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_input_hotkey_binds_list);
                break;
@@ -1136,6 +1402,9 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
             case MENU_LABEL_ONLINE_UPDATER:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_options);
                break;
+            case MENU_LABEL_NETPLAY:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_netplay);
+               break;
             case MENU_LABEL_CONTENT_SETTINGS:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_content_settings);
                break;
@@ -1166,27 +1435,59 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
             case MENU_LABEL_DEFERRED_DATABASE_MANAGER_LIST:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_database_manager_list_deferred);
                break;
+#ifdef HAVE_LIBRETRODB
             case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred);
                break;
             case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_PUBLISHER:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_DEVELOPER:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ORIGIN:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_FRANCHISE:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ENHANCEMENT_HW:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ESRB_RATING:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_BBFC_RATING:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ELSPA_RATING:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_PEGI_RATING:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_CERO_RATING:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_EDGE_MAGAZINE_RATING:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_EDGE_MAGAZINE_ISSUE:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_FAMITSU_MAGAZINE_RATING:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_MAX_USERS:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_RELEASEMONTH:
-            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_RELEASEYEAR:
-               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_subsearch);
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_publisher);
                break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_DEVELOPER:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_developer);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ORIGIN:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_origin);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_FRANCHISE:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_franchise);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ENHANCEMENT_HW:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_enhancement_hw);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ESRB_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_esrb_rating);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_BBFC_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_bbfc_rating);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ELSPA_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_elspa_rating);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_PEGI_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_pegi_rating);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_CERO_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_cero_rating);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_EDGE_MAGAZINE_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_edge_magazine_rating);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_EDGE_MAGAZINE_ISSUE:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_edge_magazine_issue);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_FAMITSU_MAGAZINE_RATING:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_famitsu_magazine_rating);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_MAX_USERS:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_max_users);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_RELEASEMONTH:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_releasemonth);
+               break;
+            case MENU_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_RELEASEYEAR:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_cursor_manager_list_deferred_query_rdb_entry_releaseyear);
+               break;
+#endif
             case MENU_LABEL_CORE_INFORMATION:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_core_information);
                break;
@@ -1196,12 +1497,12 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
             case MENU_LABEL_NETWORK_INFORMATION:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_network_information);
                break;
-            case MENU_LABEL_DEBUG_INFORMATION:
-               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_debug_information);
-               break;
             case MENU_LABEL_ACHIEVEMENT_LIST:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_achievement_list);
                break;
+            case MENU_LABEL_ACHIEVEMENT_LIST_HARDCORE:
+               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_achievement_list_hardcore);
+               break;
             case MENU_LABEL_CORE_COUNTERS:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_core_counters);
                break;
@@ -1253,9 +1554,6 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
             case MENU_LABEL_INPUT_OVERLAY:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_input_overlay);
                break;
-            case MENU_LABEL_INPUT_OSK_OVERLAY:
-               BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_input_osk_overlay);
-               break;
             case MENU_LABEL_VIDEO_FONT_PATH:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_video_font_path);
                break;
@@ -1266,7 +1564,7 @@ static int menu_cbs_init_bind_deferred_push_compare_label(
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_content_history_path);
                break;
             case MENU_LABEL_DOWNLOADED_FILE_DETECT_CORE_LIST:
-            case MENU_LABEL_DETECT_CORE_LIST:
+            case MENU_LABEL_FAVORITES:
                BIND_ACTION_DEFERRED_PUSH(cbs, deferred_push_detect_core_list);
                break;
             default:
