@@ -1,5 +1,5 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -14,26 +14,33 @@
  */
 
 #include <file/file_path.h>
+#include <compat/strl.h>
 #include <string/stdstring.h>
 #include <lists/string_list.h>
 
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
 #include "../menu_driver.h"
 #include "../menu_animation.h"
 #include "../menu_cbs.h"
 #include "../menu_shader.h"
 #include "../menu_display.h"
 
-#include "../../input/input_autodetect.h"
+#include "../../tasks/tasks_internal.h"
 #include "../../input/input_config.h"
 
+#include "../../core.h"
 #include "../../core_info.h"
+#include "../../configuration.h"
 #include "../../file_path_special.h"
 #include "../../managers/core_option_manager.h"
 #include "../../managers/cheat_manager.h"
-#include "../../general.h"
 #include "../../performance_counters.h"
-#include "../../system.h"
-#include "../../intl/intl.h"
+#include "../../paths.h"
+#include "../../runloop.h"
+#include "../../wifi/wifi_driver.h"
 
 #ifndef BIND_ACTION_GET_VALUE
 #define BIND_ACTION_GET_VALUE(cbs, name) \
@@ -41,13 +48,6 @@
    cbs->action_get_value_ident = #name;
 #endif
 
-const char axis_labels[4][128] = {
-   RETRO_LBL_ANALOG_LEFT_X,
-   RETRO_LBL_ANALOG_LEFT_Y,
-   RETRO_LBL_ANALOG_RIGHT_X,
-   RETRO_LBL_ANALOG_RIGHT_Y
-};
-
 static void menu_action_setting_disp_set_label_cheat_num_passes(
       file_list_t* list,
       unsigned *w, unsigned type, unsigned i,
@@ -119,12 +119,11 @@ static void menu_action_setting_disp_set_label_configurations(
       const char *path,
       char *s2, size_t len2)
 {
-   global_t *global = global_get_ptr();
-
    *w = 19;
    strlcpy(s2, path, len2);
-   if (global && *global->path.config)
-      fill_pathname_base(s, global->path.config,
+
+   if (!path_is_empty(RARCH_PATH_CONFIG))
+      fill_pathname_base(s, path_get(RARCH_PATH_CONFIG),
             len);
    else
       strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_DIRECTORY_DEFAULT), len);
@@ -139,26 +138,20 @@ static void menu_action_setting_disp_set_label_shader_filter_pass(
       const char *path,
       char *s2, size_t len2)
 {
-   unsigned pass = 0;
 #if defined(HAVE_CG) || defined(HAVE_GLSL) || defined(HAVE_HLSL)
-   struct video_shader *shader = NULL;
+   struct video_shader_pass *shader_pass = menu_shader_manager_get_pass(
+         type - MENU_SETTINGS_SHADER_PASS_FILTER_0);
 #endif
 
-   (void)pass;
-
    *s = '\0';
    *w = 19;
    strlcpy(s2, path, len2);
 
 #if defined(HAVE_CG) || defined(HAVE_GLSL) || defined(HAVE_HLSL)
-   menu_driver_ctl(RARCH_MENU_CTL_SHADER_GET,
-         &shader);
-   if (!shader)
+   if (!shader_pass)
       return;
 
-  pass = (type - MENU_SETTINGS_SHADER_PASS_FILTER_0);
-
-  switch (shader->pass[pass].filter)
+  switch (shader_pass->filter)
   {
      case 0:
         strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_DONT_CARE),
@@ -217,15 +210,30 @@ static void menu_action_setting_disp_set_label_pipeline(
          strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_OFF), len);
          break;
       case XMB_SHADER_PIPELINE_SIMPLE_RIBBON:
-         strlcpy(s, 
+         strlcpy(s,
                msg_hash_to_str(
                   MENU_ENUM_LABEL_VALUE_SHADER_PIPELINE_RIBBON_SIMPLIFIED), len);
          break;
       case XMB_SHADER_PIPELINE_RIBBON:
-         strlcpy(s, 
+         strlcpy(s,
                msg_hash_to_str(
                   MENU_ENUM_LABEL_VALUE_SHADER_PIPELINE_RIBBON), len);
          break;
+      case XMB_SHADER_PIPELINE_SIMPLE_SNOW:
+         strlcpy(s,
+               msg_hash_to_str(
+                  MENU_ENUM_LABEL_VALUE_SHADER_PIPELINE_SIMPLE_SNOW), len);
+         break;
+      case XMB_SHADER_PIPELINE_SNOW:
+         strlcpy(s,
+               msg_hash_to_str(
+                  MENU_ENUM_LABEL_VALUE_SHADER_PIPELINE_SNOW), len);
+         break;
+      case XMB_SHADER_PIPELINE_BOKEH:
+         strlcpy(s,
+               msg_hash_to_str(
+                  MENU_ENUM_LABEL_VALUE_SHADER_PIPELINE_BOKEH), len);
+         break;
    }
 
    strlcpy(s2, path, len2);
@@ -241,18 +249,11 @@ static void menu_action_setting_disp_set_label_shader_num_passes(
       const char *path,
       char *s2, size_t len2)
 {
-#if defined(HAVE_CG) || defined(HAVE_GLSL) || defined(HAVE_HLSL)
-   struct video_shader *shader = NULL;
-#endif
-
    *s = '\0';
    *w = 19;
    strlcpy(s2, path, len2);
 #if defined(HAVE_CG) || defined(HAVE_GLSL) || defined(HAVE_HLSL)
-   menu_driver_ctl(RARCH_MENU_CTL_SHADER_GET,
-         &shader);
-   if (shader)
-      snprintf(s, len, "%u", shader->passes);
+   snprintf(s, len, "%u", menu_shader_manager_get_amount_passes());
 #endif
 }
 
@@ -266,11 +267,9 @@ static void menu_action_setting_disp_set_label_shader_pass(
       char *s2, size_t len2)
 {
 #if defined(HAVE_CG) || defined(HAVE_GLSL) || defined(HAVE_HLSL)
-   struct video_shader *shader = NULL;
+   struct video_shader_pass *shader_pass = menu_shader_manager_get_pass(
+         type - MENU_SETTINGS_SHADER_PASS_0);
 #endif
-   unsigned pass = (type - MENU_SETTINGS_SHADER_PASS_0);
-
-   (void)pass;
 
    *s = '\0';
    *w = 19;
@@ -278,14 +277,11 @@ static void menu_action_setting_disp_set_label_shader_pass(
    strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_NOT_AVAILABLE), len);
 
 #if defined(HAVE_CG) || defined(HAVE_GLSL) || defined(HAVE_HLSL)
-   menu_driver_ctl(RARCH_MENU_CTL_SHADER_GET,
-         &shader);
-   if (!shader)
+   if (!shader_pass)
       return;
 
-   if (!string_is_empty(shader->pass[pass].source.path))
-      fill_pathname_base(s,
-            shader->pass[pass].source.path, len);
+   if (!string_is_empty(shader_pass->source.path))
+      fill_pathname_base(s, shader_pass->source.path, len);
 #endif
 }
 
@@ -308,7 +304,7 @@ static void menu_action_setting_disp_set_label_shader_default_filter(
       return;
 
    if (settings->video.smooth)
-      strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_LINEAR), len); 
+      strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_LINEAR), len);
    else
       strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_NEAREST), len);
 }
@@ -336,7 +332,7 @@ static void menu_action_setting_disp_set_label_shader_parameter(
    if (!shader_info.data)
       return;
 
-   param = &shader_info.data->parameters[type - 
+   param = &shader_info.data->parameters[type -
       MENU_SETTINGS_SHADER_PARAMETER_0];
 
    if (!param)
@@ -357,8 +353,8 @@ static void menu_action_setting_disp_set_label_shader_preset_parameter(
       char *s2, size_t len2)
 {
 #if defined(HAVE_CG) || defined(HAVE_GLSL) || defined(HAVE_HLSL)
-   const struct video_shader_parameter *param = NULL;
-   struct video_shader *shader = NULL;
+   const struct video_shader_parameter *param = menu_shader_manager_get_parameters(
+         type - MENU_SETTINGS_SHADER_PRESET_PARAMETER_0);
 #endif
 
    *s = '\0';
@@ -366,18 +362,9 @@ static void menu_action_setting_disp_set_label_shader_preset_parameter(
    strlcpy(s2, path, len2);
 
 #if defined(HAVE_CG) || defined(HAVE_GLSL) || defined(HAVE_HLSL)
-   menu_driver_ctl(RARCH_MENU_CTL_SHADER_GET,
-         &shader);
-   if (!shader)
-      return;
-
-   param = &shader->parameters[type - MENU_SETTINGS_SHADER_PRESET_PARAMETER_0];
-
-   if (!param)
-      return;
-
-   snprintf(s, len, "%.2f [%.2f %.2f]",
-         param->current, param->minimum, param->maximum);
+   if (param)
+      snprintf(s, len, "%.2f [%.2f %.2f]",
+            param->current, param->minimum, param->maximum);
 #endif
 }
 
@@ -393,7 +380,8 @@ static void menu_action_setting_disp_set_label_shader_scale_pass(
    unsigned pass               = 0;
    unsigned scale_value        = 0;
 #if defined(HAVE_CG) || defined(HAVE_GLSL) || defined(HAVE_HLSL)
-   struct video_shader *shader = NULL;
+   struct video_shader_pass *shader_pass = menu_shader_manager_get_pass(
+         type - MENU_SETTINGS_SHADER_PASS_SCALE_0);
 #endif
 
    *s = '\0';
@@ -404,13 +392,10 @@ static void menu_action_setting_disp_set_label_shader_scale_pass(
    (void)scale_value;
 
 #if defined(HAVE_CG) || defined(HAVE_GLSL) || defined(HAVE_HLSL)
-   menu_driver_ctl(RARCH_MENU_CTL_SHADER_GET,
-         &shader);
-   if (!shader)
+   if (!shader_pass)
       return;
 
-   pass        = (type - MENU_SETTINGS_SHADER_PASS_SCALE_0);
-   scale_value = shader->pass[pass].fbo.scale_x;
+   scale_value = shader_pass->fbo.scale_x;
 
    if (!scale_value)
       strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_DONT_CARE), len);
@@ -448,11 +433,11 @@ static void menu_action_setting_disp_set_label_input_desc(
       const char *path,
       char *s2, size_t len2)
 {
-   char descriptor[PATH_MAX_LENGTH];
+   char descriptor[255];
    const struct retro_keybind *auto_bind = NULL;
    const struct retro_keybind *keybind   = NULL;
    settings_t *settings                  = config_get_ptr();
-   unsigned inp_desc_index_offset        = 
+   unsigned inp_desc_index_offset        =
       type - MENU_SETTINGS_INPUT_DESC_BEGIN;
    unsigned inp_desc_user                = inp_desc_index_offset /
       (RARCH_FIRST_CUSTOM_BIND + 4);
@@ -462,14 +447,16 @@ static void menu_action_setting_disp_set_label_input_desc(
 
    if (!settings)
       return;
-   
+
+   descriptor[0] = '\0';
+
    remap_id = settings->input.remap_ids
       [inp_desc_user][inp_desc_button_index_offset];
 
    keybind = (const struct retro_keybind*)
       &settings->input.binds[inp_desc_user][remap_id];
    auto_bind = (const struct retro_keybind*)
-      input_get_auto_bind(inp_desc_user, remap_id);
+      input_config_get_bind_auto(inp_desc_user, remap_id);
 
    input_config_get_bind_string(descriptor,
       keybind, auto_bind, sizeof(descriptor));
@@ -478,19 +465,39 @@ static void menu_action_setting_disp_set_label_input_desc(
    {
       if(strstr(descriptor, "Auto") && !strstr(descriptor,
                msg_hash_to_str(MENU_ENUM_LABEL_VALUE_NOT_AVAILABLE)))
-         strlcpy(s, 
+         strlcpy(s,
             descriptor,
             len);
       else
-         strlcpy(s, 
-            settings->input.binds[inp_desc_user][remap_id].desc,
+         strlcpy(s,
+            msg_hash_to_str(settings->input.binds[inp_desc_user][remap_id].enum_idx),
             len);
    }
 
 
 
    else
-      strlcpy(s, axis_labels[remap_id], len);
+   {
+      const char *str = NULL;
+      switch (remap_id)
+      {
+         case 0:
+            str = msg_hash_to_str(MENU_ENUM_LABEL_VALUE_INPUT_ANALOG_LEFT_X);
+            break;
+         case 1:
+            str = msg_hash_to_str(MENU_ENUM_LABEL_VALUE_INPUT_ANALOG_LEFT_Y);
+            break;
+         case 2:
+            str = msg_hash_to_str(MENU_ENUM_LABEL_VALUE_INPUT_ANALOG_RIGHT_X);
+            break;
+         case 3:
+            str = msg_hash_to_str(MENU_ENUM_LABEL_VALUE_INPUT_ANALOG_RIGHT_Y);
+            break;
+      }
+
+      if (!string_is_empty(str))
+         strlcpy(s, str, len);
+   }
 
    *w = 19;
    strlcpy(s2, path, len2);
@@ -511,9 +518,9 @@ static void menu_action_setting_disp_set_label_cheat(
    if (cheat_index < cheat_manager_get_buf_size())
       snprintf(s, len, "%s : (%s)",
             (cheat_manager_get_code(cheat_index) != NULL)
-            ? cheat_manager_get_code(cheat_index) : 
+            ? cheat_manager_get_code(cheat_index) :
             msg_hash_to_str(MENU_ENUM_LABEL_VALUE_NOT_AVAILABLE),
-            cheat_manager_get_code_state(cheat_index) ? 
+            cheat_manager_get_code_state(cheat_index) ?
             msg_hash_to_str(MENU_ENUM_LABEL_VALUE_ON) :
             msg_hash_to_str(MENU_ENUM_LABEL_VALUE_OFF)
             );
@@ -603,6 +610,34 @@ static void menu_action_setting_disp_set_label_menu_more(
    strlcpy(s2, path, len2);
 }
 
+static void menu_action_setting_disp_set_label_db_entry(
+      file_list_t* list,
+      unsigned *w, unsigned type, unsigned i,
+      const char *label,
+      char *s, size_t len,
+      const char *entry_label,
+      const char *path,
+      char *s2, size_t len2)
+{
+   strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_MORE), len);
+   *w = 10;
+   strlcpy(s2, path, len2);
+}
+
+static void menu_action_setting_disp_set_label_entry(
+      file_list_t* list,
+      unsigned *w, unsigned type, unsigned i,
+      const char *label,
+      char *s, size_t len,
+      const char *entry_label,
+      const char *path,
+      char *s2, size_t len2)
+{
+   *s = '\0';
+   *w = 8;
+   strlcpy(s2, path, len2);
+}
+
 static void menu_action_setting_disp_set_label_state(
       file_list_t* list,
       unsigned *w, unsigned type, unsigned i,
@@ -694,6 +729,18 @@ static void menu_action_setting_disp_set_label_xmb_theme(
          strlcpy(s,
                msg_hash_to_str(MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_PIXEL), len);
          break;
+      case XMB_ICON_THEME_NEOACTIVE:
+         strlcpy(s,
+               msg_hash_to_str(MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_NEOACTIVE), len);
+         break;
+      case XMB_ICON_THEME_SYSTEMATIC:
+         strlcpy(s,
+               msg_hash_to_str(MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_SYSTEMATIC), len);
+	 break;
+      case XMB_ICON_THEME_DOTART:
+         strlcpy(s,
+               msg_hash_to_str(MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_DOTART), len);
+         break;
       case XMB_ICON_THEME_CUSTOM:
          strlcpy(s,
                msg_hash_to_str(MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_CUSTOM), len);
@@ -701,6 +748,22 @@ static void menu_action_setting_disp_set_label_xmb_theme(
    }
 }
 
+static void menu_action_setting_disp_set_label_wifi_is_online(
+      file_list_t* list,
+      unsigned *w, unsigned type, unsigned i,
+      const char *label,
+      char *s, size_t len,
+      const char *entry_label,
+      const char *path,
+      char *s2, size_t len2)
+{
+   strlcpy(s2, path, len2);
+   *w = 19;
+
+   if (driver_wifi_ssid_is_online(i))
+      strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_ONLINE), len);
+}
+
 static void menu_action_setting_disp_set_label_xmb_menu_color_theme(
       file_list_t* list,
       unsigned *w, unsigned type, unsigned i,
@@ -871,7 +934,7 @@ static void menu_action_setting_disp_set_label_thumbnails(
                   MENU_ENUM_LABEL_VALUE_THUMBNAIL_MODE_SCREENSHOTS), len);
          break;
       case 2:
-         strlcpy(s, 
+         strlcpy(s,
                msg_hash_to_str(
                   MENU_ENUM_LABEL_VALUE_THUMBNAIL_MODE_TITLE_SCREENS), len);
          break;
@@ -899,18 +962,22 @@ static void menu_action_setting_disp_set_label_menu_toggle_gamepad_combo(
 
    strlcpy(s2, path, len2);
    *w = 19;
+
    switch (settings->input.menu_toggle_gamepad_combo)
    {
-      case 0:
+      case INPUT_TOGGLE_NONE:
          strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_NONE), len);
          break;
-      case 1:
+      case INPUT_TOGGLE_DOWN_Y_L_R:
          strlcpy(s, "Down + L1 + R1 + Y", len);
          break;
-      case 2:
+      case INPUT_TOGGLE_L3_R3:
          strlcpy(s, "L3 + R3", len);
-	 break;
-      case 3:
+         break;
+      case INPUT_TOGGLE_L1_R1_START_SELECT:
+         strlcpy(s, "L1 + R1 + Start + Select", len);
+         break;
+      case INPUT_TOGGLE_START_SELECT:
          strlcpy(s, "Start + Select", len);
          break;
    }
@@ -1050,19 +1117,6 @@ static void menu_action_setting_disp_set_label_menu_file_plain(
          path, "(FILE)", s2, len2);
 }
 
-static void menu_action_setting_disp_set_label_menu_file_image(
-      file_list_t* list,
-      unsigned *w, unsigned type, unsigned i,
-      const char *label,
-      char *s, size_t len,
-      const char *entry_label,
-      const char *path,
-      char *s2, size_t len2)
-{
-   menu_action_setting_generic_disp_set_label(w, s, len,
-         path, "(IMG)", s2, len2);
-}
-
 static void menu_action_setting_disp_set_label_menu_file_imageviewer(
       file_list_t* list,
       unsigned *w, unsigned type, unsigned i,
@@ -1314,7 +1368,6 @@ static void menu_action_setting_disp_set_label_core_option_create(
       char *s2, size_t len2)
 {
    rarch_system_info_t *system = NULL;
-   global_t            *global = global_get_ptr();
 
    runloop_ctl(RUNLOOP_CTL_SYSTEM_INFO_GET, &system);
    if (!system)
@@ -1325,8 +1378,8 @@ static void menu_action_setting_disp_set_label_core_option_create(
 
    strlcpy(s, "", len);
 
-   if (!string_is_empty(global->name.base))
-      strlcpy(s,  path_basename(global->name.base), len);
+   if (!string_is_empty(path_get(RARCH_PATH_BASENAME)))
+      strlcpy(s,  path_basename(path_get(RARCH_PATH_BASENAME)), len);
 
    strlcpy(s2, path, len2);
 }
@@ -1339,7 +1392,7 @@ static void menu_action_setting_disp_set_label_playlist_associations(file_list_t
       const char *path,
       char *s2, size_t len2)
 {
-   char playlist_name_with_ext[PATH_MAX_LENGTH] = {0};
+   char playlist_name_with_ext[255];
    bool found_matching_core_association         = false;
    settings_t         *settings                 = config_get_ptr();
    struct string_list *str_list                 = string_split(settings->playlist_names, ";");
@@ -1347,6 +1400,7 @@ static void menu_action_setting_disp_set_label_playlist_associations(file_list_t
 
    strlcpy(s2, path, len2);
 
+   playlist_name_with_ext[0] = '\0';
    *s = '\0';
    *w = 19;
 
@@ -1416,60 +1470,6 @@ static void menu_action_setting_disp_set_label_core_options(file_list_t* list,
    strlcpy(s2, path, len2);
 }
 
-static void menu_action_setting_disp_set_label_content_history(
-      file_list_t* list,
-      unsigned *w, unsigned type, unsigned i,
-      const char *label,
-      char *s, size_t len,
-      const char *entry_label,
-      const char *path,
-      char *s2, size_t len2)
-{
-   *s = '\0';
-   *w = 19;
-
-   menu_setting_get_label(list, s,
-         len, w, type, label, entry_label, i);
-
-   strlcpy(s2, path, len2);
-}
-
-static void menu_action_setting_disp_set_label_system_information(
-      file_list_t* list,
-      unsigned *w, unsigned type, unsigned i,
-      const char *label,
-      char *s, size_t len,
-      const char *entry_label,
-      const char *path,
-      char *s2, size_t len2)
-{
-   *s = '\0';
-   *w = 2;
-
-   menu_setting_get_label(list, s,
-         len, w, type, label, entry_label, i);
-
-   strlcpy(s2, path, len2);
-}
-
-static void menu_action_setting_disp_set_label_debug_information(
-      file_list_t* list,
-      unsigned *w, unsigned type, unsigned i,
-      const char *label,
-      char *s, size_t len,
-      const char *entry_label,
-      const char *path,
-      char *s2, size_t len2)
-{
-   *s = '\0';
-   *w = 2;
-
-   menu_setting_get_label(list, s,
-         len, w, type, label, entry_label, i);
-
-   strlcpy(s2, path, len2);
-}
-
 static void menu_action_setting_disp_set_label_achievement_information(
       file_list_t* list,
       unsigned *w, unsigned type, unsigned i,
@@ -1488,6 +1488,21 @@ static void menu_action_setting_disp_set_label_achievement_information(
    strlcpy(s2, path, len2);
 }
 
+static void menu_action_setting_disp_set_label_no_items(
+      file_list_t* list,
+      unsigned *w, unsigned type, unsigned i,
+      const char *label,
+      char *s, size_t len,
+      const char *entry_label,
+      const char *path,
+      char *s2, size_t len2)
+{
+   *s = '\0';
+   *w = 19;
+
+   strlcpy(s2, path, len2);
+}
+
 static void menu_action_setting_disp_set_label(file_list_t* list,
       unsigned *w, unsigned type, unsigned i,
       const char *label,
@@ -1505,6 +1520,66 @@ static void menu_action_setting_disp_set_label(file_list_t* list,
    strlcpy(s2, path, len2);
 }
 
+static void menu_action_setting_disp_set_label_setting_bool(file_list_t* list,
+      unsigned *w, unsigned type, unsigned i,
+      const char *label,
+      char *s, size_t len,
+      const char *entry_label,
+      const char *path,
+      char *s2, size_t len2)
+{
+   rarch_setting_t *setting = menu_setting_find(list->list[i].label);
+
+   *s = '\0';
+   *w = 19;
+
+   if (setting)
+   {
+      if (*setting->value.target.boolean)
+         strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_ON), len);
+      else
+         strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_OFF), len);
+   }
+
+   strlcpy(s2, path, len2);
+}
+
+static void menu_action_setting_disp_set_label_setting_string(file_list_t* list,
+      unsigned *w, unsigned type, unsigned i,
+      const char *label,
+      char *s, size_t len,
+      const char *entry_label,
+      const char *path,
+      char *s2, size_t len2)
+{
+   rarch_setting_t *setting = menu_setting_find(list->list[i].label);
+
+   *w = 19;
+
+   strlcpy(s, setting->value.target.string, len);
+
+   strlcpy(s2, path, len2);
+}
+
+static void menu_action_setting_disp_set_label_setting_path(file_list_t* list,
+      unsigned *w, unsigned type, unsigned i,
+      const char *label,
+      char *s, size_t len,
+      const char *entry_label,
+      const char *path,
+      char *s2, size_t len2)
+{
+   rarch_setting_t *setting = menu_setting_find(list->list[i].label);
+   const char *basename     = setting ? path_basename(setting->value.target.string) : NULL;
+
+   *w = 19;
+
+   if (!string_is_empty(basename))
+      strlcpy(s, basename, len);
+
+   strlcpy(s2, path, len2);
+}
+
 static int menu_cbs_init_bind_get_string_representation_compare_label(
       menu_file_list_cbs_t *cbs)
 {
@@ -1512,6 +1587,18 @@ static int menu_cbs_init_bind_get_string_representation_compare_label(
    {
       switch (cbs->enum_idx)
       {
+         case MENU_ENUM_LABEL_VIDEO_DRIVER:
+         case MENU_ENUM_LABEL_AUDIO_DRIVER:
+         case MENU_ENUM_LABEL_INPUT_DRIVER:
+         case MENU_ENUM_LABEL_JOYPAD_DRIVER:
+         case MENU_ENUM_LABEL_AUDIO_RESAMPLER_DRIVER:
+         case MENU_ENUM_LABEL_RECORD_DRIVER:
+         case MENU_ENUM_LABEL_LOCATION_DRIVER:
+         case MENU_ENUM_LABEL_CAMERA_DRIVER:
+         case MENU_ENUM_LABEL_WIFI_DRIVER:
+         case MENU_ENUM_LABEL_MENU_DRIVER:
+            BIND_ACTION_GET_VALUE(cbs, menu_action_setting_disp_set_label);
+            break;
          case MENU_ENUM_LABEL_STATE_SLOT:
             BIND_ACTION_GET_VALUE(cbs,
                   menu_action_setting_disp_set_label_state);
@@ -1524,6 +1611,10 @@ static int menu_cbs_init_bind_get_string_representation_compare_label(
             BIND_ACTION_GET_VALUE(cbs,
                   menu_action_setting_disp_set_label_xmb_theme);
             break;
+         case MENU_ENUM_LABEL_CONNECT_WIFI:
+            BIND_ACTION_GET_VALUE(cbs,
+                  menu_action_setting_disp_set_label_wifi_is_online);
+            break;
          case MENU_ENUM_LABEL_XMB_MENU_COLOR_THEME:
             BIND_ACTION_GET_VALUE(cbs,
                   menu_action_setting_disp_set_label_xmb_menu_color_theme);
@@ -1591,8 +1682,7 @@ static int menu_cbs_init_bind_get_string_representation_compare_label(
          case MENU_ENUM_LABEL_CONTENT_COLLECTION_LIST:
          case MENU_ENUM_LABEL_LOAD_CONTENT_HISTORY:
          case MENU_ENUM_LABEL_DOWNLOADED_FILE_DETECT_CORE_LIST:
-         case MENU_ENUM_LABEL_DETECT_CORE_LIST:
-         case MENU_ENUM_LABEL_LOAD_CONTENT:
+         case MENU_ENUM_LABEL_FAVORITES:
          case MENU_ENUM_LABEL_CORE_OPTIONS:
          case MENU_ENUM_LABEL_CORE_CHEAT_OPTIONS:
          case MENU_ENUM_LABEL_SHADER_OPTIONS:
@@ -1611,8 +1701,8 @@ static int menu_cbs_init_bind_get_string_representation_compare_label(
          case MENU_ENUM_LABEL_CORE_INPUT_REMAPPING_OPTIONS:
          case MENU_ENUM_LABEL_CORE_INFORMATION:
          case MENU_ENUM_LABEL_SYSTEM_INFORMATION:
-         case MENU_ENUM_LABEL_DEBUG_INFORMATION:
          case MENU_ENUM_LABEL_ACHIEVEMENT_LIST:
+         case MENU_ENUM_LABEL_ACHIEVEMENT_LIST_HARDCORE:
          case MENU_ENUM_LABEL_SAVE_STATE:
          case MENU_ENUM_LABEL_LOAD_STATE:
             BIND_ACTION_GET_VALUE(cbs,
@@ -1666,6 +1756,7 @@ static int menu_cbs_init_bind_get_string_representation_compare_type(
                menu_action_setting_disp_set_label_core_option_create);
             break;
          case FILE_TYPE_CORE:
+         case FILE_TYPE_DIRECT_LOAD:
             BIND_ACTION_GET_VALUE(cbs,
                menu_action_setting_disp_set_label_menu_file_core);
             break;
@@ -1682,9 +1773,6 @@ static int menu_cbs_init_bind_get_string_representation_compare_type(
                menu_action_setting_disp_set_label_music);
             break;
          case FILE_TYPE_IMAGE:
-            BIND_ACTION_GET_VALUE(cbs,
-               menu_action_setting_disp_set_label_menu_file_image);
-            break;
          case FILE_TYPE_IMAGEVIEWER:
             BIND_ACTION_GET_VALUE(cbs,
                menu_action_setting_disp_set_label_menu_file_imageviewer);
@@ -1753,6 +1841,12 @@ static int menu_cbs_init_bind_get_string_representation_compare_type(
          case MENU_SETTING_SUBGROUP:
          case MENU_SETTINGS_CUSTOM_BIND_ALL:
          case MENU_SETTINGS_CUSTOM_BIND_DEFAULT_ALL:
+         case MENU_SETTING_ACTION:
+         case MENU_SETTING_ACTION_LOADSTATE:
+         case 7:   /* Run */
+         case MENU_SETTING_ACTION_DELETE_ENTRY:
+         case 117: /* Netplay settings */
+         case MENU_SETTING_ACTION_CORE_DISK_OPTIONS:
             BIND_ACTION_GET_VALUE(cbs,
                menu_action_setting_disp_set_label_menu_more);
             break;
@@ -1760,7 +1854,24 @@ static int menu_cbs_init_bind_get_string_representation_compare_type(
             BIND_ACTION_GET_VALUE(cbs,
                menu_action_setting_disp_set_label_menu_disk_index);
             break;
+         case 31: /* Database entry */
+            BIND_ACTION_GET_VALUE(cbs, menu_action_setting_disp_set_label_db_entry);
+            break;
+         case 25: /* URL directory entries */
+         case 26: /* URL entries */
+            BIND_ACTION_GET_VALUE(cbs, menu_action_setting_disp_set_label_entry);
+            break;
+         case MENU_SETTING_NO_ITEM:
+            BIND_ACTION_GET_VALUE(cbs, menu_action_setting_disp_set_label_no_items);
+            break;
+         case 32: /* Recent history entry */
+         case 65535: /* System info entry */
+            BIND_ACTION_GET_VALUE(cbs, menu_action_setting_disp_set_label_entry);
+            break;
          default:
+#if 0
+            RARCH_LOG("type: %d\n", type);
+#endif
             BIND_ACTION_GET_VALUE(cbs, menu_action_setting_disp_set_label);
             break;
       }
@@ -1775,6 +1886,38 @@ int menu_cbs_init_bind_get_string_representation(menu_file_list_cbs_t *cbs,
    if (!cbs)
       return -1;
 
+   if (strstr(label, "joypad_index") && strstr(label, "input_player"))
+   {
+      BIND_ACTION_GET_VALUE(cbs, menu_action_setting_disp_set_label);
+      return 0;
+   }
+
+#if 0
+   RARCH_LOG("MENU_SETTINGS_NONE: %d\n", MENU_SETTINGS_NONE);
+   RARCH_LOG("MENU_SETTINGS_LAST: %d\n", MENU_SETTINGS_LAST);
+#endif
+
+   if (cbs->setting)
+   {
+      switch (setting_get_type(cbs->setting))
+      {
+         case ST_BOOL:
+            BIND_ACTION_GET_VALUE(cbs,
+                  menu_action_setting_disp_set_label_setting_bool);
+            return 0;
+         case ST_STRING:
+            BIND_ACTION_GET_VALUE(cbs,
+                  menu_action_setting_disp_set_label_setting_string);
+            return 0;
+         case ST_PATH:
+            BIND_ACTION_GET_VALUE(cbs,
+                  menu_action_setting_disp_set_label_setting_path);
+            return 0;
+         default:
+            break;
+      }
+   }
+
    if (cbs->enum_idx != MSG_UNKNOWN)
    {
       switch (cbs->enum_idx)
@@ -1788,21 +1931,18 @@ int menu_cbs_init_bind_get_string_representation(menu_file_list_cbs_t *cbs,
                   menu_action_setting_disp_set_label_cheevos_locked_entry);
             return 0;
          case MENU_ENUM_LABEL_LOAD_CONTENT_HISTORY:
-            BIND_ACTION_GET_VALUE(cbs,
-                  menu_action_setting_disp_set_label_content_history);
-            return 0;
          case MENU_ENUM_LABEL_SYSTEM_INFORMATION:
             BIND_ACTION_GET_VALUE(cbs,
-                  menu_action_setting_disp_set_label_system_information);
-            return 0;
-         case MENU_ENUM_LABEL_DEBUG_INFORMATION:
-            BIND_ACTION_GET_VALUE(cbs,
-                  menu_action_setting_disp_set_label_debug_information);
+                  menu_action_setting_disp_set_label_menu_more);
             return 0;
          case MENU_ENUM_LABEL_ACHIEVEMENT_LIST:
             BIND_ACTION_GET_VALUE(cbs,
                   menu_action_setting_disp_set_label_achievement_information);
             return 0;
+         case MENU_ENUM_LABEL_ACHIEVEMENT_LIST_HARDCORE:
+            BIND_ACTION_GET_VALUE(cbs,
+                  menu_action_setting_disp_set_label_achievement_information);
+            return 0;
          default:
             break;
       }
