@@ -1,5 +1,5 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -13,10 +13,15 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <compat/strl.h>
 #include <file/file_path.h>
 #include <lists/string_list.h>
 #include <string/stdstring.h>
 
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
 #include "../menu_content.h"
 #include "../menu_driver.h"
 #include "../menu_cbs.h"
@@ -25,28 +30,29 @@
 #include "../menu_shader.h"
 #include "../menu_navigation.h"
 
+#include "../widgets/menu_list.h"
+
+#include "../../configuration.h"
+#include "../../core.h"
 #include "../../core_info.h"
 #include "../../managers/cheat_manager.h"
 #include "../../file_path_special.h"
-#include "../../general.h"
 #include "../../retroarch.h"
-#include "../../system.h"
+#include "../../runloop.h"
 #include "../../ui/ui_companion_driver.h"
 
 #ifndef BIND_ACTION_RIGHT
 #define BIND_ACTION_RIGHT(cbs, name) \
-   cbs->action_right = name; \
-   cbs->action_right_ident = #name;
+   do { \
+      cbs->action_right = name; \
+      cbs->action_right_ident = #name; \
+   } while(0)
 #endif
 
 #ifdef HAVE_SHADER_MANAGER
-static int generic_shader_action_parameter_right(
-      struct video_shader *shader, struct video_shader_parameter *param,
+static int generic_shader_action_parameter_right(struct video_shader_parameter *param,
       unsigned type, const char *label, bool wraparound)
 {
-   if (!shader)
-      return menu_cbs_exit();
-
    param->current += param->step;
    param->current  = MIN(MAX(param->minimum, param->current), param->maximum);
 
@@ -63,22 +69,19 @@ int shader_action_parameter_right(unsigned type, const char *label, bool wraparo
    video_shader_driver_get_current_shader(&shader_info);
 
    param = &shader_info.data->parameters[type - MENU_SETTINGS_SHADER_PARAMETER_0];
-   return generic_shader_action_parameter_right(shader_info.data, param, type, label, wraparound);
+   if (!param)
+      return menu_cbs_exit();
+   return generic_shader_action_parameter_right(param, type, label, wraparound);
 }
 
 int shader_action_parameter_preset_right(unsigned type, const char *label,
       bool wraparound)
 {
-   struct video_shader_parameter *param = NULL;
-   struct video_shader      *shader     = NULL;
-
-   menu_driver_ctl(RARCH_MENU_CTL_SHADER_GET,
-         &shader);
-
-   param = shader ? 
-      &shader->parameters[type - MENU_SETTINGS_SHADER_PRESET_PARAMETER_0] :
-      NULL;
-   return generic_shader_action_parameter_right(shader, param, type, label, wraparound);
+   struct video_shader_parameter *param = menu_shader_manager_get_parameters(
+         type - MENU_SETTINGS_SHADER_PRESET_PARAMETER_0);
+   if (!param)
+      return menu_cbs_exit();
+   return generic_shader_action_parameter_right(param, type, label, wraparound);
 }
 #endif
 
@@ -223,27 +226,20 @@ static int action_right_shader_scale_pass(unsigned type, const char *label,
       bool wraparound)
 {
 #ifdef HAVE_SHADER_MANAGER
-   unsigned pass = type - MENU_SETTINGS_SHADER_PASS_SCALE_0;
-   struct video_shader *shader           = NULL;
-   struct video_shader_pass *shader_pass = NULL;
+   unsigned current_scale, delta;
+   unsigned pass                         = 
+      type - MENU_SETTINGS_SHADER_PASS_SCALE_0;
+   struct video_shader_pass *shader_pass = menu_shader_manager_get_pass(pass);
 
-   menu_driver_ctl(RARCH_MENU_CTL_SHADER_GET,
-         &shader);
-   if (!shader)
-      return menu_cbs_exit();
-   shader_pass = &shader->pass[pass];
    if (!shader_pass)
       return menu_cbs_exit();
 
-   {
-      unsigned current_scale   = shader_pass->fbo.scale_x;
-      unsigned delta           = 1;
-      current_scale            = (current_scale + delta) % 6;
+   current_scale            = shader_pass->fbo.scale_x;
+   delta                    = 1;
+   current_scale            = (current_scale + delta) % 6;
 
-      shader_pass->fbo.valid   = current_scale;
-      shader_pass->fbo.scale_x = shader_pass->fbo.scale_y = current_scale;
-
-   }
+   shader_pass->fbo.valid   = current_scale;
+   shader_pass->fbo.scale_x = shader_pass->fbo.scale_y = current_scale;
 #endif
    return 0;
 }
@@ -254,14 +250,8 @@ static int action_right_shader_filter_pass(unsigned type, const char *label,
 #ifdef HAVE_SHADER_MANAGER
    unsigned pass                         = type - MENU_SETTINGS_SHADER_PASS_FILTER_0;
    unsigned delta                        = 1;
-   struct video_shader *shader           = NULL;
-   struct video_shader_pass *shader_pass = NULL;
+   struct video_shader_pass *shader_pass = menu_shader_manager_get_pass(pass);
 
-   menu_driver_ctl(RARCH_MENU_CTL_SHADER_GET,
-         &shader);
-   if (!shader)
-      return menu_cbs_exit();
-   shader_pass = &shader->pass[pass];
    if (!shader_pass)
       return menu_cbs_exit();
 
@@ -277,7 +267,8 @@ static int action_right_shader_filter_default(unsigned type, const char *label,
    rarch_setting_t *setting = menu_setting_find_enum(MENU_ENUM_LABEL_VIDEO_SMOOTH);
    if (!setting)
       return menu_cbs_exit();
-   return menu_action_handle_setting(setting, menu_setting_get_type(setting), MENU_ACTION_RIGHT,
+   return menu_action_handle_setting(setting,
+         setting_get_type(setting), MENU_ACTION_RIGHT,
          wraparound);
 #else
    return 0;
@@ -302,16 +293,15 @@ static int action_right_shader_num_passes(unsigned type, const char *label,
       bool wraparound)
 {
 #ifdef HAVE_SHADER_MANAGER
-   bool refresh      = false;
-   struct video_shader *shader = NULL;
+   bool refresh                = false;
+   struct video_shader *shader = menu_shader_get();
 
-   menu_driver_ctl(RARCH_MENU_CTL_SHADER_GET,
-         &shader);
    if (!shader)
       return menu_cbs_exit();
 
-   if ((shader->passes < GFX_MAX_SHADERS))
-      shader->passes++;
+   if ((menu_shader_manager_get_amount_passes() < GFX_MAX_SHADERS))
+      menu_shader_manager_increment_amount_passes();
+
    menu_entries_ctl(MENU_ENTRIES_CTL_SET_REFRESH, &refresh);
    menu_driver_ctl(RARCH_MENU_CTL_SET_PREVENT_POPULATE, NULL);
    video_shader_resolve_parameters(NULL, shader);
@@ -330,20 +320,22 @@ static int action_right_video_resolution(unsigned type, const char *label,
 static int playlist_association_right(unsigned type, const char *label,
       bool wraparound)
 {
-   size_t i, next, found, current = 0;
-   char core_path[PATH_MAX_LENGTH]  = {0};
+   char core_path[PATH_MAX_LENGTH];
+   char new_playlist_cores[PATH_MAX_LENGTH];
+   size_t i, next, found, current   = 0;
    core_info_t *info                = NULL;
    struct string_list *stnames      = NULL;
    struct string_list *stcores      = NULL;
-   char new_playlist_cores[PATH_MAX_LENGTH] = {0};
+   core_info_list_t           *list = NULL;
    settings_t *settings             = config_get_ptr();
    const char *path                 = path_basename(label);
-   core_info_list_t           *list = NULL;
    
    core_info_get_list(&list);
    if (!list)
       return menu_cbs_exit();
 
+   core_path[0] = new_playlist_cores[0] = '\0';
+
    stnames = string_split(settings->playlist_names, ";");
    stcores = string_split(settings->playlist_cores, ";");
 
@@ -482,6 +474,10 @@ static int menu_cbs_init_bind_right_compare_type(menu_file_list_cbs_t *cbs,
             if (  string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HISTORY_TAB))   ||
                   string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_PLAYLISTS_TAB)) ||
                   string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_ADD_TAB)) ||
+                  string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_NETPLAY_TAB)) ||
+                  string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_MUSIC_TAB)) ||
+                  string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_IMAGES_TAB)) ||
+                  string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_VIDEO_TAB)) ||
                   string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HORIZONTAL_MENU)) ||
                   string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_SETTINGS_TAB))
                   )
@@ -510,10 +506,10 @@ static int menu_cbs_init_bind_right_compare_label(menu_file_list_cbs_t *cbs,
 
    if (cbs->setting)
    {
-      const char *parent_group   = menu_setting_get_parent_group(cbs->setting);
+      const char *parent_group   = cbs->setting->parent_group;
 
       if (string_is_equal(parent_group, msg_hash_to_str(MENU_ENUM_LABEL_MAIN_MENU)) 
-               && (menu_setting_get_type(cbs->setting) == ST_GROUP))
+               && (setting_get_type(cbs->setting) == ST_GROUP))
       {
          BIND_ACTION_RIGHT(cbs, action_right_scroll);
          return 0;
@@ -523,7 +519,9 @@ static int menu_cbs_init_bind_right_compare_label(menu_file_list_cbs_t *cbs,
    for (i = 0; i < MAX_USERS; i++)
    {
       uint32_t label_setting_hash;
-      char label_setting[PATH_MAX_LENGTH] = {0};
+      char label_setting[128];
+
+      label_setting[0] = '\0';
 
       snprintf(label_setting, sizeof(label_setting), "input_player%d_joypad_index", i + 1);
       label_setting_hash = msg_hash_calculate(label_setting);
@@ -551,6 +549,9 @@ static int menu_cbs_init_bind_right_compare_label(menu_file_list_cbs_t *cbs,
       {
          switch (cbs->enum_idx)
          {
+            case MENU_ENUM_LABEL_CONNECT_NETPLAY_ROOM:
+               BIND_ACTION_RIGHT(cbs, action_right_mainmenu);
+               break;
             case MENU_ENUM_LABEL_VIDEO_SHADER_SCALE_PASS:
                BIND_ACTION_RIGHT(cbs, action_right_shader_scale_pass);
                break;
@@ -569,12 +570,41 @@ static int menu_cbs_init_bind_right_compare_label(menu_file_list_cbs_t *cbs,
             case MENU_ENUM_LABEL_SCREEN_RESOLUTION:
                BIND_ACTION_RIGHT(cbs, action_right_video_resolution);
                break;
+            case MENU_ENUM_LABEL_OPEN_ARCHIVE_DETECT_CORE:
+            case MENU_ENUM_LABEL_LOAD_ARCHIVE_DETECT_CORE:
+               BIND_ACTION_RIGHT(cbs, action_right_scroll);
+               break;
             case MENU_ENUM_LABEL_NO_ITEMS:
             case MENU_ENUM_LABEL_NO_PLAYLIST_ENTRIES_AVAILABLE:
-               if (  string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_MAIN_MENU))       ||
+               if (  
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HISTORY_TAB))   ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_MAIN_MENU))       ||
                      string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_PLAYLISTS_TAB))   ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_MUSIC_TAB)) ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_IMAGES_TAB)) ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_VIDEO_TAB)) ||
                      string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HORIZONTAL_MENU))
                   )
+               {
+                  BIND_ACTION_RIGHT(cbs, action_right_mainmenu);
+               }
+               else
+               {
+                  BIND_ACTION_RIGHT(cbs, action_right_scroll);
+               }
+               break;
+            case MENU_ENUM_LABEL_START_VIDEO_PROCESSOR:
+            case MENU_ENUM_LABEL_TAKE_SCREENSHOT:
+               if (  string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HISTORY_TAB))   ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_PLAYLISTS_TAB)) ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_ADD_TAB)) ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_NETPLAY_TAB)) ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_MUSIC_TAB)) ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_IMAGES_TAB)) ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_VIDEO_TAB)) ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HORIZONTAL_MENU)) ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_SETTINGS_TAB))
+                  )
                {
                   BIND_ACTION_RIGHT(cbs, action_right_mainmenu);
                   break;
@@ -607,7 +637,11 @@ int menu_cbs_init_bind_right(menu_file_list_cbs_t *cbs,
       if (  string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HISTORY_TAB))   ||
             string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_PLAYLISTS_TAB)) ||
             string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_ADD_TAB)) ||
+            string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_NETPLAY_TAB)) ||
             string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_MAIN_MENU)) ||
+            string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_MUSIC_TAB)) ||
+            string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_IMAGES_TAB)) ||
+            string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_VIDEO_TAB)) ||
             string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HORIZONTAL_MENU)) ||
             string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_SETTINGS_TAB))
          )
