@@ -1,5 +1,5 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2016 - Hans-Kristian Arntzen
+ *  Copyright (C) 2016-2017 - Hans-Kristian Arntzen
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -13,6 +13,10 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <retro_assert.h>
+#include <dynamic/dylib.h>
+#include <string/stdstring.h>
+
 #ifdef HAVE_CONFIG_H
 #include "../../config.h"
 #endif
@@ -23,11 +27,8 @@
 #endif
 #endif
 
-#include <retro_assert.h>
-#include <dynamic/dylib.h>
-
 #include "vulkan_common.h"
-#include "../../performance_counters.h"
+#include "../../configuration.h"
 
 static dylib_t vulkan_library;
 static VkInstance cached_instance;
@@ -265,6 +266,18 @@ void vulkan_sync_texture_to_cpu(vk_t *vk, const struct vk_texture *tex)
    vkInvalidateMappedMemoryRanges(vk->context->device, 1, &range);
 }
 
+static unsigned vulkan_num_miplevels(unsigned width, unsigned height)
+{
+   unsigned size = MAX(width, height);
+   unsigned levels = 0;
+   while (size)
+   {
+      levels++;
+      size >>= 1;
+   }
+   return levels;
+}
+
 struct vk_texture vulkan_create_texture(vk_t *vk,
       struct vk_texture *old,
       unsigned width, unsigned height,
@@ -284,6 +297,7 @@ struct vk_texture vulkan_create_texture(vk_t *vk,
    VkCommandBufferAllocateInfo cmd_info = { VK_STRUCTURE_TYPE_COMMAND_BUFFER_ALLOCATE_INFO };
    VkSubmitInfo submit_info             = { VK_STRUCTURE_TYPE_SUBMIT_INFO };
    VkCommandBufferBeginInfo begin_info  = { VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO };
+   unsigned i;
 
    memset(&tex, 0, sizeof(tex));
 
@@ -292,8 +306,19 @@ struct vk_texture vulkan_create_texture(vk_t *vk,
    info.extent.width  = width;
    info.extent.height = height;
    info.extent.depth  = 1;
-   info.mipLevels     = 1;
    info.arrayLayers   = 1;
+
+   /* For simplicity, always build mipmaps for
+    * static textures, samplers can be used to enable it dynamically.
+    */
+   if (type == VULKAN_TEXTURE_STATIC)
+   {
+      info.mipLevels = vulkan_num_miplevels(width, height);
+      tex.mipmap     = true;
+   }
+   else
+      info.mipLevels = 1;
+
    info.samples = VK_SAMPLE_COUNT_1_BIT;
 
    if (type == VULKAN_TEXTURE_STREAMED)
@@ -317,19 +342,24 @@ struct vk_texture vulkan_create_texture(vk_t *vk,
       case VULKAN_TEXTURE_STATIC:
          retro_assert(initial && "Static textures must have initial data.\n");
          info.tiling        = VK_IMAGE_TILING_OPTIMAL;
-         info.usage         = VK_IMAGE_USAGE_SAMPLED_BIT | VK_IMAGE_USAGE_TRANSFER_DST_BIT;
+         info.usage         = VK_IMAGE_USAGE_SAMPLED_BIT |
+                              VK_IMAGE_USAGE_TRANSFER_DST_BIT |
+                              VK_IMAGE_USAGE_TRANSFER_SRC_BIT;
          info.initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
          break;
 
       case VULKAN_TEXTURE_DYNAMIC:
          retro_assert(!initial && "Dynamic textures must not have initial data.\n");
          info.tiling        = VK_IMAGE_TILING_OPTIMAL;
-         info.usage         = VK_IMAGE_USAGE_SAMPLED_BIT | VK_IMAGE_USAGE_TRANSFER_DST_BIT;
+         info.usage         = VK_IMAGE_USAGE_SAMPLED_BIT |
+                              VK_IMAGE_USAGE_TRANSFER_DST_BIT |
+                              VK_IMAGE_USAGE_TRANSFER_SRC_BIT;
          info.initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
          break;
 
       case VULKAN_TEXTURE_STREAMED:
-         info.usage         = VK_IMAGE_USAGE_SAMPLED_BIT;
+         info.usage         = VK_IMAGE_USAGE_SAMPLED_BIT |
+                              VK_IMAGE_USAGE_TRANSFER_SRC_BIT;
          info.tiling        = VK_IMAGE_TILING_LINEAR;
          info.initialLayout = VK_IMAGE_LAYOUT_PREINITIALIZED;
          break;
@@ -462,7 +492,7 @@ struct vk_texture vulkan_create_texture(vk_t *vk,
          view.components.a             = VK_COMPONENT_SWIZZLE_A;
       }
       view.subresourceRange.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
-      view.subresourceRange.levelCount = 1;
+      view.subresourceRange.levelCount = info.mipLevels;
       view.subresourceRange.layerCount = 1;
 
       vkCreateImageView(device, &view, NULL, &tex.view);
@@ -526,8 +556,13 @@ struct vk_texture vulkan_create_texture(vk_t *vk,
             VK_PIPELINE_STAGE_HOST_BIT,
             VK_PIPELINE_STAGE_TRANSFER_BIT);
 
+      /* If doing mipmapping on upload, keep in general so we can easily do transfers to
+       * and transfers from the images without having to
+       * mess around with lots of extra transitions at per-level granularity.
+       */
       vulkan_image_layout_transition(vk, staging, tex.image,
-            VK_IMAGE_LAYOUT_UNDEFINED, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL,
+            VK_IMAGE_LAYOUT_UNDEFINED,
+            tex.mipmap ? VK_IMAGE_LAYOUT_GENERAL : VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL,
             0, VK_ACCESS_TRANSFER_WRITE_BIT,
             VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT,
             VK_PIPELINE_STAGE_TRANSFER_BIT);
@@ -542,16 +577,69 @@ struct vk_texture vulkan_create_texture(vk_t *vk,
 
       vkCmdCopyImage(staging,
             tmp.image, VK_IMAGE_LAYOUT_GENERAL,
-            tex.image, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL,
+            tex.image,
+            tex.mipmap ? VK_IMAGE_LAYOUT_GENERAL : VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL,
             1, &region);
 
-      vulkan_image_layout_transition(vk, staging, tex.image,
-            VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL,
-            VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL,
-            VK_ACCESS_TRANSFER_WRITE_BIT,
-            VK_ACCESS_SHADER_READ_BIT,
-            VK_PIPELINE_STAGE_TRANSFER_BIT,
-            VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT);
+      if (tex.mipmap)
+      {
+         for (i = 1; i < info.mipLevels; i++)
+         {
+            VkImageBlit blit_region;
+            unsigned src_width = MAX(width >> (i - 1), 1);
+            unsigned src_height = MAX(height >> (i - 1), 1);
+            unsigned target_width = MAX(width >> i, 1);
+            unsigned target_height = MAX(height >> i, 1);
+            memset(&blit_region, 0, sizeof(blit_region));
+
+            blit_region.srcSubresource.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
+            blit_region.srcSubresource.mipLevel = i - 1;
+            blit_region.srcSubresource.baseArrayLayer = 0;
+            blit_region.srcSubresource.layerCount = 1;
+            blit_region.dstSubresource = blit_region.srcSubresource;
+            blit_region.dstSubresource.mipLevel = i;
+            blit_region.srcOffsets[1].x = src_width;
+            blit_region.srcOffsets[1].y = src_height;
+            blit_region.srcOffsets[1].z = 1;
+            blit_region.dstOffsets[1].x = target_width;
+            blit_region.dstOffsets[1].y = target_height;
+            blit_region.dstOffsets[1].z = 1;
+
+            /* Only injects execution and memory barriers,
+             * not actual transition. */
+            vulkan_image_layout_transition(vk, staging, tex.image,
+                  VK_IMAGE_LAYOUT_GENERAL,
+                  VK_IMAGE_LAYOUT_GENERAL,
+                  VK_ACCESS_TRANSFER_WRITE_BIT,
+                  VK_ACCESS_TRANSFER_READ_BIT,
+                  VK_PIPELINE_STAGE_TRANSFER_BIT,
+                  VK_PIPELINE_STAGE_TRANSFER_BIT);
+
+            vkCmdBlitImage(staging,
+                  tex.image, VK_IMAGE_LAYOUT_GENERAL,
+                  tex.image, VK_IMAGE_LAYOUT_GENERAL,
+                  1, &blit_region, VK_FILTER_LINEAR);
+         }
+
+         /* Complete our texture. */
+         vulkan_image_layout_transition(vk, staging, tex.image,
+               VK_IMAGE_LAYOUT_GENERAL,
+               VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL,
+               VK_ACCESS_TRANSFER_WRITE_BIT,
+               VK_ACCESS_SHADER_READ_BIT,
+               VK_PIPELINE_STAGE_TRANSFER_BIT,
+               VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT);
+      }
+      else
+      {
+         vulkan_image_layout_transition(vk, staging, tex.image,
+               VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL,
+               VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL,
+               VK_ACCESS_TRANSFER_WRITE_BIT,
+               VK_ACCESS_SHADER_READ_BIT,
+               VK_PIPELINE_STAGE_TRANSFER_BIT,
+               VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT);
+      }
 
       vkEndCommandBuffer(staging);
       submit_info.commandBufferCount = 1;
@@ -1155,7 +1243,7 @@ static bool vulkan_find_extensions(const char **exts, unsigned num_exts,
       found = false;
       for (i = 0; i < property_count; i++)
       {
-         if (!strcmp(exts[ext], properties[i].extensionName))
+         if (string_is_equal(exts[ext], properties[i].extensionName))
          {
             found = true;
             break;
@@ -1368,6 +1456,8 @@ static bool vulkan_context_init_device(gfx_ctx_vulkan_data_t *vk)
    vkGetPhysicalDeviceMemoryProperties(vk->context.gpu,
          &vk->context.memory_properties);
 
+   RARCH_LOG("[Vulkan]: Using GPU: %s\n", vk->context.gpu_properties.deviceName);
+
    if (vk->context.device == VK_NULL_HANDLE)
    {
       VkQueueFamilyProperties *queue_properties = NULL;
@@ -1627,6 +1717,7 @@ bool vulkan_context_init(gfx_ctx_vulkan_data_t *vk,
       info.pfnCallback = vulkan_debug_cb;
       vkCreateDebugReportCallbackEXT(vk->context.instance, &info, NULL, &vk->context.debug_callback);
    }
+   RARCH_LOG("[Vulkan]: Enabling Vulkan debug layers.\n");
 #endif
 
    /* Try different API versions if driver has compatible
@@ -1846,7 +1937,9 @@ out:
    create_info.planeStackIndex = planes[best_plane].currentStackIndex;
    create_info.transform = VK_SURFACE_TRANSFORM_IDENTITY_BIT_KHR;
    create_info.globalAlpha = 1.0f;
-   create_info.alphaMode = alpha_mode;
+   /* TODO/FIXME - why is this cast needed for CXX_BUILD? Why is
+    * alpha_mode not just VkDisplayPlaneAlphaFlagBitsKHR to begin with? */
+   create_info.alphaMode = (VkDisplayPlaneAlphaFlagBitsKHR)alpha_mode;
    create_info.imageExtent.width = *width;
    create_info.imageExtent.height = *height;
 
@@ -2145,12 +2238,7 @@ void vulkan_acquire_next_image(gfx_ctx_vulkan_data_t *vk)
 
    if (*next_fence != VK_NULL_HANDLE)
    {
-      static struct retro_perf_counter fence_wait = {0};
-
-      performance_counter_init(&fence_wait, "fence_wait");
-      performance_counter_start(&fence_wait);
       vkWaitForFences(vk->context.device, 1, next_fence, true, UINT64_MAX);
-      performance_counter_stop(&fence_wait);
 
       vkResetFences(vk->context.device, 1, next_fence);
    }
@@ -2205,6 +2293,7 @@ bool vulkan_create_swapchain(gfx_ctx_vulkan_data_t *vk,
             present_modes[i]);
    }
 
+   vk->context.swap_interval = swap_interval;
    for (i = 0; i < present_mode_count; i++)
    {
       if (!swap_interval && present_modes[i] == VK_PRESENT_MODE_MAILBOX_KHR)
@@ -2298,7 +2387,7 @@ bool vulkan_create_swapchain(gfx_ctx_vulkan_data_t *vk,
    info.clipped                = true;
    info.oldSwapchain           = old_swapchain;
    info.imageUsage             = VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT 
-      | VK_IMAGE_USAGE_TRANSFER_SRC_BIT;
+      | VK_IMAGE_USAGE_TRANSFER_SRC_BIT | VK_IMAGE_USAGE_TRANSFER_DST_BIT;
 
    if (vkCreateSwapchainKHR(vk->context.device,
             &info, NULL, &vk->swapchain) != VK_SUCCESS)
