@@ -1,5 +1,5 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -16,11 +16,16 @@
 #include <retro_miscellaneous.h>
 #include <string/stdstring.h>
 
-#include "../../frontend/frontend_driver.h"
-#include "../../general.h"
-#include "../../verbosity.h"
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
 #include "win32_common.h"
+#include "../../frontend/frontend_driver.h"
+#include "../../configuration.h"
+#include "../../verbosity.h"
 #include "../../driver.h"
+#include "../../paths.h"
 #include "../../runloop.h"
 #include "../../tasks/tasks_internal.h"
 #include "../../core_info.h"
@@ -37,57 +42,101 @@
 #include <commdlg.h>
 #include "../../retroarch.h"
 #include "../video_thread_wrapper.h"
-#include <Shellapi.h>
+#include <shellapi.h>
+
+#ifdef HAVE_MENU
+#include "../../menu/menu_driver.h"
+#endif
+
 #ifndef _MSC_VER
 extern "C" {
 #endif
 
+#include <encodings/utf.h>
+
 LRESULT win32_menu_loop(HWND owner, WPARAM wparam);
 
 #ifndef _MSC_VER
 }
 #endif
 
-extern "C" bool dinput_handle_message(void *dinput, UINT message, WPARAM wParam, LPARAM lParam);
-
-bool doubleclick_on_titlebar = false;
-unsigned g_resize_width;
-unsigned g_resize_height;
-bool g_restore_desktop;
-static unsigned g_pos_x = CW_USEDEFAULT;
-static unsigned g_pos_y = CW_USEDEFAULT;
-static bool g_resized;
-bool g_inited;
-static bool g_quit;
-ui_window_win32_t main_window;
-
+#ifdef HAVE_D3D9
+extern "C" bool dinput_handle_message(void *dinput, UINT message,
+      WPARAM wParam, LPARAM lParam);
+extern void *dinput_gdi;
 extern void *dinput_wgl;
-static void *curD3D = NULL;
 extern void *dinput;
+#endif
+
+unsigned g_resize_width             = 0;
+unsigned g_resize_height            = 0;
+static bool g_resized               = false;
+bool g_restore_desktop              = false;
+static bool doubleclick_on_titlebar = false;
+bool g_inited                       = false;
+static bool g_quit                  = false;
+static unsigned g_pos_x             = CW_USEDEFAULT;
+static unsigned g_pos_y             = CW_USEDEFAULT;
+static void *curD3D                 = NULL;
+
+ui_window_win32_t main_window;
 
 /* Power Request APIs */
 
+#if !defined(_XBOX) && (_MSC_VER == 1310)
+typedef struct _REASON_CONTEXT 
+{
+   ULONG Version;
+   DWORD Flags;
+   union
+   {
+      struct
+      {
+         HMODULE LocalizedReasonModule;
+         ULONG LocalizedreasonId;
+         ULONG ReasonStringCount;
+         LPWSTR *ReasonStrings;
+      } Detailed;
+      LPWSTR SimpleReasonString;
+   } Reason;
+} REASON_CONTEXT, *PREASON_CONTEXT;
+
+typedef enum _POWER_REQUEST_TYPE
+{
+   PowerRequestDisplayRequired,
+   PowerRequestSystemRequired,
+   PowerRequestAwayModeRequired,
+   PowerRequestExecutionRequired
+} POWER_REQUEST_TYPE, *PPOWER_REQUEST_TYPE;
+
+#define POWER_REQUEST_CONTEXT_VERSION         0
+#define POWER_REQUEST_CONTEXT_SIMPLE_STRING   1
+#define POWER_REQUEST_CONTEXT_DETAILED_STRING 2
+#endif
+
+#ifdef _WIN32_WINNT_WIN7
 typedef REASON_CONTEXT POWER_REQUEST_CONTEXT, *PPOWER_REQUEST_CONTEXT, *LPPOWER_REQUEST_CONTEXT;
+#endif
 
 #ifndef MAX_MONITORS
 #define MAX_MONITORS 9
 #endif
 
 static HMONITOR win32_monitor_last;
-static unsigned win32_monitor_count;
 static HMONITOR win32_monitor_all[MAX_MONITORS];
+static unsigned win32_monitor_count              = 0;
 
 extern "C"
 {
-	bool doubleclick_on_titlebar_pressed(void)
-	{
-		return doubleclick_on_titlebar;
-	}
+   bool doubleclick_on_titlebar_pressed(void)
+   {
+      return doubleclick_on_titlebar;
+   }
 
-        void unset_doubleclick_on_titlebar(void)
-        {
-           doubleclick_on_titlebar = false;
-        }
+   void unset_doubleclick_on_titlebar(void)
+   {
+      doubleclick_on_titlebar = false;
+   }
 };
 
 INT_PTR CALLBACK PickCoreProc(HWND hDlg, UINT message, 
@@ -96,7 +145,6 @@ INT_PTR CALLBACK PickCoreProc(HWND hDlg, UINT message,
    size_t list_size;
    core_info_list_t *core_info_list = NULL;
    const core_info_t *core_info     = NULL;
-   char *fullpath                   = NULL;
 
    switch (message)
    {
@@ -106,10 +154,9 @@ INT_PTR CALLBACK PickCoreProc(HWND hDlg, UINT message,
             unsigned i;
             /* Add items to list.  */
 
-            runloop_ctl(RUNLOOP_CTL_GET_CONTENT_PATH, &fullpath);
             core_info_get_list(&core_info_list);
             core_info_list_get_supported_cores(core_info_list,
-                  (const char*)fullpath, &core_info, &list_size);
+                  path_get(RARCH_PATH_CONTENT), &core_info, &list_size);
 
             hwndList = GetDlgItem(hDlg, ID_CORELISTBOX);  
 
@@ -129,27 +176,23 @@ INT_PTR CALLBACK PickCoreProc(HWND hDlg, UINT message,
             case IDOK:
             case IDCANCEL:
                EndDialog(hDlg, LOWORD(wParam));
-               return FALSE;
-
+               break;
             case ID_CORELISTBOX:
-               {
-                  switch (HIWORD(wParam)) 
-                  { 
-                     case LBN_SELCHANGE:
-                        {
-                           int lbItem;
-                           const core_info_t *info = NULL;
-                           runloop_ctl(RUNLOOP_CTL_GET_CONTENT_PATH, &fullpath);
-                           HWND hwndList = GetDlgItem(hDlg, ID_CORELISTBOX); 
-                           lbItem = (int)SendMessage(hwndList, LB_GETCURSEL, 0, 0); 
-                           core_info_get_list(&core_info_list);
-                           core_info_list_get_supported_cores(core_info_list,
-                                 (const char*)fullpath, &core_info, &list_size);
-                           info = (const core_info_t*)&core_info[lbItem];
-                           runloop_ctl(RUNLOOP_CTL_SET_LIBRETRO_PATH,info->path);
-                        } 
-                        break;
-                  }
+               switch (HIWORD(wParam)) 
+               { 
+                  case LBN_SELCHANGE:
+                     {
+                        int lbItem;
+                        const core_info_t *info = NULL;
+                        HWND hwndList = GetDlgItem(hDlg, ID_CORELISTBOX); 
+                        lbItem = (int)SendMessage(hwndList, LB_GETCURSEL, 0, 0); 
+                        core_info_get_list(&core_info_list);
+                        core_info_list_get_supported_cores(core_info_list,
+                              path_get(RARCH_PATH_CONTENT), &core_info, &list_size);
+                        info = (const core_info_t*)&core_info[lbItem];
+                        runloop_ctl(RUNLOOP_CTL_SET_LIBRETRO_PATH,info->path);
+                     } 
+                     break;
                }
                return TRUE;
          }
@@ -184,7 +227,7 @@ void win32_monitor_get_info(void)
    memset(&current_mon, 0, sizeof(current_mon));
    current_mon.cbSize = sizeof(MONITORINFOEX);
 
-   GetMonitorInfo(win32_monitor_last, (MONITORINFO*)&current_mon);
+   GetMonitorInfo(win32_monitor_last, (MONITORINFOEX*)&current_mon);
    ChangeDisplaySettingsEx(current_mon.szDevice, NULL, NULL, 0, NULL);
 }
 
@@ -196,7 +239,8 @@ void win32_monitor_info(void *data, void *hm_data, unsigned *mon_id)
    HMONITOR *hm_to_use  = (HMONITOR*)hm_data;
 
    if (!win32_monitor_last)
-      win32_monitor_last = MonitorFromWindow(GetDesktopWindow(), MONITOR_DEFAULTTONEAREST);
+      win32_monitor_last = MonitorFromWindow(GetDesktopWindow(),
+            MONITOR_DEFAULTTONEAREST);
 
    *hm_to_use = win32_monitor_last;
    fs_monitor = settings->video.monitor_index;
@@ -221,13 +265,15 @@ void win32_monitor_info(void *data, void *hm_data, unsigned *mon_id)
 
    memset(mon, 0, sizeof(*mon));
    mon->cbSize = sizeof(MONITORINFOEX);
-   GetMonitorInfo(*hm_to_use, (MONITORINFO*)mon);
+   GetMonitorInfo(*hm_to_use, (MONITORINFOEX*)mon);
 }
 
 /* Get the count of the files dropped */
 static int win32_drag_query_file(HWND hwnd, WPARAM wparam)
 {
-   char szFilename[1024] = {0};
+   char szFilename[1024];
+
+   szFilename[0] = '\0';
 
    if (DragQueryFile((HDROP)wparam, 0xFFFFFFFF, NULL, 0))
    {
@@ -236,24 +282,23 @@ static int win32_drag_query_file(HWND hwnd, WPARAM wparam)
       content_ctx_info_t content_info  = {0};
       core_info_list_t *core_info_list = NULL;
       const core_info_t *core_info     = NULL;
-      settings_t *settings             = config_get_ptr();
 
-	  if (!settings)
-		  return 0;
-
-      DragQueryFile((HDROP)wparam, 0, szFilename, 1024);
+      DragQueryFile((HDROP)wparam, 0, szFilename, sizeof(szFilename));
 
       core_info_get_list(&core_info_list);
 
-	  if (!core_info_list)
-		  return 0;
+     if (!core_info_list)
+        return 0;
 
       core_info_list_get_supported_cores(core_info_list,
             (const char*)szFilename, &core_info, &list_size);
 
-      runloop_ctl(RUNLOOP_CTL_SET_CONTENT_PATH,szFilename);
+      if (!list_size)
+         return 0;
 
-      if (!string_is_empty(settings->path.libretro))
+      path_set(RARCH_PATH_CONTENT, szFilename);
+
+      if (!path_is_empty(RARCH_PATH_CONTENT))
       {
          unsigned i;
          core_info_t *current_core = NULL;
@@ -267,7 +312,7 @@ static int win32_drag_query_file(HWND hwnd, WPARAM wparam)
             if(!string_is_equal(info->systemname, current_core->systemname))
                break;
 
-            if(string_is_equal(settings->path.libretro, info->path))
+            if(string_is_equal(path_get(RARCH_PATH_CORE), info->path))
             {
                /* Our previous core supports the current rom */
                content_ctx_info_t content_info = {0};
@@ -318,7 +363,6 @@ static int win32_drag_query_file(HWND hwnd, WPARAM wparam)
 static LRESULT CALLBACK WndProcCommon(bool *quit, HWND hwnd, UINT message,
       WPARAM wparam, LPARAM lparam)
 {
-   settings_t *settings     = config_get_ptr();
 
    if (message == WM_NCLBUTTONDBLCLK)
       doubleclick_on_titlebar = true;
@@ -373,9 +417,12 @@ static LRESULT CALLBACK WndProcCommon(bool *quit, HWND hwnd, UINT message,
          }
          *quit = true;
          break;
-	  case WM_COMMAND:
-         if (settings->ui.menubar_enable)
-            win32_menu_loop(main_window.hwnd, wparam);
+     case WM_COMMAND:
+         {
+            settings_t *settings     = config_get_ptr();
+            if (settings->ui.menubar_enable)
+               win32_menu_loop(main_window.hwnd, wparam);
+         }
          break;
    }
    return 0;
@@ -383,6 +430,7 @@ static LRESULT CALLBACK WndProcCommon(bool *quit, HWND hwnd, UINT message,
 
 extern void ui_window_win32_set_droppable(void *data, bool droppable);
 
+#ifdef HAVE_D3D9
 LRESULT CALLBACK WndProcD3D(HWND hwnd, UINT message,
       WPARAM wparam, LPARAM lparam)
 {
@@ -427,13 +475,14 @@ LRESULT CALLBACK WndProcD3D(HWND hwnd, UINT message,
       return 0;
    return DefWindowProc(hwnd, message, wparam, lparam);
 }
+#endif
 
+#if defined(HAVE_OPENGL) || defined(HAVE_VULKAN)
 LRESULT CALLBACK WndProcGL(HWND hwnd, UINT message,
       WPARAM wparam, LPARAM lparam)
 {
    LRESULT ret;
    bool quit = false;
-   settings_t *settings     = config_get_ptr();
 
    if (message == WM_NCLBUTTONDBLCLK)
       doubleclick_on_titlebar = true;
@@ -468,8 +517,94 @@ LRESULT CALLBACK WndProcGL(HWND hwnd, UINT message,
          return 0;
    }
 
+#ifdef HAVE_D3D9
    if (dinput_wgl && dinput_handle_message(dinput_wgl, message, wparam, lparam))
       return 0;
+#endif
+   return DefWindowProc(hwnd, message, wparam, lparam);
+}
+#endif
+
+LRESULT CALLBACK WndProcGDI(HWND hwnd, UINT message,
+      WPARAM wparam, LPARAM lparam)
+{
+   LRESULT ret;
+   bool quit = false;
+
+   if (message == WM_NCLBUTTONDBLCLK)
+      doubleclick_on_titlebar = true;
+
+   switch (message)
+   {
+      case WM_PAINT:
+      {
+         PAINTSTRUCT ps;
+         HDC hdc = BeginPaint(hwnd, &ps);
+
+#ifdef HAVE_MENU
+         if (menu_driver_is_alive() && !gdi_has_menu_frame())
+         {
+            RECT rect;
+            GetClientRect(hwnd, &rect);
+
+            TRIVERTEX vertex[2];
+            vertex[0].x     = rect.left;
+            vertex[0].y     = rect.top;
+            vertex[0].Red   = 1 << 8;
+            vertex[0].Green = 81 << 8;
+            vertex[0].Blue  = 127 << 8;
+            vertex[0].Alpha = 0;
+
+            vertex[1].x     = rect.right;
+            vertex[1].y     = rect.bottom;
+            vertex[1].Red   = 0;
+            vertex[1].Green = 1 << 8;
+            vertex[1].Blue  = 33 << 8;
+            vertex[1].Alpha = 0;
+
+            GRADIENT_RECT gRect;
+            gRect.LowerRight = 0;
+            gRect.UpperLeft  = 1;
+
+            GradientFill(hdc, vertex, 2, &gRect, 1, GRADIENT_FILL_RECT_V);
+         }
+#endif
+
+         EndPaint(hwnd, &ps);
+         break;
+      }
+      case WM_DROPFILES:
+      case WM_SYSCOMMAND:
+      case WM_CHAR:
+      case WM_KEYDOWN:
+      case WM_KEYUP:
+      case WM_SYSKEYUP:
+      case WM_SYSKEYDOWN:
+      case WM_CLOSE:
+      case WM_DESTROY:
+      case WM_QUIT:
+      case WM_SIZE:
+      case WM_COMMAND:
+         ret = WndProcCommon(&quit, hwnd, message, wparam, lparam);
+         if (quit)
+            return ret;
+         break;
+      case WM_CREATE:
+         {
+            ui_window_win32_t win32_window;
+            win32_window.hwnd = hwnd;
+
+            create_gdi_context(hwnd, &g_quit);
+
+            ui_window_win32_set_droppable(&win32_window, true);
+         }
+         return 0;
+   }
+
+#ifdef HAVE_D3D9
+   if (dinput_gdi && dinput_handle_message(dinput_gdi, message, wparam, lparam))
+      return 0;
+#endif
    return DefWindowProc(hwnd, message, wparam, lparam);
 }
 
@@ -496,7 +631,7 @@ bool win32_window_create(void *data, unsigned style,
 #endif
 
 bool win32_get_metrics(void *data,
-	enum display_metric_types type, float *value)
+   enum display_metric_types type, float *value)
 {
 #ifdef _XBOX
    return false;
@@ -541,20 +676,24 @@ void win32_monitor_init(void)
    g_quit              = false;
 }
 
-bool win32_monitor_set_fullscreen(unsigned width, unsigned height, unsigned refresh, char *dev_name)
+static bool win32_monitor_set_fullscreen(unsigned width, unsigned height,
+      unsigned refresh, char *dev_name)
 {
 #ifndef _XBOX
    DEVMODE devmode;
 
    memset(&devmode, 0, sizeof(devmode));
-   devmode.dmSize       = sizeof(DEVMODE);
-   devmode.dmPelsWidth  = width;
-   devmode.dmPelsHeight = height;
+   devmode.dmSize             = sizeof(DEVMODE);
+   devmode.dmPelsWidth        = width;
+   devmode.dmPelsHeight       = height;
    devmode.dmDisplayFrequency = refresh;
-   devmode.dmFields     = DM_PELSWIDTH | DM_PELSHEIGHT | DM_DISPLAYFREQUENCY;
+   devmode.dmFields           = DM_PELSWIDTH | DM_PELSHEIGHT | DM_DISPLAYFREQUENCY;
 
-   RARCH_LOG("[WGL]: Setting fullscreen to %ux%u @ %uHz on device %s.\n", width, height, refresh, dev_name);
-   return ChangeDisplaySettingsEx(dev_name, &devmode, NULL, CDS_FULLSCREEN, NULL) == DISP_CHANGE_SUCCESSFUL;
+   RARCH_LOG("Setting fullscreen to %ux%u @ %uHz on device %s.\n",
+         width, height, refresh, dev_name);
+
+   return ChangeDisplaySettingsEx(dev_name, &devmode,
+         NULL, CDS_FULLSCREEN, NULL) == DISP_CHANGE_SUCCESSFUL;
 #endif
 }
 
@@ -568,10 +707,12 @@ void win32_show_cursor(bool state)
 #endif
 }
 
-void win32_check_window(bool *quit, bool *resize, unsigned *width, unsigned *height)
+void win32_check_window(bool *quit, bool *resize,
+      unsigned *width, unsigned *height)
 {
 #ifndef _XBOX
-   const ui_application_t *application = ui_companion_driver_get_application_ptr();
+   const ui_application_t *application = 
+      ui_companion_driver_get_application_ptr();
    if (application)
       application->process_events();
 #endif
@@ -605,11 +746,12 @@ bool win32_suppress_screensaver(void *data, bool enable)
 
       if (major*100+minor >= 601)
       {
+#ifdef _WIN32_WINNT_WIN7
          /* Windows 7, 8, 10 codepath */
          typedef HANDLE (WINAPI * PowerCreateRequestPtr)(REASON_CONTEXT *context);
          typedef BOOL   (WINAPI * PowerSetRequestPtr)(HANDLE PowerRequest,
                POWER_REQUEST_TYPE RequestType);
-         HMODULE kernel32 = GetModuleHandleW(L"kernel32.dll");
+         HMODULE kernel32 = GetModuleHandle("kernel32.dll");
          PowerCreateRequestPtr powerCreateRequest =
             (PowerCreateRequestPtr)GetProcAddress(kernel32, "PowerCreateRequest");
          PowerSetRequestPtr    powerSetRequest =
@@ -629,6 +771,7 @@ bool win32_suppress_screensaver(void *data, bool enable)
             powerSetRequest( Request, PowerRequestDisplayRequired);
             return true;
          }
+#endif
       }
       else
       {
@@ -642,9 +785,10 @@ bool win32_suppress_screensaver(void *data, bool enable)
 #endif
 }
 
+/* FIXME: It should not be necessary to add the W after MONITORINFOEX, but linking fails without it. */
 void win32_set_style(MONITORINFOEX *current_mon, HMONITOR *hm_to_use,
-	unsigned *width, unsigned *height, bool fullscreen, bool windowed_full,
-	RECT *rect, RECT *mon_rect, DWORD *style)
+   unsigned *width, unsigned *height, bool fullscreen, bool windowed_full,
+   RECT *rect, RECT *mon_rect, DWORD *style)
 {
 #ifndef _XBOX
    settings_t *settings = config_get_ptr();
@@ -654,7 +798,8 @@ void win32_set_style(MONITORINFOEX *current_mon, HMONITOR *hm_to_use,
     * for black frame insertion using video.refresh_rate set to half
     * of the display refresh rate, as well as higher vsync swap intervals. */
    float refresh_mod    = settings->video.black_frame_insertion ? 2.0f : 1.0f;
-   unsigned refresh     = roundf(settings->video.refresh_rate * refresh_mod * settings->video.swap_interval);
+   unsigned refresh     = roundf(settings->video.refresh_rate 
+         * refresh_mod * settings->video.swap_interval);
 
    if (fullscreen)
    {
@@ -670,10 +815,10 @@ void win32_set_style(MONITORINFOEX *current_mon, HMONITOR *hm_to_use,
 
          if (!win32_monitor_set_fullscreen(*width, *height,
                   refresh, current_mon->szDevice))
-			 {}
+          {}
 
          /* Display settings might have changed, get new coordinates. */
-         GetMonitorInfo(*hm_to_use, (MONITORINFO*)current_mon);
+         GetMonitorInfo(*hm_to_use, (MONITORINFOEX*)current_mon);
          *mon_rect = current_mon->rcMonitor;
       }
    }
@@ -703,7 +848,8 @@ void win32_set_window(unsigned *width, unsigned *height,
       if (!fullscreen && settings->ui.menubar_enable)
       {
          RECT rc_temp = {0, 0, (LONG)*height, 0x7FFF};
-         SetMenu(main_window.hwnd, LoadMenu(GetModuleHandle(NULL),MAKEINTRESOURCE(IDR_MENU)));
+         SetMenu(main_window.hwnd,
+               LoadMenu(GetModuleHandle(NULL),MAKEINTRESOURCE(IDR_MENU)));
          SendMessage(main_window.hwnd, WM_NCCALCSIZE, FALSE, (LPARAM)&rc_temp);
          g_resize_height = *height += rc_temp.top + rect->top;
          SetWindowPos(main_window.hwnd, NULL, 0, 0, *width, *height, SWP_NOMOVE);
@@ -735,6 +881,7 @@ bool win32_set_video_mode(void *data,
    RECT rect             = {0};
    HMONITOR hm_to_use    = NULL;
    settings_t *settings  = config_get_ptr();
+   int res               = 0;
 
    win32_monitor_info(&current_mon, &hm_to_use, &mon_id);
 
@@ -744,18 +891,28 @@ bool win32_set_video_mode(void *data,
 
    windowed_full   = settings->video.windowed_fullscreen;
 
-   win32_set_style(&current_mon, &hm_to_use, &width, &height, fullscreen, windowed_full, &rect, &mon_rect, &style);
+   win32_set_style(&current_mon, &hm_to_use, &width, &height,
+         fullscreen, windowed_full, &rect, &mon_rect, &style);
 
    if (!win32_window_create(data, style, &mon_rect, width, height, fullscreen))
       return false;
    
    win32_set_window(&width, &height, fullscreen, windowed_full, &rect);
 
-   /* Wait until context is created (or failed to do so ...) */
-   while (!g_inited && !g_quit && GetMessage(&msg, main_window.hwnd, 0, 0))
+   /* Wait until context is created (or failed to do so ...).
+    * Please don't remove the (res = ) as GetMessage can return -1. */
+   while (!g_inited && !g_quit && (res = GetMessage(&msg, main_window.hwnd, 0, 0)) != 0)
    {
-      TranslateMessage(&msg);
-      DispatchMessage(&msg);
+      if (res == -1)
+      {
+         RARCH_ERR("GetMessage error code %d\n", GetLastError());
+         break;
+      }
+      else
+      {
+         TranslateMessage(&msg);
+         DispatchMessage(&msg);
+      }
    }
 
    if (g_quit)
