@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -23,28 +23,39 @@
 
 #include <X11/Xatom.h>
 
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
 #include "x11_common.h"
 #include "../../frontend/frontend_driver.h"
 #include "../../input/common/input_x11_common.h"
-#include "../../configuration.h"
 #include "../../verbosity.h"
-#include "../../runloop.h"
+
+#ifdef HAVE_DBUS
+#include <dbus/dbus.h>
+static DBusConnection* dbus_connection      = NULL;
+static unsigned int dbus_screensaver_cookie = 0;
+#endif
+
+static bool xdg_screensaver_available       = true;
+bool g_x11_entered                          = false;
+static bool g_x11_has_focus                 = false;
+static bool g_x11_true_full                 = false;
+Display *g_x11_dpy                          = NULL;
+
+unsigned g_x11_screen                       = 0;
 
 Colormap g_x11_cmap;
 Window   g_x11_win;
-Display *g_x11_dpy;
 
 static Atom XA_NET_WM_STATE;
 static Atom XA_NET_WM_STATE_FULLSCREEN;
 static Atom XA_NET_MOVERESIZE_WINDOW;
 
 static Atom g_x11_quit_atom;
-static bool g_x11_has_focus;
 static XIM g_x11_xim;
 static XIC g_x11_xic;
-static bool g_x11_true_full;
-
-unsigned g_x11_screen;
 
 #define XA_INIT(x) XA##x = XInternAtom(dpy, #x, False)
 #define _NET_WM_STATE_ADD 1
@@ -52,6 +63,133 @@ unsigned g_x11_screen;
 #define MOVERESIZE_X_SHIFT 8
 #define MOVERESIZE_Y_SHIFT 9
 
+#ifdef HAVE_DBUS
+static void dbus_ensure_connection(void)
+{
+    DBusError err;
+    int ret;
+    
+    dbus_error_init(&err);
+
+    dbus_connection = dbus_bus_get_private(DBUS_BUS_SESSION, &err);
+
+    if (dbus_error_is_set(&err))
+    {
+        RARCH_LOG("[DBus]: Failed to get DBus connection. Screensaver will not be suspended via DBus.\n");
+        dbus_error_free(&err);
+    }
+
+    if (dbus_connection)
+        dbus_connection_set_exit_on_disconnect(dbus_connection, true);
+}
+
+static void dbus_close_connection(void)
+{
+   if (!dbus_connection)
+      return;
+
+   dbus_connection_close(dbus_connection);
+   dbus_connection_unref(dbus_connection);
+   dbus_connection = NULL;
+}
+
+static bool dbus_screensaver_inhibit(void)
+{
+   const char *app    = "RetroArch";
+   const char *reason = "Playing a game";
+   DBusMessage   *msg = NULL;
+   DBusMessage *reply = NULL;
+   bool ret           = false;
+
+   if (!dbus_connection)
+      return false; /* DBus connection was not obtained */
+
+   if (dbus_screensaver_cookie > 0)
+      return true; /* Already inhibited */
+
+   msg = dbus_message_new_method_call("org.freedesktop.ScreenSaver",
+         "/org/freedesktop/ScreenSaver",
+         "org.freedesktop.ScreenSaver",
+         "Inhibit");
+
+   if (!msg)
+      return false;
+
+   if (!dbus_message_append_args(msg,
+            DBUS_TYPE_STRING, &app,
+            DBUS_TYPE_STRING, &reason,
+            DBUS_TYPE_INVALID))
+   {
+      dbus_message_unref(msg);
+      return false;
+   }
+
+   reply = dbus_connection_send_with_reply_and_block(dbus_connection,
+         msg, 300, NULL);
+
+   if (reply != NULL)
+   {
+      if (!dbus_message_get_args(reply, NULL,
+               DBUS_TYPE_UINT32, &dbus_screensaver_cookie,
+               DBUS_TYPE_INVALID))
+         dbus_screensaver_cookie = 0;
+      else
+         ret = true;
+
+      dbus_message_unref(reply);
+   }
+
+   dbus_message_unref(msg);
+
+   if (dbus_screensaver_cookie == 0)
+   {
+      RARCH_ERR("[DBus]: Failed to suspend screensaver via DBus.\n");
+   }
+   else
+   {
+      RARCH_LOG("[DBus]: Suspended screensaver via DBus.\n");
+   }
+
+   return ret;
+}
+
+static void dbus_screensaver_uninhibit(void)
+{
+   DBusMessage *msg = NULL;
+
+   if (!dbus_connection)
+      return;
+
+   if (dbus_screensaver_cookie == 0)
+      return;
+
+   msg = dbus_message_new_method_call("org.freedesktop.ScreenSaver",
+         "/org/freedesktop/ScreenSaver",
+         "org.freedesktop.ScreenSaver",
+         "UnInhibit");
+   if (!msg)
+       return;
+
+   dbus_message_append_args(msg,
+         DBUS_TYPE_UINT32, &dbus_screensaver_cookie,
+         DBUS_TYPE_INVALID);
+
+   if (dbus_connection_send(dbus_connection, msg, NULL))
+      dbus_connection_flush(dbus_connection);
+   dbus_message_unref(msg);
+
+   dbus_screensaver_cookie = 0;
+}
+
+/* Returns false when fallback should be attempted */
+bool x11_suspend_screensaver_dbus(bool enable)
+{
+   if (enable) return dbus_screensaver_inhibit();
+   dbus_screensaver_uninhibit();
+   return false;
+}
+#endif
+
 static void x11_hide_mouse(Display *dpy, Window win)
 {
    static char bm_no_data[] = {0, 0, 0, 0, 0, 0, 0, 0};
@@ -90,13 +228,13 @@ void x11_windowed_fullscreen(Display *dpy, Window win)
    XA_INIT(_NET_WM_STATE);
    XA_INIT(_NET_WM_STATE_FULLSCREEN);
 
-   xev.xclient.type = ClientMessage;
-   xev.xclient.send_event = True;
+   xev.xclient.type         = ClientMessage;
+   xev.xclient.send_event   = True;
    xev.xclient.message_type = XA_NET_WM_STATE;
-   xev.xclient.window = win;
-   xev.xclient.format = 32;
-   xev.xclient.data.l[0] = _NET_WM_STATE_ADD;
-   xev.xclient.data.l[1] = XA_NET_WM_STATE_FULLSCREEN;
+   xev.xclient.window       = win;
+   xev.xclient.format       = 32;
+   xev.xclient.data.l[0]    = _NET_WM_STATE_ADD;
+   xev.xclient.data.l[1]    = XA_NET_WM_STATE_FULLSCREEN;
 
    XSendEvent(dpy, DefaultRootWindow(dpy), False,
          SubstructureRedirectMask | SubstructureNotifyMask,
@@ -129,7 +267,7 @@ void x11_move_window(Display *dpy, Window win, int x, int y,
 
 static void x11_set_window_class(Display *dpy, Window win)
 {
-   XClassHint hint = {0};
+   XClassHint hint;
 
    hint.res_name   = (char*)"retroarch"; /* Broken header. */
    hint.res_class  = (char*)"retroarch";
@@ -141,36 +279,51 @@ void x11_set_window_attr(Display *dpy, Window win)
    x11_set_window_class(dpy, win);
 }
 
-void x11_suspend_screensaver(Window wnd, bool enable)
+static void xdg_screensaver_inhibit(Window wnd)
 {
-   int ret;
-   char cmd[64] = {0};
-   static bool screensaver_na = false;
+   int  ret;
+   char cmd[64];
 
-   if (!enable)
-       return;
+   cmd[0] = '\0';
 
-   if (screensaver_na)
-      return;
-
-   RARCH_LOG("Suspending screensaver (X11).\n");
+   RARCH_LOG("Suspending screensaver (X11, xdg-screensaver).\n");
 
    snprintf(cmd, sizeof(cmd), "xdg-screensaver suspend 0x%x", (int)wnd);
 
    ret = system(cmd);
    if (ret == -1)
    {
-      screensaver_na = true;
+      xdg_screensaver_available = false;
       RARCH_WARN("Failed to launch xdg-screensaver.\n");
    }
    else if (WEXITSTATUS(ret))
    {
-      screensaver_na = true;
+      xdg_screensaver_available = false;
       RARCH_WARN("Could not suspend screen saver.\n");
    }
 }
 
-static bool get_video_mode(Display *dpy, unsigned width, unsigned height,
+void x11_suspend_screensaver_xdg_screensaver(Window wnd, bool enable)
+{
+   /* Check if screensaver suspend is enabled in config */
+   if (!enable)
+      return;
+
+   if (xdg_screensaver_available)
+      xdg_screensaver_inhibit(wnd);
+}
+
+void x11_suspend_screensaver(Window wnd, bool enable)
+{
+#ifdef HAVE_DBUS
+    if (x11_suspend_screensaver_dbus(enable))
+       return;
+#endif
+    x11_suspend_screensaver_xdg_screensaver(wnd, enable);
+}
+
+static bool get_video_mode(video_frame_info_t *video_info,
+      Display *dpy, unsigned width, unsigned height,
       XF86VidModeModeInfo *mode, XF86VidModeModeInfo *desktop_mode)
 {
    float refresh_mod;
@@ -178,7 +331,6 @@ static bool get_video_mode(Display *dpy, unsigned width, unsigned height,
    bool ret                    = false;
    float minimum_fps_diff      = 0.0f;
    XF86VidModeModeInfo **modes = NULL;
-   settings_t *settings        = config_get_ptr();
 
    XF86VidModeGetAllModeLines(dpy, DefaultScreen(dpy), &num_modes, &modes);
 
@@ -192,7 +344,7 @@ static bool get_video_mode(Display *dpy, unsigned width, unsigned height,
 
    /* If we use black frame insertion, we fake a 60 Hz monitor 
     * for 120 Hz one, etc, so try to match that. */
-   refresh_mod = settings->video.black_frame_insertion ? 0.5f : 1.0f;
+   refresh_mod = video_info->black_frame_insertion ? 0.5f : 1.0f;
 
    for (i = 0; i < num_modes; i++)
    {
@@ -208,7 +360,7 @@ static bool get_video_mode(Display *dpy, unsigned width, unsigned height,
          continue;
 
       refresh = refresh_mod * m->dotclock * 1000.0f / (m->htotal * m->vtotal);
-      diff    = fabsf(refresh - settings->video.refresh_rate);
+      diff    = fabsf(refresh - video_info->refresh_rate);
 
       if (!ret || diff < minimum_fps_diff)
       {
@@ -222,12 +374,13 @@ static bool get_video_mode(Display *dpy, unsigned width, unsigned height,
    return ret;
 }
 
-bool x11_enter_fullscreen(Display *dpy, unsigned width,
+bool x11_enter_fullscreen(video_frame_info_t *video_info,
+      Display *dpy, unsigned width,
       unsigned height, XF86VidModeModeInfo *desktop_mode)
 {
    XF86VidModeModeInfo mode;
 
-   if (!get_video_mode(dpy, width, height, &mode, desktop_mode))
+   if (!get_video_mode(video_info, dpy, width, height, &mode, desktop_mode))
       return false;
 
    if (!XF86VidModeSwitchToMode(dpy, DefaultScreen(dpy), &mode))
@@ -263,10 +416,10 @@ static XineramaScreenInfo *x11_query_screens(Display *dpy, int *num_screens)
 bool x11_get_xinerama_coord(Display *dpy, int screen,
       int *x, int *y, unsigned *w, unsigned *h)
 {
-   int i, num_screens = 0;
-   bool ret = false;
-
+   int i, num_screens       = 0;
+   bool                 ret = false;
    XineramaScreenInfo *info = x11_query_screens(dpy, &num_screens);
+
    RARCH_LOG("[X11]: Xinerama screens: %d.\n", num_screens);
 
    for (i = 0; i < num_screens; i++)
@@ -289,11 +442,11 @@ bool x11_get_xinerama_coord(Display *dpy, int screen,
 unsigned x11_get_xinerama_monitor(Display *dpy, int x, int y,
       int w, int h)
 {
-   int i, num_screens = 0;
-   unsigned monitor   = 0;
-   int largest_area   = 0;
-
+   int       i, num_screens = 0;
+   unsigned       monitor   = 0;
+   int       largest_area   = 0;
    XineramaScreenInfo *info = x11_query_screens(dpy, &num_screens);
+
    RARCH_LOG("[X11]: Xinerama screens: %d.\n", num_screens);
 
    for (i = 0; i < num_screens; i++)
@@ -304,8 +457,8 @@ unsigned x11_get_xinerama_monitor(Display *dpy, int x, int y,
       int max_ty = MAX(y, info[i].y_org);
       int min_by = MIN(y + h, info[i].y_org + info[i].height);
 
-      int len_x = min_rx - max_lx;
-      int len_y = min_by - max_ty;
+      int len_x  = min_rx - max_lx;
+      int len_y  = min_by - max_ty;
 
       /* The whole window is outside the screen. */
       if (len_x < 0 || len_y < 0)
@@ -330,8 +483,7 @@ bool x11_create_input_context(Display *dpy, Window win, XIM *xim, XIC *xic)
    x11_destroy_input_context(xim, xic);
 
    g_x11_has_focus = true;
-
-   *xim = XOpenIM(dpy, NULL, NULL, NULL);
+   *xim            = XOpenIM(dpy, NULL, NULL, NULL);
 
    if (!*xim)
    {
@@ -370,13 +522,12 @@ void x11_destroy_input_context(XIM *xim, XIC *xic)
 bool x11_get_metrics(void *data,
       enum display_metric_types type, float *value)
 {
-   int pixels_x, pixels_y, physical_width, physical_height;
    unsigned     screen_no  = 0;
    Display           *dpy  = (Display*)XOpenDisplay(NULL);
-   pixels_x                = DisplayWidth(dpy, screen_no);
-   pixels_y                = DisplayHeight(dpy, screen_no);
-   physical_width          = DisplayWidthMM(dpy, screen_no);
-   physical_height         = DisplayHeightMM(dpy, screen_no);
+   int pixels_x            = DisplayWidth(dpy, screen_no);
+   int pixels_y            = DisplayHeight(dpy, screen_no);
+   int physical_width      = DisplayWidthMM(dpy, screen_no);
+   int physical_height     = DisplayHeightMM(dpy, screen_no);
 
    (void)pixels_y;
 
@@ -404,11 +555,10 @@ bool x11_get_metrics(void *data,
 
 bool x11_alive(void *data)
 {
-   XEvent event;
-
    while (XPending(g_x11_dpy))
    {
-      bool filter;
+      XEvent event;
+      bool filter = false;
 
       /* Can get events from older windows. Check this. */
       XNextEvent(g_x11_dpy, &event);
@@ -438,7 +588,34 @@ bool x11_alive(void *data)
             break;
 
          case ButtonPress:
-            x_input_poll_wheel(&event.xbutton, true);
+            switch (event.xbutton.button)
+            {
+               case 1: /* Left click */
+#if 0
+                  RARCH_LOG("Click occurred : [%d, %d]\n",
+                        event.xbutton.x_root,
+                        event.xbutton.y_root);
+#endif
+                  break;
+               case 2: /* Grabbed  */
+                       /* Middle click */
+                  break;
+               case 3: /* Right click */
+                  break;
+               case 4: /* Grabbed  */
+                       /* Scroll up */
+               case 5: /* Scroll down */
+                  x_input_poll_wheel(&event.xbutton, true);
+                  break;
+            }
+            break;
+
+         case EnterNotify:
+            g_x11_entered = true;
+            break;
+
+         case LeaveNotify:
+            g_x11_entered = false;
             break;
 
          case ButtonRelease:
@@ -456,7 +633,8 @@ bool x11_alive(void *data)
 }
 
 void x11_check_window(void *data, bool *quit,
-   bool *resize, unsigned *width, unsigned *height, unsigned frame_count)
+   bool *resize, unsigned *width, unsigned *height,
+   bool is_shutdown)
 {
    unsigned new_width  = *width;
    unsigned new_height = *height;
@@ -480,14 +658,14 @@ void x11_get_video_size(void *data, unsigned *width, unsigned *height)
    if (!g_x11_dpy || g_x11_win == None)
    {
       Display *dpy = (Display*)XOpenDisplay(NULL);
-      *width  = 0;
-      *height = 0;
+      *width       = 0;
+      *height      = 0;
 
       if (dpy)
       {
          int screen = DefaultScreen(dpy);
-         *width  = DisplayWidth(dpy, screen);
-         *height = DisplayHeight(dpy, screen);
+         *width     = DisplayWidth(dpy, screen);
+         *height    = DisplayHeight(dpy, screen);
          XCloseDisplay(dpy);
       }
    }
@@ -529,19 +707,24 @@ bool x11_connect(void)
          return false;
    }
 
+#ifdef HAVE_DBUS
+   dbus_ensure_connection();
+#endif
+
+
    return true;
 }
 
-void x11_update_window_title(void *data)
+void x11_update_title(void *data, video_frame_info_t *video_info)
 {
-   char buf[128]           = {0};
-   char buf_fps[128]       = {0};
-   settings_t *settings    = config_get_ptr();
+   char title[128];
 
-   if (video_monitor_get_fps(buf, sizeof(buf), buf_fps, sizeof(buf_fps)))
-      XStoreName(g_x11_dpy, g_x11_win, buf);
-   if (settings->fps_show)
-      runloop_msg_queue_push(buf_fps, 1, 1, false);
+   title[0] = '\0';
+
+   video_driver_get_window_title(title, sizeof(title));
+
+   if (title[0])
+      XStoreName(g_x11_dpy, g_x11_win, title);
 }
 
 bool x11_input_ctx_new(bool true_full)
@@ -569,6 +752,11 @@ void x11_window_destroy(bool fullscreen)
    if (!fullscreen)
       XDestroyWindow(g_x11_dpy, g_x11_win);
    g_x11_win = None;
+
+#ifdef HAVE_DBUS
+    dbus_screensaver_uninhibit();
+    dbus_close_connection();
+#endif
 }
 
 void x11_colormap_destroy(void)
