@@ -1,6 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2016 - Brad Parker
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -18,16 +19,19 @@
 #define __CONFIG_DEF_H
 
 #include <boolean.h>
-#include "driver.h"
-#include "gfx/video_driver.h"
+#include "gfx/video_defines.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-enum
+#include "input/input_driver.h"
+
+#include "config.def.keybinds.h"
+
+enum video_driver_enum
 {
-   VIDEO_GL = 0,
+   VIDEO_GL                 = 0,
    VIDEO_VULKAN,
    VIDEO_DRM,
    VIDEO_XVIDEO,
@@ -35,6 +39,7 @@ enum
    VIDEO_SDL2,
    VIDEO_EXT,
    VIDEO_WII,
+   VIDEO_WIIU,
    VIDEO_XENON360,
    VIDEO_XDK_D3D,
    VIDEO_PSP1,
@@ -42,13 +47,19 @@ enum
    VIDEO_CTR,
    VIDEO_D3D9,
    VIDEO_VG,
-   VIDEO_NULL,
    VIDEO_OMAP,
    VIDEO_EXYNOS,
    VIDEO_SUNXI,
    VIDEO_DISPMANX,
+   VIDEO_CACA,
+   VIDEO_GDI,
+   VIDEO_VGA,
+   VIDEO_NULL
+};
 
-   AUDIO_RSOUND,
+enum audio_driver_enum
+{
+   AUDIO_RSOUND             = VIDEO_NULL + 1,
    AUDIO_OSS,
    AUDIO_ALSA,
    AUDIO_ALSATHREAD,
@@ -66,16 +77,24 @@ enum
    AUDIO_PS3,
    AUDIO_XENON360,
    AUDIO_WII,
+   AUDIO_WIIU,
    AUDIO_RWEBAUDIO,
    AUDIO_PSP,
    AUDIO_CTR,
-   AUDIO_NULL,
+   AUDIO_NULL
+};
 
-   AUDIO_RESAMPLER_CC,
+enum audio_resampler_driver_enum
+{
+   AUDIO_RESAMPLER_CC       = AUDIO_NULL + 1,
    AUDIO_RESAMPLER_SINC,
    AUDIO_RESAMPLER_NEAREST,
+   AUDIO_RESAMPLER_NULL
+};
 
-   INPUT_ANDROID,
+enum input_driver_enum
+{
+   INPUT_ANDROID            = AUDIO_RESAMPLER_NULL + 1,
    INPUT_SDL,
    INPUT_SDL2,
    INPUT_X,
@@ -86,17 +105,23 @@ enum
    INPUT_CTR,
    INPUT_XENON360,
    INPUT_WII,
+   INPUT_WIIU,
    INPUT_XINPUT,
    INPUT_UDEV,
    INPUT_LINUXRAW,
    INPUT_COCOA,
    INPUT_QNX,
    INPUT_RWEBINPUT,
-   INPUT_NULL,
+   INPUT_DOS,
+   INPUT_NULL
+};
 
-   JOYPAD_PS3,
+enum joypad_driver_enum
+{
+   JOYPAD_PS3               = INPUT_NULL + 1,
    JOYPAD_XINPUT,
    JOYPAD_GX,
+   JOYPAD_WIIU,
    JOYPAD_XDK,
    JOYPAD_PSP,
    JOYPAD_CTR,
@@ -105,30 +130,53 @@ enum
    JOYPAD_LINUXRAW,
    JOYPAD_ANDROID,
    JOYPAD_SDL,
+   JOYPAD_DOS,
    JOYPAD_HID,
    JOYPAD_QNX,
-   JOYPAD_NULL,
+   JOYPAD_NULL
+};
 
-   CAMERA_V4L2,
+enum camera_driver_enum
+{
+   CAMERA_V4L2              = JOYPAD_NULL + 1,
    CAMERA_RWEBCAM,
    CAMERA_ANDROID,
    CAMERA_AVFOUNDATION,
-   CAMERA_NULL,
+   CAMERA_NULL
+};
 
-   LOCATION_ANDROID,
+enum wifi_driver_enum
+{
+   WIFI_CONNMANCTL          = CAMERA_NULL + 1,
+   WIFI_NULL
+};
+
+enum location_driver_enum
+{
+   LOCATION_ANDROID         = WIFI_NULL + 1,
    LOCATION_CORELOCATION,
-   LOCATION_NULL,
+   LOCATION_NULL
+};
 
-   OSK_PS3,
-   OSK_NULL,
+enum osk_driver_enum
+{
+   OSK_PS3                  = LOCATION_NULL + 1,
+   OSK_NULL
+};
 
-   MENU_RGUI,
+enum menu_driver_enum
+{
+   MENU_RGUI                = OSK_NULL + 1,
    MENU_XUI,
    MENU_MATERIALUI,
    MENU_XMB,
    MENU_NUKLEAR,
+   MENU_NULL
+};
 
-   RECORD_FFMPEG,
+enum record_driver_enum
+{
+   RECORD_FFMPEG            = MENU_NULL + 1,
    RECORD_NULL
 };
 
@@ -144,6 +192,8 @@ enum
 #define VIDEO_DEFAULT_DRIVER VIDEO_GL
 #elif defined(GEKKO)
 #define VIDEO_DEFAULT_DRIVER VIDEO_WII
+#elif defined(WIIU)
+#define VIDEO_DEFAULT_DRIVER VIDEO_WIIU
 #elif defined(XENON)
 #define VIDEO_DEFAULT_DRIVER VIDEO_XENON360
 #elif (defined(_XBOX1) || defined(_XBOX360)) && (defined(HAVE_D3D8) || defined(HAVE_D3D9))
@@ -164,6 +214,10 @@ enum
 #define VIDEO_DEFAULT_DRIVER VIDEO_SDL
 #elif defined(HAVE_SDL2)
 #define VIDEO_DEFAULT_DRIVER VIDEO_SDL2
+#elif defined(_WIN32) && !defined(_XBOX)
+#define VIDEO_DEFAULT_DRIVER VIDEO_GDI
+#elif defined(DJGPP)
+#define VIDEO_DEFAULT_DRIVER VIDEO_VGA
 #elif defined(HAVE_DYLIB) && !defined(ANDROID)
 #define VIDEO_DEFAULT_DRIVER VIDEO_EXT
 #else
@@ -176,16 +230,18 @@ enum
 #define AUDIO_DEFAULT_DRIVER AUDIO_XENON360
 #elif defined(GEKKO)
 #define AUDIO_DEFAULT_DRIVER AUDIO_WII
+#elif defined(WIIU)
+#define AUDIO_DEFAULT_DRIVER AUDIO_WIIU
 #elif defined(PSP) || defined(VITA)
 #define AUDIO_DEFAULT_DRIVER AUDIO_PSP
 #elif defined(_3DS)
 #define AUDIO_DEFAULT_DRIVER AUDIO_CTR
+#elif defined(HAVE_PULSE)
+#define AUDIO_DEFAULT_DRIVER AUDIO_PULSE
 #elif defined(HAVE_ALSA) && defined(HAVE_VIDEOCORE)
 #define AUDIO_DEFAULT_DRIVER AUDIO_ALSATHREAD
 #elif defined(HAVE_ALSA)
 #define AUDIO_DEFAULT_DRIVER AUDIO_ALSA
-#elif defined(HAVE_PULSE)
-#define AUDIO_DEFAULT_DRIVER AUDIO_PULSE
 #elif defined(HAVE_OSS)
 #define AUDIO_DEFAULT_DRIVER AUDIO_OSS
 #elif defined(HAVE_JACK)
@@ -216,7 +272,7 @@ enum
 #define AUDIO_DEFAULT_DRIVER AUDIO_NULL
 #endif
 
-#ifdef PSP
+#if defined(PSP) || defined(EMSCRIPTEN)
 #define AUDIO_DEFAULT_RESAMPLER_DRIVER  AUDIO_RESAMPLER_CC
 #else
 #define AUDIO_DEFAULT_RESAMPLER_DRIVER  AUDIO_RESAMPLER_SINC
@@ -234,6 +290,8 @@ enum
 #define INPUT_DEFAULT_DRIVER INPUT_XINPUT
 #elif defined(ANDROID)
 #define INPUT_DEFAULT_DRIVER INPUT_ANDROID
+#elif defined(EMSCRIPTEN) && defined(HAVE_SDL2)
+#define INPUT_DEFAULT_DRIVER INPUT_SDL2
 #elif defined(EMSCRIPTEN)
 #define INPUT_DEFAULT_DRIVER INPUT_RWEBINPUT
 #elif defined(_WIN32)
@@ -246,6 +304,8 @@ enum
 #define INPUT_DEFAULT_DRIVER INPUT_CTR
 #elif defined(GEKKO)
 #define INPUT_DEFAULT_DRIVER INPUT_WII
+#elif defined(WIIU)
+#define INPUT_DEFAULT_DRIVER INPUT_WIIU
 #elif defined(HAVE_UDEV)
 #define INPUT_DEFAULT_DRIVER INPUT_UDEV
 #elif defined(__linux__) && !defined(ANDROID)
@@ -262,6 +322,8 @@ enum
 #define INPUT_DEFAULT_DRIVER INPUT_SDL
 #elif defined(HAVE_SDL2)
 #define INPUT_DEFAULT_DRIVER INPUT_SDL2
+#elif defined(DJGPP)
+#define INPUT_DEFAULT_DRIVER INPUT_DOS
 #else
 #define INPUT_DEFAULT_DRIVER INPUT_NULL
 #endif
@@ -272,6 +334,8 @@ enum
 #define JOYPAD_DEFAULT_DRIVER JOYPAD_XINPUT
 #elif defined(GEKKO)
 #define JOYPAD_DEFAULT_DRIVER JOYPAD_GX
+#elif defined(WIIU)
+#define JOYPAD_DEFAULT_DRIVER JOYPAD_WIIU
 #elif defined(_XBOX)
 #define JOYPAD_DEFAULT_DRIVER JOYPAD_XDK
 #elif defined(PSP) || defined(VITA)
@@ -288,6 +352,8 @@ enum
 #define JOYPAD_DEFAULT_DRIVER JOYPAD_ANDROID
 #elif defined(HAVE_SDL) || defined(HAVE_SDL2)
 #define JOYPAD_DEFAULT_DRIVER JOYPAD_SDL
+#elif defined(DJGPP)
+#define JOYPAD_DEFAULT_DRIVER JOYPAD_DOS
 #elif defined(HAVE_HID)
 #define JOYPAD_DEFAULT_DRIVER JOYPAD_HID
 #elif defined(__QNX__)
@@ -308,6 +374,12 @@ enum
 #define CAMERA_DEFAULT_DRIVER CAMERA_NULL
 #endif
 
+#if defined(HAVE_LAKKA)
+#define WIFI_DEFAULT_DRIVER WIFI_CONNMANCTL
+#else
+#define WIFI_DEFAULT_DRIVER WIFI_NULL
+#endif
+
 #if defined(ANDROID)
 #define LOCATION_DEFAULT_DRIVER LOCATION_ANDROID
 #elif defined(HAVE_CORELOCATION) && (defined(HAVE_COCOA) || defined(HAVE_COCOATOUCH))
@@ -324,17 +396,19 @@ enum
 
 #if defined(HAVE_XUI)
 #define MENU_DEFAULT_DRIVER MENU_XUI
-#elif defined(IOS) || defined(ANDROID) || defined(__QNX__)
+#elif defined(HAVE_MATERIALUI) && defined(RARCH_MOBILE)
 #define MENU_DEFAULT_DRIVER MENU_MATERIALUI
-#elif defined(__MACH__) && defined(HAVE_XMB) || defined(__CELLOS_LV2__)
+#elif defined(HAVE_XMB)
 #define MENU_DEFAULT_DRIVER MENU_XMB
-#else
+#elif defined(HAVE_RGUI)
 #define MENU_DEFAULT_DRIVER MENU_RGUI
+#else
+#define MENU_DEFAULT_DRIVER MENU_NULL
 #endif
 
 #if defined(XENON) || defined(_XBOX360) || defined(__CELLOS_LV2__)
 #define DEFAULT_ASPECT_RATIO 1.7778f
-#elif defined(_XBOX1) || defined(GEKKO) || defined(ANDROID) || defined(__QNX__)
+#elif defined(_XBOX1) || defined(GEKKO) || defined(ANDROID)
 #define DEFAULT_ASPECT_RATIO 1.3333f
 #else
 #define DEFAULT_ASPECT_RATIO -1.0f
@@ -359,15 +433,20 @@ static bool bundle_assets_extract_enable = false;
 #endif
 
 static const bool def_history_list_enable = true;
+static const bool def_playlist_entry_remove = true;
 
 static const unsigned int def_user_language = 0;
 
-#if (defined(_WIN32) && !defined(_XBOX))
+#if (defined(_WIN32) && !defined(_XBOX)) || (defined(__linux) && !defined(ANDROID) && !defined(HAVE_LAKKA)) || (defined(__MACH__) && !defined(IOS))
 static const bool def_mouse_enable = true;
 #else
 static const bool def_mouse_enable = false;
 #endif
 
+#ifdef HAVE_CHEEVOS
+static const bool cheevos_enable = false;
+#endif
+
 /* VIDEO */
 
 #if defined(_XBOX360)
@@ -394,6 +473,12 @@ static const bool windowed_fullscreen = true;
  * specific monitors, 1 being the first monitor. */
 static const unsigned monitor_index = 0;
 
+/* Window */
+/* Window size. A value of 0 uses window scale
+ * multiplied by the core framebuffer size. */
+static const unsigned window_x = 0;
+static const unsigned window_y = 0;
+
 /* Fullscreen resolution. A value of 0 uses the desktop
  * resolution. */
 static const unsigned fullscreen_x = 0;
@@ -404,7 +489,7 @@ static const bool load_dummy_on_core_shutdown = false;
 #else
 static const bool load_dummy_on_core_shutdown = true;
 #endif
-
+static const bool check_firmware_before_loading = false;
 /* Forcibly disable composition.
  * Only valid on Windows Vista/7/8 for now. */
 static const bool disable_composition = false;
@@ -464,8 +549,10 @@ static const bool video_shared_context = false;
 /* Sets GC/Wii screen width. */
 static const unsigned video_viwidth = 640;
 
+#ifdef GEKKO
 /* Removes 480i flicker, smooths picture a little. */
 static const bool video_vfilter = true;
+#endif
 
 /* Smooths picture. */
 static const bool video_smooth = true;
@@ -507,26 +594,40 @@ static unsigned aspect_ratio_idx = ASPECT_RATIO_CORE;
 /* Save configuration file on exit. */
 static bool config_save_on_exit = true;
 
-static const bool default_overlay_enable = false;
+static bool show_hidden_files = true;
 
 static const bool overlay_hide_in_menu = true;
 
+static const bool display_keyboard_overlay = false;
+
 #ifdef HAVE_MENU
 #include "menu/menu_display.h"
 
 static bool default_block_config_read = true;
 
+#ifdef HAVE_XMB
 static unsigned xmb_scale_factor = 100;
 static unsigned xmb_alpha_factor = 75;
-
 static unsigned xmb_icon_theme   = XMB_ICON_THEME_MONOCHROME;
 static unsigned xmb_theme        = XMB_THEME_ELECTRIC_BLUE;
-
 #ifdef HAVE_LAKKA
 static bool xmb_shadows_enable   = false;
 #else
 static bool xmb_shadows_enable   = true;
 #endif
+static bool xmb_show_settings    = true;
+#ifdef HAVE_IMAGEVIEWER
+static bool xmb_show_images      = true;
+#endif
+#ifdef HAVE_FFMPEG
+static bool xmb_show_music       = true;
+static bool xmb_show_video       = true;
+#endif
+static bool xmb_show_history     = true;
+#ifdef HAVE_LIBRETRODB
+static bool xmb_show_add     	 = true;
+#endif
+#endif
 
 static float menu_wallpaper_opacity = 0.300;
 
@@ -534,53 +635,50 @@ static float menu_footer_opacity = 1.000;
 
 static float menu_header_opacity = 1.000;
 
-static unsigned menu_background_gradient = 4;
-
 #if defined(HAVE_CG) || defined(HAVE_HLSL) || defined(HAVE_GLSL) || defined(HAVE_VULKAN)
-#if defined(HAVE_OPENGLES2) || defined(OSX_PPC)
+#if defined(HAVE_OPENGLES2) || (defined(__MACH__) && (defined(__ppc__) || defined(__ppc64__)))
 static unsigned menu_shader_pipeline = 1;
 #else
 static unsigned menu_shader_pipeline = 2;
 #endif
 #endif
 
-static bool show_advanced_settings    = true;
+static bool show_advanced_settings            = false;
 static const uint32_t menu_entry_normal_color = 0xffffffff;
 static const uint32_t menu_entry_hover_color  = 0xff64ff64;
 static const uint32_t menu_title_color        = 0xff64ff64;
+
 #else
 static bool default_block_config_read = false;
 #endif
 
-#ifdef RARCH_CONSOLE
-static bool default_core_specific_config = true;
-#else
-static bool default_core_specific_config = false;
-#endif
-
 static bool default_game_specific_options = false;
 static bool default_auto_overrides_enable = true;
 static bool default_auto_remaps_enable = true;
+static bool default_auto_shaders_enable = true;
 
 static bool default_sort_savefiles_enable = false;
 static bool default_sort_savestates_enable = false;
 
-static unsigned default_menu_btn_ok          = RETRO_DEVICE_ID_JOYPAD_A;
-static unsigned default_menu_btn_cancel      = RETRO_DEVICE_ID_JOYPAD_B;
-static unsigned default_menu_btn_search      = RETRO_DEVICE_ID_JOYPAD_X;
-static unsigned default_menu_btn_default     = RETRO_DEVICE_ID_JOYPAD_START;
-static unsigned default_menu_btn_info        = RETRO_DEVICE_ID_JOYPAD_SELECT;
-static unsigned default_menu_btn_scroll_down = RETRO_DEVICE_ID_JOYPAD_R;
-static unsigned default_menu_btn_scroll_up   = RETRO_DEVICE_ID_JOYPAD_L;
-
 #if defined(__CELLOS_LV2__) || defined(_XBOX1) || defined(_XBOX360)
-static unsigned menu_toggle_gamepad_combo    = 2;
+static unsigned menu_toggle_gamepad_combo    = INPUT_TOGGLE_L3_R3;
+#elif defined(VITA)
+static unsigned menu_toggle_gamepad_combo    = INPUT_TOGGLE_L1_R1_START_SELECT;
 #else
-static unsigned menu_toggle_gamepad_combo    = 0;
+static unsigned menu_toggle_gamepad_combo    = INPUT_TOGGLE_NONE;
 #endif
 
-#ifdef ANDROID
-static bool back_as_menu_toggle_enable = true;
+#if defined(VITA)
+static unsigned input_backtouch_enable       = false;
+static unsigned input_backtouch_toggle       = false;
+#endif
+
+static bool all_users_control_menu = false;
+
+#if defined(ANDROID) || defined(_WIN32)
+static bool menu_swap_ok_cancel_buttons = true;
+#else
+static bool menu_swap_ok_cancel_buttons = false;
 #endif
 
 /* Crop overscanned frames. */
@@ -634,10 +732,8 @@ static const bool font_enable = true;
  * disable VSync, and leave this at its default. */
 #ifdef _3DS
 static const float refresh_rate = (32730.0 * 8192.0) / 4481134.0 ;
-#elif defined(RARCH_CONSOLE)
-static const float refresh_rate = 60/1.001;
 #else
-static const float refresh_rate = 59.95;
+static const float refresh_rate = 60/1.001;
 #endif
 
 /* Allow games to set rotation. If false, rotation requests are
@@ -664,7 +760,7 @@ static const char *audio_device = NULL;
 
 /* Desired audio latency in milliseconds. Might not be honored
  * if driver can't provide given latency. */
-#ifdef ANDROID
+#if defined(ANDROID) || defined(EMSCRIPTEN)
 /* For most Android devices, 64ms is way too low. */
 static const int out_latency = 128;
 #else
@@ -709,16 +805,29 @@ static const unsigned rewind_buffer_size = 20 << 20; /* 20MiB */
 static const unsigned rewind_granularity = 1;
 
 /* Pause gameplay when gameplay loses focus. */
+#ifdef EMSCRIPTEN
+static const bool pause_nonactive = false;
+#else
 static const bool pause_nonactive = true;
+#endif
 
 /* Saves non-volatile SRAM at a regular interval.
  * It is measured in seconds. A value of 0 disables autosave. */
 static const unsigned autosave_interval = 0;
 
+/* Netplay without savestates/rewind */
+static const bool netplay_stateless_mode = false;
+
 /* When being client over netplay, use keybinds for
  * user 1 rather than user 2. */
 static const bool netplay_client_swap_input = true;
 
+static const bool netplay_nat_traversal = false;
+
+static const unsigned netplay_delay_frames = 16;
+
+static const int netplay_check_frames = 30;
+
 /* On save state load, block SRAM from being overwritten.
  * This could potentially lead to buggy games. */
 static const bool block_sram_overwrite = false;
@@ -736,6 +845,8 @@ static const bool savestate_auto_index = false;
 static const bool savestate_auto_save = false;
 static const bool savestate_auto_load = false;
 
+static const bool savestate_thumbnail_enable = false;
+
 /* Slowmotion ratio. */
 static const float slowmotion_ratio = 3.0;
 
@@ -762,6 +873,8 @@ static const bool menu_dpi_override_enable = true;
 
 #ifdef RARCH_MOBILE
 static const unsigned menu_dpi_override_value = 72;
+#elif defined(__CELLOS_LV2__)
+static const unsigned menu_dpi_override_value = 360;
 #else
 static const unsigned menu_dpi_override_value = 200;
 #endif
@@ -817,6 +930,8 @@ static char buildbot_server_url[] = "http://buildbot.libretro.com/nightly/androi
 #else
 static char buildbot_server_url[] = "";
 #endif
+#elif defined(__QNX__)
+static char buildbot_server_url[] = "http://buildbot.libretro.com/nightly/blackberry/latest/";
 #elif defined(IOS)
 static char buildbot_server_url[] = "http://buildbot.libretro.com/nightly/apple/ios/latest/";
 #elif defined(OSX)
@@ -830,7 +945,7 @@ static char buildbot_server_url[] = "http://buildbot.libretro.com/nightly/apple/
 #elif defined(_WIN32) && !defined(_XBOX)
 #if defined(__x86_64__)
 static char buildbot_server_url[] = "http://buildbot.libretro.com/nightly/win-x86_64/latest/";
-#elif defined(__i386__) || defined(__i486__) || defined(__i686__)
+#elif defined(__i386__) || defined(__i486__) || defined(__i686__) || defined(_M_IX86) || defined(_M_IA64)
 static char buildbot_server_url[] = "http://buildbot.libretro.com/nightly/win-x86/latest/";
 #endif
 #elif defined(__linux__)
@@ -847,104 +962,4 @@ static char buildbot_server_url[] = "";
 
 static char buildbot_assets_server_url[] = "http://buildbot.libretro.com/assets/";
 
-#ifndef IS_SALAMANDER
-#include "intl/intl.h"
-
-/* User 1 */
-static const struct retro_keybind retro_keybinds_1[] = {
-    /*     | RetroPad button            | desc                           | keyboard key  | js btn |     js axis   | */
-   { true, RETRO_DEVICE_ID_JOYPAD_B,      RETRO_LBL_JOYPAD_B,              RETROK_z,       NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_Y,      RETRO_LBL_JOYPAD_Y,              RETROK_a,       NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_SELECT, RETRO_LBL_JOYPAD_SELECT,         RETROK_RSHIFT,  NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_START,  RETRO_LBL_JOYPAD_START,          RETROK_RETURN,  NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_UP,     RETRO_LBL_JOYPAD_UP,             RETROK_UP,      NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_DOWN,   RETRO_LBL_JOYPAD_DOWN,           RETROK_DOWN,    NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_LEFT,   RETRO_LBL_JOYPAD_LEFT,           RETROK_LEFT,    NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_RIGHT,  RETRO_LBL_JOYPAD_RIGHT,          RETROK_RIGHT,   NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_A,      RETRO_LBL_JOYPAD_A,              RETROK_x,       NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_X,      RETRO_LBL_JOYPAD_X,              RETROK_s,       NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_L,      RETRO_LBL_JOYPAD_L,              RETROK_q,       NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_R,      RETRO_LBL_JOYPAD_R,              RETROK_w,       NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_L2,     RETRO_LBL_JOYPAD_L2,             RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_R2,     RETRO_LBL_JOYPAD_R2,             RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_L3,     RETRO_LBL_JOYPAD_L3,             RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_R3,     RETRO_LBL_JOYPAD_R3,             RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-
-   { true, RARCH_ANALOG_LEFT_X_PLUS,      RETRO_LBL_ANALOG_LEFT_X_PLUS,    RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_LEFT_X_MINUS,     RETRO_LBL_ANALOG_LEFT_X_MINUS,   RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_LEFT_Y_PLUS,      RETRO_LBL_ANALOG_LEFT_Y_PLUS,    RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_LEFT_Y_MINUS,     RETRO_LBL_ANALOG_LEFT_Y_MINUS,   RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_RIGHT_X_PLUS,     RETRO_LBL_ANALOG_RIGHT_X_PLUS,   RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_RIGHT_X_MINUS,    RETRO_LBL_ANALOG_RIGHT_X_MINUS,  RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_RIGHT_Y_PLUS,     RETRO_LBL_ANALOG_RIGHT_Y_PLUS,   RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_RIGHT_Y_MINUS,    RETRO_LBL_ANALOG_RIGHT_Y_MINUS,  RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-
-   { true, RARCH_TURBO_ENABLE,             RETRO_LBL_TURBO_ENABLE,         RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_FAST_FORWARD_KEY,         RETRO_LBL_FAST_FORWARD_KEY,     RETROK_SPACE,   NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_FAST_FORWARD_HOLD_KEY,    RETRO_LBL_FAST_FORWARD_HOLD_KEY,RETROK_l,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_LOAD_STATE_KEY,           RETRO_LBL_LOAD_STATE_KEY,       RETROK_F4,      NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_SAVE_STATE_KEY,           RETRO_LBL_SAVE_STATE_KEY,       RETROK_F2,      NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_FULLSCREEN_TOGGLE_KEY,    RETRO_LBL_FULLSCREEN_TOGGLE_KEY,RETROK_f,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_QUIT_KEY,                 RETRO_LBL_QUIT_KEY,             RETROK_ESCAPE,  NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_STATE_SLOT_PLUS,          RETRO_LBL_STATE_SLOT_PLUS,      RETROK_F7,      NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_STATE_SLOT_MINUS,         RETRO_LBL_STATE_SLOT_MINUS,     RETROK_F6,      NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_REWIND,                   RETRO_LBL_REWIND,               RETROK_r,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_MOVIE_RECORD_TOGGLE,      RETRO_LBL_MOVIE_RECORD_TOGGLE,  RETROK_o,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_PAUSE_TOGGLE,             RETRO_LBL_PAUSE_TOGGLE,         RETROK_p,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_FRAMEADVANCE,             RETRO_LBL_FRAMEADVANCE,         RETROK_k,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_RESET,                    RETRO_LBL_RESET,                RETROK_h,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_SHADER_NEXT,              RETRO_LBL_SHADER_NEXT,          RETROK_m,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_SHADER_PREV,              RETRO_LBL_SHADER_PREV,          RETROK_n,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_CHEAT_INDEX_PLUS,         RETRO_LBL_CHEAT_INDEX_PLUS,     RETROK_y,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_CHEAT_INDEX_MINUS,        RETRO_LBL_CHEAT_INDEX_MINUS,    RETROK_t,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_CHEAT_TOGGLE,             RETRO_LBL_CHEAT_TOGGLE,         RETROK_u,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_SCREENSHOT,               RETRO_LBL_SCREENSHOT,           RETROK_F8,      NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_MUTE,                     RETRO_LBL_MUTE,                 RETROK_F9,      NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_OSK,                      RETRO_LBL_OSK,                  RETROK_F12,      NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_NETPLAY_FLIP,             RETRO_LBL_NETPLAY_FLIP,         RETROK_i,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_SLOWMOTION,               RETRO_LBL_SLOWMOTION,           RETROK_e,       NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ENABLE_HOTKEY,            RETRO_LBL_ENABLE_HOTKEY,        RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_VOLUME_UP,                RETRO_LBL_VOLUME_UP,            RETROK_KP_PLUS, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_VOLUME_DOWN,              RETRO_LBL_VOLUME_DOWN,          RETROK_KP_MINUS,NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_OVERLAY_NEXT,             RETRO_LBL_OVERLAY_NEXT,         RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_DISK_EJECT_TOGGLE,        RETRO_LBL_DISK_EJECT_TOGGLE,    RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_DISK_NEXT,                RETRO_LBL_DISK_NEXT,            RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_DISK_PREV,                RETRO_LBL_DISK_PREV,            RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_GRAB_MOUSE_TOGGLE,        RETRO_LBL_GRAB_MOUSE_TOGGLE,    RETROK_F11,     NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_MENU_TOGGLE,              RETRO_LBL_MENU_TOGGLE,          RETROK_F1,      NO_BTN, 0, AXIS_NONE },
-};
-
-/* Users 2 to MAX_USERS */
-static const struct retro_keybind retro_keybinds_rest[] = {
-    /*     | RetroPad button            | desc                           | keyboard key  | js btn |     js axis   | */
-   { true, RETRO_DEVICE_ID_JOYPAD_B,      RETRO_LBL_JOYPAD_B,              RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_Y,      RETRO_LBL_JOYPAD_Y,              RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_SELECT, RETRO_LBL_JOYPAD_SELECT,         RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_START,  RETRO_LBL_JOYPAD_START,          RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_UP,     RETRO_LBL_JOYPAD_UP,             RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_DOWN,   RETRO_LBL_JOYPAD_DOWN,           RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_LEFT,   RETRO_LBL_JOYPAD_LEFT,           RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_RIGHT,  RETRO_LBL_JOYPAD_RIGHT,          RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_A,      RETRO_LBL_JOYPAD_A,              RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_X,      RETRO_LBL_JOYPAD_X,              RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_L,      RETRO_LBL_JOYPAD_L,              RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_R,      RETRO_LBL_JOYPAD_R,              RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_L2,     RETRO_LBL_JOYPAD_L2,             RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_R2,     RETRO_LBL_JOYPAD_R2,             RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_L3,     RETRO_LBL_JOYPAD_L3,             RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RETRO_DEVICE_ID_JOYPAD_R3,     RETRO_LBL_JOYPAD_R3,             RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-
-   { true, RARCH_ANALOG_LEFT_X_PLUS,      RETRO_LBL_ANALOG_LEFT_X_PLUS,    RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_LEFT_X_MINUS,     RETRO_LBL_ANALOG_LEFT_X_MINUS,   RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_LEFT_Y_PLUS,      RETRO_LBL_ANALOG_LEFT_Y_PLUS,    RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_LEFT_Y_MINUS,     RETRO_LBL_ANALOG_LEFT_Y_MINUS,   RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_RIGHT_X_PLUS,     RETRO_LBL_ANALOG_RIGHT_X_PLUS,   RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_RIGHT_X_MINUS,    RETRO_LBL_ANALOG_RIGHT_X_MINUS,  RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_RIGHT_Y_PLUS,     RETRO_LBL_ANALOG_RIGHT_Y_PLUS,   RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_ANALOG_RIGHT_Y_MINUS,    RETRO_LBL_ANALOG_RIGHT_Y_MINUS,  RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-   { true, RARCH_TURBO_ENABLE,            RETRO_LBL_TURBO_ENABLE,          RETROK_UNKNOWN, NO_BTN, 0, AXIS_NONE },
-};
-
-#endif
-
 #endif
