@@ -22,7 +22,8 @@ enum rarch_core_type
    CORE_TYPE_DUMMY,
    CORE_TYPE_FFMPEG,
    CORE_TYPE_IMAGEVIEWER,
-   CORE_TYPE_NETRETROPAD
+   CORE_TYPE_NETRETROPAD,
+   CORE_TYPE_VIDEO_PROCESSOR
 };
 
 #endif
