@@ -2,6 +2,7 @@
 #define _CTR_DEBUG_H__
 
 #include <stdio.h>
+#include <3ds/types.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -18,8 +19,9 @@ void dump_result_value(Result val);
 #define DEBUG_VAR(X) printf( "%-20s: 0x%08X\n", #X, (u32)(X))
 #define DEBUG_INT(X) printf( "%-20s: %10i\n", #X, (s32)(X))
 #define DEBUG_VAR64(X) printf( #X"\r\t\t\t\t : 0x%016llX\n", (u64)(X))
-#define DEBUG_ERROR(X) do{if(X)dump_result_value(X)}while(0)
+#define DEBUG_ERROR(X) do{if(X)dump_result_value(X);}while(0)
 #define PRINTFPOS(X,Y) "\x1b["#X";"#Y"H"
 #define PRINTFPOS_STR(X,Y) "\x1b["X";"Y"H"
+#define PRINTF_LINE(X) "\x1b["X";0H"
 
 #endif //_CTR_DEBUG_H__
