@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *  Copyright (C) 2012-2015 - Michael Lelli
  *  Copyright (C) 2013-2014 - Steven Crowe
  *
@@ -25,19 +25,25 @@
 #include <retro_inline.h>
 #include <string/stdstring.h>
 
-#include "../../frontend/drivers/platform_linux.h"
-#include "../input_autodetect.h"
-#include "../input_config.h"
-#include "../input_joypad_driver.h"
-#include "../drivers_keyboard/keyboard_event_android.h"
-#include "../../performance_counters.h"
-#include "../../general.h"
-#include "../../driver.h"
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
 
 #ifdef HAVE_MENU
 #include "../../menu/menu_display.h"
 #endif
 
+#include "../input_config.h"
+#include "../input_driver.h"
+
+#include "../../frontend/drivers/platform_linux.h"
+#include "../../gfx/video_driver.h"
+#include "../input_joypad_driver.h"
+#include "../drivers_keyboard/keyboard_event_android.h"
+#include "../../tasks/tasks_internal.h"
+#include "../../performance_counters.h"
+#include "../../configuration.h"
+
 #define MAX_TOUCH 16
 #define MAX_NUM_KEYBOARDS 3
 
@@ -339,9 +345,7 @@ static void android_input_poll_main_cmd(void)
 
             runloop_ctl(RUNLOOP_CTL_SET_PAUSED, &boolean);
             runloop_ctl(RUNLOOP_CTL_SET_IDLE,   &boolean);
-#ifdef HAVE_MENU
             video_driver_unset_stub_frame();
-#endif
 
             if ((android_app->sensor_state_mask
                      & (UINT64_C(1) << RETRO_SENSOR_ACCELEROMETER_ENABLE))
@@ -361,9 +365,7 @@ static void android_input_poll_main_cmd(void)
 
             runloop_ctl(RUNLOOP_CTL_SET_PAUSED, &boolean);
             runloop_ctl(RUNLOOP_CTL_SET_IDLE,   &boolean);
-#ifdef HAVE_MENU
             video_driver_set_stub_frame();
-#endif
 
             /* Avoid draining battery while app is not being used. */
             if ((android_app->sensor_state_mask
@@ -456,10 +458,9 @@ static bool android_input_init_handle(void)
    return true;
 }
 
-static void *android_input_init(void)
+static void *android_input_init(const char *joypad_driver)
 {
    int32_t sdk;
-   settings_t *settings = config_get_ptr();
    struct android_app *android_app = (struct android_app*)g_android;
    android_input_t *android = (android_input_t*)
       calloc(1, sizeof(*android));
@@ -469,8 +470,7 @@ static void *android_input_init(void)
 
    android->thread.pads_connected = 0;
    android->copy.pads_connected = 0;
-   android->joypad         = input_joypad_init_driver(
-         settings->input.joypad_driver, android);
+   android->joypad         = input_joypad_init_driver(joypad_driver, android);
  
    input_keymaps_init_keyboard_lut(rarch_key_map_android);
  
@@ -503,7 +503,8 @@ static INLINE int android_input_poll_event_type_motion(
    size_t motion_ptr;
    bool keyup;
 
-   if (source & ~(AINPUT_SOURCE_TOUCHSCREEN | AINPUT_SOURCE_MOUSE))
+   // Only handle events from a touchscreen or mouse
+   if (!(source & (AINPUT_SOURCE_TOUCHSCREEN | AINPUT_SOURCE_MOUSE)))
       return 1;
 
    getaction  = AMotionEvent_getAction(event);
@@ -526,15 +527,17 @@ static INLINE int android_input_poll_event_type_motion(
    }
    else
    {
-      float x, y;
       int pointer_max = MIN(AMotionEvent_getPointerCount(event), MAX_TOUCH);
 
       for (motion_ptr = 0; motion_ptr < pointer_max; motion_ptr++)
       {
-         x = AMotionEvent_getX(event, motion_ptr);
-         y = AMotionEvent_getY(event, motion_ptr);
+         struct video_viewport vp = {0};
+         float x = AMotionEvent_getX(event, motion_ptr);
+         float y = AMotionEvent_getY(event, motion_ptr);
 
-         input_translate_coord_viewport(x, y,
+         video_driver_translate_coord_viewport_wrap(
+               &vp,
+               x, y,
                &android_data->pointer[motion_ptr].x,
                &android_data->pointer[motion_ptr].y,
                &android_data->pointer[motion_ptr].full_x,
@@ -637,13 +640,13 @@ static void handle_hotplug(android_input_data_t *android_data,
       struct android_app *android_app, int *port, int id,
       int source)
 {
-   char device_name[256]        = {0};
-   char device_model[256]       = {0};
-   char name_buf[256]           = {0};
+   char device_name[256];
+   char device_model[256];
+   char name_buf[256];
    int vendorId                 = 0;
    int productId                = 0;
-   bool back_mapped             = false;
-   settings_t         *settings = config_get_ptr();
+
+   device_name[0] = device_model[0] = name_buf[0] = '\0';
 
    frontend_android_get_name(device_model, sizeof(device_model));
 
@@ -753,6 +756,14 @@ static void handle_hotplug(android_input_data_t *android_data,
       }
    }
 
+   /* Other ATV Devices
+    * Add other common ATV devices that will follow the Android
+    * Gaempad convention as "Android Gamepad"
+    */
+    /* to-do: add DS4 on Bravia ATV */
+   else if (strstr(device_name, "NVIDIA"))
+      strlcpy (name_buf, "Android Gamepad", sizeof(name_buf));
+
    /* GPD XD
     * This is a simple hack, basically groups the "back"
     * button with the rest of the gamepad
@@ -878,43 +889,22 @@ static void handle_hotplug(android_input_data_t *android_data,
    if (*port < 0)
       *port = android_data->pads_connected;
 
-   if (settings->input.autodetect_enable)
-   {
-      bool      autoconfigured;
-      autoconfig_params_t params   = {{0}};
-
-      RARCH_LOG("Pads Connected: %d Port: %d\n %s VID/PID: %d/%d\n",
-            android_data->pads_connected, *port, name_buf,
-            params.vid, params.pid);
-
-      strlcpy(params.name, name_buf, sizeof(params.name));
-      params.idx = *port;
-      params.vid = vendorId;
-      params.pid = productId;
-      settings->input.pid[*port] = params.pid;
-      settings->input.vid[*port] = params.vid;
-
-      strlcpy(params.driver, android_joypad.ident, sizeof(params.driver));
-      autoconfigured = input_config_autoconfigure_joypad(&params);
-
-      if (autoconfigured)
-      {
-         if (settings->input.autoconf_binds[*port]
-               [RARCH_MENU_TOGGLE].joykey != 0)
-            back_mapped = true;
-      }
-   }
+   if (!input_autoconfigure_connect(
+         name_buf,
+         NULL,
+         android_joypad.ident,
+         *port,
+         vendorId,
+         productId))
+      input_config_set_device_name(*port, name_buf);
 
    if (!string_is_empty(name_buf))
    {
+      settings_t         *settings = config_get_ptr();
       strlcpy(settings->input.device_names[*port],
             name_buf, sizeof(settings->input.device_names[*port]));
    }
 
-   if (!back_mapped && settings->input.back_as_menu_toggle_enable)
-      settings->input.autoconf_binds[*port]
-         [RARCH_MENU_TOGGLE].joykey = AKEYCODE_BACK;
-
    android_data->pad_states[android_data->pads_connected].id = id;
    android_data->pad_states[android_data->pads_connected].port = *port;
    strlcpy(android_data->pad_states[*port].name, name_buf,
@@ -1026,6 +1016,28 @@ static void android_input_poll_memcpy(void *data)
    }
 }
 
+static bool android_input_key_pressed(void *data, int key)		
+{		
+   rarch_joypad_info_t joypad_info;
+   android_input_t *android = (android_input_t*)data;		
+   settings_t *settings     = config_get_ptr();		
+
+   if(       settings->input.binds[0][key].valid 
+         && android_keyboard_port_input_pressed(settings->input.binds[0],key))		
+      return true;		
+
+   joypad_info.joy_idx        = 0;
+   joypad_info.auto_binds     = settings->input.autoconf_binds[0];
+   joypad_info.axis_threshold = settings->input.axis_threshold;
+
+   if (settings->input.binds[0][key].valid &&		
+         input_joypad_pressed(android->joypad, joypad_info,
+            0, settings->input.binds[0], key))		
+      return true;		
+
+   return false;		
+}
+
 /* Handle all events. If our activity is in pause state,
  * block until we're unpaused.
  */
@@ -1036,7 +1048,7 @@ static void android_input_poll(void *data)
    struct android_app *android_app = (struct android_app*)g_android;
 
    while ((ident =
-            ALooper_pollAll((input_driver_key_pressed(&key))
+            ALooper_pollAll((android_input_key_pressed(data, key))
                ? -1 : 1,
                NULL, NULL, NULL)) >= 0)
    {
@@ -1097,21 +1109,25 @@ bool android_run_events(void *data)
 }
 
 static int16_t android_input_state(void *data,
+      rarch_joypad_info_t joypad_info,
       const struct retro_keybind **binds, unsigned port, unsigned device,
       unsigned idx, unsigned id)
 {
-   settings_t *settings = config_get_ptr();
-   android_input_t *android = (android_input_t*)data;
+   settings_t *settings               = config_get_ptr();
+   android_input_t *android           = (android_input_t*)data;
    android_input_data_t *android_data = (android_input_data_t*)&android->copy;
 
    switch (device)
    {
       case RETRO_DEVICE_JOYPAD:
-         return input_joypad_pressed(android->joypad, port, binds[port], id) ||
-                android_keyboard_port_input_pressed(binds[port],id);
+         return input_joypad_pressed(android->joypad, joypad_info,
+               port, binds[port], id) ||
+            android_keyboard_port_input_pressed(binds[port],id);
       case RETRO_DEVICE_ANALOG:
-         return input_joypad_analog(android->joypad, port, idx, id,
-               binds[port]);
+         if (binds[port])
+            return input_joypad_analog(android->joypad, joypad_info,
+                  port, idx, id, binds[port]);
+         break;
       case RETRO_DEVICE_POINTER:
          switch (id)
          {
@@ -1149,21 +1165,6 @@ static int16_t android_input_state(void *data,
    return 0;
 }
 
-static bool android_input_key_pressed(void *data, int key)
-{
-   android_input_t *android = (android_input_t*)data;
-   settings_t *settings     = config_get_ptr();
-
-   if(android_keyboard_port_input_pressed(settings->input.binds[0],key))
-      return true;
-
-   if (input_joypad_pressed(android->joypad,
-         0, settings->input.binds[0], key))
-      return true;
-
-   return false;
-}
-
 static bool android_input_meta_key_pressed(void *data, int key)
 {
    return false;
@@ -1321,7 +1322,6 @@ input_driver_t input_android = {
    android_input_init,
    android_input_poll,
    android_input_state,
-   android_input_key_pressed,
    android_input_meta_key_pressed,
    android_input_free_input,
    android_input_set_sensor_state,
