@@ -1,5 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2013-2014 - Jason Fetters
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *  Copyright (C) 2014-2015 - Jay McCarthy
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
@@ -31,7 +32,7 @@
 #include "../../../runloop.h"
 
 #ifdef HAVE_MENU
-#include "../../../menu/menu_entry.h"
+#include "../../../menu/widgets/menu_entry.h"
 #include "../../../menu/menu_navigation.h"
 #include "../../../menu/drivers/menu_generic.h"
 #endif
@@ -121,7 +122,7 @@ static void RunActionSheet(const char* title, const struct string_list* items,
                                     reuseIdentifier:cell_id];
 
   menu_entry_get_path(self.i, label, sizeof(label));
-  menu_entry_get_value(self.i, buffer, sizeof(buffer));
+  menu_entry_get_value(self.i, NULL, buffer, sizeof(buffer));
 
   result.textLabel.text = BOXSTRING(label);
 
@@ -274,15 +275,14 @@ static void RunActionSheet(const char* title, const struct string_list* items,
 
 - (void)alertView:(UIAlertView*)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
 {
-  if (buttonIndex == alertView.firstOtherButtonIndex) {
-    menu_entry_bind_key_set(self.i, RETROK_UNKNOWN);
-  } else if(buttonIndex == alertView.firstOtherButtonIndex + 1) {
-    menu_entry_bind_joykey_set(self.i, NO_BTN);
-  } else if(buttonIndex == alertView.firstOtherButtonIndex + 2) {
-    menu_entry_bind_joyaxis_set(self.i, AXIS_NONE);
-  }
-   
-  [self finishWithClickedButton:true];
+   if (buttonIndex == alertView.firstOtherButtonIndex)
+      menu_entry_bind_key_set(self.i, RETROK_UNKNOWN);
+   else if(buttonIndex == alertView.firstOtherButtonIndex + 1)
+      menu_entry_bind_joykey_set(self.i, NO_BTN);
+   else if(buttonIndex == alertView.firstOtherButtonIndex + 2)
+      menu_entry_bind_joyaxis_set(self.i, AXIS_NONE);
+
+   [self finishWithClickedButton:true];
 }
 
 - (void)checkBind:(NSTimer*)send
@@ -290,15 +290,14 @@ static void RunActionSheet(const char* title, const struct string_list* items,
    int32_t value = 0;
    int32_t idx = menu_entry_bind_index(self.i);
 
-   if ((value = cocoa_input_find_any_key())) {
-     menu_entry_bind_key_set(self.i, input_keymaps_translate_keysym_to_rk(value));
-   } else if ((value = cocoa_input_find_any_button(idx)) >= 0) {
-     menu_entry_bind_joykey_set(self.i, value);
-   } else if ((value = cocoa_input_find_any_axis(idx))) {
-     menu_entry_bind_joyaxis_set(self.i, (value > 0) ? AXIS_POS(value - 1) : AXIS_NEG(abs(value) - 1));
-   } else {
+   if ((value = cocoa_input_find_any_key()))
+      menu_entry_bind_key_set(self.i, input_keymaps_translate_keysym_to_rk(value));
+   else if ((value = cocoa_input_find_any_button(idx)) >= 0)
+      menu_entry_bind_joykey_set(self.i, value);
+   else if ((value = cocoa_input_find_any_axis(idx)))
+      menu_entry_bind_joyaxis_set(self.i, (value > 0) ? AXIS_POS(value - 1) : AXIS_NEG(abs(value) - 1));
+   else
       return;
-   }
 
    [self finishWithClickedButton:false];
 }
@@ -429,7 +428,7 @@ replacementString:(NSString *)string
    field          = [alertView textFieldAtIndex:0];
    field.delegate = self.formatter;
 
-   menu_entry_get_value(self.i, buffer, sizeof(buffer));
+   menu_entry_get_value(self.i, NULL, buffer, sizeof(buffer));
    if (string_is_empty(buffer))
       strlcpy(buffer,
             msg_hash_to_str(MENU_ENUM_LABEL_VALUE_NOT_AVAILABLE),
@@ -496,10 +495,9 @@ replacementString:(NSString *)string
   [super initialize:main idx:i];
   self.formatter = [RANumberFormatter new];
 
-  if (menu_entry_num_has_range(self.i)) {
-    [self.formatter setRangeFrom:BOXFLOAT(menu_entry_num_min(self.i))
-                              To:BOXFLOAT(menu_entry_num_max(self.i))];
-  }
+  if (menu_entry_num_has_range(self.i))
+     [self.formatter setRangeFrom:BOXFLOAT(menu_entry_num_min(self.i))
+                               To:BOXFLOAT(menu_entry_num_max(self.i))];
 }
 @end
 
