@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -25,6 +25,10 @@
 #define WM_MOUSEHWHEEL 0x20e
 #endif
 
+#ifndef WM_MOUSEWHEEL
+#define WM_MOUSEWHEEL 0x020A
+#endif
+
 #include <dinput.h>
 
 #include <stdlib.h>
@@ -34,13 +38,22 @@
 
 #include <windowsx.h>
 
-#include "../../general.h"
-#include "../../verbosity.h"
-#include "../input_autodetect.h"
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
+#include <string/stdstring.h>
+
 #include "../input_config.h"
+#include "../input_driver.h"
 #include "../input_joypad_driver.h"
 #include "../input_keymaps.h"
 
+#include "../../gfx/video_driver.h"
+
+#include "../../verbosity.h"
+#include "../../tasks/tasks_internal.h"
+
 /* Keep track of which pad indexes are 360 controllers.
  * Not static, will be read in xinput_joypad.c
  * -1 = not xbox pad, otherwise 0..3
@@ -62,6 +75,7 @@ struct pointer_status
 
 struct dinput_input
 {
+   char *joypad_driver_name;
    bool blocked;
    LPDIRECTINPUTDEVICE8 keyboard;
    LPDIRECTINPUTDEVICE8 mouse;
@@ -116,10 +130,9 @@ error:
    return false;
 }
 
-static void *dinput_init(void)
+static void *dinput_init(const char *joypad_driver)
 {
    struct dinput_input *di = NULL;
-   settings_t *settings = config_get_ptr();
 
    if (!dinput_init_context())
    {
@@ -131,6 +144,9 @@ static void *dinput_init(void)
    if (!di)
       return NULL;
 
+   if (!string_is_empty(joypad_driver))
+      di->joypad_driver_name = strdup(joypad_driver);
+
 #ifdef __cplusplus
    if (FAILED(IDirectInput8_CreateDevice(g_dinput_ctx, GUID_SysKeyboard, &di->keyboard, NULL)))
    {
@@ -176,7 +192,7 @@ static void *dinput_init(void)
    }
 
    input_keymaps_init_keyboard_lut(rarch_key_map_dinput);
-   di->joypad = input_joypad_init_driver(settings->input.joypad_driver, di);
+   di->joypad = input_joypad_init_driver(joypad_driver, di);
 
    return di;
 }
@@ -196,6 +212,9 @@ static void dinput_poll(void *data)
 {
    struct dinput_input *di = (struct dinput_input*)data;
 
+   if (!di)
+      return;
+
    memset(di->state, 0, sizeof(di->state));
    if (di->keyboard)
    {
@@ -264,16 +283,18 @@ static bool dinput_keyboard_pressed(struct dinput_input *di, unsigned key)
 }
 
 static bool dinput_is_pressed(struct dinput_input *di,
+      rarch_joypad_info_t joypad_info,
       const struct retro_keybind *binds,
       unsigned port, unsigned id)
 {
    const struct retro_keybind *bind = &binds[id];
+
    if (id >= RARCH_BIND_LIST_END)
       return false;
 
    if (!di->blocked && dinput_keyboard_pressed(di, bind->key))
       return true;
-   if (input_joypad_pressed(di->joypad, port, binds, id))
+   if (binds && binds[id].valid && input_joypad_pressed(di->joypad, joypad_info, port, binds, id))
       return true;
 
    return false;
@@ -303,12 +324,6 @@ static int16_t dinput_pressed_analog(struct dinput_input *di,
    return pressed_plus + pressed_minus;
 }
 
-static bool dinput_key_pressed(void *data, int key)
-{
-   settings_t *settings = config_get_ptr();
-   return dinput_is_pressed((struct dinput_input*)data, settings->input.binds[0], 0, key);
-}
-
 static bool dinput_meta_key_pressed(void *data, int key)
 {
    return false;
@@ -396,10 +411,11 @@ static int16_t dinput_mouse_state_screen(struct dinput_input *di, unsigned id)
 static int16_t dinput_pointer_state(struct dinput_input *di,
       unsigned idx, unsigned id, bool screen)
 {
-   bool pointer_down, valid, inside;
+   bool pointer_down, inside;
    int x, y;
+   struct video_viewport vp = {0};
    int16_t res_x = 0, res_y = 0, res_screen_x = 0, res_screen_y = 0;
-   unsigned num = 0;
+   unsigned num  = 0;
    struct pointer_status *check_pos = di->pointer_head.next;
 
    while (check_pos && num < idx)
@@ -421,10 +437,8 @@ static int16_t dinput_pointer_state(struct dinput_input *di,
       pointer_down = true;
    }
 
-   valid           = input_translate_coord_viewport(x, y,
-         &res_x, &res_y, &res_screen_x, &res_screen_y);
-
-   if (!valid)
+   if (!(video_driver_translate_coord_viewport_wrap(&vp, x, y,
+         &res_x, &res_y, &res_screen_x, &res_screen_y)))
       return 0;
 
    if (screen)
@@ -454,27 +468,30 @@ static int16_t dinput_pointer_state(struct dinput_input *di,
 }
 
 static int16_t dinput_input_state(void *data,
+      rarch_joypad_info_t joypad_info,
       const struct retro_keybind **binds, unsigned port,
       unsigned device, unsigned idx, unsigned id)
 {
    int16_t ret;
-   struct dinput_input *di = (struct dinput_input*)data;
-   settings_t *settings    = config_get_ptr();
+   struct dinput_input *di    = (struct dinput_input*)data;
 
    switch (device)
    {
       case RETRO_DEVICE_JOYPAD:
-         return dinput_is_pressed(di, binds[port], port, id);
-
+         return dinput_is_pressed(di, joypad_info, binds[port], port, id);
       case RETRO_DEVICE_KEYBOARD:
          return dinput_keyboard_pressed(di, id);
 
       case RETRO_DEVICE_ANALOG:
-         ret = dinput_pressed_analog(di, binds[port], idx, id);
-         if (!ret)
-            ret = input_joypad_analog(di->joypad, port,
-                  idx, id, settings->input.binds[port]);
-         return ret;
+         if (binds[port])
+         {
+            ret = dinput_pressed_analog(di, binds[port], idx, id);
+            if (!ret)
+               ret = input_joypad_analog(di->joypad, joypad_info,
+                     port, idx, id, binds[port]);
+            return ret;
+         }
+         return 0;
 
       case RETRO_DEVICE_MOUSE:
          return dinput_mouse_state(di, id);
@@ -589,7 +606,6 @@ extern "C"
 bool dinput_handle_message(void *dinput, UINT message, WPARAM wParam, LPARAM lParam)
 {
    struct dinput_input *di = (struct dinput_input *)dinput;
-   settings_t *settings    = config_get_ptr();
    /* WM_POINTERDOWN   : Arrives for each new touch event
     *                    with a new ID - add to list.
     * WM_POINTERUP     : Arrives once the pointer is no
@@ -635,19 +651,15 @@ bool dinput_handle_message(void *dinput, UINT message, WPARAM wParam, LPARAM lPa
             return true;
          }
       case WM_DEVICECHANGE:
-         {
             if (di->joypad)
                di->joypad->destroy();
-            di->joypad = input_joypad_init_driver(settings->input.joypad_driver, di);
-            break;
-         }
+            di->joypad = input_joypad_init_driver(di->joypad_driver_name, di);
+         break;
       case WM_MOUSEWHEEL:
-         {
             if (((short) HIWORD(wParam))/120 > 0)
                di->mouse_wu = true;
             if (((short) HIWORD(wParam))/120 < 0)
                di->mouse_wd = true;
-         }
          break;
       case WM_MOUSEHWHEEL:
          {
@@ -684,6 +696,9 @@ static void dinput_free(void *data)
       if (di->mouse)
          IDirectInputDevice8_Release(di->mouse);
 
+      if (string_is_empty(di->joypad_driver_name))
+         free(di->joypad_driver_name);
+
       free(di);
    }
 
@@ -754,7 +769,6 @@ input_driver_t input_dinput = {
    dinput_init,
    dinput_poll,
    dinput_input_state,
-   dinput_key_pressed,
    dinput_meta_key_pressed,
    dinput_free,
    NULL,
