@@ -1,5 +1,6 @@
 /*  RetroArch - A frontend for libretro.
- *  Plain DRM diver: Copyright (C) 2016 - Manuel Alfayate
+ *  Copyright (C) 2015-2017 - Manuel Alfayate
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -11,8 +12,11 @@
  *
  *  You should have received a copy of the GNU General Public License along with RetroArch.
  *  If not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
+ /*  Plain DRM diver */
+
 #include <xf86drm.h>
 #include <xf86drmMode.h>
 #include <libdrm/drm_fourcc.h>
@@ -20,17 +24,21 @@
 #include <sys/mman.h>
 #include <errno.h>
 
+#include <compat/strl.h>
 #include <rthreads/rthreads.h>
+#include <string/stdstring.h>
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "../../config.h"
+#endif
+
+#ifdef HAVE_MENU
+#include "../../menu/menu_driver.h"
 #endif
 
-#include "../../driver.h"
-#include "../../general.h"
-#include "../../retroarch.h"
-#include "../video_context_driver.h"
 #include "../font_driver.h"
+#include "../video_context_driver.h"
+#include "../../retroarch.h"
 
 #include "drm_pixformats.h"
 
@@ -360,7 +368,7 @@ static uint32_t get_plane_prop_id(uint32_t obj_id, const char *name)
       /* We look for the prop_id we need */
       for (j = 0; j < props->count_props; j++)
       {
-         if (!strcmp(props_info[j]->name, name))
+         if (string_is_equal(props_info[j]->name, name))
             return props_info[j]->prop_id;
       }
       RARCH_ERR ("DRM: plane %d fb property ID with name %s not found\n",
@@ -376,7 +384,7 @@ static void drm_format_name(const unsigned int fourcc, char *format_str)
 	for (i = 0; i < ARRAY_SIZE(format_info); i++)
    {
 		if (format_info[i].format == fourcc)
-			strcpy(format_str, format_info[i].name);
+			strlcpy(format_str, format_info[i].name, sizeof(format_str));
 	}
 }
 
@@ -407,7 +415,7 @@ static uint64_t drm_plane_type(drmModePlane *plane)
    for (j = 0; j < props->count_props; j++)
    {
       /* found the type property */
-      if ( !strcmp(drmModeGetProperty(drm.fd, props->props[j])->name, "type"))
+      if (string_is_equal(drmModeGetProperty(drm.fd, props->props[j])->name, "type"))
          return (props->prop_values[j]);
    }
    return (0);
@@ -508,7 +516,7 @@ static void drm_plane_setup(struct drm_surface *surface)
     * crtc_x and crtc_y are the position of the plane
     * pw and ph are the input size: the size of the area we read from the fb. */
    uint32_t plane_flags = 0;
-   uint32_t plane_w = drm.current_mode->vdisplay * surface->aspect;;
+   uint32_t plane_w = drm.current_mode->vdisplay * surface->aspect;
    uint32_t plane_h = drm.current_mode->vdisplay;
    /* If we obtain a scaled image width that is bigger than the physical screen width,
     * then we keep the physical screen width as our maximun width. */
@@ -727,10 +735,13 @@ static void *drm_gfx_init(const video_info_t *video,
    if (!init_drm())
    {
       RARCH_ERR ("DRM: Failed to initialize DRM\n");
+      free(_drmvars);
       return NULL;
    }
    else
-      RARCH_LOG ("DRM: Init succesful.\n");
+   {
+      RARCH_LOG ("DRM: Init successful.\n");
+   }
 
    _drmvars->kms_width  = drm.current_mode->hdisplay;  
    _drmvars->kms_height = drm.current_mode->vdisplay;  
@@ -739,11 +750,13 @@ static void *drm_gfx_init(const video_info_t *video,
 }
 
 static bool drm_gfx_frame(void *data, const void *frame, unsigned width,
-      unsigned height, uint64_t frame_count, unsigned pitch, const char *msg)
+      unsigned height, uint64_t frame_count, unsigned pitch, const char *msg,
+      video_frame_info_t *video_info)
 {
    struct drm_video *_drmvars = data;
 
-   if (width != _drmvars->core_width || height != _drmvars->core_height)
+   if (  ( width != _drmvars->core_width) || 
+         (height != _drmvars->core_height))
    {
       /* Sanity check. */
       if (width == 0 || height == 0)
@@ -773,11 +786,9 @@ static bool drm_gfx_frame(void *data, const void *frame, unsigned width,
       drm_plane_setup(_drmvars->main_surface);
    }
 
-   if (_drmvars->menu_active)
-   {
-      char buf[128];
-      video_monitor_get_fps(buf, sizeof(buf), NULL, 0);
-   }
+#ifdef HAVE_MENU
+   menu_driver_frame(video_info);
+#endif
 
    /* Update main surface: locate free page, blit and flip. */
    drm_surface_update(_drmvars, frame, _drmvars->main_surface);
@@ -928,7 +939,7 @@ static void drm_gfx_set_rotation(void *data, unsigned rotation)
    (void)rotation;
 }
 
-static bool drm_gfx_read_viewport(void *data, uint8_t *buffer)
+static bool drm_gfx_read_viewport(void *data, uint8_t *buffer, bool is_idle)
 {
    (void)data;
    (void)buffer;
