@@ -1,6 +1,6 @@
 #include "shaders_common.h"
 
-static const char *stock_fragment_xmb_simple = GLSL(
+static const char *stock_fragment_xmb_ribbon_simple = GLSL(
    uniform float time;
 
    void main()
