@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *  Copyright (C) 2012-2015 - Michael Lelli
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
@@ -23,19 +23,27 @@
 
 #include <streams/file_stream.h>
 
-#include "../../driver.h"
-#include "../../general.h"
-#include "../drivers_font_renderer/bitmap.h"
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
+#ifdef HAVE_MENU
 #include "../../menu/menu_driver.h"
 #include "../../menu/menu_display.h"
+#endif
 
 #ifdef HW_RVL
 #include "../../memory/wii/mem2_manager.h"
 #endif
 
-#include "../../defines/gx_defines.h"
+#include "../font_driver.h"
 
-extern syssram* __SYS_LockSram();
+#include "../drivers_font_renderer/bitmap.h"
+#include "../../defines/gx_defines.h"
+#include "../../configuration.h"
+#include "../../driver.h"
+
+extern syssram* __SYS_LockSram(void);
 extern u32 __SYS_UnlockSram(u32 write);
 
 struct gx_overlay_data
@@ -65,13 +73,6 @@ typedef struct gx_video
    void *framebuf[2];
 } gx_video_t;
 
-unsigned g_current_framebuf;
-
-bool g_vsync;
-OSCond g_video_cond;
-volatile bool g_draw_done;
-uint32_t g_orientation;
-
 static struct
 {
    uint32_t *data; /* needs to be resizable. */
@@ -86,13 +87,19 @@ static struct
    GXTexObj obj;
 } menu_tex ATTRIBUTE_ALIGN(32);
 
-uint8_t gx_fifo[256 * 1024] ATTRIBUTE_ALIGN(32);
-uint8_t display_list[1024] ATTRIBUTE_ALIGN(32);
+static OSCond g_video_cond;
+static unsigned g_current_framebuf;
+static volatile bool g_draw_done = false;
+static bool              g_vsync = false;
+static uint32_t g_orientation    = 0;
+
+static uint8_t gx_fifo[256 * 1024] ATTRIBUTE_ALIGN(32);
+static uint8_t display_list[1024]  ATTRIBUTE_ALIGN(32);
+static size_t display_list_size;
 uint16_t gx_xOrigin, gx_yOrigin;
 int8_t gx_system_xOrigin, gx_used_system_xOrigin;
 int8_t gx_xOriginNeg, gx_xOriginPos;
 int8_t gx_yOriginNeg, gx_yOriginPos;
-size_t display_list_size;
 GXRModeObj gx_mode;
 unsigned gx_old_width, gx_old_height;
 
@@ -539,27 +546,27 @@ static void setup_video_mode(gx_video_t *gx)
 
 static void init_texture(void *data, unsigned width, unsigned height)
 {
-   unsigned g_filter, menu_w, menu_h;
+   unsigned g_filter;
+   size_t fb_pitch;
+   unsigned fb_width, fb_height;
    gx_video_t *gx       = (gx_video_t*)data;
    GXTexObj *fb_ptr   	= (GXTexObj*)&g_tex.obj;
    GXTexObj *menu_ptr 	= (GXTexObj*)&menu_tex.obj;
    settings_t *settings = config_get_ptr();
 
-   width &= ~3;
-   height &= ~3;
-   g_filter = settings->video.smooth ? GX_LINEAR : GX_NEAR;
-   menu_w = 320;
-   menu_h = 240;
+   width               &= ~3;
+   height              &= ~3;
+   g_filter             = settings->video.smooth ? GX_LINEAR : GX_NEAR;
 
-   menu_w = menu_display_get_width();
-   menu_h = menu_display_get_height();
+   menu_display_get_fb_size(&fb_width, &fb_height,
+         &fb_pitch);
 
    GX_InitTexObj(fb_ptr, g_tex.data, width, height,
          (gx->rgb32) ? GX_TF_RGBA8 : gx->menu_texture_enable ?
          GX_TF_RGB5A3 : GX_TF_RGB565,
          GX_CLAMP, GX_CLAMP, GX_FALSE);
    GX_InitTexObjFilterMode(fb_ptr, g_filter, g_filter);
-   GX_InitTexObj(menu_ptr, menu_tex.data, menu_w, menu_h,
+   GX_InitTexObj(menu_ptr, menu_tex.data, fb_width, fb_height,
          GX_TF_RGB5A3, GX_CLAMP, GX_CLAMP, GX_FALSE);
    GX_InitTexObjFilterMode(menu_ptr, g_filter, g_filter);
    GX_InvalidateTexAll();
@@ -689,12 +696,13 @@ static void gx_efb_screenshot(void)
 static void *gx_init(const video_info_t *video,
       const input_driver_t **input, void **input_data)
 {
-   void *gxinput  = NULL;
-   gx_video_t *gx = (gx_video_t*)calloc(1, sizeof(gx_video_t));
+   settings_t *settings = config_get_ptr();
+   void *gxinput        = NULL;
+   gx_video_t *gx       = (gx_video_t*)calloc(1, sizeof(gx_video_t));
    if (!gx)
       return NULL;
 
-   gxinput     = input_gx.init();
+   gxinput     = input_gx.init(settings->input.joypad_driver);
    *input      = gxinput ? &input_gx : NULL;
    *input_data = gxinput;
 
@@ -990,7 +998,7 @@ static void gx_resize(void *data)
    }
    guMtxIdentity(m2);
    guMtxRotDeg(m2, 'Z', degrees);
-   guMtxConcat(m1, m2, m1);
+   c_guMtxConcat(m1, m2, m1);
    GX_LoadPosMtxImm(m1, GX_PNMTX0);
 
    init_texture(data, 4, 4);
@@ -1171,7 +1179,7 @@ static void gx_viewport_info(void *data, struct video_viewport *vp)
    *vp = gx->vp;
 }
 
-static bool gx_read_viewport(void *data, uint8_t *buffer)
+static bool gx_read_viewport(void *data, uint8_t *buffer, bool is_idle)
 {
    (void)data;
    (void)buffer;
@@ -1433,17 +1441,18 @@ static void gx_free(void *data)
 static bool gx_frame(void *data, const void *frame,
       unsigned width, unsigned height,
       uint64_t frame_count, unsigned pitch,
-      const char *msg)
+      const char *msg,
+      video_frame_info_t *video_info)
 {
+   char fps_text_buf[128];
    static struct retro_perf_counter gx_frame = {0};
-   char fps_txt[128]                  = {0};
-   char fps_text_buf[128]             = {0};
    gx_video_t *gx                     = (gx_video_t*)data;
    u8                       clear_efb = GX_FALSE;
-   settings_t               *settings = config_get_ptr();
 
-   performance_counter_init(&gx_frame, "gx_frame");
-   performance_counter_start(&gx_frame);
+   fps_text_buf[0]                    = '\0';
+
+   performance_counter_init(gx_frame, "gx_frame");
+   performance_counter_start_plus(video_info->is_perfcnt_enable, gx_frame);
 
    if(!gx || (!frame && !gx->menu_texture_enable))
       return true;
@@ -1478,8 +1487,8 @@ static bool gx_frame(void *data, const void *frame,
    {
       static struct retro_perf_counter gx_frame_convert = {0};
 
-      performance_counter_init(&gx_frame_convert, "gx_frame_convert");
-      performance_counter_start(&gx_frame_convert);
+      performance_counter_init(gx_frame_convert, "gx_frame_convert");
+      performance_counter_start_plus(video_info->is_perfcnt_enable, gx_frame_convert);
 
       if (gx->rgb32)
          convert_texture32(frame, g_tex.data, width, height, pitch);
@@ -1489,7 +1498,7 @@ static bool gx_frame(void *data, const void *frame,
          convert_texture16(frame, g_tex.data, width, height, pitch);
       DCFlushRange(g_tex.data, height * (width << (gx->rgb32 ? 2 : 1)));
 
-      performance_counter_stop(&gx_frame_convert);
+      performance_counter_stop_plus(video_info->is_perfcnt_enable, gx_frame_convert);
    }
 
    if (gx->menu_texture_enable && gx->menu_data)
@@ -1497,9 +1506,8 @@ static bool gx_frame(void *data, const void *frame,
       size_t fb_pitch;
       unsigned fb_width, fb_height;
 
-      fb_width  = menu_display_get_width();
-      fb_height = menu_display_get_height();
-      fb_pitch  = menu_display_get_framebuffer_pitch();
+      menu_display_get_fb_size(&fb_width, &fb_height,
+            &fb_pitch);
 
       convert_texture16(
             gx->menu_data,
@@ -1512,6 +1520,10 @@ static bool gx_frame(void *data, const void *frame,
             fb_width * fb_pitch);
    }
 
+#ifdef HAVE_MENU
+   menu_driver_frame(video_info);
+#endif
+
    GX_InvalidateTexAll();
 
    GX_SetCurrentMtx(GX_PNMTX0);
@@ -1532,16 +1544,15 @@ static bool gx_frame(void *data, const void *frame,
 
    GX_DrawDone();
 
-   video_monitor_get_fps(fps_txt, sizeof(fps_txt),
-         fps_text_buf, sizeof(fps_text_buf));
-
-   if (settings->fps_show)
+   if (video_info->fps_show)
    {
-      char mem1_txt[128] = {0};
-      char mem2_txt[128] = {0};
+      char mem1_txt[128];
+      char mem2_txt[128];
       unsigned x         = 15;
       unsigned y         = 35;
 
+      mem1_txt[0] = mem2_txt[0] = '\0';
+
       (void)mem2_txt;
 
       gx_blit_line(gx, x, y, fps_text_buf);
@@ -1571,7 +1582,7 @@ static bool gx_frame(void *data, const void *frame,
    VIDEO_SetNextFramebuffer(gx->framebuf[g_current_framebuf]);
    VIDEO_Flush();
 
-   performance_counter_stop(&gx_frame);
+   performance_counter_stop_plus(video_info->is_perfcnt_enable, gx_frame);
 
    return true;
 }
