@@ -1,6 +1,6 @@
 /* RetroArch - A frontend for libretro.
  * Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- * Copyright (C) 2011-2016 - Daniel De Matteis
+ * Copyright (C) 2011-2017 - Daniel De Matteis
  * Copyright (C) 2012-2014 - Jason Fetters
  * Copyright (C) 2014-2015 - Jay McCarthy
  *
@@ -22,9 +22,15 @@
 
 #include <sys/utsname.h>
 
+#include <mach/mach_host.h>
+
 #include <CoreFoundation/CoreFoundation.h>
 #include <CoreFoundation/CFArray.h>
 
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
 #ifdef __OBJC__
 #include <Foundation/NSPathUtilities.h>
 #include <objc/message.h>
@@ -41,21 +47,23 @@
 #endif
 
 #include <boolean.h>
+#include <compat/apple_compat.h>
 #include <retro_miscellaneous.h>
 #include <file/file_path.h>
 #include <rhash.h>
 
-#include "../frontend_driver.h"
-#include "../../file_path_special.h"
-#include "../../defaults.h"
-#include "../../general.h"
-#include "../../verbosity.h"
-#include "../../ui/ui_companion_driver.h"
-
 #ifdef HAVE_MENU
 #include "../../menu/menu_driver.h"
 #endif
 
+#include "../frontend_driver.h"
+#include "../../file_path_special.h"
+#include "../../configuration.h"
+#include "../../defaults.h"
+#include "../../runloop.h"
+#include "../../verbosity.h"
+#include "../../ui/ui_companion_driver.h"
+
 #if 1
 #define RELEASE_BUILD
 #endif
@@ -100,32 +108,6 @@ typedef enum
    CFAllDomainsMask     = 0x0ffff  /* All domains: all of the above and future items */
 } CFDomainMask;
 
-#ifndef __has_feature
-/* Compatibility with non-Clang compilers. */
-#define __has_feature(x) 0
-#endif
-
-#ifndef CF_RETURNS_RETAINED
-#if __has_feature(attribute_cf_returns_retained)
-#define CF_RETURNS_RETAINED __attribute__((cf_returns_retained))
-#else
-#define CF_RETURNS_RETAINED
-#endif
-#endif
-
-#ifndef NS_INLINE
-#define NS_INLINE inline
-#endif
-
-NS_INLINE CF_RETURNS_RETAINED CFTypeRef CFBridgingRetainCompat(id X)
-{
-#if __has_feature(objc_arc)
-   return (__bridge_retained CFTypeRef)X;
-#else
-   return X;
-#endif
-}
-
 static NSSearchPathDirectory NSConvertFlagsCF(unsigned flags)
 {
    switch (flags)
@@ -372,8 +354,6 @@ static void frontend_darwin_get_environment_settings(int *argc, char *argv[],
    fill_pathname_join(g_defaults.dir.assets, home_dir_buf, "assets", sizeof(g_defaults.dir.assets));
    fill_pathname_join(g_defaults.dir.system, home_dir_buf, "system", sizeof(g_defaults.dir.system));
    fill_pathname_join(g_defaults.dir.menu_config, home_dir_buf, "config", sizeof(g_defaults.dir.menu_config));
-   fill_pathname_join(g_defaults.path.config, g_defaults.dir.menu_config,
-        file_path_str(FILE_PATH_MAIN_CONFIG), sizeof(g_defaults.path.config));
    fill_pathname_join(g_defaults.dir.remap, g_defaults.dir.menu_config, "remaps", sizeof(g_defaults.dir.remap));
    fill_pathname_join(g_defaults.dir.database, home_dir_buf, "database/rdb", sizeof(g_defaults.dir.database));
    fill_pathname_join(g_defaults.dir.cursor, home_dir_buf, "database/cursors", sizeof(g_defaults.dir.cursor));
@@ -381,7 +361,6 @@ static void frontend_darwin_get_environment_settings(int *argc, char *argv[],
    fill_pathname_join(g_defaults.dir.thumbnails, home_dir_buf, "thumbnails", sizeof(g_defaults.dir.thumbnails));
    fill_pathname_join(g_defaults.dir.sram, home_dir_buf, "saves", sizeof(g_defaults.dir.sram));
    fill_pathname_join(g_defaults.dir.savestate, home_dir_buf, "states", sizeof(g_defaults.dir.savestate));
-   fill_pathname_join(g_defaults.dir.menu_config, home_dir_buf, "config", sizeof(g_defaults.dir.menu_config));
 #if defined(IOS)
    fill_pathname_join(g_defaults.dir.playlist, home_dir_buf, "playlists", sizeof(g_defaults.dir.playlist));
 #endif
@@ -398,8 +377,6 @@ static void frontend_darwin_get_environment_settings(int *argc, char *argv[],
    fill_pathname_join(g_defaults.dir.playlist, application_data, "playlists", sizeof(g_defaults.dir.playlist));
    fill_pathname_join(g_defaults.dir.thumbnails, application_data, "thumbnails", sizeof(g_defaults.dir.thumbnails));
    fill_pathname_join(g_defaults.dir.menu_config, application_data, "config", sizeof(g_defaults.dir.menu_config));
-   fill_pathname_join(g_defaults.path.config, g_defaults.dir.menu_config,
-        file_path_str(FILE_PATH_MAIN_CONFIG), sizeof(g_defaults.path.config));
    fill_pathname_join(g_defaults.dir.remap, g_defaults.dir.menu_config, "remaps", sizeof(g_defaults.dir.remap));
    fill_pathname_join(g_defaults.dir.core_assets, application_data, "downloads", sizeof(g_defaults.dir.core_assets));
    fill_pathname_join(g_defaults.dir.screenshot, application_data, "screenshots", sizeof(g_defaults.dir.screenshot));
@@ -660,9 +637,14 @@ static int frontend_darwin_parse_drive_list(void *data)
    CFSearchPathForDirectoriesInDomains(CFDocumentDirectory, CFUserDomainMask, 1, home_dir_buf, sizeof(home_dir_buf));
 
    menu_entries_append_enum(list,
-         home_dir_buf, "", MSG_UNKNOWN, FILE_TYPE_DIRECTORY, 0, 0);
-   menu_entries_append_enum(list, "/", "",
-         MSG_UNKNOWN, FILE_TYPE_DIRECTORY, 0, 0);
+         home_dir_buf,
+        msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
+        MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
+        MENU_SETTING_ACTION, 0, 0);
+   menu_entries_append_enum(list, "/",
+         msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
+         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
+        MENU_SETTING_ACTION, 0, 0);
 
    ret = 0;
 
@@ -692,7 +674,7 @@ static uint64_t frontend_darwin_get_mem_total(void)
 
 static uint64_t frontend_darwin_get_mem_used(void)
 {
-#if defined(OSX) && !defined(OSX_PPC)
+#if (defined(OSX) && !(defined(__ppc__) || defined(__ppc64__)))
     vm_size_t page_size;
     vm_statistics64_data_t vm_stats;
     mach_port_t mach_port        = mach_host_self();
@@ -734,5 +716,7 @@ frontend_ctx_driver_t frontend_ctx_darwin = {
    NULL,                         /* get_sighandler_state */
    NULL,                         /* set_sighandler_state */
    NULL,                         /* destroy_signal_handler_state */
+   NULL,                         /* attach_console */
+   NULL,                         /* detach_console */
    "darwin",
 };
