@@ -1,6 +1,6 @@
 /* RetroArch - A frontend for libretro.
  * Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- * Copyright (C) 2011-2016 - Daniel De Matteis
+ * Copyright (C) 2011-2017 - Daniel De Matteis
  * Copyright (C) 2012-2015 - Jason Fetters
  * Copyright (C) 2012-2015 - Michael Lelli
  *
@@ -29,6 +29,10 @@
 #include <signal.h>
 #include <pthread.h>
 
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
 #ifdef ANDROID
 #include <sys/system_properties.h>
 #ifdef __arm__
@@ -41,6 +45,8 @@
 #include <retro_inline.h>
 #include <compat/strl.h>
 #include <rhash.h>
+#include <retro_stat.h>
+#include <lists/file_list.h>
 #include <file/file_path.h>
 #include <streams/file_stream.h>
 #include <string/stdstring.h>
@@ -49,11 +55,17 @@
 #include "../frontend.h"
 #include "../frontend_driver.h"
 #include "../../defaults.h"
-#include "../../general.h"
 #include "../../retroarch.h"
 #include "../../verbosity.h"
+#include "../../paths.h"
 #include "platform_linux.h"
 
+#ifdef HAVE_MENU
+#include "../../menu/menu_driver.h"
+#include "../../menu/menu_display.h"
+#include "../../menu/menu_entries.h"
+#endif
+
 #ifdef ANDROID
 enum
 {
@@ -359,7 +371,6 @@ static void android_app_entry(void *data)
 {
    char arguments[]  = "retroarch";
    char      *argv[] = {arguments,   NULL};
-   int          ret  = 0;
    int          argc = 1;
 
    if (rarch_main(argc, argv, data) != 0)
@@ -368,12 +379,14 @@ static void android_app_entry(void *data)
    do
    {
       unsigned sleep_ms = 0;
-      ret = runloop_iterate(&sleep_ms);
+      int           ret = runloop_iterate(&sleep_ms);
 
       if (ret == 1 && sleep_ms > 0)
          retro_sleep(sleep_ms);
       task_queue_ctl(TASK_QUEUE_CTL_CHECK, NULL);
-   }while (ret != -1);
+      if (ret == -1)
+         break;
+   }while(1);
 
    main_exit(data);
 #endif
@@ -542,8 +555,8 @@ static bool device_is_game_console(const char *name)
 
 bool test_permissions(const char *path)
 {
-   char buf[PATH_MAX_LENGTH];
-   bool ret;
+   bool ret                  = false;
+   char buf[PATH_MAX_LENGTH] = {0};
 
    __android_log_print(ANDROID_LOG_INFO,
       "RetroArch", "Testing permissions for %s\n",path);
@@ -636,6 +649,8 @@ static void check_proc_acpi_battery(const char * node, bool * have_battery,
    int          secs = -1;
    int           pct = -1;
 
+   path[0]           = '\0';
+
    snprintf(path, sizeof(path), "%s/%s/%s", base, node, "state");
 
    if (!path_file_exists(path))
@@ -747,9 +762,7 @@ static void check_proc_acpi_sysfs_battery(const char *node,
       bool *have_battery, bool *charging,
       int *seconds, int *percent)
 {
-   unsigned capacity;
-   char path[1024]   = {0};
-   char info[1024]   = {0};
+   char path[1024];
    const char *base  = proc_acpi_sysfs_battery_path;
    char        *buf  = NULL;
    char         *ptr = NULL;
@@ -757,25 +770,37 @@ static void check_proc_acpi_sysfs_battery(const char *node,
    char         *val = NULL;
    bool       charge = false;
    bool       choose = false;
+   unsigned capacity = 0;
    ssize_t length    = 0;
    int       maximum = -1;
    int     remaining = -1;
    int          secs = -1;
    int           pct = -1;
 
-   if (!strstr(node, "BAT"))
-      return;
+   path[0]           = '\0';
 
    snprintf(path, sizeof(path), "%s/%s/%s", base, node, "status");
+
    if (!path_file_exists(path))
       return;
    if (filestream_read_file(path, (void**)&buf, &length) != 1)
       return;
 
-   if (strstr((char*)buf, "Discharging"))
-      *have_battery = true;
-   else if (strstr((char*)buf, "Full"))
-      *have_battery = true;
+   if (buf)
+   {
+      if (strstr((char*)buf, "Discharging"))
+         *have_battery = true;
+      else if (strstr((char*)buf, "Charging"))
+      {
+         *have_battery = true;
+         *charging = true;
+      }
+      else if (strstr((char*)buf, "Full"))
+         *have_battery = true;
+      free(buf);
+   }
+
+   buf = NULL;
 
    snprintf(path, sizeof(path), "%s/%s/%s", base, node, "capacity");
    if (filestream_read_file(path, (void**)&buf, &length) != 1)
@@ -786,14 +811,13 @@ static void check_proc_acpi_sysfs_battery(const char *node,
    *percent = capacity;
 
 end:
-   if (buf)
-      free(buf);
+   free(buf);
    buf = NULL;
 }
 
 static void check_proc_acpi_ac_adapter(const char * node, bool *have_ac)
 {
-   char path[1024]  = {0};
+   char path[1024];
    const char *base = proc_acpi_ac_adapter_path;
    char       *buf  = NULL;
    char        *ptr = NULL;
@@ -801,6 +825,8 @@ static void check_proc_acpi_ac_adapter(const char * node, bool *have_ac)
    char        *val = NULL;
    ssize_t length   = 0;
 
+   path[0]          = '\0';
+
    snprintf(path, sizeof(path), "%s/%s/%s", base, node, "state");
    if (!path_file_exists(path))
       return;
@@ -825,11 +851,13 @@ static void check_proc_acpi_ac_adapter(const char * node, bool *have_ac)
 
 static void check_proc_acpi_sysfs_ac_adapter(const char * node, bool *have_ac)
 {
-   char  path[1024] = {0};
+   char  path[1024];
    ssize_t length   = 0;
    char     *buf    = NULL;
    const char *base = proc_acpi_sysfs_ac_adapter_path;
 
+   path[0]          = '\0';
+
    snprintf(path, sizeof(path), "%s/%s", base, "online");
    if (!path_file_exists(path))
       return;
@@ -845,7 +873,6 @@ static void check_proc_acpi_sysfs_ac_adapter(const char * node, bool *have_ac)
 static bool next_string(char **_ptr, char **_str)
 {
    char *ptr = *_ptr;
-   char *str = *_str;
 
    while (*ptr == ' ')       /* skip any spaces... */
       ptr++;
@@ -853,14 +880,12 @@ static bool next_string(char **_ptr, char **_str)
    if (*ptr == '\0')
       return false;
 
-   str = ptr;
    while ((*ptr != ' ') && (*ptr != '\n') && (*ptr != '\0'))
       ptr++;
 
    if (*ptr != '\0')
       *(ptr++) = '\0';
 
-   *_str = str;
    *_ptr = ptr;
    return true;
 }
@@ -965,20 +990,18 @@ static bool frontend_linux_powerstate_check_acpi(
       enum frontend_powerstate *state,
       int *seconds, int *percent)
 {
-   bool ret            = false;
-   struct RDIR *entry  = NULL;
    bool have_battery   = false;
    bool have_ac        = false;
    bool charging       = false;
-
-   *state = FRONTEND_POWERSTATE_NONE;
-
-   entry = retro_opendir(proc_acpi_battery_path);
+   struct RDIR *entry  = retro_opendir(proc_acpi_battery_path);
    if (!entry)
-      goto end;
+      return false;
 
    if (retro_dirent_error(entry))
-      goto end;
+   {
+      retro_closedir(entry);
+      return false;
+   }
 
    while (retro_readdir(entry))
       check_proc_acpi_battery(retro_dirent_get_name(entry),
@@ -988,12 +1011,14 @@ static bool frontend_linux_powerstate_check_acpi(
 
    entry = retro_opendir(proc_acpi_ac_adapter_path);
    if (!entry)
-      goto end;
+      return false;
 
    while (retro_readdir(entry))
       check_proc_acpi_ac_adapter(
             retro_dirent_get_name(entry), &have_ac);
 
+   retro_closedir(entry);
+
    if (!have_battery)
       *state = FRONTEND_POWERSTATE_NO_SOURCE;
    else if (charging)
@@ -1003,28 +1028,17 @@ static bool frontend_linux_powerstate_check_acpi(
    else
       *state = FRONTEND_POWERSTATE_ON_POWER_SOURCE;
 
-   ret = true;
-
-end:
-   if (entry)
-      retro_closedir(entry);
-
-   return ret;
+   return true;
 }
 
 static bool frontend_linux_powerstate_check_acpi_sysfs(
       enum frontend_powerstate *state,
       int *seconds, int *percent)
 {
-   bool ret            = false;
-   struct RDIR *entry  = NULL;
    bool have_battery   = false;
    bool have_ac        = false;
    bool charging       = false;
-
-   *state = FRONTEND_POWERSTATE_NONE;
-
-   entry = retro_opendir(proc_acpi_sysfs_battery_path);
+   struct RDIR *entry  = retro_opendir(proc_acpi_sysfs_battery_path);
    if (!entry)
       goto error;
 
@@ -1032,8 +1046,13 @@ static bool frontend_linux_powerstate_check_acpi_sysfs(
       goto error;
 
    while (retro_readdir(entry))
-      check_proc_acpi_sysfs_battery(retro_dirent_get_name(entry),
-            &have_battery, &charging, seconds, percent);
+   {
+      const char *node = retro_dirent_get_name(entry);
+
+      if (node && strstr(node, "BAT"))
+         check_proc_acpi_sysfs_battery(node,
+               &have_battery, &charging, seconds, percent);
+   }
 
    retro_closedir(entry);
 
@@ -1093,6 +1112,8 @@ static enum frontend_powerstate frontend_linux_get_powerstate(
    if (frontend_linux_powerstate_check_acpi_sysfs(&ret, seconds, percent))
       return ret;
 
+   ret = FRONTEND_POWERSTATE_NONE;
+
    if (frontend_linux_powerstate_check_acpi(&ret, seconds, percent))
       return ret;
 
@@ -1109,6 +1130,7 @@ static enum frontend_powerstate frontend_linux_get_powerstate(
 #define LINUX_ARCH_PPC64      0x1028cf52U
 #define LINUX_ARCH_MIPS       0x7c9aa25eU
 #define LINUX_ARCH_TILE       0x7c9e7873U
+#define LINUX_ARCH_AARCH64    0x191bfc0eU
 #define LINUX_ARCH_ARMV7B     0xf27015f4U
 #define LINUX_ARCH_ARMV7L     0xf27015feU
 #define LINUX_ARCH_ARMV6L     0xf27015ddU
@@ -1130,6 +1152,8 @@ static enum frontend_architecture frontend_linux_get_architecture(void)
 
    switch (buffer_hash)
    {
+      case LINUX_ARCH_AARCH64:
+         return FRONTEND_ARCH_ARMV8;
       case LINUX_ARCH_ARMV7L:
       case LINUX_ARCH_ARMV7B:
          return FRONTEND_ARCH_ARMV7;
@@ -1453,8 +1477,6 @@ static void frontend_linux_get_env(int *argc,
             "RetroArch", "[ENV]: application location: [%s]\n", app_dir);
          if (args && *app_dir)
          {
-            char buf[PATH_MAX_LENGTH] = {0};
-
             fill_pathname_join(g_defaults.dir.assets, app_dir,
                   "assets", sizeof(g_defaults.dir.assets));
             fill_pathname_join(g_defaults.dir.cache, app_dir,
@@ -1463,8 +1485,6 @@ static void frontend_linux_get_env(int *argc,
                   "shaders", sizeof(g_defaults.dir.shader));
             fill_pathname_join(g_defaults.dir.overlay, app_dir,
                   "overlays", sizeof(g_defaults.dir.overlay));
-            fill_pathname_join(g_defaults.dir.osk_overlay, app_dir,
-                  "overlays", sizeof(g_defaults.dir.osk_overlay));
             fill_pathname_join(g_defaults.dir.core, app_dir,
                   "cores", sizeof(g_defaults.dir.core));
             fill_pathname_join(g_defaults.dir.core_info,
@@ -1483,7 +1503,7 @@ static void frontend_linux_get_env(int *argc,
                   app_dir, "database/cursors", sizeof(g_defaults.dir.cursor));
             fill_pathname_join(g_defaults.dir.wallpapers,
                   app_dir, "assets/wallpapers", sizeof(g_defaults.dir.wallpapers));
-            if(*downloads_dir && test_permissions(downloads_dir))
+            if(!string_is_empty(downloads_dir) && test_permissions(downloads_dir))
             {
                fill_pathname_join(g_defaults.dir.core_assets,
                      downloads_dir, "", sizeof(g_defaults.dir.core_assets));
@@ -1499,24 +1519,10 @@ static void frontend_linux_get_env(int *argc,
                "RetroArch", "[ENV]: default download folder: [%s]",
                g_defaults.dir.core_assets);
 
-            if(*screenshot_dir && test_permissions(screenshot_dir))
-            {
-               fill_pathname_join(g_defaults.dir.screenshot,
-                     screenshot_dir, "", sizeof(g_defaults.dir.screenshot));
-            }
-            else
-            {
-               fill_pathname_join(g_defaults.dir.screenshot,
-                     app_dir, "screenshots", sizeof(g_defaults.dir.screenshot));
-               path_mkdir(g_defaults.dir.screenshot);
-            }
-
-            __android_log_print(ANDROID_LOG_INFO,
-               "RetroArch", "[ENV]: default screenshot folder: [%s]",
-               g_defaults.dir.screenshot);
-
             switch (perms)
             {
+               /* Set defaults for this since we can't guarantee saving on content dir will
+                  work in this case */
                case INTERNAL_STORAGE_APPDIR_WRITABLE:
                   fill_pathname_join(g_defaults.dir.sram,
                         internal_storage_app_path, "saves", sizeof(g_defaults.dir.sram));
@@ -1536,19 +1542,25 @@ static void frontend_linux_get_env(int *argc,
                   fill_pathname_join(g_defaults.dir.cheats,
                         internal_storage_app_path, "cheats", sizeof(g_defaults.dir.cheats));
 
-                  /* TODO/FIXME - Test if this is needed at all, as far as I know,
-                   * every directory we set in g_defaults already gets created if it
-                   * doesn't exist already */
+                  if(!string_is_empty(screenshot_dir) && test_permissions(screenshot_dir))
+                  {
+                     fill_pathname_join(g_defaults.dir.screenshot,
+                           screenshot_dir, "", sizeof(g_defaults.dir.screenshot));
+                  }
+                  else
+                  {
+                     fill_pathname_join(g_defaults.dir.screenshot,
+                           internal_storage_app_path, "screenshots", sizeof(g_defaults.dir.screenshot));
+                     path_mkdir(g_defaults.dir.screenshot);
+                  }
+
                   path_mkdir(g_defaults.dir.sram);
                   path_mkdir(g_defaults.dir.savestate);
                   path_mkdir(g_defaults.dir.system);
-                  path_mkdir(g_defaults.dir.menu_config);
-                  path_mkdir(g_defaults.dir.remap);
-                  path_mkdir(g_defaults.dir.thumbnails);
-                  path_mkdir(g_defaults.dir.playlist);
-                  path_mkdir(g_defaults.dir.cheats);
                   break;
                case INTERNAL_STORAGE_NOT_WRITABLE:
+                  /* Set defaults for this since we can't guarantee saving on content dir will
+                     work in this case */
                   fill_pathname_join(g_defaults.dir.sram,
                         app_dir, "saves", sizeof(g_defaults.dir.sram));
                   fill_pathname_join(g_defaults.dir.savestate,
@@ -1567,19 +1579,25 @@ static void frontend_linux_get_env(int *argc,
                   fill_pathname_join(g_defaults.dir.cheats,
                         app_dir, "cheats", sizeof(g_defaults.dir.cheats));
 
-                  /* TODO/FIXME - Test if this is needed at all, as far as I know,
-                   * every directory we set in g_defaults already gets created if it
-                   * doesn't exist already */
+                  if(!string_is_empty(screenshot_dir) && test_permissions(screenshot_dir))
+                  {
+                     fill_pathname_join(g_defaults.dir.screenshot,
+                           screenshot_dir, "", sizeof(g_defaults.dir.screenshot));
+                  }
+                  else
+                  {
+                     fill_pathname_join(g_defaults.dir.screenshot,
+                           app_dir, "screenshots", sizeof(g_defaults.dir.screenshot));
+                     path_mkdir(g_defaults.dir.screenshot);
+                  }
+
                   path_mkdir(g_defaults.dir.sram);
                   path_mkdir(g_defaults.dir.savestate);
                   path_mkdir(g_defaults.dir.system);
-                  path_mkdir(g_defaults.dir.menu_config);
-                  path_mkdir(g_defaults.dir.remap);
-                  path_mkdir(g_defaults.dir.thumbnails);
-                  path_mkdir(g_defaults.dir.playlist);
-                  path_mkdir(g_defaults.dir.cheats);
                   break;
                case INTERNAL_STORAGE_WRITABLE:
+                  /* Don't set defaults for saves, states, system or screenshots
+                     in this case to be able to honour saving on content dir */
                   fill_pathname_join(g_defaults.dir.menu_config,
                         internal_storage_path, "RetroArch/config", sizeof(g_defaults.dir.menu_config));
                   fill_pathname_join(g_defaults.dir.remap,
@@ -1590,38 +1608,14 @@ static void frontend_linux_get_env(int *argc,
                         internal_storage_path, "RetroArch/playlists", sizeof(g_defaults.dir.playlist));
                   fill_pathname_join(g_defaults.dir.cheats,
                         internal_storage_path, "RetroArch/cheats", sizeof(g_defaults.dir.cheats));
-
-                  /* TODO/FIXME - Test if this is needed at all, as far as I know,
-                   * every directory we set in g_defaults already gets created if it
-                   * doesn't exist already */
-                  path_mkdir(g_defaults.dir.menu_config);
-                  path_mkdir(g_defaults.dir.remap);
-                  path_mkdir(g_defaults.dir.thumbnails);
-                  path_mkdir(g_defaults.dir.playlist);
-                  path_mkdir(g_defaults.dir.cheats);
                default:
                   break;
             }
-
-            /* create save and system directories in the internal dir too */
-            fill_pathname_join(buf, app_dir, "saves", sizeof(buf));
-            path_mkdir(buf);
-
-            fill_pathname_join(buf, app_dir, "states", sizeof(buf));
-            path_mkdir(buf);
-
-            fill_pathname_join(buf, app_dir, "system", sizeof(buf));
-            path_mkdir(buf);
-
-            /* create save and system directories in the internal sd too */
-            fill_pathname_join(buf, internal_storage_app_path, "saves", sizeof(buf));
-            path_mkdir(buf);
-
-            fill_pathname_join(buf, internal_storage_app_path, "states", sizeof(buf));
-            path_mkdir(buf);
-
-            fill_pathname_join(buf, internal_storage_app_path, "system", sizeof(buf));
-            path_mkdir(buf);
+            path_mkdir(g_defaults.dir.menu_config);
+            path_mkdir(g_defaults.dir.remap);
+            path_mkdir(g_defaults.dir.thumbnails);
+            path_mkdir(g_defaults.dir.playlist);
+            path_mkdir(g_defaults.dir.cheats);
 
             __android_log_print(ANDROID_LOG_INFO,
                "RetroArch", "[ENV]: default savefile folder: [%s]",
@@ -1632,6 +1626,9 @@ static void frontend_linux_get_env(int *argc,
             __android_log_print(ANDROID_LOG_INFO,
                "RetroArch", "[ENV]: default system folder: [%s]",
                g_defaults.dir.system);
+            __android_log_print(ANDROID_LOG_INFO,
+               "RetroArch", "[ENV]: default screenshot folder: [%s]",
+               g_defaults.dir.screenshot);
          }
       }
    }
@@ -1651,17 +1648,40 @@ static void frontend_linux_get_env(int *argc,
    else if (strstr(device_model, "GAMEMID_BT"))
       g_defaults.settings.out_latency = 160;
    else if (strstr(device_model, "SHIELD"))
+   {
       g_defaults.settings.video_refresh_rate = 60.0;
+#ifdef HAVE_MENU
+#ifdef HAVE_MATERIALUI
+      g_defaults.menu.materialui.menu_color_theme_enable = true;
+      g_defaults.menu.materialui.menu_color_theme        = MATERIALUI_THEME_NVIDIA_SHIELD;
+#endif
+#endif
+
+#if 0
+      /* Set the OK/cancel menu buttons to the default 
+       * ones used for Shield */
+      g_defaults.menu.controls.set = true;
+      g_defaults.menu.controls.menu_btn_ok     = RETRO_DEVICE_ID_JOYPAD_B;
+      g_defaults.menu.controls.menu_btn_cancel = RETRO_DEVICE_ID_JOYPAD_A;
+#endif
+   }
    else if (strstr(device_model, "JSS15J"))
       g_defaults.settings.video_refresh_rate = 59.65;
 
 
-   /* Explicitly disable input overlay by default
-    * for gamepad-like/console devices. */
+   /* For gamepad-like/console devices:
+    *
+    * - Explicitly disable input overlay by default
+    * - Use XMB menu driver by default
+    *
+    * */
 
    if (device_is_game_console(device_model))
+   {
+      g_defaults.overlay.set    = true;
+      g_defaults.overlay.enable = false;
       snprintf(g_defaults.settings.menu, sizeof(g_defaults.settings.menu), "xmb");
-
+   }
 #else
    char base_path[PATH_MAX] = {0};
    const char *xdg          = getenv("XDG_CONFIG_HOME");
@@ -1682,8 +1702,23 @@ static void frontend_linux_get_env(int *argc,
          "cores", sizeof(g_defaults.dir.core_info));
    fill_pathname_join(g_defaults.dir.autoconfig, base_path,
          "autoconfig", sizeof(g_defaults.dir.autoconfig));
-   fill_pathname_join(g_defaults.dir.assets, base_path,
-         "assets", sizeof(g_defaults.dir.assets));
+
+   if (path_is_directory("/usr/local/share/retroarch/assets"))
+      fill_pathname_join(g_defaults.dir.assets, "/usr/local/share/retroarch",
+            "assets", sizeof(g_defaults.dir.assets));
+   else if (path_is_directory("/usr/share/retroarch/assets"))
+      fill_pathname_join(g_defaults.dir.assets, "/usr/share/retroarch",
+            "assets", sizeof(g_defaults.dir.assets));
+   else if (path_is_directory("/usr/local/share/games/retroarch/assets"))
+      fill_pathname_join(g_defaults.dir.assets, "/usr/local/share/games/retroarch",
+            "assets", sizeof(g_defaults.dir.assets));
+   else if (path_is_directory("/usr/share/games/retroarch/assets"))
+      fill_pathname_join(g_defaults.dir.assets, "/usr/share/games/retroarch",
+            "assets", sizeof(g_defaults.dir.assets));
+   else
+      fill_pathname_join(g_defaults.dir.assets, base_path,
+            "assets", sizeof(g_defaults.dir.assets));
+
    fill_pathname_join(g_defaults.dir.menu_config, base_path,
          "config", sizeof(g_defaults.dir.menu_config));
    fill_pathname_join(g_defaults.dir.remap, g_defaults.dir.menu_config,
@@ -1700,8 +1735,6 @@ static void frontend_linux_get_env(int *argc,
          "cheats", sizeof(g_defaults.dir.cheats));
    fill_pathname_join(g_defaults.dir.overlay, base_path,
          "overlay", sizeof(g_defaults.dir.overlay));
-   fill_pathname_join(g_defaults.dir.osk_overlay, base_path,
-         "overlay", sizeof(g_defaults.dir.osk_overlay));
    fill_pathname_join(g_defaults.dir.core_assets, base_path,
          "downloads", sizeof(g_defaults.dir.core_assets));
    fill_pathname_join(g_defaults.dir.screenshot, base_path,
@@ -1826,31 +1859,42 @@ static void frontend_linux_init(void *data)
 
 }
 
-#ifdef ANDROID
-static int frontend_android_parse_drive_list(void *data)
+static int frontend_linux_parse_drive_list(void *data)
 {
+#ifdef HAVE_MENU
    file_list_t *list = (file_list_t*)data;
 
+#ifdef ANDROID
    menu_entries_append_enum(list,
          app_dir,
          msg_hash_to_str(MSG_APPLICATION_DIR),
-         MSG_APPLICATION_DIR, FILE_TYPE_DIRECTORY, 0, 0);
+         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
+         MENU_SETTING_ACTION, 0, 0);
    menu_entries_append_enum(list,
          internal_storage_app_path,
          msg_hash_to_str(MSG_EXTERNAL_APPLICATION_DIR),
-         MSG_EXTERNAL_APPLICATION_DIR,
-         FILE_TYPE_DIRECTORY, 0, 0);
+         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
+         MENU_SETTING_ACTION, 0, 0);
    menu_entries_append_enum(list,
          internal_storage_path,
-         msg_hash_to_str(MSG_INTERNAL_MEMORY),
-         MSG_INTERNAL_MEMORY, FILE_TYPE_DIRECTORY, 0, 0);
+         msg_hash_to_str(MSG_INTERNAL_STORAGE),
+         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
+         MENU_SETTING_ACTION, 0, 0);
+   menu_entries_append_enum(list,
+         "/storage",
+         msg_hash_to_str(MSG_REMOVABLE_STORAGE),
+         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
+         MENU_SETTING_ACTION, 0, 0);
+#endif
 
-   menu_entries_append_enum(list, "/", "",
-         MSG_UNKNOWN, FILE_TYPE_DIRECTORY, 0, 0);
+   menu_entries_append_enum(list, "/",
+         msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
+         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
+         MENU_SETTING_ACTION, 0, 0);
+#endif
 
    return 0;
 }
-#endif
 
 #ifndef HAVE_DYNAMIC
 
@@ -1872,10 +1916,8 @@ static bool frontend_linux_set_fork(enum frontend_fork fork_mode)
 
          {
             char executable_path[PATH_MAX_LENGTH] = {0};
-            settings_t                  *settings = config_get_ptr();
-
             fill_pathname_application_path(executable_path, sizeof(executable_path));
-            strlcpy(settings->path.libretro, executable_path, sizeof(settings->path.libretro));
+            path_set(RARCH_PATH_CORE, executable_path);
          }
          command_event(CMD_EVENT_QUIT, NULL);
          break;
@@ -1967,12 +2009,17 @@ static uint64_t frontend_linux_get_mem_used(void)
    return total - freemem - buffers - cached;
 }
 
+/*#include <valgrind/valgrind.h>*/
 static void frontend_linux_sighandler(int sig)
 {
+#ifdef VALGRIND_PRINTF_BACKTRACE
+VALGRIND_PRINTF_BACKTRACE("SIGINT");
+#endif
    (void)sig;
-   if (linux_sighandler_quit)
-      exit(1);
-   linux_sighandler_quit = 1;
+   linux_sighandler_quit++;
+   if (linux_sighandler_quit == 1) {}
+   if (linux_sighandler_quit == 2) exit(1);
+   if (linux_sighandler_quit >= 3) abort(); /* in case there's a second deadlock in a C++ destructor or something */
 }
 
 static void frontend_linux_install_signal_handlers(void)
@@ -2031,17 +2078,15 @@ frontend_ctx_driver_t frontend_ctx_linux = {
    NULL,                         /* load_content */
    frontend_linux_get_architecture,
    frontend_linux_get_powerstate,
-#ifdef ANDROID
-   frontend_android_parse_drive_list, /* parse_drive_list */
-#else
-   NULL,                         /* parse_drive_list */
-#endif
+   frontend_linux_parse_drive_list,
    frontend_linux_get_mem_total,
    frontend_linux_get_mem_used,
    frontend_linux_install_signal_handlers,
    frontend_linux_get_signal_handler_state,
    frontend_linux_set_signal_handler_state,
    frontend_linux_destroy_signal_handler_state,
+   NULL,                         /* attach_console */
+   NULL,                         /* detach_console */
 #ifdef ANDROID
    "android"
 #else
