@@ -15,7 +15,7 @@ void *initGui(void *arg)
 {
     char **arguments = (char**)arg;
     t = ctrWimp(i,arguments);
-    CreateMainWindow(t); //-->uncomment this to open the QT gui
+    CreateMainWindow(t); //-->uncomment this to open the Qt GUI
     return 0;
 }
 
@@ -37,8 +37,6 @@ int main(int argc, char *argv[])
         Sleep(1000);
         printf("test = %d\n",i);
         i++;
-        if(j < 2)
-            t->SetTitle("test");
     }
 
     pthread_join(gui,NULL);
