@@ -1,7 +1,8 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *  Copyright (C) 2012-2015 - Michael Lelli
+ *  Copyright (C) 2016-2017 - Brad Parker
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -23,21 +24,29 @@
 
 #include <string/stdstring.h>
 #include <lists/string_list.h>
+#include <compat/strl.h>
 #include <compat/posix_string.h>
 #include <encodings/utf.h>
 #include <file/file_path.h>
 #include <retro_inline.h>
 #include <string/stdstring.h>
+#include <encodings/utf.h>
+
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
 
 #include "menu_generic.h"
 
-#include "../../general.h"
-
 #include "../menu_driver.h"
 #include "../menu_animation.h"
 #include "../menu_display.h"
 #include "../menu_navigation.h"
 
+#include "../widgets/menu_input_dialog.h"
+
+#include "../../configuration.h"
+#include "../../runloop.h"
 #include "../../gfx/drivers_font_renderer/bitmap.h"
 
 #define RGUI_TERM_START_X(width)        (width / 21)
@@ -52,6 +61,8 @@ typedef struct
    unsigned last_width;
    unsigned last_height;
    float scroll_y;
+   bool mouse_show;
+   unsigned int frame_count;
 } rgui_t;
 
 static uint16_t *rgui_framebuf_data      = NULL;
@@ -164,39 +175,6 @@ static void rgui_color_rect(
             data[j * (pitch >> 1) + i] = color;
 }
 
-static uint8_t string_walkbyte(const char **string)
-{
-   return *((*string)++);
-}
-
-#ifdef HAVE_UTF8
-/* Does not validate the input, returns garbage if it's not UTF-8. */
-static uint32_t string_walk(const char **string)
-{
-   uint8_t first = string_walkbyte(string);
-   uint32_t ret;
-   
-   if (first<128)
-      return first;
-   
-   ret = 0;
-   ret = (ret<<6) | (string_walkbyte(string)    & 0x3F);
-   if (first >= 0xE0)
-      ret = (ret<<6) | (string_walkbyte(string) & 0x3F);
-   if (first >= 0xF0)
-      ret = (ret<<6) | (string_walkbyte(string) & 0x3F);
-   
-   if (first >= 0xF0)
-      return ret | (first&31)<<18;
-   if (first >= 0xE0)
-      return ret | (first&15)<<12;
-   return ret | (first&7)<<6;
-}
-
-#else
-#define string_walk string_walkbyte
-#endif
-
 static void blit_line(int x, int y,
       const char *message, uint16_t color)
 {
@@ -209,7 +187,7 @@ static void blit_line(int x, int y,
    while (!string_is_empty(message))
    {
       const uint8_t *font_fb = menu_display_get_font_framebuffer();
-      uint32_t symbol        = string_walk(&message);
+      uint32_t symbol        = utf8_walk(&message);
       
       for (j = 0; j < FONT_HEIGHT; j++)
       {
@@ -275,14 +253,14 @@ static bool rguidisp_init_font(menu_handle_t *menu)
 
 static void rgui_render_background(void)
 {
-   size_t pitch_in_pixels, size, fb_pitch;
+   size_t pitch_in_pixels, size;
+   size_t fb_pitch;
    unsigned fb_width, fb_height;
    uint16_t             *src  = NULL;
    uint16_t             *dst  = NULL;
 
-   fb_width = menu_display_get_width();
-   fb_height = menu_display_get_height();
-   fb_pitch = menu_display_get_framebuffer_pitch();
+   menu_display_get_fb_size(&fb_width, &fb_height,
+         &fb_pitch);
 
    pitch_in_pixels = fb_pitch >> 1;
    size            = fb_pitch * 4;
@@ -338,15 +316,14 @@ static void rgui_render_messagebox(const char *message)
    width        = 0;
    glyphs_width = 0;
 
-   fb_width  = menu_display_get_width();
-   fb_height = menu_display_get_height();
-   fb_pitch  = menu_display_get_framebuffer_pitch();
+   menu_display_get_fb_size(&fb_width, &fb_height,
+         &fb_pitch);
 
    for (i = 0; i < list->size; i++)
    {
       unsigned line_width;
       char     *msg   = list->elems[i].data;
-      unsigned msglen = strlen(msg);
+      unsigned msglen = utf8len(msg);
 
       if (msglen > RGUI_TERM_WIDTH(fb_width))
       {
@@ -381,7 +358,7 @@ static void rgui_render_messagebox(const char *message)
    for (i = 0; i < list->size; i++)
    {
       const char *msg = list->elems[i].data;
-      int offset_x    = FONT_WIDTH_STRIDE * (glyphs_width - strlen(msg)) / 2;
+      int offset_x    = FONT_WIDTH_STRIDE * (glyphs_width - utf8len(msg)) / 2;
       int offset_y    = FONT_HEIGHT_STRIDE * i;
 
       blit_line(x + 8 + offset_x, y + 8 + offset_y, msg, color);
@@ -393,42 +370,42 @@ end:
 
 static void rgui_blit_cursor(void)
 {
-   size_t   fb_pitch;
+   size_t fb_pitch;
    unsigned fb_width, fb_height;
-   int16_t        x = menu_input_mouse_state(MENU_MOUSE_X_AXIS);
-   int16_t        y = menu_input_mouse_state(MENU_MOUSE_Y_AXIS);
+   int16_t        x   = menu_input_mouse_state(MENU_MOUSE_X_AXIS);
+   int16_t        y   = menu_input_mouse_state(MENU_MOUSE_Y_AXIS);
 
-   fb_width  = menu_display_get_width();
-   fb_height = menu_display_get_height();
-   fb_pitch  = menu_display_get_framebuffer_pitch();
+   menu_display_get_fb_size(&fb_width, &fb_height,
+         &fb_pitch);
 
    rgui_color_rect(fb_pitch, fb_width, fb_height, x, y - 5, 1, 11, 0xFFFF);
    rgui_color_rect(fb_pitch, fb_width, fb_height, x - 5, y, 11, 1, 0xFFFF);
 }
 
+static void rgui_frame(void *data, video_frame_info_t *video_info)
+{
+   rgui_t *rgui                   = (rgui_t*)data;
+   rgui->frame_count              = video_info->frame_count;
+}
+
 static void rgui_render(void *data)
 {
    menu_animation_ctx_ticker_t ticker;
    unsigned x, y;
-   bool display_kb, msg_force;
    uint16_t hover_color, normal_color;
    size_t i, end, fb_pitch, old_start;
    unsigned fb_width, fb_height;
    int bottom;
-   uint64_t *frame_count          = NULL;
-   char title[256]                = {0};
-   char title_buf[256]            = {0};
-   char title_msg[64]             = {0};
-   char msg[PATH_MAX_LENGTH]      = {0};
+   char title[255];
+   char title_buf[255];
+   char title_msg[64];
+   char msg[255];
+   bool msg_force                 = false;
    settings_t *settings           = config_get_ptr();
    rgui_t *rgui                   = (rgui_t*)data;
+   uint64_t frame_count           = rgui ? rgui->frame_count : 0;
 
-   frame_count = video_driver_get_frame_count_ptr();
-
-   msg[0]       = '\0';
-   title[0]     = '\0';
-   title_buf[0] = '\0';
-   title_msg[0] = '\0';
+   msg[0] = title[0] = title_buf[0] = title_msg[0] = '\0';
 
    if (!rgui)
       return;
@@ -438,7 +415,7 @@ static void rgui_render(void *data)
       msg_force = menu_display_get_msg_force();
 
       if (menu_entries_ctl(MENU_ENTRIES_CTL_NEEDS_REFRESH, NULL)
-            && menu_driver_ctl(RARCH_MENU_CTL_IS_ALIVE, NULL) && !msg_force)
+            && menu_driver_is_alive() && !msg_force)
          return;
 
       if (runloop_ctl(RUNLOOP_CTL_IS_IDLE, NULL))
@@ -448,9 +425,8 @@ static void rgui_render(void *data)
          return;
    }
 
-   fb_width  = menu_display_get_width();
-   fb_height = menu_display_get_height();
-   fb_pitch  = menu_display_get_framebuffer_pitch();
+   menu_display_get_fb_size(&fb_width, &fb_height,
+         &fb_pitch);
 
    /* if the framebuffer changed size, recache the background */
    if (rgui->last_width != fb_width || rgui->last_height != fb_height)
@@ -531,27 +507,36 @@ static void rgui_render(void *data)
 
    ticker.s        = title_buf;
    ticker.len      = RGUI_TERM_WIDTH(fb_width) - 10;
-   ticker.idx      = *frame_count / RGUI_TERM_START_X(fb_width);
+   ticker.idx      = frame_count / RGUI_TERM_START_X(fb_width);
    ticker.str      = title;
    ticker.selected = true;
 
-   menu_animation_ctl(MENU_ANIMATION_CTL_TICKER, &ticker);
+   menu_animation_ticker(&ticker);
 
    hover_color  = HOVER_COLOR(settings);
    normal_color = NORMAL_COLOR(settings);
 
    if (menu_entries_ctl(MENU_ENTRIES_CTL_SHOW_BACK, NULL))
+   {
+      char back_buf[32];
+      char back_msg[32];
+
+      back_buf[0] = back_msg[0] = '\0';
+
+      strlcpy(back_buf, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_BASIC_MENU_CONTROLS_BACK), sizeof(back_buf));
+      strlcpy(back_msg, string_to_upper(back_buf), sizeof(back_msg));
       blit_line(
             RGUI_TERM_START_X(fb_width),
             RGUI_TERM_START_X(fb_width),
-            msg_hash_to_str(MENU_ENUM_LABEL_VALUE_BACK),
+            back_msg,
             TITLE_COLOR(settings));
+   }
 
    strlcpy(title_buf, string_to_upper(title_buf), sizeof(title_buf));
 
    blit_line(
          RGUI_TERM_START_X(fb_width) + (RGUI_TERM_WIDTH(fb_width)
-            - strlen(title_buf)) * FONT_WIDTH_STRIDE / 2,
+            - utf8len(title_buf)) * FONT_WIDTH_STRIDE / 2,
          RGUI_TERM_START_X(fb_width),
          title_buf, TITLE_COLOR(settings));
 
@@ -564,7 +549,9 @@ static void rgui_render(void *data)
    if (settings->menu.timedate_enable)
    {
       menu_display_ctx_datetime_t datetime;
-      char timedate[PATH_MAX_LENGTH] = {0};
+      char timedate[255];
+
+      timedate[0] = '\0';
 
       datetime.s         = timedate;
       datetime.len       = sizeof(timedate);
@@ -587,11 +574,11 @@ static void rgui_render(void *data)
    {
       menu_animation_ctx_ticker_t ticker;
       size_t selection;
-      char entry_path[PATH_MAX_LENGTH]      = {0};
-      char entry_value[PATH_MAX_LENGTH]     = {0};
-      char message[PATH_MAX_LENGTH]         = {0};
-      char entry_title_buf[PATH_MAX_LENGTH] = {0};
-      char type_str_buf[PATH_MAX_LENGTH]    = {0};
+      char entry_path[255];
+      char entry_value[255];
+      char message[255];
+      char entry_title_buf[255];
+      char type_str_buf[255];
       unsigned                entry_spacing = menu_entry_get_spacing(i);
       bool                entry_selected    = menu_entry_is_currently_selected(i);
       
@@ -607,22 +594,22 @@ static void rgui_render(void *data)
       entry_title_buf[0] = '\0';
       type_str_buf[0]    = '\0';
 
-      menu_entry_get_value(i, entry_value, sizeof(entry_value));
+      menu_entry_get_value(i, NULL, entry_value, sizeof(entry_value));
       menu_entry_get_rich_label(i, entry_path, sizeof(entry_path));
 
       ticker.s        = entry_title_buf;
       ticker.len      = RGUI_TERM_WIDTH(fb_width) - (entry_spacing + 1 + 2);
-      ticker.idx      = *frame_count / RGUI_TERM_START_X(fb_width);
+      ticker.idx      = frame_count / RGUI_TERM_START_X(fb_width);
       ticker.str      = entry_path;
       ticker.selected = entry_selected;
 
-      menu_animation_ctl(MENU_ANIMATION_CTL_TICKER, &ticker);
+      menu_animation_ticker(&ticker);
 
       ticker.s        = type_str_buf;
       ticker.len      = entry_spacing;
       ticker.str      = entry_value;
 
-      menu_animation_ctl(MENU_ANIMATION_CTL_TICKER, &ticker);
+      menu_animation_ticker(&ticker);
 
       snprintf(message, sizeof(message), "%c %-*.*s %-*s",
             entry_selected ? '>' : ' ',
@@ -636,17 +623,11 @@ static void rgui_render(void *data)
             entry_selected ? hover_color : normal_color);
    }
 
-   menu_input_ctl(MENU_INPUT_CTL_KEYBOARD_DISPLAY, &display_kb);
-
-   if (display_kb)
+   if (menu_input_dialog_get_display_kb())
    {
-      const char *str   = NULL;
-      const char *label = NULL;
-      menu_input_ctl(MENU_INPUT_CTL_KEYBOARD_BUFF_PTR, &str);
-      menu_input_ctl(MENU_INPUT_CTL_KEYBOARD_LABEL,    &label);
+      const char *str   = menu_input_dialog_get_buffer();
+      const char *label = menu_input_dialog_get_label_buffer();
 
-      if (!str)
-         str = "";
       snprintf(msg, sizeof(msg), "%s\n%s", label, str);
       rgui_render_messagebox(msg);
    }
@@ -658,11 +639,15 @@ static void rgui_render(void *data)
       rgui->force_redraw = true;
    }
 
-   if (settings->menu.mouse.enable 
-         && (settings->video.fullscreen 
-            || !video_driver_has_windowed())
-         )
-      rgui_blit_cursor();
+   if (rgui->mouse_show)
+   {
+      settings_t *settings = config_get_ptr();
+      bool cursor_visible  = settings->video.fullscreen ||
+         !video_driver_has_windowed();
+
+      if (settings->menu.mouse.enable && cursor_visible)
+         rgui_blit_cursor();
+   }
 }
 
 static void rgui_framebuffer_free(void)
@@ -748,13 +733,14 @@ static void rgui_free(void *data)
 
 static void rgui_set_texture(void)
 {
+   size_t fb_pitch;
    unsigned fb_width, fb_height;
 
    if (!menu_display_get_framebuffer_dirty_flag())
       return;
 
-   fb_width  = menu_display_get_width();
-   fb_height = menu_display_get_height();
+   menu_display_get_fb_size(&fb_width, &fb_height,
+         &fb_pitch);
 
    menu_display_unset_framebuffer_dirty_flag();
 
@@ -776,7 +762,7 @@ static void rgui_navigation_clear(void *data, bool pending_push)
 
 static void rgui_navigation_set(void *data, bool scroll)
 {
-   size_t selection, start;
+   size_t selection, start, fb_pitch;
    unsigned fb_width, fb_height;
    bool do_set_start              = false;
    size_t end                     = menu_entries_get_end();
@@ -785,8 +771,8 @@ static void rgui_navigation_set(void *data, bool scroll)
    if (!scroll)
       return;
 
-   fb_width  = menu_display_get_width();
-   fb_height = menu_display_get_height();
+   menu_display_get_fb_size(&fb_width, &fb_height,
+         &fb_pitch);
 
    if (selection < RGUI_TERM_HEIGHT(fb_width, fb_height) /2)
    {
@@ -834,15 +820,28 @@ static void rgui_populate_entries(void *data,
 static int rgui_environ(enum menu_environ_cb type,
       void *data, void *userdata)
 {
+   rgui_t           *rgui = (rgui_t*)userdata;
+
    switch (type)
    {
-      case 0:
+      case MENU_ENVIRON_ENABLE_MOUSE_CURSOR:
+         if (!rgui)
+            return -1;
+         rgui->mouse_show = true;
+         menu_display_set_framebuffer_dirty_flag();
          break;
+      case MENU_ENVIRON_DISABLE_MOUSE_CURSOR:
+         if (!rgui)
+            return -1;
+         rgui->mouse_show = false;
+         menu_display_unset_framebuffer_dirty_flag();
+         break;
+      case 0:
       default:
-         return -1;
+         break;
    }
 
-   return 0;
+   return -1;
 }
 
 static int rgui_pointer_tap(void *data,
@@ -880,7 +879,7 @@ menu_ctx_driver_t menu_ctx_rgui = {
    rgui_set_message,
    generic_menu_iterate,
    rgui_render,
-   NULL,
+   rgui_frame,
    rgui_init,
    rgui_free,
    NULL,
