@@ -1,6 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2015 - Higor Euripedes
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Higor Euripedes
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -20,15 +21,9 @@
 #include <retro_inline.h>
 #include <gfx/scaler/scaler.h>
 
-#include "SDL.h"
-#include "SDL_syswm.h"
-#include "../../driver.h"
-#include "../../general.h"
-#include "../../retroarch.h"
-#include "../../performance_counters.h"
-#include "../../verbosity.h"
-#include "../video_context_driver.h"
-#include "../font_driver.h"
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
 
 #ifdef HAVE_X11
 #include "../common/x11_common.h"
@@ -38,9 +33,16 @@
 #include "../../menu/menu_driver.h"
 #endif
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "SDL.h"
+#include "SDL_syswm.h"
+
+#include "../font_driver.h"
+
+#include "../../configuration.h"
+#include "../../retroarch.h"
+#include "../../performance_counters.h"
+#include "../../verbosity.h"
+#include "../video_context_driver.h"
 
 typedef struct sdl2_tex
 {
@@ -343,13 +345,9 @@ static void sdl_refresh_input_size(sdl2_video_t *vid, bool menu, bool rgb32,
        || target->rgb32 != rgb32 || target->pitch != pitch)
    {
       unsigned format;
-      static struct retro_perf_counter sdl_create_texture = {0};
 
       sdl_tex_zero(target);
 
-      performance_counter_init(&sdl_create_texture, "sdl_create_texture");
-      performance_counter_start(&sdl_create_texture);
-
       if (menu)
          format = rgb32 ? SDL_PIXELFORMAT_ARGB8888 : SDL_PIXELFORMAT_RGBA4444;
       else /* this assumes the frontend will convert 0RGB1555 to RGB565 */
@@ -362,8 +360,6 @@ static void sdl_refresh_input_size(sdl2_video_t *vid, bool menu, bool rgb32,
       target->tex = SDL_CreateTexture(vid->renderer, format,
                                       SDL_TEXTUREACCESS_STREAMING, width, height);
 
-      performance_counter_stop(&sdl_create_texture);
-
       if (!target->tex)
       {
          RARCH_ERR("Failed to create %s texture: %s\n", menu ? "menu" : "main",
@@ -387,8 +383,8 @@ static void *sdl2_gfx_init(const video_info_t *video,
 {
    int i;
    unsigned flags;
+   sdl2_video_t *vid    = NULL;
    settings_t *settings = config_get_ptr();
-   sdl2_video_t *vid;
 
 #ifdef HAVE_X11
    XInitThreads();
@@ -445,7 +441,7 @@ static void *sdl2_gfx_init(const video_info_t *video,
    }
 
    vid->video         = *video;
-   vid->video.smooth  = settings->video.smooth;
+   vid->video.smooth  = video->smooth;
    vid->should_resize = true;
 
    sdl_tex_zero(&vid->frame);
@@ -497,32 +493,32 @@ static void check_window(sdl2_video_t *vid)
 
 static bool sdl2_gfx_frame(void *data, const void *frame, unsigned width,
       unsigned height, uint64_t frame_count,
-      unsigned pitch, const char *msg)
+      unsigned pitch, const char *msg, video_frame_info_t *video_info)
 {
-   char buf[128]     = {0};
    sdl2_video_t *vid = (sdl2_video_t*)data;
+   char title[128];
 
    if (vid->should_resize)
       sdl_refresh_viewport(vid);
 
-   if (frame)
+   if (frame && video_info->libretro_running)
    {
       static struct retro_perf_counter sdl_copy_frame = {0};
 
       sdl_refresh_input_size(vid, false, vid->video.rgb32, width, height, pitch);
 
-      performance_counter_init(&sdl_copy_frame, "sdl_copy_frame");
-      performance_counter_start(&sdl_copy_frame);
+      performance_counter_init(sdl_copy_frame, "sdl_copy_frame");
+      performance_counter_start_plus(video_info->is_perfcnt_enable, sdl_copy_frame);
 
       SDL_UpdateTexture(vid->frame.tex, NULL, frame, pitch);
 
-      performance_counter_stop(&sdl_copy_frame);
+      performance_counter_stop_plus(video_info->is_perfcnt_enable, sdl_copy_frame);
    }
 
    SDL_RenderCopyEx(vid->renderer, vid->frame.tex, NULL, NULL, vid->rotation, NULL, SDL_FLIP_NONE);
 
 #ifdef HAVE_MENU
-   menu_driver_ctl(RARCH_MENU_CTL_FRAME, NULL);
+   menu_driver_frame(video_info);
 #endif
 
    if (vid->menu.active)
@@ -533,8 +529,12 @@ static bool sdl2_gfx_frame(void *data, const void *frame, unsigned width,
 
    SDL_RenderPresent(vid->renderer);
 
-   if (video_monitor_get_fps(buf, sizeof(buf), NULL, 0))
-      SDL_SetWindowTitle(vid->window, buf);
+   title[0] = '\0';
+
+   video_driver_get_window_title(title, sizeof(title));
+
+   if (title[0])
+      SDL_SetWindowTitle(vid->window, title);
 
    return true;
 }
@@ -620,16 +620,13 @@ static void sdl2_gfx_viewport_info(void *data, struct video_viewport *vp)
    *vp = vid->vp;
 }
 
-static bool sdl2_gfx_read_viewport(void *data, uint8_t *buffer)
+static bool sdl2_gfx_read_viewport(void *data, uint8_t *buffer, bool is_idle)
 {
    SDL_Surface *surf = NULL, *bgr24 = NULL;
    sdl2_video_t *vid = (sdl2_video_t*)data;
-   static struct retro_perf_counter sdl2_gfx_read_viewport = {0};
 
-   performance_counter_init(&sdl2_gfx_read_viewport, "sdl2_gfx_read_viewport");
-   performance_counter_start(&sdl2_gfx_read_viewport);
-
-   video_driver_cached_frame_render();
+   if (!is_idle)
+      video_driver_cached_frame();
 
    surf  = SDL_GetWindowSurface(vid->window);
    bgr24 = SDL_ConvertSurfaceFormat(surf, SDL_PIXELFORMAT_BGR24, 0);
@@ -642,8 +639,6 @@ static bool sdl2_gfx_read_viewport(void *data, uint8_t *buffer)
 
    memcpy(buffer, bgr24->pixels, bgr24->h * bgr24->pitch);
 
-   performance_counter_stop(&sdl2_gfx_read_viewport);
-
    return true;
 }
 
@@ -693,21 +688,14 @@ static void sdl2_poke_apply_state_changes(void *data)
 static void sdl2_poke_set_texture_frame(void *data, const void *frame, bool rgb32,
       unsigned width, unsigned height, float alpha)
 {
-   sdl2_video_t *vid = (sdl2_video_t*)data;
-
    if (frame)
    {
-      static struct retro_perf_counter copy_texture_frame = {0};
+      sdl2_video_t *vid = (sdl2_video_t*)data;
 
       sdl_refresh_input_size(vid, true, rgb32, width, height,
-                             width * (rgb32 ? 4 : 2));
-
-      performance_counter_init(&copy_texture_frame, "copy_texture_frame");
-      performance_counter_start(&copy_texture_frame);
+            width * (rgb32 ? 4 : 2));
 
       SDL_UpdateTexture(vid->menu.tex, NULL, frame, vid->menu.pitch);
-
-      performance_counter_stop(&copy_texture_frame);
    }
 }
 
@@ -719,7 +707,7 @@ static void sdl2_poke_texture_enable(void *data, bool enable, bool full_screen)
 }
 
 static void sdl2_poke_set_osd_msg(void *data, const char *msg,
-      const struct font_params *params, void *font)
+      const void *params, void *font)
 {
    sdl2_video_t *vid = (sdl2_video_t*)data;
    sdl2_render_msg(vid, msg);
