@@ -1,5 +1,6 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2015 - Manuel Alfayate
+ *  Copyright (C) 2015-2017 - Manuel Alfayate
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -24,10 +25,18 @@
 #include <rthreads/rthreads.h>
 #include <string/stdstring.h>
 
-#include "../../general.h"
-#include "../../retroarch.h"
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
+#ifdef HAVE_MENU
+#include "../../menu/menu_driver.h"
+#endif
+
 #include "../font_driver.h"
 
+#include "../../retroarch.h"
+
 #define NUMPAGES 2
 
 /* Lowlevel SunxiG2D functions block */
@@ -758,7 +767,8 @@ static void sunxi_setup_scale (void *data,
 }
 
 static bool sunxi_gfx_frame(void *data, const void *frame, unsigned width,
-      unsigned height, uint64_t frame_count, unsigned pitch, const char *msg)
+      unsigned height, uint64_t frame_count, unsigned pitch, const char *msg,
+      video_frame_info_t *video_info)
 {
    struct sunxi_video *_dispvars = (struct sunxi_video*)data;
 
@@ -774,11 +784,12 @@ static bool sunxi_gfx_frame(void *data, const void *frame, unsigned width,
       sunxi_setup_scale(_dispvars, width, height, pitch);
    }
 
+#ifdef HAVE_MENU
+   menu_driver_frame(video_info);
+#endif
+
    if (_dispvars->menu_active)
    {
-      char buf[128] = {0};
-
-      video_monitor_get_fps(buf, sizeof(buf), NULL, 0);
       ioctl(_dispvars->sunxi_disp->fd_fb, FBIO_WAITFORVSYNC, 0);
       return true;
    }
@@ -916,9 +927,11 @@ static void sunxi_set_texture_frame(void *data, const void *frame, bool rgb32,
 static void sunxi_set_aspect_ratio (void *data, unsigned aspect_ratio_idx) 
 {
    struct sunxi_video *_dispvars = (struct sunxi_video*)data;
-   /* Here we obtain the new aspect ratio. */
-   float new_aspect = aspectratio_lut[aspect_ratio_idx].value;
-   if (new_aspect != _dispvars->aspect_ratio) {
+   float              new_aspect = aspectratio_lut[aspect_ratio_idx].value;
+
+   if (new_aspect != _dispvars->aspect_ratio)
+   {
+      /* Here we set the new aspect ratio. */
       _dispvars->aspect_ratio = new_aspect;
       sunxi_setup_scale(_dispvars, _dispvars->src_width, _dispvars->src_height, _dispvars->src_pitch);
    }
