@@ -1,5 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2015 - Hans-Kristian Arntzen
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -24,7 +25,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include <sys/poll.h>
 #include <sys/epoll.h>
 
 #include <libudev.h>
@@ -32,22 +32,27 @@
 #include <linux/input.h>
 #include <linux/kd.h>
 
-#include <file/file_path.h>
-#include <string/stdstring.h>
-
-#include "../drivers_keyboard/keyboard_event_udev.h"
-#include "../common/linux_common.h"
-
-#include "../input_config.h"
-#include "../input_joypad_driver.h"
-#include "../input_keymaps.h"
-#include "../../general.h"
-#include "../../verbosity.h"
-
 #ifdef HAVE_CONFIG_H
 #include "../../config.h"
 #endif
 
+#include <file/file_path.h>
+#include <compat/strl.h>
+#include <string/stdstring.h>
+
+#include "../input_config.h"
+#include "../input_driver.h"
+#include "../input_joypad_driver.h"
+#include "../input_keymaps.h"
+
+#include "../drivers_keyboard/keyboard_event_udev.h"
+#include "../../gfx/video_driver.h"
+#include "../common/linux_common.h"
+#include "../common/udev_common.h"
+#include "../common/epoll_common.h"
+
+#include "../../verbosity.h"
+
 typedef struct udev_input udev_input_t;
 
 typedef void (*device_handle_cb)(void *data,
@@ -96,7 +101,7 @@ struct udev_input
 };
 
 #ifdef HAVE_XKBCOMMON
-int  init_xkb(void);
+int init_xkb(int fd, size_t size);
 #endif
 
 static void udev_handle_touchpad(void *data,
@@ -224,27 +229,13 @@ static void udev_handle_mouse(void *data,
    }
 }
 
-static bool udev_input_hotplug_available(udev_input_t *udev)
-{
-   struct pollfd fds = {0};
-
-   if (!udev || !udev->monitor)
-      return false;
-
-   fds.fd     = udev_monitor_get_fd(udev->monitor);
-   fds.events = POLLIN;
-
-   return (poll(&fds, 1, 0) == 1) && (fds.revents & POLLIN);
-}
-
-static bool add_device(udev_input_t *udev,
+static bool udev_input_add_device(udev_input_t *udev,
       const char *devnode, device_handle_cb cb)
 {
    int fd;
    udev_input_device_t **tmp;
    udev_input_device_t *device = NULL;
    struct stat st              = {0};
-   struct epoll_event event    = {0};
 
    if (stat(devnode, &st) < 0)
       return false;
@@ -278,13 +269,7 @@ static bool add_device(udev_input_t *udev,
    tmp[udev->num_devices++] = device;
    udev->devices            = tmp;
 
-   event.events             = EPOLLIN;
-   event.data.ptr           = device;
-
-   /* Shouldn't happen, but just check it. */
-   if (epoll_ctl(udev->epfd, EPOLL_CTL_ADD, fd, &event) < 0)
-      RARCH_ERR("Failed to add FD (%d) to epoll list (%s).\n",
-            fd, strerror(errno));
+   epoll_add(&udev->epfd, fd, device);
 
    return true;
 
@@ -347,15 +332,15 @@ static void udev_input_handle_hotplug(udev_input_t *udev)
    if (string_is_equal(action, "add"))
    {
       device_handle_cb cb      = NULL;
-      if (!strcmp(devtype, "keyboard"))
+      if (string_is_equal(devtype, "keyboard"))
          cb = udev_handle_keyboard;
-      else if (!strcmp(devtype, "touchpad"))
+      else if (string_is_equal(devtype, "touchpad"))
          cb = udev_handle_touchpad;
-      else if (!strcmp(devtype, "mouse"))
+      else if (string_is_equal(devtype, "mouse"))
          cb = udev_handle_mouse;
 
       RARCH_LOG("[udev]: Hotplug add %s: %s.\n", devtype, devnode);
-      add_device(udev, devnode, cb);
+      udev_input_add_device(udev, devnode, cb);
    }
    else if (string_is_equal(action, "remove"))
    {
@@ -373,14 +358,17 @@ static void udev_input_poll(void *data)
    struct epoll_event events[32];
    udev_input_t *udev = (udev_input_t*)data;
 
+   if (!udev)
+      return;
+
    udev->mouse_x   = udev->mouse_y   = 0;
    udev->mouse_wu  = udev->mouse_wd  = 0;
    udev->mouse_whu = udev->mouse_whd = 0;
 
-   while (udev_input_hotplug_available(udev))
+   while (udev->monitor && udev_hotplug_available(udev->monitor))
       udev_input_handle_hotplug(udev);
 
-   ret = epoll_wait(udev->epfd, events, ARRAY_SIZE(events), 0);
+   ret = epoll_waiting(&udev->epfd, events, ARRAY_SIZE(events), 0);
 
    for (i = 0; i < ret; i++)
    {
@@ -455,15 +443,16 @@ static int16_t udev_lightgun_state(udev_input_t *udev, unsigned id)
 
 static int16_t udev_analog_pressed(const struct retro_keybind *binds, unsigned idx, unsigned id)
 {
-   unsigned id_minus = 0;
-   unsigned id_plus  = 0;
-   int16_t pressed_minus = 0, pressed_plus = 0;
+   unsigned id_minus     = 0;
+   unsigned id_plus      = 0;
+   int16_t pressed_minus = 0;
+   int16_t pressed_plus  = 0;
 
    input_conv_analog_id_to_bind_id(idx, id, &id_minus, &id_plus);
 
-   if (udev_input_is_pressed(binds, id_minus))
+   if (binds && binds[id_minus].valid && udev_input_is_pressed(binds, id_minus))
       pressed_minus = -0x7fff;
-   if (udev_input_is_pressed(binds, id_plus))
+   if (binds && binds[id_plus].valid && udev_input_is_pressed(binds, id_plus))
       pressed_plus = 0x7fff;
 
    return pressed_plus + pressed_minus;
@@ -472,16 +461,12 @@ static int16_t udev_analog_pressed(const struct retro_keybind *binds, unsigned i
 static int16_t udev_pointer_state(udev_input_t *udev,
       unsigned idx, unsigned id, bool screen)
 {
-   bool valid, inside;
+   bool inside              = false;
+   struct video_viewport vp = {0};
    int16_t res_x = 0, res_y = 0, res_screen_x = 0, res_screen_y = 0;
 
-   if (idx != 0)
-      return 0;
-
-   valid = input_translate_coord_viewport(udev->mouse_x, udev->mouse_y,
-         &res_x, &res_y, &res_screen_x, &res_screen_y);
-
-   if (!valid)
+   if (!(video_driver_translate_coord_viewport_wrap(&vp, udev->mouse_x, udev->mouse_y,
+         &res_x, &res_y, &res_screen_x, &res_screen_y)))
       return 0;
 
    if (screen)
@@ -508,22 +493,26 @@ static int16_t udev_pointer_state(udev_input_t *udev,
    return 0;
 }
 
-static int16_t udev_input_state(void *data, const struct retro_keybind **binds,
+static int16_t udev_input_state(void *data,
+      rarch_joypad_info_t joypad_info,
+      const struct retro_keybind **binds,
       unsigned port, unsigned device, unsigned idx, unsigned id)
 {
    int16_t ret;
-   udev_input_t *udev = (udev_input_t*)data;
+   udev_input_t *udev         = (udev_input_t*)data;
+
+   if (!udev)
+      return 0;
 
    switch (device)
    {
       case RETRO_DEVICE_JOYPAD:
          return udev_input_is_pressed(binds[port], id) ||
-            input_joypad_pressed(udev->joypad, port, binds[port], id);
-
+            input_joypad_pressed(udev->joypad, joypad_info, port, binds[port], id);
       case RETRO_DEVICE_ANALOG:
          ret = udev_analog_pressed(binds[port], idx, id);
-         if (!ret)
-            ret = input_joypad_analog(udev->joypad, port, idx, id, binds[port]);
+         if (!ret && binds[port])
+            ret = input_joypad_analog(udev->joypad, joypad_info, port, idx, id, binds[port]);
          return ret;
 
       case RETRO_DEVICE_KEYBOARD:
@@ -533,9 +522,10 @@ static int16_t udev_input_state(void *data, const struct retro_keybind **binds,
 
       case RETRO_DEVICE_POINTER:
       case RARCH_DEVICE_POINTER_SCREEN:
-         return udev_pointer_state(udev, idx, id,
-               device == RARCH_DEVICE_POINTER_SCREEN);
-
+         if (idx == 0)
+            return udev_pointer_state(udev, idx, id,
+                  device == RARCH_DEVICE_POINTER_SCREEN);
+         break;
       case RETRO_DEVICE_LIGHTGUN:
          return udev_lightgun_state(udev, id);
    }
@@ -543,19 +533,6 @@ static int16_t udev_input_state(void *data, const struct retro_keybind **binds,
    return 0;
 }
 
-static bool udev_input_key_pressed(void *data, int key)
-{
-   udev_input_t *udev    = (udev_input_t*)data;
-   settings_t *settings  = config_get_ptr();
-
-   if (udev_input_is_pressed(settings->input.binds[0], key))
-      return true;
-   if (input_joypad_pressed(udev->joypad, 0, settings->input.binds[0], key))
-      return true;
-
-   return false;
-}
-
 static bool udev_input_meta_key_pressed(void *data, int key)
 {
    return false;
@@ -572,8 +549,7 @@ static void udev_input_free(void *data)
    if (udev->joypad)
       udev->joypad->destroy();
 
-   if (udev->epfd >= 0)
-      close(udev->epfd);
+   epoll_free(&udev->epfd);
 
    for (i = 0; i < udev->num_devices; i++)
    {
@@ -619,7 +595,7 @@ static bool open_devices(udev_input_t *udev, const char *type, device_handle_cb
          int fd = open(devnode, O_RDONLY | O_NONBLOCK);
 
          RARCH_LOG("[udev] Adding device %s as type %s.\n", devnode, type);
-         if (!add_device(udev, devnode, cb))
+         if (!udev_input_add_device(udev, devnode, cb))
             RARCH_ERR("[udev] Failed to open device: %s (%s).\n", devnode, strerror(errno));
          close(fd);
       }
@@ -631,10 +607,9 @@ static bool open_devices(udev_input_t *udev, const char *type, device_handle_cb
    return true;
 }
 
-static void *udev_input_init(void)
+static void *udev_input_init(const char *joypad_driver)
 {
-   settings_t *settings = config_get_ptr();
-   udev_input_t *udev = (udev_input_t*)calloc(1, sizeof(*udev));
+   udev_input_t *udev   = (udev_input_t*)calloc(1, sizeof(*udev));
 
    if (!udev)
       return NULL;
@@ -654,12 +629,11 @@ static void *udev_input_init(void)
    }
 
 #ifdef HAVE_XKBCOMMON
-   if (init_xkb() == -1)
+   if (init_xkb(-1, 0) == -1)
       goto error;
 #endif
 
-   udev->epfd = epoll_create(32);
-   if (udev->epfd < 0)
+   if (!epoll_new(&udev->epfd))
    {
       RARCH_ERR("Failed to create epoll FD.\n");
       goto error;
@@ -688,7 +662,7 @@ static void *udev_input_init(void)
    if (!udev->num_devices)
       RARCH_WARN("[udev]: Couldn't open any keyboard, mouse or touchpad. Are permissions set correctly for /dev/input/event*?\n");
 
-   udev->joypad = input_joypad_init_driver(settings->input.joypad_driver, udev);
+   udev->joypad = input_joypad_init_driver(joypad_driver, udev);
    input_keymaps_init_keyboard_lut(rarch_key_map_linux);
 
    linux_terminal_disable_input();
@@ -755,7 +729,6 @@ input_driver_t input_udev = {
    udev_input_init,
    udev_input_poll,
    udev_input_state,
-   udev_input_key_pressed,
    udev_input_meta_key_pressed,
    udev_input_free,
    NULL,
