@@ -1,6 +1,6 @@
 /* RetroArch - A frontend for libretro.
- *  Copyright (C) 2015-2016 - Ali Bouhlel
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2015-2017 - Ali Bouhlel
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  * RetroArch is free software: you can redistribute it and/or modify it under the terms
  * of the GNU General Public License as published by the Free Software Found-
@@ -24,33 +24,37 @@
 #pragma comment( lib, "comctl32" )
 #endif
 
+#define TITLE_MAX PATH_MAX
+#define FULLPATH_MAX 32768
+
 #define IDI_ICON 1
 
 #ifndef _WIN32_WINNT
 #define _WIN32_WINNT 0x0500 //_WIN32_WINNT_WIN2K
 #endif
 
-#ifndef _WIN32_IE
-#define _WIN32_IE 0x0300
-#endif
-
+#include "../../gfx/common/win32_common.h"
 #include <windows.h>
 #include <commdlg.h>
 #include <commctrl.h>
 
 #include <retro_inline.h>
+#include <retro_miscellaneous.h>
 #include <file/file_path.h>
+#include <string/stdstring.h>
+#include <compat/strl.h>
 
 #include "../ui_companion_driver.h"
 #include "../../msg_hash.h"
+#include "../../configuration.h"
 #include "../../driver.h"
+#include "../../paths.h"
 #include "../../runloop.h"
 #include "../../gfx/video_context_driver.h"
 #include "../../gfx/video_shader_driver.h"
 #include "../../tasks/tasks_internal.h"
 
 #include "../../gfx/common/gl_common.h"
-#include "../../gfx/common/win32_common.h"
 #include "ui_win32.h"
 
 #define SHADER_DLG_WIDTH                  220
@@ -112,32 +116,28 @@ typedef struct
 
 static shader_dlg_t g_shader_dlg = {{0}};
 
-static void shader_dlg_refresh_trackbar_label(int index)
+static bool shader_dlg_refresh_trackbar_label(int index, 
+      video_shader_ctx_t *shader_info)
 {
-   video_shader_ctx_t shader_info;
    char val_buffer[32]         = {0};
 
-   video_shader_driver_get_current_shader(&shader_info);
-
-   if (floorf(shader_info.data->parameters[index].current) 
-         == shader_info.data->parameters[index].current)
+   if (floorf(shader_info->data->parameters[index].current) 
+         == shader_info->data->parameters[index].current)
       snprintf(val_buffer, sizeof(val_buffer), "%.0f",
-            shader_info.data->parameters[index].current);
+            shader_info->data->parameters[index].current);
    else
       snprintf(val_buffer, sizeof(val_buffer), "%.2f",
-            shader_info.data->parameters[index].current);
+            shader_info->data->parameters[index].current);
 
    SendMessage(g_shader_dlg.controls[index].trackbar.label_val,
          WM_SETTEXT, 0, (LPARAM)val_buffer);
 
+   return true;
 }
 
 static void shader_dlg_params_refresh(void)
 {
    int i;
-   video_shader_ctx_t shader_info;
-
-   video_shader_driver_get_current_shader(&shader_info);
 
    for (i = 0; i < GFX_MAX_PARAMETERS; i++)
    {
@@ -150,26 +150,37 @@ static void shader_dlg_params_refresh(void)
       {
          case SHADER_PARAM_CTRL_CHECKBOX:
             {
-               bool checked = 
+               video_shader_ctx_t shader_info;
+               video_shader_driver_get_current_shader(&shader_info);
+
+               bool checked = shader_info.data ?
                   (shader_info.data->parameters[i].current == 
-                   shader_info.data->parameters[i].maximum);
+                   shader_info.data->parameters[i].maximum) : false;
                SendMessage(control->checkbox.hwnd, BM_SETCHECK, checked, 0);
             }
             break;
          case SHADER_PARAM_CTRL_TRACKBAR:
-            shader_dlg_refresh_trackbar_label(i);
+            {
+               video_shader_ctx_t shader_info;
+               video_shader_driver_get_current_shader(&shader_info);
+               if (shader_info.data && !shader_dlg_refresh_trackbar_label(i, &shader_info))
+                  break;
 
-            SendMessage(control->trackbar.hwnd,
-                  TBM_SETRANGEMIN, (WPARAM)TRUE, (LPARAM)0);
-            SendMessage(control->trackbar.hwnd,
-                  TBM_SETRANGEMAX, (WPARAM)TRUE,
-                  (LPARAM)((shader_info.data->parameters[i].maximum - 
-                        shader_info.data->parameters[i].minimum) 
-                     / shader_info.data->parameters[i].step));
-            SendMessage(control->trackbar.hwnd, TBM_SETPOS, (WPARAM)TRUE,
-                  (LPARAM)((shader_info.data->parameters[i].current - 
-                        shader_info.data->parameters[i].minimum) / 
-                     shader_info.data->parameters[i].step));
+               if (shader_info.data)
+               {
+                  SendMessage(control->trackbar.hwnd,
+                        TBM_SETRANGEMIN, (WPARAM)TRUE, (LPARAM)0);
+                  SendMessage(control->trackbar.hwnd,
+                        TBM_SETRANGEMAX, (WPARAM)TRUE,
+                        (LPARAM)((shader_info.data->parameters[i].maximum - 
+                              shader_info.data->parameters[i].minimum) 
+                           / shader_info.data->parameters[i].step));
+                  SendMessage(control->trackbar.hwnd, TBM_SETPOS, (WPARAM)TRUE,
+                        (LPARAM)((shader_info.data->parameters[i].current - 
+                              shader_info.data->parameters[i].minimum) / 
+                           shader_info.data->parameters[i].step));
+               }
+            }
             break;
          case SHADER_PARAM_CTRL_NONE:
          default:
@@ -184,10 +195,9 @@ static void shader_dlg_params_clear(void)
 
    for (i = 0; i < GFX_MAX_PARAMETERS; i++)
    {
-      const ui_window_t *window = ui_companion_driver_get_window_ptr();
       shader_param_ctrl_t*control = &g_shader_dlg.controls[i];
 
-      if (control->type == SHADER_PARAM_CTRL_NONE)
+      if (!control || control->type == SHADER_PARAM_CTRL_NONE)
          break;
 
       switch (control->type)
@@ -195,8 +205,11 @@ static void shader_dlg_params_clear(void)
          case SHADER_PARAM_CTRL_NONE:
             break;
          case SHADER_PARAM_CTRL_CHECKBOX:
-            if (window)
-               window->destroy(&control->checkbox);
+            {
+               const ui_window_t *window = ui_companion_driver_get_window_ptr();
+               if (window)
+                  window->destroy(&control->checkbox);
+            }
             break;
          case SHADER_PARAM_CTRL_TRACKBAR:
             DestroyWindow(control->trackbar.label_title);
@@ -211,23 +224,24 @@ static void shader_dlg_params_clear(void)
 
 void shader_dlg_params_reload(void)
 {
+#ifdef HAVE_SHADERPIPELINE
    HFONT hFont;
    RECT parent_rect;
    int i, pos_x, pos_y;
    video_shader_ctx_t shader_info;
-   const ui_window_t *window = ui_companion_driver_get_window_ptr();
-   video_shader_driver_get_current_shader(&shader_info);
-
+   const ui_window_t *window = NULL;
+   
    shader_dlg_params_clear();
 
-   if (!shader_info.data)
-      return;
-   if (shader_info.data->num_parameters > GFX_MAX_PARAMETERS)
+   video_shader_driver_get_current_shader(&shader_info);
+
+   if (!shader_info.data || shader_info.data->num_parameters > GFX_MAX_PARAMETERS)
       return;
 
-   hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
-   pos_y = g_shader_dlg.parameters_start_y;
-   pos_x = SHADER_DLG_CTRL_X;
+   window = ui_companion_driver_get_window_ptr();
+   hFont  = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
+   pos_y  = g_shader_dlg.parameters_start_y;
+   pos_x  = SHADER_DLG_CTRL_X;
 
    for (i = 0; i < (int)shader_info.data->num_parameters; i++)
    {
@@ -312,7 +326,7 @@ void shader_dlg_params_reload(void)
          (pos_x - SHADER_DLG_CTRL_X) + SHADER_DLG_WIDTH,
          (pos_x == SHADER_DLG_CTRL_X) ? pos_y + 30 : SHADER_DLG_MAX_HEIGHT,
          SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
-
+#endif
 }
 
 static void shader_dlg_update_on_top_state(void)
@@ -351,15 +365,13 @@ void shader_dlg_show(HWND parent_hwnd)
    window->set_focused(&g_shader_dlg.window);
 }
 
+#if defined(HAVE_OPENGL) || defined(HAVE_VULKAN)
 static LRESULT CALLBACK ShaderDlgWndProc(HWND hwnd, UINT message,
       WPARAM wparam, LPARAM lparam)
 {
    int i, pos;
-   video_shader_ctx_t shader_info;
    const ui_window_t *window = ui_companion_driver_get_window_ptr();
 
-   video_shader_driver_get_current_shader(&shader_info);
-
    switch (message)
    {
       case WM_CREATE:
@@ -387,36 +399,50 @@ static LRESULT CALLBACK ShaderDlgWndProc(HWND hwnd, UINT message,
          if (g_shader_dlg.controls[i].type != SHADER_PARAM_CTRL_CHECKBOX)
             break;
 
-         if (SendMessage(g_shader_dlg.controls[i].checkbox.hwnd,
-                  BM_GETCHECK, 0, 0) == BST_CHECKED)
-            shader_info.data->parameters[i].current = 
-               shader_info.data->parameters[i].maximum;
-         else
-            shader_info.data->parameters[i].current = 
-               shader_info.data->parameters[i].minimum;
+         {
+            video_shader_ctx_t shader_info;
+            video_shader_driver_get_current_shader(&shader_info);
 
+            if (SendMessage(g_shader_dlg.controls[i].checkbox.hwnd,
+                     BM_GETCHECK, 0, 0) == BST_CHECKED)
+               shader_info.data->parameters[i].current = 
+                  shader_info.data->parameters[i].maximum;
+            else
+               shader_info.data->parameters[i].current = 
+                  shader_info.data->parameters[i].minimum;
+         }
          break;
 
       case WM_HSCROLL:
-         i = GetWindowLong((HWND)lparam, GWL_ID);
+         {
+            video_shader_ctx_t shader_info;
+            video_shader_driver_get_current_shader(&shader_info);
+            i = GetWindowLong((HWND)lparam, GWL_ID);
 
-         if (i >= GFX_MAX_PARAMETERS)
-            break;
+            if (i >= GFX_MAX_PARAMETERS)
+               break;
 
-         if (g_shader_dlg.controls[i].type != SHADER_PARAM_CTRL_TRACKBAR)
-            break;
+            if (g_shader_dlg.controls[i].type != SHADER_PARAM_CTRL_TRACKBAR)
+               break;
 
-         pos = (int)SendMessage(g_shader_dlg.controls[i].trackbar.hwnd, TBM_GETPOS, 0, 0);
-         shader_info.data->parameters[i].current = 
-            shader_info.data->parameters[i].minimum + pos * shader_info.data->parameters[i].step;
+            pos = (int)SendMessage(g_shader_dlg.controls[i].trackbar.hwnd, TBM_GETPOS, 0, 0);
 
-         shader_dlg_refresh_trackbar_label(i);
+            {
+
+               shader_info.data->parameters[i].current = 
+                  shader_info.data->parameters[i].minimum + pos * shader_info.data->parameters[i].step;
+            }
+
+            if (shader_info.data)
+               shader_dlg_refresh_trackbar_label(i, &shader_info);
+         }
          break;
 
    }
 
    return DefWindowProc(hwnd, message, wparam, lparam);
 }
+#endif
 
 bool win32_window_init(WNDCLASSEX *wndclass,
       bool fullscreen, const char *class_name)
@@ -444,13 +470,15 @@ bool win32_window_init(WNDCLASSEX *wndclass,
    if (class_name != NULL) 
       return true;
 
+   /* Shader dialog is disabled for now, until video_threaded issues are fixed.
    if (!win32_shader_dlg_init())
-      RARCH_ERR("[WGL]: wgl_shader_dlg_init() failed.\n");
+      RARCH_ERR("[WGL]: wgl_shader_dlg_init() failed.\n");*/
    return true;
 }
 
 bool win32_shader_dlg_init(void)
 {
+#if defined(HAVE_OPENGL) || defined(HAVE_VULKAN)
    static bool inited = false;
    int pos_y;
    HFONT hFont;
@@ -498,7 +526,7 @@ bool win32_shader_dlg_init(void)
    pos_y +=  SHADER_DLG_SEPARATOR_HEIGHT + SHADER_DLG_CTRL_MARGIN;
 
    g_shader_dlg.parameters_start_y = pos_y;
-
+#endif
    return true;
 }
 
@@ -516,19 +544,32 @@ static bool win32_browser(
    if (browser)
    {
       ui_browser_window_state_t browser_state;
+      /* OPENFILENAME.lpstrFilter requires a null separated list of name/ext pairs terminated by a second null at the end. */
+      char *all_files[] = {"All Files (*.*)", "*.*", ""};
 
-      browser_state.filters  = strdup(extensions);
-      browser_state.title    = strdup(title);
-      browser_state.startdir = strdup(initial_dir);
-      browser_state.path     = strdup(filename);
+      /* These need to be big enough to hold the path/name of any file the user may select.
+       * FIXME: We should really handle the error case when this isn't big enough. */
+      char new_title[TITLE_MAX];
+      char new_file[FULLPATH_MAX];
+
+      new_title[0] = '\0';
+      new_file[0] = '\0';
+
+      if (!string_is_empty(title))
+         strlcpy(new_title, title, sizeof(new_title));
+
+      if (filename && *filename)
+         strlcpy(new_file, filename, sizeof(new_file));
+
+      browser_state.filters  = all_files[0];
+      browser_state.title    = new_title;
+      browser_state.startdir = strdup("");
+      browser_state.path     = new_file;
       browser_state.window   = owner;
 
       result = browser->open(&browser_state);
 
-      free(browser_state.filters);
-      free(browser_state.title);
       free(browser_state.startdir);
-      free(browser_state.path);
    }
 
    return result;
@@ -541,15 +582,18 @@ LRESULT win32_menu_loop(HWND owner, WPARAM wparam)
    bool do_wm_close     = false;
    settings_t *settings = config_get_ptr();
 
-	switch (mode)
+   switch (mode)
    {
       case ID_M_LOAD_CORE:
       case ID_M_LOAD_CONTENT:
          {
             char win32_file[PATH_MAX_LENGTH] = {0};
+            wchar_t title_wide[PATH_MAX];
+            char title_cp[PATH_MAX];
             const char *extensions  = NULL;
             const char *title       = NULL;
             const char *initial_dir = NULL;
+            size_t converted        = 0;
 
             switch (mode)
             {
@@ -559,15 +603,21 @@ LRESULT win32_menu_loop(HWND owner, WPARAM wparam)
                   initial_dir = settings->directory.libretro;
                   break;
                case ID_M_LOAD_CONTENT:
-                  extensions  = "All Files\0*.*\0";
+                  extensions  = "All Files (*.*)\0*.*\0";
                   title       = msg_hash_to_str(
                         MENU_ENUM_LABEL_VALUE_LOAD_CONTENT_LIST);
                   initial_dir = settings->directory.menu_content;
                   break;
             }
 
+            /* Convert UTF8 to UTF16, then back to the local code page.
+             * This is needed for proper multi-byte string display until Unicode is fully supported.
+             */
+            MultiByteToWideChar(CP_UTF8, 0, title, -1, title_wide, sizeof(title_wide) / sizeof(title_wide[0]));
+            wcstombs(title_cp, title_wide, sizeof(title_cp) - 1);
+
             if (!win32_browser(owner, win32_file,
-                     extensions, title, initial_dir))
+                     extensions, title_cp, initial_dir))
                break;
 
             switch (mode)
@@ -579,7 +629,8 @@ LRESULT win32_menu_loop(HWND owner, WPARAM wparam)
                case ID_M_LOAD_CONTENT:
                   {
                      content_ctx_info_t content_info = {0};
-                     runloop_ctl(RUNLOOP_CTL_SET_CONTENT_PATH, win32_file);
+
+                     path_set(RARCH_PATH_CONTENT, win32_file);
 
                      do_wm_close = true;
                      task_push_content_load_default(
@@ -658,13 +709,13 @@ LRESULT win32_menu_loop(HWND owner, WPARAM wparam)
          break;
    }
 
-	if (cmd != CMD_EVENT_NONE)
-		command_event(cmd, NULL);
+   if (cmd != CMD_EVENT_NONE)
+      command_event(cmd, NULL);
 
-	if (do_wm_close)
-		PostMessage(owner, WM_CLOSE, 0, 0);
-	
-	return 0L;
+   if (do_wm_close)
+      PostMessage(owner, WM_CLOSE, 0, 0);
+   
+   return 0L;
 }
 
 static void ui_companion_win32_deinit(void *data)
