@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2015-2016 - Sergi Granell (xerpi)
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2015-2017 - Sergi Granell (xerpi)
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -18,72 +18,39 @@
 
 #include <retro_inline.h>
 #include <string/stdstring.h>
+#include <formats/image.h>
+
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
+#ifdef HAVE_MENU
+#include "../../menu/menu_driver.h"
+#endif
+
+#include "../font_driver.h"
+
+#include "../common/vita2d_common.h"
+#include "../../driver.h"
+#include "../video_coord_array.h"
+#include "../video_context_driver.h"
+#include "../../verbosity.h"
+#include "../../configuration.h"
 
 #include "../../defines/psp_defines.h"
-#include "../../general.h"
-#include "../../driver.h"
 
-typedef struct vita_menu_frame
-{
-   bool active;
-   int width;
-   int height;
-   vita2d_texture *texture;
-} vita_menu_t;
+extern void *memcpy_neon(void *dst, const void *src, size_t n);
 
-#ifdef HAVE_OVERLAY
-struct vita_overlay_data
-{
-   vita2d_texture *tex;
-   float x;
-   float y; 
-   float w; 
-   float h;
-   float tex_x;
-   float tex_y; 
-   float tex_w; 
-   float tex_h;
-   float alpha_mod;
-   float width;
-   float height;
-};
-#endif
-
-typedef struct vita_video
-{
-   vita2d_texture *texture;
-   SceGxmTextureFormat format;
-   int width;
-   int height;
-   SceGxmTextureFilter tex_filter;
-
-   bool fullscreen;
-   bool vsync;
-   bool rgb32;
-
-   video_viewport_t vp;
-
-   unsigned rotation;
-   bool vblank_not_reached;
-   bool keep_aspect;
-   bool should_resize;
-
-   vita_menu_t menu;
-   
-#ifdef HAVE_OVERLAY
-   struct vita_overlay_data *overlay;
-   unsigned overlays;
-   bool overlay_enable;
-   bool overlay_full_screen;
-#endif
-   
-} vita_video_t;
 
+static void vita2d_gfx_set_viewport(void *data, unsigned viewport_width,
+      unsigned viewport_height, bool force_full, bool allow_rotate);
+      
 static void *vita2d_gfx_init(const video_info_t *video,
       const input_driver_t **input, void **input_data)
 {
    vita_video_t *vita   = (vita_video_t *)calloc(1, sizeof(vita_video_t));
-   settings_t *settings = config_get_ptr();
+   unsigned temp_width                = PSP_FB_WIDTH;
+   unsigned temp_height               = PSP_FB_HEIGHT;
 
    if (!vita)
       return NULL;
@@ -93,13 +60,13 @@ static void *vita2d_gfx_init(const video_info_t *video,
 
    RARCH_LOG("vita2d_gfx_init: w: %i  h: %i\n", video->width, video->height);
    RARCH_LOG("RARCH_SCALE_BASE: %i input_scale: %i = %i\n",
-	RARCH_SCALE_BASE, video->input_scale, RARCH_SCALE_BASE * video->input_scale);
+	 RARCH_SCALE_BASE, video->input_scale, RARCH_SCALE_BASE * video->input_scale);
 
    vita2d_init();
    vita2d_set_clear_color(RGBA8(0x00, 0x00, 0x00, 0xFF));
    vita2d_set_vblank_wait(video->vsync);
 
-   if (vita->rgb32)
+   if (video->rgb32)
    {
       RARCH_LOG("Format: SCE_GXM_TEXTURE_FORMAT_X8U8U8U8_1RGB\n");
       vita->format = SCE_GXM_TEXTURE_FORMAT_X8U8U8U8_1RGB;
@@ -124,11 +91,16 @@ static void *vita2d_gfx_init(const video_info_t *video,
    vita->tex_filter   = video->smooth 
       ? SCE_GXM_TEXTURE_FILTER_LINEAR : SCE_GXM_TEXTURE_FILTER_POINT;
 
+   video_driver_set_size(&temp_width, &temp_height);
+   vita2d_gfx_set_viewport(vita, temp_width, temp_height, false, true);
+
+
    if (input && input_data)
    {
-      void *pspinput = input_psp.init();
-      *input      = pspinput ? &input_psp : NULL;
-      *input_data = pspinput;
+      settings_t *settings = config_get_ptr();
+      void *pspinput       = input_psp.init(settings->input.joypad_driver);
+      *input               = pspinput ? &input_psp : NULL;
+      *input_data          = pspinput;
    }
 
    vita->keep_aspect        = true;
@@ -136,13 +108,8 @@ static void *vita2d_gfx_init(const video_info_t *video,
 #ifdef HAVE_OVERLAY
    vita->overlay_enable     = false;
 #endif
-   if (!font_driver_init_first(NULL, NULL, vita, *settings->path.font 
-          ? settings->path.font : NULL, settings->video.font_size, false,
-          FONT_DRIVER_RENDER_VITA2D))
-   {
-      RARCH_ERR("Font: Failed to initialize font renderer.\n");
-        return false;
-   }
+   font_driver_init_osd(vita, false, FONT_DRIVER_RENDER_VITA2D);
+
    return vita;
 }
 
@@ -151,6 +118,9 @@ static void vita2d_render_overlay(void *data);
 static void vita2d_free_overlay(vita_video_t *vita)
 {
    unsigned i;
+
+   vita2d_wait_rendering_done();
+
    for (i = 0; i < vita->overlays; i++)
       vita2d_free_texture(vita->overlay[i].tex);
    free(vita->overlay);
@@ -163,60 +133,55 @@ static void vita2d_gfx_update_viewport(vita_video_t* vita);
 
 static bool vita2d_gfx_frame(void *data, const void *frame,
       unsigned width, unsigned height, uint64_t frame_count,
-      unsigned pitch, const char *msg)
+      unsigned pitch, const char *msg, video_frame_info_t *video_info)
 {
    void *tex_p;
    vita_video_t *vita = (vita_video_t *)data;
 
-   (void)frame;
-   (void)width;
-   (void)height;
-   (void)pitch;
-   (void)msg;
-
    if (frame)
    {
-      unsigned i, j;
-      unsigned int stride;
-
-      if ((width != vita->width || height != vita->height) && vita->texture)
+      if(!(vita->texture&&vita2d_texture_get_datap(vita->texture)==frame))
       {
-         vita2d_free_texture(vita->texture);
-         vita->texture = NULL;
-      }
+         unsigned i;
+         unsigned int stride;
 
-      if (!vita->texture)
-      {
-         RARCH_LOG("Creating texture: %ix%i\n", width, height);
-         vita->width = width;
-         vita->height = height;
-         vita->texture = vita2d_create_empty_texture_format(width, height, vita->format);
-         vita2d_texture_set_filters(vita->texture,vita->tex_filter,vita->tex_filter);
-      }
-      tex_p = vita2d_texture_get_datap(vita->texture);
-      stride = vita2d_texture_get_stride(vita->texture);
+         if ((width != vita->width || height != vita->height) && vita->texture)
+         {
+            vita2d_free_texture(vita->texture);
+            vita->texture = NULL;
+         }
 
-      if (vita->format == SCE_GXM_TEXTURE_FORMAT_X8U8U8U8_1RGB)
-      {
-         stride                     /= 4;
-         pitch                      /= 4;
-         uint32_t             *tex32 = tex_p;
-         const uint32_t     *frame32 = frame;
+         if (!vita->texture)
+         {
+            RARCH_LOG("Creating texture: %ix%i\n", width, height);
+            vita->width = width;
+            vita->height = height;
+            vita->texture = vita2d_create_empty_texture_format(width, height, vita->format);
+            vita2d_texture_set_filters(vita->texture,vita->tex_filter,vita->tex_filter);
+         }
+         tex_p = vita2d_texture_get_datap(vita->texture);
+         stride = vita2d_texture_get_stride(vita->texture);
 
-         for (i = 0; i < height; i++)
-            for (j = 0; j < width; j++)
-               tex32[j + i*stride] = frame32[j + i*pitch];
-      }
-      else
-      {
-         stride                 /= 2;
-         pitch                  /= 2;
-         uint16_t *tex16         = tex_p;
-         const uint16_t *frame16 = frame;
+         if (vita->format == SCE_GXM_TEXTURE_FORMAT_X8U8U8U8_1RGB)
+         {
+            stride                     /= 4;
+            pitch                      /= 4;
+            uint32_t             *tex32 = tex_p;
+            const uint32_t     *frame32 = frame;
 
-         for (i = 0; i < height; i++)
-            for (j = 0; j < width; j++)
-               tex16[j + i*stride] = frame16[j + i*pitch];
+            for (i = 0; i < height; i++)
+               memcpy_neon(&tex32[i*stride],&frame32[i*pitch],pitch*sizeof(uint32_t));
+         }
+         else
+         {
+            stride                 /= 2;
+            pitch                  /= 2;
+            uint16_t *tex16         = tex_p;
+            const uint16_t *frame16 = frame;
+
+            for (i = 0; i < height; i++)
+               memcpy_neon(&tex16[i*stride],&frame16[i*pitch],width*sizeof(uint16_t));
+         }
       }
    }
 
@@ -224,7 +189,8 @@ static bool vita2d_gfx_frame(void *data, const void *frame,
       vita2d_gfx_update_viewport(vita);
 
    vita2d_start_drawing();
-   vita2d_clear_screen();
+
+   vita2d_draw_rectangle(0,0,PSP_FB_WIDTH,PSP_FB_HEIGHT,vita2d_get_clear_color());
 
    if (vita->texture)
    {
@@ -239,7 +205,7 @@ static bool vita2d_gfx_frame(void *data, const void *frame,
          const float rad = vita->rotation * radian;
          float scalex = vita->vp.width / (float)vita->width;
          float scaley = vita->vp.height / (float)vita->height;
-         vita2d_draw_texture_scale_rotate(vita->texture, vita->vp.x,
+         vita2d_draw_texture_scale_rotate(vita->texture,vita->vp.x,
                vita->vp.y, scalex, scaley, rad);
       }
    }
@@ -249,37 +215,44 @@ static bool vita2d_gfx_frame(void *data, const void *frame,
       vita2d_render_overlay(vita);
 #endif
 
-   if (vita->menu.active && vita->menu.texture)
+   if (vita->menu.active)
    {
-      if (vita->fullscreen)
-         vita2d_draw_texture_scale(vita->menu.texture,
-               0, 0,
-               PSP_FB_WIDTH  / (float)vita->menu.width,
-               PSP_FB_HEIGHT / (float)vita->menu.height);
-      else
+#ifdef HAVE_MENU
+      menu_driver_frame(video_info);
+#endif
+
+      if(vita->menu.texture)
       {
-         if (vita->menu.width > vita->menu.height)
-         {
-            float scale = PSP_FB_HEIGHT / (float)vita->menu.height;
-            float w = vita->menu.width * scale;
+         if (vita->fullscreen)
             vita2d_draw_texture_scale(vita->menu.texture,
-                  PSP_FB_WIDTH / 2.0f - w/2.0f, 0.0f,
-                  scale, scale);
-         }
+                  0, 0,
+                  PSP_FB_WIDTH  / (float)vita->menu.width,
+                  PSP_FB_HEIGHT / (float)vita->menu.height);
          else
          {
-            float scale = PSP_FB_WIDTH / (float)vita->menu.width;
-            float h = vita->menu.height * scale;
-            vita2d_draw_texture_scale(vita->menu.texture,
-                  0.0f, PSP_FB_HEIGHT / 2.0f - h/2.0f,
-                  scale, scale);
+            if (vita->menu.width > vita->menu.height)
+            {
+               float scale = PSP_FB_HEIGHT / (float)vita->menu.height;
+               float w = vita->menu.width * scale;
+               vita2d_draw_texture_scale(vita->menu.texture,
+                     PSP_FB_WIDTH / 2.0f - w/2.0f, 0.0f,
+                     scale, scale);
+            }
+            else
+            {
+               float scale = PSP_FB_WIDTH / (float)vita->menu.width;
+               float h = vita->menu.height * scale;
+               vita2d_draw_texture_scale(vita->menu.texture,
+                     0.0f, PSP_FB_HEIGHT / 2.0f - h/2.0f,
+                     scale, scale);
+            }
          }
       }
    }
-   
+
    if(!string_is_empty(msg))
-     font_driver_render_msg(NULL, msg, NULL);
-   
+      font_driver_render_msg(video_info, NULL, msg, NULL);
+
    vita2d_end_drawing();
    vita2d_swap_buffers();
 
@@ -340,9 +313,7 @@ static void vita2d_gfx_free(void *data)
       vita->texture = NULL;
    }
 
-   font_driver_free(NULL);
-
-   RARCH_LOG("vita2d_gfx_free() done\n");
+   font_driver_free_osd();
 }
 
 static bool vita2d_gfx_set_shader(void *data,
@@ -355,6 +326,25 @@ static bool vita2d_gfx_set_shader(void *data,
    return false;
 }
 
+static void vita2d_set_projection(vita_video_t *vita,
+      struct video_ortho *ortho, bool allow_rotate)
+{
+   math_matrix_4x4 rot;
+
+   /* Calculate projection. */
+   matrix_4x4_ortho(&vita->mvp_no_rot, ortho->left, ortho->right,
+         ortho->bottom, ortho->top, ortho->znear, ortho->zfar);
+
+   if (!allow_rotate)
+   {
+      vita->mvp = vita->mvp_no_rot;
+      return;
+   }
+
+   matrix_4x4_rotate_z(&rot, M_PI * vita->rotation / 180.0f);
+   matrix_4x4_multiply(&vita->mvp, &rot, &vita->mvp_no_rot);
+}
+
 static void vita2d_gfx_update_viewport(vita_video_t* vita)
 {
    int x                = 0;
@@ -374,12 +364,13 @@ static void vita2d_gfx_update_viewport(vita_video_t* vita)
    else if (vita->keep_aspect)
    {
       float desired_aspect = video_driver_get_aspect_ratio();
-      if (vita->rotation == ORIENTATION_VERTICAL ||
-            vita->rotation == ORIENTATION_FLIPPED_ROTATED){
-              device_aspect = 1.0 / device_aspect;
-              width = PSP_FB_HEIGHT;
-              height = PSP_FB_WIDTH;
-            }
+      if ( (vita->rotation == ORIENTATION_VERTICAL) ||
+           (vita->rotation == ORIENTATION_FLIPPED_ROTATED))
+      {
+         device_aspect = 1.0 / device_aspect;
+         width = PSP_FB_HEIGHT;
+         height = PSP_FB_WIDTH;
+      }
 #if defined(HAVE_MENU)
       if (settings->video.aspect_ratio_idx == ASPECT_RATIO_CUSTOM)
       {
@@ -420,11 +411,13 @@ static void vita2d_gfx_update_viewport(vita_video_t* vita)
             height = (unsigned)roundf(2.0f * height * delta);
          }
 
-         if (vita->rotation == ORIENTATION_VERTICAL ||
-               vita->rotation == ORIENTATION_FLIPPED_ROTATED){
-                 x = (PSP_FB_WIDTH - width) * 0.5f;
-                 y = (PSP_FB_HEIGHT - height) * 0.5f;
-               }
+         if ( (vita->rotation == ORIENTATION_VERTICAL) ||
+              (vita->rotation == ORIENTATION_FLIPPED_ROTATED)
+            )
+         {
+            x = (PSP_FB_WIDTH - width) * 0.5f;
+            y = (PSP_FB_HEIGHT - height) * 0.5f;
+         }
       }
 
       vita->vp.x      = x;
@@ -446,6 +439,111 @@ static void vita2d_gfx_update_viewport(vita_video_t* vita)
 
 }
 
+static void vita2d_gfx_set_viewport(void *data, unsigned viewport_width,
+      unsigned viewport_height, bool force_full, bool allow_rotate)
+{
+   gfx_ctx_aspect_t aspect_data;
+   unsigned width, height;
+   int x                    = 0;
+   int y                    = 0;
+   float device_aspect      = (float)viewport_width / viewport_height;
+   struct video_ortho ortho = {0, 1, 1, 0, -1, 1};
+   settings_t *settings     = config_get_ptr();
+   vita_video_t *vita       = (vita_video_t*)data;
+
+   video_driver_get_size(&width, &height);
+
+   aspect_data.aspect     = &device_aspect;
+   aspect_data.width      = viewport_width;
+   aspect_data.height     = viewport_height;
+
+   video_context_driver_translate_aspect(&aspect_data);
+
+   if (settings->video.scale_integer && !force_full)
+   {
+      video_viewport_get_scaled_integer(&vita->vp,
+            viewport_width, viewport_height,
+            video_driver_get_aspect_ratio(), vita->keep_aspect);
+      viewport_width  = vita->vp.width;
+      viewport_height = vita->vp.height;
+   }
+   else if (vita->keep_aspect && !force_full)
+   {
+      float desired_aspect = video_driver_get_aspect_ratio();
+
+#if defined(HAVE_MENU)
+      if (settings->video.aspect_ratio_idx == ASPECT_RATIO_CUSTOM)
+      {
+         const struct video_viewport *custom = video_viewport_get_custom();
+
+         /* Vukan has top-left origin viewport. */
+         x               = custom->x;
+         y               = custom->y;
+         viewport_width  = custom->width;
+         viewport_height = custom->height;
+      }
+      else
+#endif
+      {
+         float delta;
+
+         if (fabsf(device_aspect - desired_aspect) < 0.0001f)
+         {
+            /* If the aspect ratios of screen and desired aspect 
+             * ratio are sufficiently equal (floating point stuff), 
+             * assume they are actually equal.
+             */
+         }
+         else if (device_aspect > desired_aspect)
+         {
+            delta          = (desired_aspect / device_aspect - 1.0f) 
+               / 2.0f + 0.5f;
+            x              = (int)roundf(viewport_width * (0.5f - delta));
+            viewport_width = (unsigned)roundf(2.0f * viewport_width * delta);
+         }
+         else
+         {
+            delta           = (device_aspect / desired_aspect - 1.0f) 
+               / 2.0f + 0.5f;
+            y               = (int)roundf(viewport_height * (0.5f - delta));
+            viewport_height = (unsigned)roundf(2.0f * viewport_height * delta);
+         }
+      }
+
+      vita->vp.x      = x;
+      vita->vp.y      = y;
+      vita->vp.width  = viewport_width;
+      vita->vp.height = viewport_height;
+   }
+   else
+   {
+      vita->vp.x      = 0;
+      vita->vp.y      = 0;
+      vita->vp.width  = viewport_width;
+      vita->vp.height = viewport_height;
+   }
+
+   vita2d_set_projection(vita, &ortho, allow_rotate);
+
+   /* Set last backbuffer viewport. */
+   if (!force_full)
+   {
+      vita->vp.width  = viewport_width;
+      vita->vp.height = viewport_height;
+   }
+
+#if 0
+   vita->vp.x          = (float)vita->vp.x;
+   vita->vp.y          = (float)vita->vp.y;
+   vita->vp.width      = (float)vita->vp.width;
+   vita->vp.height     = (float)vita->vp.height;
+   vita->vp.minDepth   = 0.0f;
+   vita->vp.maxDepth   = 1.0f;
+
+   RARCH_LOG("Setting viewport @ %ux%u\n", viewport_width, viewport_height);
+#endif
+}
+
 static void vita2d_gfx_set_rotation(void *data,
       unsigned rotation)
 {
@@ -467,7 +565,7 @@ static void vita2d_gfx_viewport_info(void *data,
        *vp = vita->vp;
 }
 
-static bool vita2d_gfx_read_viewport(void *data, uint8_t *buffer)
+static bool vita2d_gfx_read_viewport(void *data, uint8_t *buffer, bool is_idle)
 {
    (void)data;
    (void)buffer;
@@ -481,8 +579,10 @@ static void vita_set_filtering(void *data, unsigned index, bool smooth)
 
    if (vita)
    {
-      vita->tex_filter = smooth? SCE_GXM_TEXTURE_FILTER_LINEAR : SCE_GXM_TEXTURE_FILTER_POINT;
-      vita2d_texture_set_filters(vita->texture,vita->tex_filter,vita->tex_filter);
+      vita->tex_filter = smooth ? 
+         SCE_GXM_TEXTURE_FILTER_LINEAR : SCE_GXM_TEXTURE_FILTER_POINT;
+      vita2d_texture_set_filters(vita->texture,vita->tex_filter,
+            vita->tex_filter);
    }
 }
 
@@ -508,8 +608,11 @@ static void vita_set_aspect_ratio(void *data, unsigned aspect_ratio_idx)
          break;
    }
 
-   video_driver_set_aspect_ratio_value(aspectratio_lut[aspect_ratio_idx].value);
+   video_driver_set_aspect_ratio_value(
+         aspectratio_lut[aspect_ratio_idx].value);
 
+   if (!vita)
+      return;
    vita->keep_aspect = true;
    vita->should_resize = true;
 }
@@ -543,18 +646,23 @@ static void vita_set_texture_frame(void *data, const void *frame, bool rgb32,
       if (rgb32)
       {
          vita->menu.texture = vita2d_create_empty_texture(width, height);
-         RARCH_LOG("Creating Frame RGBA8 texture: w: %i  h: %i\n", width, height);
+         RARCH_LOG("Creating Frame RGBA8 texture: w: %i  h: %i\n",
+               width, height);
       }
       else
       {
-         vita->menu.texture = vita2d_create_empty_texture_format(width, height, SCE_GXM_TEXTURE_FORMAT_U4U4U4U4_RGBA);
-         RARCH_LOG("Creating Frame R5G6B5 texture: w: %i  h: %i\n", width, height);
+         vita->menu.texture = vita2d_create_empty_texture_format(
+               width, height, SCE_GXM_TEXTURE_FORMAT_U4U4U4U4_RGBA);
+         RARCH_LOG("Creating Frame R5G6B5 texture: w: %i  h: %i\n",
+               width, height);
       }
       vita->menu.width  = width;
       vita->menu.height = height;
    }
 
-   vita2d_texture_set_filters(vita->menu.texture,SCE_GXM_TEXTURE_FILTER_LINEAR,SCE_GXM_TEXTURE_FILTER_LINEAR);
+   vita2d_texture_set_filters(vita->menu.texture,
+         SCE_GXM_TEXTURE_FILTER_LINEAR,
+         SCE_GXM_TEXTURE_FILTER_LINEAR);
 
    tex_p  = vita2d_texture_get_datap(vita->menu.texture);
    stride = vita2d_texture_get_stride(vita->menu.texture);
@@ -591,9 +699,98 @@ static void vita_set_texture_enable(void *data, bool state, bool full_screen)
    vita->menu.active = state;
 }
 
+static uintptr_t vita_load_texture(void *video_data, void *data,
+      bool threaded, enum texture_filter_type filter_type)
+{
+   unsigned int stride, pitch, j;
+   uint32_t             *tex32    = NULL;
+   const uint32_t *frame32        = NULL;
+   struct texture_image *image    = (struct texture_image*)data;
+   struct vita2d_texture *texture = vita2d_create_empty_texture_format(image->width, 
+     image->height,SCE_GXM_TEXTURE_FORMAT_U8U8U8U8_ARGB);
+
+   if (!texture)
+      return 0;
+
+   if ((filter_type == TEXTURE_FILTER_MIPMAP_LINEAR) || 
+      (filter_type == TEXTURE_FILTER_LINEAR))
+      vita2d_texture_set_filters(texture,
+            SCE_GXM_TEXTURE_FILTER_LINEAR,
+            SCE_GXM_TEXTURE_FILTER_LINEAR);
+
+   stride                      = vita2d_texture_get_stride(texture);
+   stride                     /= 4;
+
+   tex32                       = vita2d_texture_get_datap(texture);
+   frame32                     = image->pixels;
+   pitch                       = image->width;
+
+   for (j = 0; j < image->height; j++)
+         memcpy_neon(
+               &tex32[j*stride],
+               &frame32[j*pitch],
+               pitch * sizeof(uint32_t));
+
+   return (uintptr_t)texture;
+}
+
+static void vita_unload_texture(void *data, uintptr_t handle)
+{
+   struct vita2d_texture *texture = (struct vita2d_texture*)handle;
+   if (!texture)
+      return;
+
+   /* TODO: We really want to defer this deletion instead,
+    * but this will do for now. */
+   vita2d_wait_rendering_done();
+   vita2d_free_texture(texture);
+
+   //free(texture);
+}
+
+static void vita_set_osd_msg(void *data, const char *msg,
+      const void *params, void *font)
+{
+   video_frame_info_t video_info;
+   video_driver_build_info(&video_info);
+   font_driver_render_msg(&video_info, font, msg, params);
+}
+
+static bool vita_get_current_sw_framebuffer(void *data,
+      struct retro_framebuffer *framebuffer)
+{
+   vita_video_t *vita = (vita_video_t*)data;
+
+   if (!vita->texture || vita->width != framebuffer->width ||
+         vita->height != framebuffer->height)
+   {
+      if(vita->texture)
+      {
+         vita2d_wait_rendering_done();
+         vita2d_free_texture(vita->texture);
+         vita->texture = NULL;
+      }
+
+      vita->width = framebuffer->width;
+      vita->height = framebuffer->height;
+      vita->texture = vita2d_create_empty_texture_format(
+            vita->width, vita->height, vita->format);
+      vita2d_texture_set_filters(vita->texture,
+            vita->tex_filter,vita->tex_filter);
+   }
+
+   framebuffer->data         = vita2d_texture_get_datap(vita->texture);
+   framebuffer->pitch        = vita2d_texture_get_stride(vita->texture);
+   framebuffer->format       = vita->rgb32 
+      ? RETRO_PIXEL_FORMAT_XRGB8888 : RETRO_PIXEL_FORMAT_RGB565;
+   framebuffer->memory_flags = 0;
+
+   return true;
+}
+
 static const video_poke_interface_t vita_poke_interface = {
-   NULL,
-   NULL,
+   vita_load_texture,
+   vita_unload_texture,
    NULL,
    vita_set_filtering,
    NULL, /* get_video_output_size */
@@ -606,10 +803,18 @@ static const video_poke_interface_t vita_poke_interface = {
 #ifdef HAVE_MENU
    vita_set_texture_frame,
    vita_set_texture_enable,
-#endif
-   NULL,
-   NULL,
-   NULL
+   #else
+      NULL,
+      NULL,
+   #endif
+   #ifdef HAVE_MENU
+      vita_set_osd_msg,
+   #endif
+      NULL,
+      NULL,
+      NULL,
+      vita_get_current_sw_framebuffer,
+      NULL,
  };
 
 static void vita2d_gfx_get_poke_interface(void *data,
@@ -767,7 +972,7 @@ video_driver_t video_vita2d = {
    vita2d_gfx_set_shader,
    vita2d_gfx_free,
    "vita2d",
-   NULL, /* set_viewport */
+   vita2d_gfx_set_viewport,
    vita2d_gfx_set_rotation,
    vita2d_gfx_viewport_info,
    vita2d_gfx_read_viewport,
