@@ -1,5 +1,6 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2016 - Hans-Kristian Arntzen
+ *  Copyright (C) 2016-2017 - Hans-Kristian Arntzen
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -18,7 +19,6 @@
 #include <math.h>
 #include <string.h>
 
-
 #include <compat/strl.h>
 #include <gfx/scaler/scaler.h>
 #include <formats/image.h>
@@ -27,17 +27,8 @@
 #include <retro_assert.h>
 #include <libretro.h>
 
-#include "../common/vulkan_common.h"
-
-#include "../../driver.h"
-#include "../../record/record_driver.h"
-#include "../../performance_counters.h"
-
-#include "../../general.h"
-#include "../../retroarch.h"
-
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "../../config.h"
 #endif
 
 #ifdef HAVE_MENU
@@ -45,6 +36,17 @@
 #endif
 
 #include "../font_driver.h"
+
+#include "../common/vulkan_common.h"
+
+#include "../../driver.h"
+#include "../../configuration.h"
+#include "../../record/record_driver.h"
+#include "../../performance_counters.h"
+
+#include "../../retroarch.h"
+#include "../../verbosity.h"
+
 #include "../video_context_driver.h"
 #include "../video_coord_array.h"
 
@@ -53,7 +55,7 @@ static void vulkan_set_viewport(void *data, unsigned viewport_width,
 
 #ifdef HAVE_OVERLAY
 static void vulkan_overlay_free(vk_t *vk);
-static void vulkan_render_overlay(vk_t *vk);
+static void vulkan_render_overlay(vk_t *vk, video_frame_info_t *video_info);
 #endif
 static void vulkan_viewport_info(void *data, struct video_viewport *vp);
 
@@ -207,20 +209,20 @@ static void vulkan_init_pipelines(
 #include "vulkan_shaders/font.frag.inc"
       ;
 
-   static const uint32_t ribbon_vert[] =
-#include "vulkan_shaders/ribbon.vert.inc"
+   static const uint32_t pipeline_ribbon_vert[] =
+#include "vulkan_shaders/pipeline_ribbon.vert.inc"
       ;
 
-   static const uint32_t ribbon_frag[] =
-#include "vulkan_shaders/ribbon.frag.inc"
+   static const uint32_t pipeline_ribbon_frag[] =
+#include "vulkan_shaders/pipeline_ribbon.frag.inc"
       ;
 
-   static const uint32_t ribbon_simple_vert[] =
-#include "vulkan_shaders/ribbon_simple.vert.inc"
+   static const uint32_t pipeline_ribbon_simple_vert[] =
+#include "vulkan_shaders/pipeline_ribbon_simple.vert.inc"
       ;
 
-   static const uint32_t ribbon_simple_frag[] =
-#include "vulkan_shaders/ribbon_simple.frag.inc"
+   static const uint32_t pipeline_ribbon_simple_frag[] =
+#include "vulkan_shaders/pipeline_ribbon_simple.frag.inc"
       ;
 
    unsigned i;
@@ -392,13 +394,13 @@ static void vulkan_init_pipelines(
    {
       if (i & 2)
       {
-         module_info.codeSize   = sizeof(ribbon_simple_vert);
-         module_info.pCode      = ribbon_simple_vert;
+         module_info.codeSize   = sizeof(pipeline_ribbon_simple_vert);
+         module_info.pCode      = pipeline_ribbon_simple_vert;
       }
       else
       {
-         module_info.codeSize   = sizeof(ribbon_vert);
-         module_info.pCode      = ribbon_vert;
+         module_info.codeSize   = sizeof(pipeline_ribbon_vert);
+         module_info.pCode      = pipeline_ribbon_vert;
       }
 
       shader_stages[0].stage = VK_SHADER_STAGE_VERTEX_BIT;
@@ -408,13 +410,13 @@ static void vulkan_init_pipelines(
 
       if (i & 2)
       {
-         module_info.codeSize   = sizeof(ribbon_simple_frag);
-         module_info.pCode      = ribbon_simple_frag;
+         module_info.codeSize   = sizeof(pipeline_ribbon_simple_frag);
+         module_info.pCode      = pipeline_ribbon_simple_frag;
       }
       else
       {
-         module_info.codeSize   = sizeof(ribbon_frag);
-         module_info.pCode      = ribbon_frag;
+         module_info.codeSize   = sizeof(pipeline_ribbon_frag);
+         module_info.pCode      = pipeline_ribbon_frag;
       }
 
       shader_stages[1].stage = VK_SHADER_STAGE_FRAGMENT_BIT;
@@ -483,16 +485,31 @@ static void vulkan_init_samplers(vk_t *vk)
    vkCreateSampler(vk->context->device,
          &info, NULL, &vk->samplers.nearest);
 
-   info.magFilter               = VK_FILTER_LINEAR;
-   info.minFilter               = VK_FILTER_LINEAR;
+   info.magFilter = VK_FILTER_LINEAR;
+   info.minFilter = VK_FILTER_LINEAR;
    vkCreateSampler(vk->context->device,
          &info, NULL, &vk->samplers.linear);
+
+   info.maxLod     = VK_LOD_CLAMP_NONE;
+   info.magFilter  = VK_FILTER_NEAREST;
+   info.minFilter  = VK_FILTER_NEAREST;
+   info.mipmapMode = VK_SAMPLER_MIPMAP_MODE_NEAREST;
+   vkCreateSampler(vk->context->device,
+         &info, NULL, &vk->samplers.mipmap_nearest);
+
+   info.magFilter  = VK_FILTER_LINEAR;
+   info.minFilter  = VK_FILTER_LINEAR;
+   info.mipmapMode = VK_SAMPLER_MIPMAP_MODE_LINEAR;
+   vkCreateSampler(vk->context->device,
+         &info, NULL, &vk->samplers.mipmap_linear);
 }
 
 static void vulkan_deinit_samplers(vk_t *vk)
 {
    vkDestroySampler(vk->context->device, vk->samplers.nearest, NULL);
    vkDestroySampler(vk->context->device, vk->samplers.linear, NULL);
+   vkDestroySampler(vk->context->device, vk->samplers.mipmap_nearest, NULL);
+   vkDestroySampler(vk->context->device, vk->samplers.mipmap_linear, NULL);
 }
 
 static void vulkan_init_buffers(vk_t *vk)
@@ -651,6 +668,8 @@ static bool vulkan_init_default_filter_chain(vk_t *vk)
    info.gpu                   = vk->context->gpu;
    info.memory_properties     = &vk->context->memory_properties;
    info.pipeline_cache        = vk->pipelines.cache;
+   info.queue                 = vk->context->queue;
+   info.command_pool          = vk->swapchain[vk->context->current_swapchain_index].cmd_pool;
    info.max_input_size.width  = vk->tex_w;
    info.max_input_size.height = vk->tex_h;
    info.swapchain.viewport    = vk->vk_vp;
@@ -683,6 +702,8 @@ static bool vulkan_init_filter_chain_preset(vk_t *vk, const char *shader_path)
    info.gpu                   = vk->context->gpu;
    info.memory_properties     = &vk->context->memory_properties;
    info.pipeline_cache        = vk->pipelines.cache;
+   info.queue                 = vk->context->queue;
+   info.command_pool          = vk->swapchain[vk->context->current_swapchain_index].cmd_pool;
    info.max_input_size.width  = vk->tex_w;
    info.max_input_size.height = vk->tex_h;
    info.swapchain.viewport    = vk->vk_vp;
@@ -826,13 +847,14 @@ static void vulkan_free(void *data)
 
       /* No need to init this since textures are create on-demand. */
       vulkan_deinit_menu(vk);
-      font_driver_free(NULL);
+
+      font_driver_free_osd();
 
       vulkan_deinit_static_resources(vk);
       vulkan_overlay_free(vk);
 
       if (vk->filter_chain)
-         vulkan_filter_chain_free(vk->filter_chain);
+         vulkan_filter_chain_free((vulkan_filter_chain_t*)vk->filter_chain);
 
       video_context_driver_free();
    }
@@ -1008,7 +1030,6 @@ static void *vulkan_init(const video_info_t *video,
    unsigned temp_width                = 0;
    unsigned temp_height               = 0;
    const gfx_ctx_driver_t *ctx_driver = NULL;
-   settings_t *settings               = config_get_ptr();
    vk_t *vk                           = (vk_t*)calloc(1, sizeof(*vk));
    if (!vk)
       return NULL;
@@ -1031,7 +1052,7 @@ static void *vulkan_init(const video_info_t *video,
    mode.height = 0;
 
    RARCH_LOG("Detecting screen resolution %ux%u.\n", full_x, full_y);
-   interval = video->vsync ? settings->video.swap_interval : 0;
+   interval = video->vsync ? video->swap_interval : 0;
    video_context_driver_swap_interval(&interval);
 
    win_width  = video->width;
@@ -1092,13 +1113,8 @@ static void *vulkan_init(const video_info_t *video,
    inp.input_data = input_data;
    video_context_driver_input_driver(&inp);
 
-   if (settings->video.font_enable)
-   {
-      if (!font_driver_init_first(NULL, NULL, vk, *settings->path.font 
-            ? settings->path.font : NULL, settings->video.font_size, false,
-            FONT_DRIVER_RENDER_VULKAN_API))
-         RARCH_ERR("[Vulkan]: Failed to initialize font renderer.\n");
-   }
+   if (video->font_enable)
+      font_driver_init_osd(vk, false, FONT_DRIVER_RENDER_VULKAN_API);
 
    vulkan_init_readback(vk);
    return vk;
@@ -1117,7 +1133,7 @@ static void vulkan_update_filter_chain(vk_t *vk)
       vk->context->num_swapchain_images,
    };
 
-   if (!vulkan_filter_chain_update_swapchain_info(vk->filter_chain, &info))
+   if (!vulkan_filter_chain_update_swapchain_info((vulkan_filter_chain_t*)vk->filter_chain, &info))
       RARCH_ERR("Failed to update filter chain info. This will probably lead to a crash ...\n");
 }
 
@@ -1220,7 +1236,7 @@ static bool vulkan_set_shader(void *data,
    }
 
    if (vk->filter_chain)
-      vulkan_filter_chain_free(vk->filter_chain);
+      vulkan_filter_chain_free((vulkan_filter_chain_t*)vk->filter_chain);
    vk->filter_chain = NULL;
 
    if (!path)
@@ -1266,7 +1282,7 @@ static void vulkan_set_rotation(void *data, unsigned rotation)
    if (!vk)
       return;
 
-   vk->rotation = 90 * rotation;
+   vk->rotation = 270 * rotation;
    vulkan_set_projection(vk, &ortho, true);
 }
 
@@ -1448,15 +1464,69 @@ static void vulkan_readback(vk_t *vk)
          VK_PIPELINE_STAGE_HOST_BIT);
 }
 
+static void vulkan_inject_black_frame(vk_t *vk, video_frame_info_t *video_info)
+{
+   VkCommandBufferBeginInfo begin_info           = {
+      VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO };
+   VkSubmitInfo submit_info                      = {
+      VK_STRUCTURE_TYPE_SUBMIT_INFO };
+
+   const VkClearColorValue clear_color = {{ 0.0f, 0.0f, 0.0f, 1.0f }};
+   const VkImageSubresourceRange range = { VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1 };
+   unsigned frame_index                = vk->context->current_swapchain_index;
+   struct vk_per_frame *chain          = &vk->swapchain[frame_index];
+   vk->chain                           = chain;
+   vk->cmd                             = chain->cmd;
+   begin_info.flags                    = VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT;
+   vkResetCommandBuffer(vk->cmd, 0);
+   vkBeginCommandBuffer(vk->cmd, &begin_info);
+
+   vulkan_image_layout_transition(vk, vk->cmd, chain->backbuffer.image,
+         VK_IMAGE_LAYOUT_UNDEFINED, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL,
+         0, VK_ACCESS_TRANSFER_WRITE_BIT,
+         VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT,
+         VK_PIPELINE_STAGE_TRANSFER_BIT);
+
+   vkCmdClearColorImage(vk->cmd, chain->backbuffer.image, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL,
+         &clear_color, 1, &range);
+
+   vulkan_image_layout_transition(vk, vk->cmd, chain->backbuffer.image,
+         VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, VK_IMAGE_LAYOUT_PRESENT_SRC_KHR,
+         VK_ACCESS_TRANSFER_WRITE_BIT, VK_ACCESS_MEMORY_READ_BIT,
+         VK_PIPELINE_STAGE_TRANSFER_BIT,
+         VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT);
+
+   vkEndCommandBuffer(vk->cmd);
+
+   submit_info.commandBufferCount   = 1;
+   submit_info.pCommandBuffers      = &vk->cmd;
+   if (vk->context->swapchain_semaphores[frame_index] != VK_NULL_HANDLE)
+   {
+      submit_info.signalSemaphoreCount = 1;
+      submit_info.pSignalSemaphores = &vk->context->swapchain_semaphores[frame_index];
+   }
+
+#ifdef HAVE_THREADS
+   slock_lock(vk->context->queue_lock);
+#endif
+   vkQueueSubmit(vk->context->queue, 1,
+         &submit_info, vk->context->swapchain_fences[frame_index]);
+#ifdef HAVE_THREADS
+   slock_unlock(vk->context->queue_lock);
+#endif
+
+   video_context_driver_swap_buffers(video_info);
+}
+
 static bool vulkan_frame(void *data, const void *frame,
       unsigned frame_width, unsigned frame_height,
       uint64_t frame_count,
-      unsigned pitch, const char *msg)
+      unsigned pitch, const char *msg, video_frame_info_t *video_info)
 {
-   struct vk_per_frame *chain;
-   unsigned width, height;
+   VkSemaphore signal_semaphores[2];
    VkClearValue clear_value;
    vk_t *vk                                      = (vk_t*)data;
+   struct vk_per_frame *chain                    = NULL;
    static struct retro_perf_counter frame_run    = {0};
    static struct retro_perf_counter begin_cmd    = {0};
    static struct retro_perf_counter build_cmd    = {0};
@@ -1465,7 +1535,8 @@ static bool vulkan_frame(void *data, const void *frame,
    static struct retro_perf_counter swapbuffers  = {0};
    static struct retro_perf_counter queue_submit = {0};
    bool waits_for_semaphores                     = false;
-   VkSemaphore signal_semaphores[2];
+   unsigned width                                = video_info->width;
+   unsigned height                               = video_info->height;
 
    VkCommandBufferBeginInfo begin_info           = { 
       VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO };
@@ -1476,16 +1547,15 @@ static bool vulkan_frame(void *data, const void *frame,
    unsigned frame_index                          = 
       vk->context->current_swapchain_index;
 
-   performance_counter_init(&frame_run, "frame_run");
-   performance_counter_init(&copy_frame, "copy_frame");
-   performance_counter_init(&swapbuffers, "swapbuffers");
-   performance_counter_init(&queue_submit, "queue_submit");
-   performance_counter_init(&begin_cmd, "begin_command");
-   performance_counter_init(&build_cmd, "build_command");
-   performance_counter_init(&end_cmd, "end_command");
-   performance_counter_start(&frame_run);
+   performance_counter_init(frame_run, "frame_run");
+   performance_counter_init(copy_frame, "copy_frame");
+   performance_counter_init(swapbuffers, "swapbuffers");
+   performance_counter_init(queue_submit, "queue_submit");
+   performance_counter_init(begin_cmd, "begin_command");
+   performance_counter_init(build_cmd, "build_command");
+   performance_counter_init(end_cmd, "end_command");
 
-   video_driver_get_size(&width, &height);
+   performance_counter_start_plus(video_info->is_perfcnt_enable, frame_run);
 
    /* Bookkeeping on start of frame. */
    chain     = &vk->swapchain[frame_index];
@@ -1495,14 +1565,14 @@ static bool vulkan_frame(void *data, const void *frame,
    vulkan_buffer_chain_discard(&chain->vbo);
    vulkan_buffer_chain_discard(&chain->ubo);
 
-   performance_counter_start(&begin_cmd);
+   performance_counter_start_plus(video_info->is_perfcnt_enable, begin_cmd);
    /* Start recording the command buffer. */
    vk->cmd          = chain->cmd;
    begin_info.flags = VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT;
    vkResetCommandBuffer(vk->cmd, 0);
 
    vkBeginCommandBuffer(vk->cmd, &begin_info);
-   performance_counter_stop(&begin_cmd);
+   performance_counter_stop_plus(video_info->is_perfcnt_enable, begin_cmd);
 
    memset(&vk->tracker, 0, sizeof(vk->tracker));
 
@@ -1527,7 +1597,7 @@ static bool vulkan_frame(void *data, const void *frame,
    }
 
    /* Upload texture */
-   performance_counter_start(&copy_frame);
+   performance_counter_start_plus(video_info->is_perfcnt_enable, copy_frame);
    if (frame && !vk->hw.enable)
    {
       unsigned y;
@@ -1578,13 +1648,13 @@ static bool vulkan_frame(void *data, const void *frame,
 
       vk->last_valid_index = frame_index;
    }
-   performance_counter_stop(&copy_frame);
+   performance_counter_stop_plus(video_info->is_perfcnt_enable, copy_frame);
 
    /* Notify filter chain about the new sync index. */
-   vulkan_filter_chain_notify_sync_index(vk->filter_chain, frame_index);
-   vulkan_filter_chain_set_frame_count(vk->filter_chain, frame_count);
+   vulkan_filter_chain_notify_sync_index((vulkan_filter_chain_t*)vk->filter_chain, frame_index);
+   vulkan_filter_chain_set_frame_count((vulkan_filter_chain_t*)vk->filter_chain, frame_count);
 
-   performance_counter_start(&build_cmd);
+   performance_counter_start_plus(video_info->is_perfcnt_enable, build_cmd);
    /* Render offscreen filter chain passes. */
    {
       /* Set the source texture in the filter chain */
@@ -1632,13 +1702,14 @@ static bool vulkan_frame(void *data, const void *frame,
          input.height = tex->height;
       }
 
-      vulkan_filter_chain_set_input_texture(vk->filter_chain, &input);
+      vulkan_filter_chain_set_input_texture((vulkan_filter_chain_t*)vk->filter_chain, &input);
    }
 
    vulkan_set_viewport(vk, width, height, false, true);
 
    vulkan_filter_chain_build_offscreen_passes(
-         vk->filter_chain, vk->cmd, &vk->vk_vp);
+         (vulkan_filter_chain_t*)vk->filter_chain,
+         vk->cmd, &vk->vk_vp);
    /* Render to backbuffer. */
    clear_value.color.float32[0]     = 0.0f;
    clear_value.color.float32[1]     = 0.0f;
@@ -1661,13 +1732,14 @@ static bool vulkan_frame(void *data, const void *frame,
    /* Begin render pass and set up viewport */
    vkCmdBeginRenderPass(vk->cmd, &rp_info, VK_SUBPASS_CONTENTS_INLINE);
 
-   vulkan_filter_chain_build_viewport_pass(vk->filter_chain, vk->cmd,
+   vulkan_filter_chain_build_viewport_pass(
+         (vulkan_filter_chain_t*)vk->filter_chain, vk->cmd,
          &vk->vk_vp, vk->mvp.data);
 
 #if defined(HAVE_MENU)
    if (vk->menu.enable)
    {
-      menu_driver_ctl(RARCH_MENU_CTL_FRAME, NULL);
+      menu_driver_frame(video_info);
 
       if (vk->menu.textures[vk->menu.last_index].image != VK_NULL_HANDLE)
       {
@@ -1690,7 +1762,9 @@ static bool vulkan_frame(void *data, const void *frame,
             quad.texture = optimal;
          }
 
-         quad.sampler = vk->samplers.linear;
+         quad.sampler = optimal->mipmap ?
+            vk->samplers.mipmap_linear : vk->samplers.linear;
+
          quad.mvp     = &vk->mvp_no_rot;
          quad.color.r = 1.0f;
          quad.color.g = 1.0f;
@@ -1702,17 +1776,22 @@ static bool vulkan_frame(void *data, const void *frame,
 #endif
 
    if (msg)
-      font_driver_render_msg(NULL, msg, NULL);
+      font_driver_render_msg(video_info, NULL, msg, NULL);
 
 #ifdef HAVE_OVERLAY
    if (vk->overlay.enable)
-      vulkan_render_overlay(vk);
+      vulkan_render_overlay(vk, video_info);
 #endif
-   performance_counter_stop(&build_cmd);
+   performance_counter_stop_plus(video_info->is_perfcnt_enable, build_cmd);
 
    /* End the render pass. We're done rendering to backbuffer now. */
    vkCmdEndRenderPass(vk->cmd);
 
+   /* End the filter chain frame.
+    * This must happen outside a render pass.
+    */
+   vulkan_filter_chain_end_frame((vulkan_filter_chain_t*)vk->filter_chain, vk->cmd);
+
    if (vk->readback.pending || vk->readback.streamed)
    {
       /* We cannot safely read back from an image which 
@@ -1738,7 +1817,7 @@ static bool vulkan_frame(void *data, const void *frame,
             VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL,
             VK_IMAGE_LAYOUT_PRESENT_SRC_KHR,
             0,
-            0,
+            VK_ACCESS_MEMORY_READ_BIT,
             VK_PIPELINE_STAGE_TRANSFER_BIT,
             VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT);
 
@@ -1752,7 +1831,7 @@ static bool vulkan_frame(void *data, const void *frame,
             VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL,
             VK_IMAGE_LAYOUT_PRESENT_SRC_KHR,
             VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT,
-            0,
+            VK_ACCESS_MEMORY_READ_BIT,
             VK_PIPELINE_STAGE_ALL_GRAPHICS_BIT,
             VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT);
    }
@@ -1772,9 +1851,9 @@ static bool vulkan_frame(void *data, const void *frame,
             vk->context->graphics_queue_index, vk->hw.src_queue_family);
    }
 
-   performance_counter_start(&end_cmd);
+   performance_counter_start_plus(video_info->is_perfcnt_enable, end_cmd);
    vkEndCommandBuffer(vk->cmd);
-   performance_counter_stop(&end_cmd);
+   performance_counter_stop_plus(video_info->is_perfcnt_enable, end_cmd);
 
    /* Submit command buffers to GPU. */
 
@@ -1816,9 +1895,9 @@ static bool vulkan_frame(void *data, const void *frame,
    }
    submit_info.pSignalSemaphores = submit_info.signalSemaphoreCount ? signal_semaphores : NULL;
 
-   performance_counter_stop(&frame_run);
+   performance_counter_stop_plus(video_info->is_perfcnt_enable, frame_run);
 
-   performance_counter_start(&queue_submit);
+   performance_counter_start_plus(video_info->is_perfcnt_enable, queue_submit);
 
 #ifdef HAVE_THREADS
    slock_lock(vk->context->queue_lock);
@@ -1828,13 +1907,14 @@ static bool vulkan_frame(void *data, const void *frame,
 #ifdef HAVE_THREADS
    slock_unlock(vk->context->queue_lock);
 #endif
-   performance_counter_stop(&queue_submit);
+   performance_counter_stop_plus(video_info->is_perfcnt_enable, queue_submit);
 
-   performance_counter_start(&swapbuffers);
-   video_context_driver_swap_buffers();
-   performance_counter_stop(&swapbuffers);
+   performance_counter_start_plus(video_info->is_perfcnt_enable, swapbuffers);
+   video_context_driver_swap_buffers(video_info);
+   performance_counter_stop_plus(video_info->is_perfcnt_enable, swapbuffers);
 
-   video_context_driver_update_window_title();
+   if (!vk->context->swap_interval_emulation_lock)
+      video_context_driver_update_window_title(video_info);
 
    /* Handle spurious swapchain invalidations as soon as we can,
     * i.e. right after swap buffers. */
@@ -1843,12 +1923,40 @@ static bool vulkan_frame(void *data, const void *frame,
       gfx_ctx_mode_t mode;
       mode.width  = width;
       mode.height = height;
-      video_context_driver_set_resize(&mode);
+      video_context_driver_set_resize(mode);
 
       vk->should_resize = false;
    }
    vulkan_check_swapchain(vk);
 
+   /* Disable BFI during fast forward, slow-motion,
+    * and pause to prevent flicker. */
+   if (
+         video_info->black_frame_insertion
+         && !input_driver_is_nonblock_state()
+         && !video_info->runloop_is_slowmotion
+         && !video_info->runloop_is_paused)
+   {
+      vulkan_inject_black_frame(vk, video_info);
+   }
+
+   /* Vulkan doesn't directly support swap_interval > 1, so we fake it by duping out more frames. */
+   if (vk->context->swap_interval > 1 && !vk->context->swap_interval_emulation_lock)
+   {
+      unsigned i;
+      vk->context->swap_interval_emulation_lock = true;
+      for (i = 1; i < vk->context->swap_interval; i++)
+      {
+         if (!vulkan_frame(vk, NULL, 0, 0, frame_count, 0, msg,
+                  video_info))
+         {
+            vk->context->swap_interval_emulation_lock = false;
+            return false;
+         }
+      }
+      vk->context->swap_interval_emulation_lock = false;
+   }
+
    return true;
 }
 
@@ -1903,7 +2011,7 @@ static struct video_shader *vulkan_get_current_shader(void *data)
    if (!vk || !vk->filter_chain)
       return NULL;
 
-   return vulkan_filter_chain_get_preset(vk->filter_chain);
+   return vulkan_filter_chain_get_preset((vulkan_filter_chain_t*)vk->filter_chain);
 }
 
 static bool vulkan_get_current_sw_framebuffer(void *data,
@@ -1968,9 +2076,9 @@ static void vulkan_set_texture_frame(void *data,
    uint8_t *dst                       = NULL;
    const uint8_t *src                 = NULL;
    vk_t *vk                           = (vk_t*)data;
-   unsigned index                     = vk->context->current_swapchain_index;
-   struct vk_texture *texture         = &vk->menu.textures[index];
-   struct vk_texture *texture_optimal = &vk->menu.textures_optimal[index];
+   unsigned index                     = 0;
+   struct vk_texture *texture         = NULL;
+   struct vk_texture *texture_optimal = NULL;
    const VkComponentMapping br_swizzle = {
       VK_COMPONENT_SWIZZLE_B,
       VK_COMPONENT_SWIZZLE_G,
@@ -1981,6 +2089,10 @@ static void vulkan_set_texture_frame(void *data,
    if (!vk)
       return;
 
+   index           = vk->context->current_swapchain_index;
+   texture         = &vk->menu.textures[index];
+   texture_optimal = &vk->menu.textures_optimal[index];
+
    /* B4G4R4A4 must be supported, but R4G4B4A4 is optional,
     * just apply the swizzle in the image view instead. */
    *texture = vulkan_create_texture(vk,
@@ -2030,10 +2142,11 @@ static void vulkan_set_texture_enable(void *data, bool state, bool full_screen)
 }
 
 static void vulkan_set_osd_msg(void *data, const char *msg,
-      const struct font_params *params, void *font)
+      const void *params, void *font)
 {
-   (void)data;
-   font_driver_render_msg(font, msg, params);
+   video_frame_info_t video_info;
+   video_driver_build_info(&video_info);
+   font_driver_render_msg(&video_info, font, msg, params);
 }
 #endif
 
@@ -2043,17 +2156,42 @@ static uintptr_t vulkan_load_texture(void *video_data, void *data,
    vk_t *vk                    = (vk_t*)video_data;
    struct texture_image *image = (struct texture_image*)data;
    struct vk_texture *texture = (struct vk_texture*)calloc(1, sizeof(*texture));
-   if (!texture)
+   if (!image || !texture)
       return 0;
 
-   *texture = vulkan_create_texture(vk, NULL,
-         image->width, image->height, VK_FORMAT_B8G8R8A8_UNORM,
-         image->pixels, NULL, VULKAN_TEXTURE_STATIC);
+   if (!image->pixels || !image->width || !image->height)
+   {
+      /* Create a dummy texture instead. */
+#define T0 0xff000000u
+#define T1 0xffffffffu
+      static const uint32_t checkerboard[] = {
+         T0, T1, T0, T1, T0, T1, T0, T1,
+         T1, T0, T1, T0, T1, T0, T1, T0,
+         T0, T1, T0, T1, T0, T1, T0, T1,
+         T1, T0, T1, T0, T1, T0, T1, T0,
+         T0, T1, T0, T1, T0, T1, T0, T1,
+         T1, T0, T1, T0, T1, T0, T1, T0,
+         T0, T1, T0, T1, T0, T1, T0, T1,
+         T1, T0, T1, T0, T1, T0, T1, T0,
+      };
+#undef T0
+#undef T1
+      *texture = vulkan_create_texture(vk, NULL,
+            8, 8, VK_FORMAT_B8G8R8A8_UNORM,
+            checkerboard, NULL, VULKAN_TEXTURE_STATIC);
+      texture->default_smooth = false;
+      texture->mipmap = false;
+   }
+   else
+   {
+      *texture = vulkan_create_texture(vk, NULL,
+            image->width, image->height, VK_FORMAT_B8G8R8A8_UNORM,
+            image->pixels, NULL, VULKAN_TEXTURE_STATIC);
 
-   /* TODO: Actually add mipmapping support.
-    * Optimal tiling would make sense here as well ... */
-   texture->default_smooth =
-      filter_type == TEXTURE_FILTER_MIPMAP_LINEAR || filter_type == TEXTURE_FILTER_LINEAR;
+      texture->default_smooth =
+         filter_type == TEXTURE_FILTER_MIPMAP_LINEAR || filter_type == TEXTURE_FILTER_LINEAR;
+      texture->mipmap = filter_type == TEXTURE_FILTER_MIPMAP_LINEAR;
+   }
 
    return (uintptr_t)texture;
 }
@@ -2124,7 +2262,7 @@ static void vulkan_viewport_info(void *data, struct video_viewport *vp)
    vp->full_height = height;
 }
 
-static bool vulkan_read_viewport(void *data, uint8_t *buffer)
+static bool vulkan_read_viewport(void *data, uint8_t *buffer, bool is_idle)
 {
    struct vk_texture *staging       = NULL;
    vk_t *vk                         = (vk_t*)data;
@@ -2137,14 +2275,10 @@ static bool vulkan_read_viewport(void *data, uint8_t *buffer)
    if (vk->readback.streamed)
    {
       const uint8_t *src;
-      static struct retro_perf_counter stream_readback = {0};
 
       if (staging->memory == VK_NULL_HANDLE)
          return false;
 
-      performance_counter_init(&stream_readback, "stream_readback");
-      performance_counter_start(&stream_readback);
-
       buffer += 3 * (vk->vp.height - 1) * vk->vp.width;
       vkMapMemory(vk->context->device, staging->memory,
             staging->offset, staging->size, 0, (void**)&src);
@@ -2156,8 +2290,6 @@ static bool vulkan_read_viewport(void *data, uint8_t *buffer)
       scaler_ctx_scale(&vk->readback.scaler, buffer, src);
 
       vkUnmapMemory(vk->context->device, staging->memory);
-
-      performance_counter_stop(&stream_readback);
    }
    else
    {
@@ -2167,7 +2299,10 @@ static bool vulkan_read_viewport(void *data, uint8_t *buffer)
        * with conversion. */
 
       vk->readback.pending = true;
-      video_driver_cached_frame_render();
+
+      if (!is_idle)
+         video_driver_cached_frame();
+
       vkQueueWaitIdle(vk->context->queue);
 
       if (!staging->mapped)
@@ -2256,17 +2391,17 @@ static void vulkan_overlay_set_alpha(void *data,
    }
 }
 
-static void vulkan_render_overlay(vk_t *vk)
+static void vulkan_render_overlay(vk_t *vk, video_frame_info_t *video_info)
 {
-   unsigned width, height;
    unsigned i;
    struct video_viewport vp;
+   unsigned width           = video_info->width;
+   unsigned height          = video_info->height;
 
    if (!vk)
       return;
 
-   video_driver_get_size(&width, &height);
-   vp = vk->vp;
+   vp                       = vk->vp;
    vulkan_set_viewport(vk, width, height, vk->overlay.full_screen, false);
 
    for (i = 0; i < vk->overlay.count; i++)
@@ -2283,7 +2418,8 @@ static void vulkan_render_overlay(vk_t *vk)
       memset(&call, 0, sizeof(call));
       call.pipeline     = vk->display.pipelines[3]; /* Strip with blend */
       call.texture      = &vk->overlay.images[i];
-      call.sampler      = vk->samplers.linear;
+      call.sampler      = call.texture->mipmap ?
+         vk->samplers.mipmap_linear : vk->samplers.linear;
       call.uniform      = &vk->mvp;
       call.uniform_size = sizeof(vk->mvp);
       call.vbo          = &range;
