@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -30,12 +30,20 @@
 
 #include <retro_inline.h>
 
-#include "../../driver.h"
-#include "../../frontend/frontend_driver.h"
-#include "../../general.h"
-#include "../../verbosity.h"
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
+#ifdef HAVE_MENU
+#include "../../menu/menu_driver.h"
+#endif
+
 #include "../font_driver.h"
 
+#include "../../configuration.h"
+#include "../../frontend/frontend_driver.h"
+#include "../../verbosity.h"
+
 #include "../common/x11_common.h"
 
 /* Adapted from bSNES and MPlayer source. */
@@ -413,8 +421,9 @@ static void *xv_init(const video_info_t *video,
 {
    unsigned i;
    XWindowAttributes target;
+   video_frame_info_t video_info;
    char buf[128]                          = {0};
-   char buf_fps[128]                      = {0};
+   char title[128]                        = {0};
    XSetWindowAttributes attributes        = {0};
    XVisualInfo visualtemplate             = {0};
    unsigned width                         = 0;
@@ -509,10 +518,16 @@ static void *xv_init(const video_info_t *video,
    attributes.event_mask   = StructureNotifyMask | KeyPressMask | 
       KeyReleaseMask | ButtonReleaseMask | ButtonPressMask | DestroyNotify | ClientMessage;
 
-   width      = video->fullscreen ? ( (video->width  == 0) ? 
-         geom->base_width : video->width) : video->width;
-   height     = video->fullscreen ? ((video->height == 0)  ? 
-         geom->base_height : video->height) : video->height;
+   if (video->fullscreen)
+   {
+      width      = (((video->width  == 0) && geom) ? geom->base_width : video->width);
+      height     = (((video->height == 0) && geom) ? geom->base_height : video->height);
+   }
+   else
+   {
+      width      = video->width;
+      height     = video->height;
+   }
    g_x11_win  = XCreateWindow(g_x11_dpy, DefaultRootWindow(g_x11_dpy),
          0, 0, width, height,
          0, xv->depth, InputOutput, visualinfo->visual,
@@ -523,8 +538,12 @@ static void *xv_init(const video_info_t *video,
 
    XMapWindow(g_x11_dpy, g_x11_win);
 
-   if (video_monitor_get_fps(buf, sizeof(buf), NULL, 0))
-      XStoreName(g_x11_dpy, g_x11_win, buf);
+   video_driver_build_info(&video_info);
+
+   video_driver_get_window_title(title, sizeof(title));
+
+   if (title[0])
+      XStoreName(g_x11_dpy, g_x11_win, title);
 
    x11_set_window_attr(g_x11_dpy, g_x11_win);
 
@@ -541,8 +560,11 @@ static void *xv_init(const video_info_t *video,
    if (atom != None)
       XvSetPortAttribute(g_x11_dpy, xv->port, atom, 1);
 
-   xv->width  = geom->max_width;
-   xv->height = geom->max_height;
+   if (geom)
+   {
+      xv->width  = geom->max_width;
+      xv->height = geom->max_height;
+   }
 
    xv->image = XvShmCreateImage(g_x11_dpy, xv->port, xv->fourcc,
          NULL, xv->width, xv->height, &xv->shminfo);
@@ -575,7 +597,7 @@ static void *xv_init(const video_info_t *video,
 
    if (input && input_data)
    {
-      xinput = input_x.init();
+      xinput = input_x.init(settings->input.joypad_driver);
       if (xinput)
       {
          *input = &input_x;
@@ -774,7 +796,7 @@ static void xv_render_msg(xv_t *xv, const char *msg,
 
 static bool xv_frame(void *data, const void *frame, unsigned width,
       unsigned height, uint64_t frame_count,
-      unsigned pitch, const char *msg)
+      unsigned pitch, const char *msg, video_frame_info_t *video_info)
 {
    XWindowAttributes target;
    xv_t *xv                  = (xv_t*)data;
@@ -792,6 +814,10 @@ static bool xv_frame(void *data, const void *frame, unsigned width,
    xv->vp.full_width  = target.width;
    xv->vp.full_height = target.height;
 
+#ifdef HAVE_MENU
+   menu_driver_frame(video_info);
+#endif
+
    if (msg)
       xv_render_msg(xv, msg, width << 1, height << 1);
 
@@ -801,7 +827,7 @@ static bool xv_frame(void *data, const void *frame, unsigned width,
          true);
    XSync(g_x11_dpy, False);
 
-   x11_update_window_title(NULL);
+   x11_update_title(NULL, video_info);
 
    return true;
 }
@@ -866,7 +892,7 @@ static void xv_set_rotation(void *data, unsigned rotation)
    (void)rotation;
 }
 
-static bool xv_read_viewport(void *data, uint8_t *buffer)
+static bool xv_read_viewport(void *data, uint8_t *buffer, bool is_idle)
 {
    (void)data;
    (void)buffer;
