@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2011-2016 - Higor Euripedes
- *  Copyright (C) 2011-2016 - Daniel De Matteis
- *  Copyright (C) 2014-2015 - Jean-André Santoni
+ *  Copyright (C) 2011-2017 - Higor Euripedes
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *  Copyright (C) 2014-2017 - Jean-André Santoni
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -41,7 +41,7 @@
 
 #include "../menu_driver.h"
 #include "../menu_animation.h"
-#include "../menu_entry.h"
+#include "../widgets/menu_entry.h"
 #include "../menu_display.h"
 #include "../menu_navigation.h"
 #include "../../retroarch.h"
@@ -141,7 +141,7 @@ typedef struct zarch_handle
    const core_info_t *pick_cores;
    size_t pick_supported;
 
-   void *fb_buf;
+   void *font;
    int font_size;
    int header_height;
    unsigned next_id;
@@ -166,43 +166,41 @@ struct zui_tabbed
 };
 
 
-static enum zarch_layout_type zarch_layout;
+static enum zarch_layout_type zarch_layout = LAY_HOME;
 
 static float zarch_zui_strwidth(void *fb_buf, const char *text, float scale)
 {
    return font_driver_get_message_width(fb_buf, text, strlen(text), scale);
 }
 
-
 static int16_t zarch_zui_input_state(zui_t *zui, enum zarch_zui_input_state state)
 {
-    switch (state)
-    {
-        case MENU_ZARCH_MOUSE_X:
-            return menu_input_mouse_state(MENU_MOUSE_X_AXIS);
-        case MENU_ZARCH_MOUSE_Y:
-            return menu_input_mouse_state(MENU_MOUSE_Y_AXIS);
-        case MENU_POINTER_ZARCH_X:
-            return menu_input_pointer_state(MENU_POINTER_X_AXIS);
-        case MENU_POINTER_ZARCH_Y:
-            return menu_input_pointer_state(MENU_POINTER_Y_AXIS);
-        case MENU_ZARCH_PRESSED:
-            if (     menu_input_mouse_state(MENU_MOUSE_LEFT_BUTTON) 
-                  || menu_input_pointer_state(MENU_POINTER_PRESSED))
-                return 1;
-            if (zui->action == MENU_ACTION_OK)
-               return 1;
-            break;
-    }
-    
-    return 0;
+   switch (state)
+   {
+      case MENU_ZARCH_MOUSE_X:
+         return menu_input_mouse_state(MENU_MOUSE_X_AXIS);
+      case MENU_ZARCH_MOUSE_Y:
+         return menu_input_mouse_state(MENU_MOUSE_Y_AXIS);
+      case MENU_POINTER_ZARCH_X:
+         return menu_input_pointer_state(MENU_POINTER_X_AXIS);
+      case MENU_POINTER_ZARCH_Y:
+         return menu_input_pointer_state(MENU_POINTER_Y_AXIS);
+      case MENU_ZARCH_PRESSED:
+         if (     menu_input_mouse_state(MENU_MOUSE_LEFT_BUTTON) 
+               || menu_input_pointer_state(MENU_POINTER_PRESSED))
+            return 1;
+         if (zui->action == MENU_ACTION_OK)
+            return 1;
+         break;
+   }
+
+   return 0;
 }
 
 static bool zarch_zui_check_button_down(zui_t *zui,
       unsigned id, int x1, int y1, int x2, int y2)
 {
    menu_input_ctx_hitbox_t hitbox;
-   bool result = false;
 
    hitbox.x1   = x1;
    hitbox.x2   = x2;
@@ -215,11 +213,11 @@ static bool zarch_zui_check_button_down(zui_t *zui,
    if (     zui->item.hot == id 
          && zarch_zui_input_state(zui, MENU_ZARCH_PRESSED))
    {
-      result = true;
       zui->item.active = id;
+      return true;
    }
 
-   return result;
+   return false;
 }
 
 static bool zarch_zui_check_button_up(zui_t *zui,
@@ -271,7 +269,7 @@ static void zarch_zui_draw_text(zui_t *zui,
 {
    struct font_params params;
 
-   if (!zui || !zui->fb_buf || string_is_empty(text))
+   if (!zui || !zui->font || string_is_empty(text))
       return;
 
    /* need to use height-y because the font renderer 
@@ -286,7 +284,7 @@ static void zarch_zui_draw_text(zui_t *zui,
    params.full_screen = true;
    params.text_align  = TEXT_ALIGN_LEFT;
 
-   video_driver_set_osd_msg(text, &params, zui->fb_buf);
+   video_driver_set_osd_msg(text, &params, zui->font);
 }
 
 static bool zarch_zui_button_full(zui_t *zui,
@@ -307,30 +305,30 @@ static bool zarch_zui_button_full(zui_t *zui,
 
 static bool zarch_zui_button(zui_t *zui, int x1, int y1, const char *label)
 {
-   if (!zui || !zui->fb_buf)
+   if (!zui || !zui->font)
       return false;
    return zarch_zui_button_full(zui, x1, y1, x1 
-         + zarch_zui_strwidth(zui->fb_buf, label, 1.0) + 24, y1 + 64, label);
+         + zarch_zui_strwidth(zui->font, label, 1.0) + 24, y1 + 64, label);
 }
 
-static bool zarch_zui_list_item(zui_t *zui, struct zui_tabbed *tab, int x1, int y1,
+static bool zarch_zui_list_item(video_frame_info_t *video_info,
+      zui_t *zui, struct zui_tabbed *tab, int x1, int y1,
       const char *label, unsigned item_id, const char *entry, bool selected)
 {
    menu_animation_ctx_ticker_t ticker;
    unsigned ticker_size;
-   char title_buf[PATH_MAX_LENGTH] = {0};
-   uint64_t *frame_count = NULL;
+   char title_buf[PATH_MAX_LENGTH];
    unsigned           id = zarch_zui_hash(zui, label);
    int                x2 = x1 + zui->width - 290 - 40;
    int                y2 = y1 + 50;
    bool           active = zarch_zui_check_button_up(zui, id, x1, y1, x2, y2);
    const float       *bg = zui_bg_panel;
-   frame_count           = video_driver_get_frame_count_ptr();
+   uint64_t frame_count  = video_info->frame_count;
+
+   title_buf[0] = '\0';
 
    if (tab->active_id != tab->prev_id)
-   {
       tab->prev_id         = tab->active_id;
-   }
 
    if (selected)
    {
@@ -348,11 +346,11 @@ static bool zarch_zui_list_item(zui_t *zui, struct zui_tabbed *tab, int x1, int
 
    ticker.s        = title_buf;
    ticker.len      = ticker_size;
-   ticker.idx      = *frame_count / 50;
+   ticker.idx      = frame_count / 50;
    ticker.str      = label;
    ticker.selected = (bg == zui_bg_hilite || bg == zui_bg_pad_hilite);
 
-   menu_animation_ctl(MENU_ANIMATION_CTL_TICKER, &ticker);
+   menu_animation_ticker(&ticker);
 
    menu_display_push_quad(zui->width, zui->height, bg, x1, y1, x2, y2);
    zarch_zui_draw_text(zui, ZUI_FG_NORMAL, 12, y1 + 35, title_buf);
@@ -382,9 +380,9 @@ static bool zarch_zui_tab(zui_t *zui, struct zui_tabbed *tab,
 
    if (!width)
    {
-      if (!zui->fb_buf)
+      if (!zui->font)
          return false;
-      width          = zarch_zui_strwidth(zui->fb_buf, label, 1.0) + 24;
+      width          = zarch_zui_strwidth(zui->font, label, 1.0) + 24;
    }
 
    x1                = tab->x;
@@ -398,9 +396,7 @@ static bool zarch_zui_tab(zui_t *zui, struct zui_tabbed *tab,
    if (zui->item.active == id || tab->active_id == ~0U || !tab->inited)
       tab->active_id    = id;
    else if (id > tab->active_id)
-   {
       tab->next_id            = id;
-   }
 
    if (!tab->inited)
       tab->inited = true;
@@ -504,7 +500,9 @@ static bool zarch_zui_gamepad_input(zui_t *zui,
    return false;
 }
 
-static int zarch_zui_render_lay_root_recent(zui_t *zui, struct zui_tabbed *tabbed)
+static int zarch_zui_render_lay_root_recent(
+      video_frame_info_t *video_info,
+      zui_t *zui, struct zui_tabbed *tabbed)
 {
    if (zarch_zui_tab(zui, tabbed, "Recent", 0))
    {
@@ -518,15 +516,21 @@ static int zarch_zui_render_lay_root_recent(zui_t *zui, struct zui_tabbed *tabbe
 
       for (i = zui->recent_dlist_first; i < size; ++i)
       {
-         char rich_label[PATH_MAX_LENGTH] = {0};
-         menu_entry_t entry               = {{0}};
+         char rich_label[PATH_MAX_LENGTH];
+         char entry_value[PATH_MAX_LENGTH];
+         menu_entry_t entry                = {{0}};
+
+         rich_label[0] = entry_value[0]    = '\0';
 
          menu_entry_get(&entry, 0, i, NULL, true);
          menu_entry_get_rich_label(i, rich_label, sizeof(rich_label));
+         menu_entry_get_value(i, NULL, entry_value,sizeof(entry_value));
 
-         if (zarch_zui_list_item(zui, tabbed, 0, 
+         if (zarch_zui_list_item(
+                  video_info,
+                  zui, tabbed, 0, 
                   tabbed->tabline_size + j * ZUI_ITEM_SIZE_PX,
-                  rich_label, i, entry.value, gamepad_index == (signed)i))
+                  rich_label, i, entry_value, gamepad_index == (signed)i))
          {
             if (menu_entry_action(&entry, i, MENU_ACTION_OK))
                return 1;
@@ -563,13 +567,17 @@ static void zarch_zui_render_lay_root_load_set_new_path(zui_t *zui,
    zui->load_dlist = NULL;
 }
 
-static int zarch_zui_render_lay_root_load(zui_t *zui,
+static int zarch_zui_render_lay_root_load(
+      video_frame_info_t *video_info,
+      zui_t *zui,
       struct zui_tabbed *tabbed)
 {
-   char parent_dir[PATH_MAX_LENGTH] = {0};
+   char parent_dir[PATH_MAX_LENGTH];
    settings_t           *settings   = config_get_ptr();
    core_info_list_t           *list = NULL;
 
+   parent_dir[0] = '\0';
+
    if (zarch_zui_tab(zui, tabbed, "Load", 1))
    {
       unsigned cwd_offset;
@@ -583,7 +591,7 @@ static int zarch_zui_render_lay_root_load(zui_t *zui,
          core_info_get_current_core(&core_info);
 
          zui->load_dlist = dir_list_new(zui->load_cwd,
-               core_info->supported_extensions, true, true);
+               core_info->supported_extensions, true, true, false, true);
          dir_list_sort(zui->load_dlist, true);
          zui->load_dlist_first  = 0;
       }
@@ -603,7 +611,9 @@ static int zarch_zui_render_lay_root_load(zui_t *zui,
          fill_pathname_parent_dir(parent_dir,
                zui->load_cwd, sizeof(parent_dir));
          if (!string_is_empty(parent_dir) &&
-               zarch_zui_list_item(zui, tabbed, 0,
+               zarch_zui_list_item(
+                  video_info,
+                  zui, tabbed, 0,
                   tabbed->tabline_size + 73, " ..", 0, NULL, false /* TODO/FIXME */))
          {
             zarch_zui_render_lay_root_load_set_new_path(zui, parent_dir);
@@ -630,14 +640,16 @@ static int zarch_zui_render_lay_root_load(zui_t *zui,
 
             for (i = skip + zui->load_dlist_first; i < size; ++i)
             {
-               char label[PATH_MAX_LENGTH] = {0};
+               char label[PATH_MAX_LENGTH];
                const char        *path     = NULL;
                const char        *basename = NULL;
 
                if (j > 10)
                   break;
 
-               path = zui->load_dlist->elems[i].data;
+               label[0] = '\0';
+
+               path     = zui->load_dlist->elems[i].data;
                basename = path_basename(path);
 
                *label = 0;
@@ -647,7 +659,9 @@ static int zarch_zui_render_lay_root_load(zui_t *zui,
                if (path_is_directory(path))
                   strncat(label, "/", sizeof(label)-1);
 
-               if (zarch_zui_list_item(zui, tabbed, 0,
+               if (zarch_zui_list_item(
+                        video_info,
+                        zui, tabbed, 0,
                         tabbed->tabline_size + 73 + j * ZUI_ITEM_SIZE_PX,
                         label, i, NULL, gamepad_index == (signed)(i-skip)))
                {
@@ -705,9 +719,10 @@ static int zarch_zui_render_lay_root_downloads(
    return 0;
 }
 
-static int zarch_zui_render_lay_root(zui_t *zui)
+static int zarch_zui_render_lay_root(video_frame_info_t *video_info,
+      zui_t *zui)
 {
-   char item[PATH_MAX_LENGTH]      = {0};
+   char item[PATH_MAX_LENGTH];
    static struct zui_tabbed tabbed = {~0U};
 
    zarch_zui_tabbed_begin(zui, &tabbed, 0, 0);
@@ -715,17 +730,18 @@ static int zarch_zui_render_lay_root(zui_t *zui)
    tabbed.width            = zui->width - 290 - 40;
    zui->next_selection_set = false;
 
-   if (zarch_zui_render_lay_root_recent(zui, &tabbed))
+   if (zarch_zui_render_lay_root_recent(video_info, zui, &tabbed))
       return 0;
-   if (zarch_zui_render_lay_root_load  (zui, &tabbed))
+   if (zarch_zui_render_lay_root_load(video_info, zui, &tabbed))
       return 0;
    if (zarch_zui_render_lay_root_collections(zui, &tabbed))
       return 0;
    if (zarch_zui_render_lay_root_downloads(zui, &tabbed))
       return 0;
 
-   (void)item;
 #ifdef ZARCH_DEBUG
+   item[0] = '\0';
+
    snprintf(item, sizeof(item), "item id: %d\n", zui->active_id);
    zarch_zui_draw_text(zui, ZUI_FG_NORMAL, 1600 +12, 300 + 41, item); 
    snprintf(item, sizeof(item), "tab  idx: %d\n", tabbed.active_id);
@@ -787,7 +803,8 @@ static void zarch_zui_draw_cursor(float x, float y)
 {
 }
 
-static int zarch_zui_render_pick_core(zui_t *zui)
+static int zarch_zui_render_pick_core(video_frame_info_t *video_info,
+      zui_t *zui)
 {
    static struct zui_tabbed tabbed = {~0U};
    unsigned i, j = 0;
@@ -809,7 +826,9 @@ static int zarch_zui_render_pick_core(zui_t *zui)
 
    if (!zui->pick_supported)
    {
-      zarch_zui_list_item(zui, &tabbed, 0, ZUI_ITEM_SIZE_PX,
+      zarch_zui_list_item(
+            video_info,
+            zui, &tabbed, 0, ZUI_ITEM_SIZE_PX,
             "Content unsupported", 0, NULL, false /* TODO/FIXME */);
       return 1;
    }
@@ -823,7 +842,9 @@ static int zarch_zui_render_pick_core(zui_t *zui)
       if (j > 10)
          break;
 
-      if (zarch_zui_list_item(zui, &tabbed, 0, ZUI_ITEM_SIZE_PX + j * ZUI_ITEM_SIZE_PX,
+      if (zarch_zui_list_item(
+               video_info,
+               zui, &tabbed, 0, ZUI_ITEM_SIZE_PX + j * ZUI_ITEM_SIZE_PX,
                zui->pick_cores[i].display_name, i, NULL, false))
       {
          int ret = zarch_zui_load_content(zui, i);
@@ -841,7 +862,7 @@ static int zarch_zui_render_pick_core(zui_t *zui)
    return 0;
 }
 
-static void zarch_frame(void *data)
+static void zarch_frame(void *data, video_frame_info_t *video_info)
 {
    unsigned i;
    float coord_color[16];
@@ -850,17 +871,14 @@ static void zarch_frame(void *data)
    menu_display_ctx_coord_draw_t coord_draw;
    settings_t *settings    = config_get_ptr();
    zui_t *zui              = (zui_t*)data;
-   video_coord_array_t *ca   = NULL;
-
-   ca = menu_display_get_coords_array();
+   video_coord_array_t *ca = menu_display_get_coords_array();
    
    if (!zui)
       return;
 
    video_driver_get_size(&zui->width, &zui->height);
 
-   menu_display_set_viewport();
-   zui->fb_buf = menu_display_get_font_buffer();
+   menu_display_set_viewport(video_info->width, video_info->height);
 
    for (i = 0; i < 16; i++)
    {
@@ -886,7 +904,7 @@ static void zarch_frame(void *data)
 
    zui->tmp_block.carr.coords.vertices = 0;
 
-   menu_display_font_bind_block(&zui->tmp_block);
+   menu_display_font_bind_block((font_data_t*)zui->font, &zui->tmp_block);
 
    menu_display_push_quad(zui->width, zui->height, zui_bg_screen,
          0, 0, zui->width, zui->height);
@@ -897,7 +915,7 @@ static void zarch_frame(void *data)
       case LAY_HOME:
          if (zarch_zui_render_sidebar(zui))
             return;
-         if (zarch_zui_render_lay_root(zui))
+         if (zarch_zui_render_lay_root(video_info, zui))
             return;
          break;
       case LAY_SETTINGS:
@@ -906,7 +924,7 @@ static void zarch_frame(void *data)
       case LAY_PICK_CORE:
          if (zarch_zui_render_sidebar(zui))
             return;
-         if (zarch_zui_render_pick_core(zui))
+         if (zarch_zui_render_pick_core(video_info, zui))
             return;
          break;
    }
@@ -952,25 +970,25 @@ static void zarch_frame(void *data)
    draw.vertex_count       = 4;
    draw.prim_type          = MENU_DISPLAY_PRIM_TRIANGLESTRIP;
 
-   if (!menu_display_libretro_running() && draw.texture)
-      draw.color             = &coord_color2[0];
+   if (!video_info->libretro_running && draw.texture)
+      draw.color           = &coord_color2[0];
 
    menu_display_blend_begin();
    draw.x              = 0;
    draw.y              = 0;
-   menu_display_draw_bg(&draw);
+   menu_display_draw_bg(&draw, video_info, false);
    menu_display_draw(&draw);
    menu_display_blend_end();
 
    zui->rendering = false;
 
-   menu_display_font_flush_block();
-   menu_display_unset_viewport();
+   menu_display_font_flush_block(video_info->width, video_info->height, 
+         (font_data_t*)zui->font);
+   menu_display_unset_viewport(video_info->width, video_info->height);
 }
 
 static void *zarch_init(void **userdata)
 {
-   int unused;
    zui_t *zui                              = NULL;
    settings_t *settings                    = config_get_ptr();
    menu_handle_t *menu                     = (menu_handle_t*)
@@ -995,26 +1013,11 @@ static void *zarch_init(void **userdata)
       settings->menu.mouse.enable = false;
    }
 
-   unused = 1000;
-   menu_display_set_header_height(unused);
-
-   unused = 28;
-   menu_display_set_font_size(unused);
-
-   (void)unused;
-
    zui->header_height  = 1000; /* dpi / 3; */
    zui->font_size       = 28;
 
-   if (!string_is_empty(settings->path.menu_wallpaper))
-      task_push_image_load(settings->path.menu_wallpaper,
-            MENU_ENUM_LABEL_CB_MENU_WALLPAPER,
-            menu_display_handle_wallpaper_upload, NULL);
-
    matrix_4x4_ortho(&zui->mvp, 0, 1, 1, 0, 0, 1);
 
-   menu_display_font(APPLICATION_SPECIAL_DIRECTORY_ASSETS_ZARCH_FONT);
-
    return menu;
 error:
    if (menu)
@@ -1043,8 +1046,16 @@ static void zarch_context_bg_destroy(void *data)
 
 static void zarch_context_destroy(void *data)
 {
-   menu_display_font_main_deinit();
+   zui_t        *zui = (zui_t*)data;
+
+   /* why on earth is this called twice on exit? */
+   if (!zui)
+      return;
+
+   menu_display_font_free((font_data_t*)zui->font);
    zarch_context_bg_destroy(data);
+
+   zui->font = NULL;
 }
 
 static bool zarch_load_image(void *userdata, 
@@ -1067,6 +1078,9 @@ static bool zarch_load_image(void *userdata,
          break;
       case MENU_IMAGE_THUMBNAIL:
          break;
+      case MENU_IMAGE_SAVESTATE_THUMBNAIL:
+         /* TODO/FIXME -implement */
+         break;
    }
 
    return true;
@@ -1074,32 +1088,21 @@ static bool zarch_load_image(void *userdata,
 
 static void zarch_context_reset(void *data)
 {
-   menu_display_ctx_font_t font_info;
    settings_t *settings  = config_get_ptr();
    zui_t          *zui   = (zui_t*)data;
 
    if (!zui || !settings)
       return;
 
-   font_info.path    = NULL;
-   font_info.size    = zui->font_size;
-
-   if (settings->video.font_enable)
-      font_info.path = settings->path.font;
-
-   if (!menu_display_font_main_init(&font_info))
-      RARCH_WARN("Failed to load font.");
-
    zarch_context_bg_destroy(zui);
 
    task_push_image_load(settings->path.menu_wallpaper,
-         MENU_ENUM_LABEL_CB_MENU_WALLPAPER,
          menu_display_handle_wallpaper_upload, NULL);
 
    menu_display_allocate_white_texture();
 
-   menu_display_set_font_size(zui->font_size);
-   menu_display_font(APPLICATION_SPECIAL_DIRECTORY_ASSETS_ZARCH_FONT);
+   menu_display_set_header_height(zui->header_height);
+   zui->font = menu_display_font(APPLICATION_SPECIAL_DIRECTORY_ASSETS_ZARCH_FONT, zui->font_size);
 }
 
 static int zarch_iterate(void *data, void *userdata, enum menu_action action)
@@ -1127,8 +1130,8 @@ static int zarch_iterate(void *data, void *userdata, enum menu_action action)
 static bool zarch_menu_init_list(void *data)
 {
    menu_displaylist_info_t info = {0};
-   file_list_t *menu_stack    = menu_entries_get_menu_stack_ptr(0);
-   file_list_t *selection_buf = menu_entries_get_selection_buf_ptr(0);
+   file_list_t *menu_stack      = menu_entries_get_menu_stack_ptr(0);
+   file_list_t *selection_buf   = menu_entries_get_selection_buf_ptr(0);
 
    strlcpy(info.label,
          msg_hash_to_str(MENU_ENUM_LABEL_HISTORY_TAB), sizeof(info.label));
