@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -20,6 +20,7 @@
 
 #include <stdint.h>
 #include <errno.h>
+#include <string.h>
 #include <unistd.h>
 #include <math.h>
 
@@ -36,8 +37,6 @@
 #include <streams/file_stream.h>
 
 #include "../../verbosity.h"
-#include "../../driver.h"
-#include "../../runloop.h"
 #include "../../frontend/frontend_driver.h"
 #include "../common/drm_common.h"
 
@@ -61,14 +60,14 @@
 
 #endif
 
-static enum gfx_ctx_api drm_api;
+static enum gfx_ctx_api drm_api           = GFX_CTX_NONE;
 
-static struct gbm_bo *g_bo;
-static struct gbm_bo *g_next_bo;
-static struct gbm_surface *g_gbm_surface;
-static struct gbm_device *g_gbm_dev;
+static struct gbm_bo *g_bo                = NULL;
+static struct gbm_bo *g_next_bo           = NULL;
+static struct gbm_surface *g_gbm_surface  = NULL;
+static struct gbm_device *g_gbm_dev       = NULL;
 
-static bool waiting_for_flip;
+static bool waiting_for_flip              = false;
 
 typedef struct gfx_ctx_drm_data
 {
@@ -103,11 +102,8 @@ static struct drm_fb *drm_fb_get_from_bo(struct gbm_bo *bo)
 {
    int ret;
    unsigned width, height, stride, handle;
-   struct drm_fb *fb = (struct drm_fb*)gbm_bo_get_user_data(bo);
-   if (fb)
-      return fb;
+   struct drm_fb *fb = (struct drm_fb*)calloc(1, sizeof(*fb));
 
-   fb     = (struct drm_fb*)calloc(1, sizeof(*fb));
    fb->bo = bo;
 
    width  = gbm_bo_get_width(bo);
@@ -142,10 +138,9 @@ static void gfx_ctx_drm_swap_interval(void *data, unsigned interval)
 }
 
 static void gfx_ctx_drm_check_window(void *data, bool *quit,
-      bool *resize, unsigned *width, unsigned *height, unsigned frame_count)
+      bool *resize, unsigned *width, unsigned *height, bool is_shutdown)
 {
    (void)data;
-   (void)frame_count;
    (void)width;
    (void)height;
 
@@ -157,12 +152,13 @@ static void gfx_ctx_drm_check_window(void *data, bool *quit,
 static void drm_flip_handler(int fd, unsigned frame,
       unsigned sec, unsigned usec, void *data)
 {
-   static unsigned first_page_flip;
-   static unsigned last_page_flip;
-
    (void)fd;
    (void)sec;
    (void)usec;
+  
+#if 0
+   static unsigned first_page_flip;
+   static unsigned last_page_flip;
 
    if (!first_page_flip)
       first_page_flip = frame;
@@ -176,6 +172,8 @@ static void drm_flip_handler(int fd, unsigned frame,
    }
 
    last_page_flip = frame;
+#endif
+
    *(bool*)data = false;
 }
 
@@ -211,8 +209,12 @@ static bool gfx_ctx_drm_wait_flip(bool block)
 static bool gfx_ctx_drm_queue_flip(void)
 {
    struct drm_fb *fb = NULL;
+
    g_next_bo         = gbm_surface_lock_front_buffer(g_gbm_surface);
-   fb                = (struct drm_fb*)drm_fb_get_from_bo(g_next_bo);
+   fb                = (struct drm_fb*)gbm_bo_get_user_data(g_next_bo);
+
+   if (!fb)
+      fb             = (struct drm_fb*)drm_fb_get_from_bo(g_next_bo);
 
    if (drmModePageFlip(g_drm_fd, g_crtc_id, fb->fb_id,
          DRM_MODE_PAGE_FLIP_EVENT, &waiting_for_flip) == 0)
@@ -222,10 +224,9 @@ static bool gfx_ctx_drm_queue_flip(void)
    return false;
 }
 
-static void gfx_ctx_drm_swap_buffers(void *data)
+static void gfx_ctx_drm_swap_buffers(void *data, video_frame_info_t *video_info)
 {
    gfx_ctx_drm_data_t *drm = (gfx_ctx_drm_data_t*)data;
-   settings_t    *settings = config_get_ptr();
 
    switch (drm_api)
    {
@@ -251,35 +252,13 @@ static void gfx_ctx_drm_swap_buffers(void *data)
    waiting_for_flip = gfx_ctx_drm_queue_flip();
 
    /* Triple-buffered page flips */
-   if (settings->video.max_swapchain_images >= 3 &&
+   if (video_info->max_swapchain_images >= 3 &&
          gbm_surface_has_free_buffers(g_gbm_surface))
       return;
 
    gfx_ctx_drm_wait_flip(true);  
 }
 
-static bool gfx_ctx_drm_set_resize(void *data,
-      unsigned width, unsigned height)
-{
-   (void)data;
-   (void)width;
-   (void)height;
-
-   return false;
-}
-
-static void gfx_ctx_drm_update_window_title(void *data)
-{
-   char buf[128];
-   char buf_fps[128];
-   settings_t *settings = config_get_ptr();
-
-   video_monitor_get_fps(buf, sizeof(buf),
-         buf_fps, sizeof(buf_fps));
-   if (settings->fps_show)
-      runloop_msg_queue_push( buf_fps, 1, 1, false);
-}
-
 static void gfx_ctx_drm_get_video_size(void *data,
       unsigned *width, unsigned *height)
 {
@@ -297,6 +276,9 @@ static void free_drm_resources(gfx_ctx_drm_data_t *drm)
    if (!drm)
       return;
 
+   /* Restore original CRTC. */
+   drm_restore_crtc();
+
    if (g_gbm_surface)
       gbm_surface_destroy(g_gbm_surface);
 
@@ -307,7 +289,10 @@ static void free_drm_resources(gfx_ctx_drm_data_t *drm)
 
    if (drm->drm)
       if (g_drm_fd >= 0)
+      {
+         drmDropMaster(g_drm_fd);
          filestream_close(drm->drm);
+      }
 
    g_gbm_surface      = NULL;
    g_gbm_dev          = NULL;
@@ -336,8 +321,6 @@ static void gfx_ctx_drm_destroy_resources(gfx_ctx_drm_data_t *drm)
          break;
    }
 
-   /* Restore original CRTC. */
-   drm_restore_crtc();
    free_drm_resources(drm);
 
    g_drm_mode          = NULL;
@@ -351,24 +334,22 @@ static void gfx_ctx_drm_destroy_resources(gfx_ctx_drm_data_t *drm)
    g_next_bo           = NULL;
 }
 
-static void *gfx_ctx_drm_init(void *video_driver)
+static void *gfx_ctx_drm_init(video_frame_info_t *video_info, void *video_driver)
 {
    int fd, i;
    unsigned monitor_index;
    unsigned gpu_index                   = 0;
    const char *gpu                      = NULL;
    struct string_list *gpu_descriptors  = NULL;
-   gfx_ctx_drm_data_t *drm          = (gfx_ctx_drm_data_t*)
+   gfx_ctx_drm_data_t *drm              = (gfx_ctx_drm_data_t*)
       calloc(1, sizeof(gfx_ctx_drm_data_t));
 
    if (!drm)
       return NULL;
 
-   fd   = -1;
-   gpu_descriptors = dir_list_new("/dev/dri", NULL, false, false);
+   gpu_descriptors = dir_list_new("/dev/dri", NULL, false, true, false, false);
 
 nextgpu:
-   drm_restore_crtc();
    free_drm_resources(drm);
 
    if (!gpu_descriptors || gpu_index == gpu_descriptors->size)
@@ -390,7 +371,7 @@ nextgpu:
    if (!drm_get_resources(fd))
       goto nextgpu;
 
-   if (!drm_get_connector(fd))
+   if (!drm_get_connector(fd, video_info))
       goto nextgpu;
 
    if (!drm_get_encoder(fd))
@@ -403,6 +384,8 @@ nextgpu:
    drm->fb_width    = g_drm_connector->modes[0].hdisplay;
    drm->fb_height   = g_drm_connector->modes[0].vdisplay;
 
+   drmSetMaster(g_drm_fd);
+
    g_gbm_dev        = gbm_create_device(fd);
 
    if (!g_gbm_dev)
@@ -614,13 +597,13 @@ error:
 #endif
 
 static bool gfx_ctx_drm_set_video_mode(void *data,
+      video_frame_info_t *video_info,
       unsigned width, unsigned height,
       bool fullscreen)
 {
    float refresh_mod;
    int i, ret                  = 0;
    struct drm_fb *fb           = NULL;
-   settings_t *settings        = config_get_ptr();
    gfx_ctx_drm_data_t *drm     = (gfx_ctx_drm_data_t*)data;
 
    if (!drm)
@@ -631,7 +614,7 @@ static bool gfx_ctx_drm_set_video_mode(void *data,
    /* If we use black frame insertion, 
     * we fake a 60 Hz monitor for 120 Hz one, 
     * etc, so try to match that. */
-   refresh_mod = settings->video.black_frame_insertion 
+   refresh_mod = video_info->black_frame_insertion 
       ? 0.5f : 1.0f;
 
    /* Find desired video mode, and use that.
@@ -641,8 +624,7 @@ static bool gfx_ctx_drm_set_video_mode(void *data,
       g_drm_mode = &g_drm_connector->modes[0];
    else
    {
-      /* Try to match settings->video.refresh_rate 
-       * as closely as possible.
+      /* Try to match refresh_rate as closely as possible.
        *
        * Lower resolutions tend to have multiple supported 
        * refresh rates as well.
@@ -658,7 +640,7 @@ static bool gfx_ctx_drm_set_video_mode(void *data,
             continue;
 
          diff = fabsf(refresh_mod * g_drm_connector->modes[i].vrefresh
-               - settings->video.refresh_rate);
+               - video_info->refresh_rate);
 
          if (!g_drm_mode || diff < minimum_fps_diff)
          {
@@ -709,9 +691,13 @@ static bool gfx_ctx_drm_set_video_mode(void *data,
    }
 
    g_bo = gbm_surface_lock_front_buffer(g_gbm_surface);
-   fb   = drm_fb_get_from_bo(g_bo);
 
-   ret  = drmModeSetCrtc(g_drm_fd,
+   fb = (struct drm_fb*)gbm_bo_get_user_data(g_bo);
+
+   if (!fb)
+      fb   = drm_fb_get_from_bo(g_bo);
+
+   ret     = drmModeSetCrtc(g_drm_fd,
          g_crtc_id, fb->fb_id, 0, 0, &g_connector_id, 1, g_drm_mode);
    if (ret < 0)
       goto error;
@@ -740,10 +726,10 @@ static void gfx_ctx_drm_destroy(void *data)
 }
 
 static void gfx_ctx_drm_input_driver(void *data,
+      const char *name,
       const input_driver_t **input, void **input_data)
 {
-   (void)data;
-   *input = NULL;
+   *input      = NULL;
    *input_data = NULL;
 }
 
@@ -759,12 +745,6 @@ static bool gfx_ctx_drm_suppress_screensaver(void *data, bool enable)
    return false;
 }
 
-static bool gfx_ctx_drm_has_windowed(void *data)
-{
-   (void)data;
-   return false;
-}
-
 static bool gfx_ctx_drm_bind_api(void *video_driver,
       enum gfx_ctx_api api, unsigned major, unsigned minor)
 {
@@ -883,12 +863,12 @@ const gfx_ctx_driver_t gfx_ctx_drm = {
    NULL, /* get_video_output_next */
    NULL, /* get_metrics */
    NULL,
-   gfx_ctx_drm_update_window_title,
+   NULL, /* update_window_title */
    gfx_ctx_drm_check_window,
-   gfx_ctx_drm_set_resize,
+   NULL, /* set_resize */
    gfx_ctx_drm_has_focus,
    gfx_ctx_drm_suppress_screensaver,
-   gfx_ctx_drm_has_windowed,
+   NULL, /* has_windowed */
    gfx_ctx_drm_swap_buffers,
    gfx_ctx_drm_input_driver,
    gfx_ctx_drm_get_proc_address,
@@ -898,5 +878,7 @@ const gfx_ctx_driver_t gfx_ctx_drm = {
    "kms",
    gfx_ctx_drm_get_flags,
    gfx_ctx_drm_set_flags,
-   gfx_ctx_drm_bind_hw_render
+   gfx_ctx_drm_bind_hw_render,
+   NULL,
+   NULL
 };
