@@ -1,5 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *  Copyright (C) 2012-2015 - Michael Lelli
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
@@ -24,12 +25,17 @@
 #include <VG/openvg.h>
 #include <bcm_host.h>
 
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
 #include <retro_inline.h>
 
-#include "../../driver.h"
-#include "../../runloop.h"
+#include "../../configuration.h"
 #include "../video_context_driver.h"
 
+#include "../../frontend/frontend_driver.h"
+
 #ifdef HAVE_EGL
 #include "../common/egl_common.h"
 #endif
@@ -80,10 +86,10 @@ static INLINE bool gfx_ctx_vc_egl_query_extension(vc_ctx_data_t *vc, const char
 }
 
 static void gfx_ctx_vc_check_window(void *data, bool *quit,
-      bool *resize, unsigned *width, unsigned *height, unsigned frame_count)
+      bool *resize, unsigned *width, unsigned *height,
+      bool is_shutdown)
 {
    (void)data;
-   (void)frame_count;
    (void)width;
    (void)height;
 
@@ -91,28 +97,6 @@ static void gfx_ctx_vc_check_window(void *data, bool *quit,
    *quit   = (bool)frontend_driver_get_signal_handler_state();
 }
 
-static bool gfx_ctx_vc_set_resize(void *data, unsigned width, unsigned height)
-{
-   (void)data;
-   (void)width;
-   (void)height;
-   return false;
-}
-
-static void gfx_ctx_vc_update_window_title(void *data)
-{
-   char buf[128]        = {0};
-   char buf_fps[128]    = {0};
-   settings_t *settings = config_get_ptr();
-
-   (void)data;
-
-   video_monitor_get_fps(buf, sizeof(buf),
-         buf_fps, sizeof(buf_fps));
-   if (settings->fps_show)
-      runloop_msg_queue_push(buf_fps, 1, 1, false);
-}
-
 static void gfx_ctx_vc_get_video_size(void *data,
       unsigned *width, unsigned *height)
 {
@@ -152,7 +136,7 @@ static void gfx_ctx_vc_get_video_size(void *data,
 
 static void gfx_ctx_vc_destroy(void *data);
 
-static void *gfx_ctx_vc_init(void *video_driver)
+static void *gfx_ctx_vc_init(video_frame_info_t *video_info, void *video_driver)
 {
    VC_DISPMANX_ALPHA_T alpha;
    EGLint n, major, minor;
@@ -171,6 +155,7 @@ static void *gfx_ctx_vc_init(void *video_driver)
       EGL_GREEN_SIZE, 8,
       EGL_BLUE_SIZE, 8,
       EGL_ALPHA_SIZE, 8,
+      EGL_DEPTH_SIZE, 16,
       EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
       EGL_NONE
    };
@@ -313,6 +298,7 @@ static void gfx_ctx_vc_set_swap_interval(void *data, unsigned swap_interval)
 }
 
 static bool gfx_ctx_vc_set_video_mode(void *data,
+      video_frame_info_t *video_info,
       unsigned width, unsigned height,
       bool fullscreen)
 {
@@ -444,10 +430,10 @@ static void gfx_ctx_vc_destroy(void *data)
 }
 
 static void gfx_ctx_vc_input_driver(void *data,
+      const char *name,
       const input_driver_t **input, void **input_data)
 {
-   (void)data;
-   *input = NULL;
+   *input      = NULL;
    *input_data = NULL;
 }
 
@@ -464,12 +450,6 @@ static bool gfx_ctx_vc_suppress_screensaver(void *data, bool enable)
    return false;
 }
 
-static bool gfx_ctx_vc_has_windowed(void *data)
-{
-   (void)data;
-   return false;
-}
-
 static float gfx_ctx_vc_translate_aspect(void *data,
       unsigned width, unsigned height)
 {
@@ -601,7 +581,7 @@ error:
    return false;
 }
 
-static void gfx_ctx_vc_swap_buffers(void *data)
+static void gfx_ctx_vc_swap_buffers(void *data, video_frame_info_t *video_info)
 {
    vc_ctx_data_t *vc = (vc_ctx_data_t*)data;
 
@@ -652,12 +632,12 @@ const gfx_ctx_driver_t gfx_ctx_videocore = {
    NULL, /* get_video_output_next */
    NULL, /* get_metrics */
    gfx_ctx_vc_translate_aspect,
-   gfx_ctx_vc_update_window_title,
+   NULL, /* update_title */
    gfx_ctx_vc_check_window,
-   gfx_ctx_vc_set_resize,
+   NULL, /* set_resize */
    gfx_ctx_vc_has_focus,
    gfx_ctx_vc_suppress_screensaver,
-   gfx_ctx_vc_has_windowed,
+   NULL, /* has_windowed */
    gfx_ctx_vc_swap_buffers,
    gfx_ctx_vc_input_driver,
    gfx_ctx_vc_get_proc_address,
@@ -667,5 +647,8 @@ const gfx_ctx_driver_t gfx_ctx_videocore = {
    "videocore",
    gfx_ctx_vc_get_flags,
    gfx_ctx_vc_set_flags,
-   gfx_ctx_vc_bind_hw_render
+   gfx_ctx_vc_bind_hw_render,
+   NULL,
+   NULL
+
 };
