@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2013-2014 - Jason Fetters
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -16,6 +16,7 @@
 
 #include <stdio.h>
 #include <stdint.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/sysctl.h>
 #include <string.h>
@@ -31,6 +32,7 @@
 #include <dynamic/dylib.h>
 #endif
 
+#include "../input_defines.h"
 #include "../input_hid_driver.h"
 #define BUILDING_BTDYNAMIC
 #include "../connect/joypad_connection.h"
@@ -1370,9 +1372,6 @@ static bool btstack_hid_joypad_button(void *data, unsigned port, uint16_t joykey
 {
    uint64_t buttons          = btstack_hid_joypad_get_buttons(data, port);
 
-   if (joykey == NO_BTN)
-      return false;
-
    /* Check hat. */
    if (GET_HAT_DIR(joykey))
       return false;
