@@ -14,6 +14,7 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <stdarg.h>
 #include <string.h>
 #include <ctype.h>
 
@@ -31,6 +32,14 @@
 #include "config.h"
 #endif
 
+#ifdef HAVE_CHEEVOS
+#include "cheevos.h"
+#endif
+
+#ifdef HAVE_NETWORKING
+#include "network/netplay/netplay.h"
+#endif
+
 #include "dynamic.h"
 #include "command.h"
 
@@ -39,19 +48,18 @@
 #include "location/location_driver.h"
 #include "record/record_driver.h"
 #include "core.h"
+#include "driver.h"
 #include "performance_counters.h"
-#include "system.h"
 #include "gfx/video_context_driver.h"
 
 #include "cores/internal_cores.h"
 #include "frontend/frontend_driver.h"
 #include "content.h"
-#ifdef HAVE_CHEEVOS
-#include "cheevos.h"
-#endif
+#include "dirs.h"
+#include "paths.h"
 #include "retroarch.h"
+#include "runloop.h"
 #include "configuration.h"
-#include "general.h"
 #include "msg_hash.h"
 #include "verbosity.h"
 
@@ -77,11 +85,16 @@ static dylib_t lib_handle;
 #define SYMBOL_IMAGEVIEWER(x) current_core->x = libretro_imageviewer_##x
 #endif
 
-#if defined(HAVE_NETWORKGAMEPAD) && defined(HAVE_NETPLAY)
+#if defined(HAVE_NETWORKING) && defined(HAVE_NETWORKGAMEPAD)
 #define SYMBOL_NETRETROPAD(x) current_core->x = libretro_netretropad_##x
 #endif
 
-static bool ignore_environment_cb;
+#if defined(HAVE_VIDEO_PROCESSOR)
+#define SYMBOL_VIDEOPROCESSOR(x) current_core->x = libretro_videoprocessor_##x
+#endif
+
+static bool ignore_environment_cb = false;
+static bool *load_no_content_hook = NULL;
 
 const struct retro_subsystem_info *libretro_find_subsystem_info(
       const struct retro_subsystem_info *info, unsigned num_info,
@@ -144,7 +157,6 @@ void libretro_free_system_info(struct retro_system_info *info)
    memset(info, 0, sizeof(*info));
 }
 
-static bool *load_no_content_hook;
 
 static bool environ_cb_get_system_info(unsigned cmd, void *data)
 {
@@ -161,41 +173,6 @@ static bool environ_cb_get_system_info(unsigned cmd, void *data)
    return true;
 }
 
-#ifndef HAVE_DYNAMIC
-bool libretro_get_system_info_static(struct retro_system_info *info,
-      bool *load_no_content)
-{
-   struct retro_system_info dummy_info = {0};
-
-   if (load_no_content)
-   {
-      load_no_content_hook = load_no_content;
-
-      /* load_no_content gets set in this callback. */
-      retro_set_environment(environ_cb_get_system_info);
-
-      /* It's possible that we just set get_system_info callback 
-       * to the currently running core.
-       *
-       * Make sure we reset it to the actual environment callback.
-       * Ignore any environment callbacks here in case we're running 
-       * on the non-current core. */
-      ignore_environment_cb = true;
-      retro_set_environment(rarch_environment_cb);
-      ignore_environment_cb = false;
-   }
-
-   retro_get_system_info(&dummy_info);
-   memcpy(info, &dummy_info, sizeof(*info));
-
-   info->library_name    = strdup(dummy_info.library_name);
-   info->library_version = strdup(dummy_info.library_version);
-   if (dummy_info.valid_extensions)
-      info->valid_extensions = strdup(dummy_info.valid_extensions);
-   return true;
-}
-#endif
-
 #ifdef HAVE_DYNAMIC
 /**
  * libretro_get_environment_info:
@@ -233,6 +210,53 @@ void libretro_get_environment_info(void (*func)(retro_environment_t),
    ignore_environment_cb = false;
 }
 
+static bool load_dynamic_core(void)
+{
+   function_t sym       = dylib_proc(NULL, "retro_init");
+
+   if (sym)
+   {
+      /* Try to verify that -lretro was not linked in from other modules
+       * since loading it dynamically and with -l will fail hard. */
+      RARCH_ERR("Serious problem. RetroArch wants to load libretro cores"
+            " dynamically, but it is already linked.\n");
+      RARCH_ERR("This could happen if other modules RetroArch depends on "
+            "link against libretro directly.\n");
+      RARCH_ERR("Proceeding could cause a crash. Aborting ...\n");
+      retroarch_fail(1, "init_libretro_sym()");
+   }
+
+   if (string_is_empty(path_get(RARCH_PATH_CORE)))
+   {
+      RARCH_ERR("RetroArch is built for dynamic libretro cores, but "
+            "libretro_path is not set. Cannot continue.\n");
+      retroarch_fail(1, "init_libretro_sym()");
+   }
+
+   /* Need to use absolute path for this setting. It can be
+    * saved to content history, and a relative path would
+    * break in that scenario. */
+   path_resolve_realpath(
+         path_get_ptr(RARCH_PATH_CORE),
+         path_get_realsize(RARCH_PATH_CORE));
+
+   RARCH_LOG("Loading dynamic libretro core from: \"%s\"\n",
+         path_get(RARCH_PATH_CORE));
+   lib_handle = dylib_load(path_get(RARCH_PATH_CORE));
+
+   if (lib_handle)
+      return true;
+
+   RARCH_ERR("Failed to open libretro core: \"%s\"\n",
+         path_get(RARCH_PATH_CORE));
+   RARCH_ERR("Error(s): %s\n", dylib_error());
+
+   runloop_msg_queue_push(msg_hash_to_str(MSG_FAILED_TO_OPEN_LIBRETRO_CORE), 1, 180, true);
+   retroarch_fail(1, "load_dynamic_core()");
+
+   return false;
+}
+
 static dylib_t libretro_get_system_info_lib(const char *path,
       struct retro_system_info *info, bool *load_no_content)
 {
@@ -274,6 +298,43 @@ static dylib_t libretro_get_system_info_lib(const char *path,
 
    return lib;
 }
+#else
+static bool libretro_get_system_info_static(struct retro_system_info *info,
+      bool *load_no_content)
+{
+   struct retro_system_info dummy_info = {0};
+
+   if (load_no_content)
+   {
+      load_no_content_hook = load_no_content;
+
+      /* load_no_content gets set in this callback. */
+      retro_set_environment(environ_cb_get_system_info);
+
+      /* It's possible that we just set get_system_info callback 
+       * to the currently running core.
+       *
+       * Make sure we reset it to the actual environment callback.
+       * Ignore any environment callbacks here in case we're running 
+       * on the non-current core. */
+      ignore_environment_cb = true;
+      retro_set_environment(rarch_environment_cb);
+      ignore_environment_cb = false;
+   }
+
+   retro_get_system_info(&dummy_info);
+   memcpy(info, &dummy_info, sizeof(*info));
+
+   if (!string_is_empty(dummy_info.library_name))
+      info->library_name    = strdup(dummy_info.library_name);
+   if (!string_is_empty(dummy_info.library_version))
+      info->library_version    = strdup(dummy_info.library_version);
+
+   if (dummy_info.valid_extensions)
+      info->valid_extensions = strdup(dummy_info.valid_extensions);
+   return true;
+}
+#endif
 
 /**
  * libretro_get_system_info:
@@ -290,62 +351,31 @@ static dylib_t libretro_get_system_info_lib(const char *path,
 bool libretro_get_system_info(const char *path,
       struct retro_system_info *info, bool *load_no_content)
 {
+#ifdef HAVE_DYNAMIC
    struct retro_system_info dummy_info = {0};
-   dylib_t lib = libretro_get_system_info_lib(path,
-         &dummy_info, load_no_content);
+   dylib_t lib                         = libretro_get_system_info_lib(
+         path, &dummy_info, load_no_content);
+
    if (!lib)
       return false;
 
    memcpy(info, &dummy_info, sizeof(*info));
-   info->library_name    = strdup(dummy_info.library_name);
-   info->library_version = strdup(dummy_info.library_version);
+   if (!string_is_empty(dummy_info.library_name))
+      info->library_name        = strdup(dummy_info.library_name);
+   if (!string_is_empty(dummy_info.library_version))
+      info->library_version     = strdup(dummy_info.library_version);
+
    if (dummy_info.valid_extensions)
-      info->valid_extensions = strdup(dummy_info.valid_extensions);
+      info->valid_extensions    = strdup(dummy_info.valid_extensions);
+
    dylib_close(lib);
+#else
+   if (!libretro_get_system_info_static(info, load_no_content))
+      return false;
+#endif
    return true;
 }
 
-static void load_dynamic_core(void)
-{
-   settings_t *settings = config_get_ptr();
-   function_t sym       = dylib_proc(NULL, "retro_init");
-
-   if (sym)
-   {
-      /* Try to verify that -lretro was not linked in from other modules
-       * since loading it dynamically and with -l will fail hard. */
-      RARCH_ERR("Serious problem. RetroArch wants to load libretro cores"
-            "dyamically, but it is already linked.\n");
-      RARCH_ERR("This could happen if other modules RetroArch depends on "
-            "link against libretro directly.\n");
-      RARCH_ERR("Proceeding could cause a crash. Aborting ...\n");
-      retroarch_fail(1, "init_libretro_sym()");
-   }
-
-   if (!*settings->path.libretro)
-   {
-      RARCH_ERR("RetroArch is built for dynamic libretro cores, but "
-            "libretro_path is not set. Cannot continue.\n");
-      retroarch_fail(1, "init_libretro_sym()");
-   }
-
-   /* Need to use absolute path for this setting. It can be
-    * saved to content history, and a relative path would
-    * break in that scenario. */
-   path_resolve_realpath(settings->path.libretro, sizeof(settings->path.libretro));
-
-   RARCH_LOG("Loading dynamic libretro core from: \"%s\"\n",
-         settings->path.libretro);
-   lib_handle = dylib_load(settings->path.libretro);
-   if (!lib_handle)
-   {
-      RARCH_ERR("Failed to open libretro core: \"%s\"\n",
-            settings->path.libretro);
-      RARCH_ERR("Error(s): %s\n", dylib_error());
-      retroarch_fail(1, "load_dynamic()");
-   }
-}
-#endif
 
 /**
  * load_symbols:
@@ -353,15 +383,17 @@ static void load_dynamic_core(void)
  *                                If CORE_TYPE_DUMMY, will
  *                                load dummy symbols.
  *
- * Setup libretro callback symbols.
+ * Setup libretro callback symbols. Returns true on success,
+ * or false if symbols could not be loaded.
  **/
-static void load_symbols(enum rarch_core_type type, struct retro_core_t *current_core)
+static bool load_symbols(enum rarch_core_type type, struct retro_core_t *current_core)
 {
    switch (type)
    {
       case CORE_TYPE_PLAIN:
 #ifdef HAVE_DYNAMIC
-         load_dynamic_core();
+         if (!load_dynamic_core())
+            return false;
 #endif
 
          SYMBOL(retro_init);
@@ -508,7 +540,7 @@ static void load_symbols(enum rarch_core_type type, struct retro_core_t *current
 #endif
          break;
       case CORE_TYPE_NETRETROPAD:
-#if defined(HAVE_NETWORKGAMEPAD) && defined(HAVE_NETPLAY)
+#if defined(HAVE_NETWORKING) && defined(HAVE_NETWORKGAMEPAD)
          SYMBOL_NETRETROPAD(retro_init);
          SYMBOL_NETRETROPAD(retro_deinit);
 
@@ -542,9 +574,48 @@ static void load_symbols(enum rarch_core_type type, struct retro_core_t *current
          SYMBOL_NETRETROPAD(retro_get_region);
          SYMBOL_NETRETROPAD(retro_get_memory_data);
          SYMBOL_NETRETROPAD(retro_get_memory_size);
+#endif
+         break;
+      case CORE_TYPE_VIDEO_PROCESSOR:
+#if defined(HAVE_VIDEO_PROCESSOR)
+         SYMBOL_VIDEOPROCESSOR(retro_init);
+         SYMBOL_VIDEOPROCESSOR(retro_deinit);
+
+         SYMBOL_VIDEOPROCESSOR(retro_api_version);
+         SYMBOL_VIDEOPROCESSOR(retro_get_system_info);
+         SYMBOL_VIDEOPROCESSOR(retro_get_system_av_info);
+
+         SYMBOL_VIDEOPROCESSOR(retro_set_environment);
+         SYMBOL_VIDEOPROCESSOR(retro_set_video_refresh);
+         SYMBOL_VIDEOPROCESSOR(retro_set_audio_sample);
+         SYMBOL_VIDEOPROCESSOR(retro_set_audio_sample_batch);
+         SYMBOL_VIDEOPROCESSOR(retro_set_input_poll);
+         SYMBOL_VIDEOPROCESSOR(retro_set_input_state);
+
+         SYMBOL_VIDEOPROCESSOR(retro_set_controller_port_device);
+
+         SYMBOL_VIDEOPROCESSOR(retro_reset);
+         SYMBOL_VIDEOPROCESSOR(retro_run);
+
+         SYMBOL_VIDEOPROCESSOR(retro_serialize_size);
+         SYMBOL_VIDEOPROCESSOR(retro_serialize);
+         SYMBOL_VIDEOPROCESSOR(retro_unserialize);
+
+         SYMBOL_VIDEOPROCESSOR(retro_cheat_reset);
+         SYMBOL_VIDEOPROCESSOR(retro_cheat_set);
+
+         SYMBOL_VIDEOPROCESSOR(retro_load_game);
+         SYMBOL_VIDEOPROCESSOR(retro_load_game_special);
+
+         SYMBOL_VIDEOPROCESSOR(retro_unload_game);
+         SYMBOL_VIDEOPROCESSOR(retro_get_region);
+         SYMBOL_VIDEOPROCESSOR(retro_get_memory_data);
+         SYMBOL_VIDEOPROCESSOR(retro_get_memory_size);
 #endif
          break;
    }
+
+   return true;
 }
 
 /**
@@ -594,15 +665,19 @@ void libretro_get_current_core_pathname(char *name, size_t size)
  *                                load dummy symbols.
  *
  * Initializes libretro symbols and
- * setups environment callback functions.
+ * setups environment callback functions. Returns true on success,
+ * or false if symbols could not be loaded.
  **/
-void init_libretro_sym(enum rarch_core_type type, struct retro_core_t *current_core)
+bool init_libretro_sym(enum rarch_core_type type, struct retro_core_t *current_core)
 {
    /* Guarantee that we can do "dirty" casting.
     * Every OS that this program supports should pass this. */
    retro_assert(sizeof(void*) == sizeof(void (*)(void)));
 
-   load_symbols(type, current_core);
+   if (!load_symbols(type, current_core))
+      return false;
+
+   return true;
 }
 
 /**
@@ -690,9 +765,10 @@ static size_t mmap_inflate(size_t addr, size_t mask)
     while (mask)
    {
       size_t tmp = (mask - 1) & ~mask;
+
       /* to put in an 1 bit instead, OR in tmp+1 */
-      addr = ((addr & ~tmp) << 1) | (addr & tmp);
-      mask = mask & (mask - 1);
+      addr       = ((addr & ~tmp) << 1) | (addr & tmp);
+      mask       = mask & (mask - 1);
    }
    
    return addr;
@@ -703,8 +779,8 @@ static size_t mmap_reduce(size_t addr, size_t mask)
    while (mask)
    {
       size_t tmp = (mask - 1) & ~mask;
-      addr = (addr & tmp) | ((addr >> 1) & ~tmp);
-      mask = (mask & (mask - 1)) >> 1;
+      addr       = (addr & tmp) | ((addr >> 1) & ~tmp);
+      mask       = (mask & (mask - 1)) >> 1;
    }
    
    return addr;
@@ -716,60 +792,181 @@ static size_t mmap_highest_bit(size_t n)
    return n ^ (n >> 1);
 }
 
-static bool mmap_preprocess_descriptors(struct retro_memory_descriptor *first, unsigned count)
+static bool mmap_preprocess_descriptors(rarch_memory_descriptor_t *first, unsigned count)
 {
-   struct retro_memory_descriptor *desc;
-   const struct retro_memory_descriptor *end;
-   size_t top_addr, disconnect_mask;
-   
-   end = first + count;
-   top_addr = 1;
+   size_t                      top_addr = 1;
+   rarch_memory_descriptor_t *desc      = NULL;
+   const rarch_memory_descriptor_t *end = first + count;
    
    for (desc = first; desc < end; desc++)
    {
-      if (desc->select != 0)
-         top_addr |= desc->select;
+      if (desc->core.select != 0)
+         top_addr |= desc->core.select;
       else
-         top_addr |= desc->start + desc->len - 1;
+         top_addr |= desc->core.start + desc->core.len - 1;
    }
    
    top_addr = mmap_add_bits_down(top_addr);
    
    for (desc = first; desc < end; desc++)
    {
-      if (desc->select == 0)
+      if (desc->core.select == 0)
       {
-         if (desc->len == 0)
+         if (desc->core.len == 0)
             return false;
          
-         if ((desc->len & (desc->len - 1)) != 0)
+         if ((desc->core.len & (desc->core.len - 1)) != 0)
             return false;
          
-         desc->select = top_addr & ~mmap_inflate(mmap_add_bits_down(desc->len - 1), desc->disconnect);
+         desc->core.select = top_addr & ~mmap_inflate(mmap_add_bits_down(desc->core.len - 1),
+               desc->core.disconnect);
       }
       
-      if (desc->len == 0)
-         desc->len = mmap_add_bits_down(mmap_reduce(top_addr & ~desc->select, desc->disconnect)) + 1;
+      if (desc->core.len == 0)
+         desc->core.len = mmap_add_bits_down(mmap_reduce(top_addr & ~desc->core.select,
+                  desc->core.disconnect)) + 1;
       
-      if (desc->start & ~desc->select)
+      if (desc->core.start & ~desc->core.select)
          return false;
        
-      while (mmap_reduce(top_addr & ~desc->select, desc->disconnect) >> 1 > desc->len - 1)
-         desc->disconnect |= mmap_highest_bit(top_addr & ~desc->select & ~desc->disconnect);
+      while (mmap_reduce(top_addr & ~desc->core.select, desc->core.disconnect) >> 1 > desc->core.len - 1)
+         desc->core.disconnect |= mmap_highest_bit(top_addr & ~desc->core.select & ~desc->core.disconnect);
       
-      disconnect_mask = mmap_add_bits_down(desc->len - 1);
-      desc->disconnect &= disconnect_mask;
+      desc->disconnect_mask = mmap_add_bits_down(desc->core.len - 1);
+      desc->core.disconnect &= desc->disconnect_mask;
       
-      while ((~disconnect_mask) >> 1 & desc->disconnect)
+      while ((~desc->disconnect_mask) >> 1 & desc->core.disconnect)
       {
-         disconnect_mask >>= 1;
-         desc->disconnect &= disconnect_mask;
+         desc->disconnect_mask >>= 1;
+         desc->core.disconnect &= desc->disconnect_mask;
       }
    }
    
    return true;
 }
 
+static bool dynamic_request_hw_context(enum retro_hw_context_type type,
+      unsigned minor, unsigned major)
+{
+   switch (type)
+   {
+      case RETRO_HW_CONTEXT_NONE:
+         RARCH_LOG("Requesting no HW context.\n");
+         break;
+
+      case RETRO_HW_CONTEXT_VULKAN:
+#ifdef HAVE_VULKAN
+         RARCH_LOG("Requesting Vulkan context.\n");
+         break;
+#else
+         RARCH_ERR("Requesting Vulkan context, but RetroArch is not compiled against Vulkan. Cannot use HW context.\n");
+         return false;
+#endif
+
+#if defined(HAVE_OPENGLES)
+
+#if (defined(HAVE_OPENGLES2) || defined(HAVE_OPENGLES3))
+      case RETRO_HW_CONTEXT_OPENGLES2:
+      case RETRO_HW_CONTEXT_OPENGLES3:
+         RARCH_LOG("Requesting OpenGLES%u context.\n",
+               type == RETRO_HW_CONTEXT_OPENGLES2 ? 2 : 3);
+         break;
+
+#if defined(HAVE_OPENGLES3)
+      case RETRO_HW_CONTEXT_OPENGLES_VERSION:
+         RARCH_LOG("Requesting OpenGLES%u.%u context.\n",
+               major, minor);
+         break;
+#endif
+
+#endif
+      case RETRO_HW_CONTEXT_OPENGL:
+      case RETRO_HW_CONTEXT_OPENGL_CORE:
+         RARCH_ERR("Requesting OpenGL context, but RetroArch "
+               "is compiled against OpenGLES. Cannot use HW context.\n");
+         return false;
+
+#elif defined(HAVE_OPENGL)
+      case RETRO_HW_CONTEXT_OPENGLES2:
+      case RETRO_HW_CONTEXT_OPENGLES3:
+         RARCH_ERR("Requesting OpenGLES%u context, but RetroArch "
+               "is compiled against OpenGL. Cannot use HW context.\n",
+               type == RETRO_HW_CONTEXT_OPENGLES2 ? 2 : 3);
+         return false;
+
+      case RETRO_HW_CONTEXT_OPENGLES_VERSION:
+         RARCH_ERR("Requesting OpenGLES%u.%u context, but RetroArch "
+               "is compiled against OpenGL. Cannot use HW context.\n",
+               major, minor);
+         return false;
+
+      case RETRO_HW_CONTEXT_OPENGL:
+         RARCH_LOG("Requesting OpenGL context.\n");
+         break;
+
+      case RETRO_HW_CONTEXT_OPENGL_CORE:
+         {
+            gfx_ctx_flags_t flags;
+            flags.flags = 0;
+            BIT32_SET(flags.flags, GFX_CTX_FLAGS_GL_CORE_CONTEXT);
+
+            video_context_driver_set_flags(&flags);
+
+            RARCH_LOG("Requesting core OpenGL context (%u.%u).\n",
+                  major, minor);
+         }
+         break;
+#endif
+
+      default:
+         RARCH_LOG("Requesting unknown context.\n");
+         return false;
+   }
+
+   return true;
+}
+
+static bool dynamic_verify_hw_context(enum retro_hw_context_type type,
+      unsigned minor, unsigned major)
+{
+   const char *video_ident = video_driver_get_ident();
+
+   switch (type)
+   {
+      case RETRO_HW_CONTEXT_VULKAN:
+         if (!string_is_equal(video_ident, "vulkan"))
+            return false;
+         break;
+      case RETRO_HW_CONTEXT_OPENGLES2:
+      case RETRO_HW_CONTEXT_OPENGLES3:
+      case RETRO_HW_CONTEXT_OPENGLES_VERSION:
+      case RETRO_HW_CONTEXT_OPENGL:
+      case RETRO_HW_CONTEXT_OPENGL_CORE:
+         if (!string_is_equal(video_ident, "gl"))
+            return false;
+         break;
+      default:
+         break;
+   }
+
+   return true;
+}
+
+static void core_performance_counter_start(struct retro_perf_counter *perf)
+{
+   if (runloop_ctl(RUNLOOP_CTL_IS_PERFCNT_ENABLE, NULL))
+   {
+      perf->call_cnt++;
+      perf->start      = cpu_features_get_perf_counter();
+   }
+}
+
+static void core_performance_counter_stop(struct retro_perf_counter *perf)
+{
+   if (runloop_ctl(RUNLOOP_CTL_IS_PERFCNT_ENABLE, NULL))
+      perf->total += cpu_features_get_perf_counter() - perf->start;
+}
+
 /**
  * rarch_environment_cb:
  * @cmd                          : Identifier of command.
@@ -784,7 +981,6 @@ bool rarch_environment_cb(unsigned cmd, void *data)
 {
    unsigned p;
    settings_t         *settings = config_get_ptr();
-   global_t         *global     = global_get_ptr();
    rarch_system_info_t *system  = NULL;
    
    runloop_ctl(RUNLOOP_CTL_SYSTEM_INFO_GET, &system);
@@ -810,7 +1006,8 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          {
             struct retro_variable *var = (struct retro_variable*)data;
 
-            if (var) {
+            if (var)
+            {
                RARCH_LOG("Environ GET_VARIABLE %s: not implemented.\n", var->key);
                var->value = NULL;
             }
@@ -834,7 +1031,7 @@ bool rarch_environment_cb(unsigned cmd, void *data)
       {
          const struct retro_message *msg = (const struct retro_message*)data;
          RARCH_LOG("Environ SET_MESSAGE: %s\n", msg->msg);
-         runloop_msg_queue_push(msg->msg, 1, msg->frames, true);
+         runloop_msg_queue_push(msg->msg, 3, msg->frames, true);
          break;
       }
 
@@ -871,19 +1068,22 @@ bool rarch_environment_cb(unsigned cmd, void *data)
       case RETRO_ENVIRONMENT_GET_SYSTEM_DIRECTORY:
          if (string_is_empty(settings->directory.system))
          {
-            char *fullpath = NULL;
-            if (runloop_ctl(RUNLOOP_CTL_GET_CONTENT_PATH, &fullpath) &&
-                  fullpath)
+            const char *fullpath = path_get(RARCH_PATH_CONTENT);
+            if (!string_is_empty(fullpath))
             {
+               char temp_path[PATH_MAX_LENGTH];
+
+               temp_path[0] = '\0';
+
                RARCH_WARN("SYSTEM DIR is empty, assume CONTENT DIR %s\n",
                      fullpath);
-               fill_pathname_basedir(global->dir.systemdir, fullpath,
-                     sizeof(global->dir.systemdir));
+               fill_pathname_basedir(temp_path, fullpath, sizeof(temp_path));
+               dir_set(RARCH_DIR_SYSTEM, temp_path);
             }
 
-            *(const char**)data = global->dir.systemdir;
+            *(const char**)data = dir_get_ptr(RARCH_DIR_SYSTEM);
             RARCH_LOG("Environ SYSTEM_DIRECTORY: \"%s\".\n",
-                  global->dir.systemdir);
+                  dir_get(RARCH_DIR_SYSTEM));
          }
          else
          {
@@ -895,7 +1095,7 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          break;
 
       case RETRO_ENVIRONMENT_GET_SAVE_DIRECTORY:
-         *(const char**)data = retroarch_get_current_savefile_dir();
+         *(const char**)data = dir_get(RARCH_DIR_CURRENT_SAVEFILE);
          break;
 
       case RETRO_ENVIRONMENT_GET_USERNAME:
@@ -1068,87 +1268,19 @@ bool rarch_environment_cb(unsigned cmd, void *data)
       case RETRO_ENVIRONMENT_SET_HW_RENDER:
       case RETRO_ENVIRONMENT_SET_HW_RENDER | RETRO_ENVIRONMENT_EXPERIMENTAL:
       {
-         struct retro_hw_render_callback *hwr = NULL;
          struct retro_hw_render_callback *cb =
             (struct retro_hw_render_callback*)data;
-
-         hwr = video_driver_get_hw_context();
+         struct retro_hw_render_callback *hwr = 
+            video_driver_get_hw_context();
 
          RARCH_LOG("Environ SET_HW_RENDER.\n");
 
-         switch (cb->context_type)
-         {
-            case RETRO_HW_CONTEXT_NONE:
-               RARCH_LOG("Requesting no HW context.\n");
-               break;
+         if (!dynamic_request_hw_context(cb->context_type, cb->version_minor, cb->version_major))
+            return false;
 
-#ifdef HAVE_VULKAN
-            case RETRO_HW_CONTEXT_VULKAN:
-               RARCH_LOG("Requesting Vulkan context.\n");
-               break;
-#else
-            case RETRO_HW_CONTEXT_VULKAN:
-               RARCH_ERR("Requesting Vulkan context, but RetroArch is not compiled against Vulkan. Cannot use HW context.\n");
-               return false;
-#endif
+         if (!dynamic_verify_hw_context(cb->context_type, cb->version_minor, cb->version_major))
+            return false;
 
-#if defined(HAVE_OPENGLES2)
-            case RETRO_HW_CONTEXT_OPENGLES2:
-#if defined(HAVE_OPENGLES3)
-            case RETRO_HW_CONTEXT_OPENGLES3:
-#endif
-               RARCH_LOG("Requesting OpenGLES%u context.\n",
-                     cb->context_type == RETRO_HW_CONTEXT_OPENGLES2 ? 2 : 3);
-               break;
-
-#if defined(HAVE_OPENGLES3)
-            case RETRO_HW_CONTEXT_OPENGLES_VERSION:
-               RARCH_LOG("Requesting OpenGLES%u.%u context.\n",
-                     cb->version_major, cb->version_minor);
-               break;
-#endif
-
-            case RETRO_HW_CONTEXT_OPENGL:
-            case RETRO_HW_CONTEXT_OPENGL_CORE:
-               RARCH_ERR("Requesting OpenGL context, but RetroArch "
-                     "is compiled against OpenGLES2. Cannot use HW context.\n");
-               return false;
-#elif defined(HAVE_OPENGL)
-            case RETRO_HW_CONTEXT_OPENGLES2:
-            case RETRO_HW_CONTEXT_OPENGLES3:
-               RARCH_ERR("Requesting OpenGLES%u context, but RetroArch "
-                     "is compiled against OpenGL. Cannot use HW context.\n",
-                     cb->context_type == RETRO_HW_CONTEXT_OPENGLES2 ? 2 : 3);
-               return false;
-
-            case RETRO_HW_CONTEXT_OPENGLES_VERSION:
-               RARCH_ERR("Requesting OpenGLES%u.%u context, but RetroArch "
-                     "is compiled against OpenGL. Cannot use HW context.\n",
-                     cb->version_major, cb->version_minor);
-               return false;
-
-            case RETRO_HW_CONTEXT_OPENGL:
-               RARCH_LOG("Requesting OpenGL context.\n");
-               break;
-
-            case RETRO_HW_CONTEXT_OPENGL_CORE:
-               {
-                  gfx_ctx_flags_t flags;
-                  flags.flags = 0;
-                  BIT32_SET(flags.flags, GFX_CTX_FLAGS_GL_CORE_CONTEXT);
-
-                  video_context_driver_set_flags(&flags);
-
-                  RARCH_LOG("Requesting core OpenGL context (%u.%u).\n",
-                        cb->version_major, cb->version_minor);
-               }
-               break;
-#endif
-
-            default:
-               RARCH_LOG("Requesting unknown context.\n");
-               return false;
-         }
          cb->get_current_framebuffer = video_driver_get_current_framebuffer;
          cb->get_proc_address        = video_driver_get_proc_address;
 
@@ -1176,17 +1308,24 @@ bool rarch_environment_cb(unsigned cmd, void *data)
       case RETRO_ENVIRONMENT_GET_LIBRETRO_PATH:
       {
          const char **path = (const char**)data;
-         *path = NULL;
 #ifdef HAVE_DYNAMIC
-         *path = settings->path.libretro;
+         *path = path_get(RARCH_PATH_CORE);
+#else
+         *path = NULL;
 #endif
          break;
       }
 
       case RETRO_ENVIRONMENT_SET_AUDIO_CALLBACK:
-#if defined(HAVE_THREADS) && !defined(__CELLOS_LV2__)
+#ifdef HAVE_THREADS
       {
          RARCH_LOG("Environ SET_AUDIO_CALLBACK.\n");
+#ifdef HAVE_NETWORKING
+         if (netplay_driver_ctl(RARCH_NETPLAY_CTL_IS_ENABLED, NULL))
+            return false;
+#endif
+         if (recording_driver_get_data_ptr()) /* A/V sync is a must. */
+            return false;
          audio_driver_set_callback(data);
       }
 #endif
@@ -1286,8 +1425,8 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          cb->get_perf_counter = cpu_features_get_perf_counter;
 
          cb->perf_register    = performance_counter_register; 
-         cb->perf_start       = performance_counter_start;
-         cb->perf_stop        = performance_counter_stop;
+         cb->perf_start       = core_performance_counter_start;
+         cb->perf_stop        = core_performance_counter_stop;
          cb->perf_log         = retro_perf_log;
          break;
       }
@@ -1334,13 +1473,18 @@ bool rarch_environment_cb(unsigned cmd, void *data)
 
          if (system)
          {
+            struct retro_subsystem_info *info_ptr = NULL;
             free(system->subsystem.data);
-            system->subsystem.data = (struct retro_subsystem_info*)
-               calloc(i, sizeof(*system->subsystem.data));
+            system->subsystem.data = NULL;
 
-            if (!system->subsystem.data)
+            info_ptr = (struct retro_subsystem_info*)
+               calloc(i, sizeof(*info_ptr));
+
+            if (!info_ptr)
                return false;
 
+            system->subsystem.data = info_ptr;
+
             memcpy(system->subsystem.data, info,
                   i * sizeof(*system->subsystem.data));
             system->subsystem.size = i;
@@ -1366,12 +1510,16 @@ bool rarch_environment_cb(unsigned cmd, void *data)
 
          if (system)
          {
+            struct retro_controller_info *info_ptr = NULL;
+
             free(system->ports.data);
-            system->ports.data = (struct retro_controller_info*)
-               calloc(i, sizeof(*system->ports.data));
-            if (!system->ports.data)
+            system->ports.data = NULL;
+
+            info_ptr = (struct retro_controller_info*)calloc(i, sizeof(*info_ptr));
+            if (!info_ptr)
                return false;
 
+            system->ports.data = info_ptr;
             memcpy(system->ports.data, info,
                   i * sizeof(*system->ports.data));
             system->ports.size = i;
@@ -1386,22 +1534,24 @@ bool rarch_environment_cb(unsigned cmd, void *data)
             unsigned i;
             const struct retro_memory_map *mmaps        =
                (const struct retro_memory_map*)data;
-            struct retro_memory_descriptor *descriptors = NULL;
+            rarch_memory_descriptor_t *descriptors = NULL;
 
             RARCH_LOG("Environ SET_MEMORY_MAPS.\n");
             free((void*)system->mmaps.descriptors);
             system->mmaps.num_descriptors = 0;
-            descriptors = (struct retro_memory_descriptor*)
+            descriptors = (rarch_memory_descriptor_t*)
                calloc(mmaps->num_descriptors,
-                     sizeof(*system->mmaps.descriptors));
+                     sizeof(*descriptors));
 
             if (!descriptors)
                return false;
 
-            system->mmaps.descriptors = descriptors;
-            memcpy((void*)system->mmaps.descriptors, mmaps->descriptors,
-                  mmaps->num_descriptors * sizeof(*system->mmaps.descriptors));
+            system->mmaps.descriptors     = descriptors;
             system->mmaps.num_descriptors = mmaps->num_descriptors;
+
+            for (i = 0; i < mmaps->num_descriptors; i++)
+               system->mmaps.descriptors[i].core = mmaps->descriptors[i];
+
             mmap_preprocess_descriptors(descriptors, mmaps->num_descriptors);
 
             if (sizeof(void *) == 8)
@@ -1411,38 +1561,38 @@ bool rarch_environment_cb(unsigned cmd, void *data)
 
             for (i = 0; i < system->mmaps.num_descriptors; i++)
             {
-               const struct retro_memory_descriptor *desc =
+               const rarch_memory_descriptor_t *desc =
                   &system->mmaps.descriptors[i];
                char flags[7];
 
                flags[0] = 'M';
-               if ((desc->flags & RETRO_MEMDESC_MINSIZE_8) == RETRO_MEMDESC_MINSIZE_8)
+               if ((desc->core.flags & RETRO_MEMDESC_MINSIZE_8) == RETRO_MEMDESC_MINSIZE_8)
                   flags[1] = '8';
-               else if ((desc->flags & RETRO_MEMDESC_MINSIZE_4) == RETRO_MEMDESC_MINSIZE_4)
+               else if ((desc->core.flags & RETRO_MEMDESC_MINSIZE_4) == RETRO_MEMDESC_MINSIZE_4)
                   flags[1] = '4';
-               else if ((desc->flags & RETRO_MEMDESC_MINSIZE_2) == RETRO_MEMDESC_MINSIZE_2)
+               else if ((desc->core.flags & RETRO_MEMDESC_MINSIZE_2) == RETRO_MEMDESC_MINSIZE_2)
                   flags[1] = '2';
                else
                   flags[1] = '1';
 
                flags[2] = 'A';
-               if ((desc->flags & RETRO_MEMDESC_ALIGN_8) == RETRO_MEMDESC_ALIGN_8)
+               if ((desc->core.flags & RETRO_MEMDESC_ALIGN_8) == RETRO_MEMDESC_ALIGN_8)
                   flags[3] = '8';
-               else if ((desc->flags & RETRO_MEMDESC_ALIGN_4) == RETRO_MEMDESC_ALIGN_4)
+               else if ((desc->core.flags & RETRO_MEMDESC_ALIGN_4) == RETRO_MEMDESC_ALIGN_4)
                   flags[3] = '4';
-               else if ((desc->flags & RETRO_MEMDESC_ALIGN_2) == RETRO_MEMDESC_ALIGN_2)
+               else if ((desc->core.flags & RETRO_MEMDESC_ALIGN_2) == RETRO_MEMDESC_ALIGN_2)
                   flags[3] = '2';
                else
                   flags[3] = '1';
 
-               flags[4] = (desc->flags & RETRO_MEMDESC_BIGENDIAN) ? 'B' : 'b';
-               flags[5] = (desc->flags & RETRO_MEMDESC_CONST) ? 'C' : 'c';
+               flags[4] = (desc->core.flags & RETRO_MEMDESC_BIGENDIAN) ? 'B' : 'b';
+               flags[5] = (desc->core.flags & RETRO_MEMDESC_CONST) ? 'C' : 'c';
                flags[6] = 0;
 
                RARCH_LOG("   %03u %s %p %08X %08X %08X %08X %08X %s\n",
-                     i + 1, flags, desc->ptr, desc->offset, desc->start,
-                     desc->select, desc->disconnect, desc->len,
-                     desc->addrspace ? desc->addrspace : "");
+                     i + 1, flags, desc->core.ptr, desc->core.offset, desc->core.start,
+                     desc->core.select, desc->core.disconnect, desc->core.len,
+                     desc->core.addrspace ? desc->core.addrspace : "");
             }
          }
          else
@@ -1519,6 +1669,13 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          break;
       }
 
+      case RETRO_ENVIRONMENT_SET_SERIALIZATION_QUIRKS:
+      {
+         uint64_t *quirks = (uint64_t *) data;
+         core_set_serialization_quirks(*quirks);
+         break;
+      }
+
       /* Default */
       default:
          RARCH_LOG("Environ UNSUPPORTED (#%u).\n", cmd);
