@@ -1,4 +1,4 @@
-/* Copyright  (C) 2010-2016 The RetroArch team
+/* Copyright  (C) 2010-2017 The RetroArch team
  *
  * ---------------------------------------------------------------------------------------
  * The following license statement only applies to this file (file_path.c).
@@ -34,9 +34,11 @@
 #include <compat/strl.h>
 #include <compat/posix_string.h>
 #endif
+#include <compat/strcasestr.h>
 #include <retro_assert.h>
 #include <retro_stat.h>
 #include <retro_miscellaneous.h>
+#include <string/stdstring.h>
 
 /**
  * path_mkdir:
@@ -84,6 +86,41 @@ end:
    return ret;
 }
 
+/**
+ * path_get_archive_delim:
+ * @path               : path
+ *
+ * Find delimiter of an archive file. Only the first '#'
+ * after a compression extension is considered.
+ *
+ * Returns: pointer to the delimiter in the path if it contains
+ * a path inside a compressed file, otherwise NULL.
+ */
+const char *path_get_archive_delim(const char *path)
+{
+   const char *last = find_last_slash(path);
+   const char *delim = NULL;
+
+   if (last)
+   {
+      delim = strcasestr(last, ".zip#");
+
+      if (!delim)
+         delim = strcasestr(last, ".apk#");
+   }
+
+   if (delim)
+      return delim + 4;
+
+   if (last)
+      delim = strcasestr(last, ".7z#");
+
+   if (delim)
+      return delim + 3;
+
+   return NULL;
+}
+
 /**
  * path_get_extension:
  * @path               : path
@@ -121,25 +158,6 @@ char *path_remove_extension(char *path)
    return last;
 }
 
-/**
- * path_contains_compressed_file:
- * @path               : path
- *
- * Checks if path contains a compressed file.
- *
- * Currently we only check for hash symbol (#) inside the pathname.
- * If path is ever expanded to a general URI, we should check for that here.
- *
- * Example:  Somewhere in the path there might be a compressed file
- * E.g.: /path/to/file.7z#mygame.img
- *
- * Returns: true (1) if path contains compressed file, otherwise false (0).
- **/
-bool path_contains_compressed_file(const char *path)
-{
-   return (strchr(path,'#') != NULL);
-}
-
 /**
  * path_is_compressed_file:
  * @path               : path
@@ -150,20 +168,13 @@ bool path_contains_compressed_file(const char *path)
  **/
 bool path_is_compressed_file(const char* path)
 {
-#if defined(HAVE_COMPRESSION) && (defined(HAVE_ZLIB) || defined(HAVE_7ZIP))
-   const char* file_ext   = path_get_extension(path);
+   const char *ext = path_get_extension(path);
 
-#ifdef HAVE_ZLIB
-   if (!strcmp(file_ext, "zip"))
+   if (     string_is_equal_noncase(ext, "zip") 
+         || string_is_equal_noncase(ext, "apk")
+         || string_is_equal_noncase(ext, "7z"))
       return true;
-#endif
 
-#ifdef HAVE_7ZIP
-   if (!strcmp(file_ext, "7z"))
-      return true;
-#endif
-
-#endif
    return false;
 }
 
@@ -210,9 +221,11 @@ bool path_file_exists(const char *path)
 void fill_pathname(char *out_path, const char *in_path,
       const char *replace, size_t size)
 {
-   char tmp_path[PATH_MAX_LENGTH] = {0};
+   char tmp_path[PATH_MAX_LENGTH];
    char *tok                      = NULL;
 
+   tmp_path[0] = '\0';
+
    retro_assert(strlcpy(tmp_path, in_path,
             sizeof(tmp_path)) < sizeof(tmp_path));
    if ((tok = (char*)strrchr(path_basename(tmp_path), '.')))
@@ -272,6 +285,9 @@ void fill_pathname_slash(char *path, size_t size)
    if (last_slash && (last_slash != (path + path_len - 1)))
    {
       char join_str[2];
+
+      join_str[0] = '\0';
+
       strlcpy(join_str, last_slash, sizeof(join_str));
       retro_assert(strlcat(path, join_str, size) < size);
    }
@@ -317,31 +333,11 @@ void fill_pathname_dir(char *in_dir, const char *in_basename,
  **/
 void fill_pathname_base(char *out, const char *in_path, size_t size)
 {
-   const char *ptr_bak = NULL;
-   const char     *ptr = find_last_slash(in_path);
+   const char     *ptr = path_basename(in_path);
 
-   (void)ptr_bak;
-
-   if (ptr)
-      ptr++;
-   else
+   if (!ptr)
       ptr = in_path;
 
-#ifdef HAVE_COMPRESSION
-   /* In case of compression, we also have to consider paths like
-    *   /path/to/archive.7z#mygame.img
-    *   and
-    *   /path/to/archive.7z#folder/mygame.img
-    *   basename would be mygame.img in both cases
-    */
-   ptr_bak = ptr;
-   ptr     = strchr(ptr_bak,'#');
-   if (ptr)
-      ptr++;
-   else
-      ptr = ptr_bak;
-#endif
-
    retro_assert(strlcpy(out, ptr, size) < size);
 }
 
@@ -415,14 +411,40 @@ void fill_pathname_parent_dir(char *out_dir,
 void fill_dated_filename(char *out_filename,
       const char *ext, size_t size)
 {
-   time_t cur_time;
-   time(&cur_time);
+   time_t cur_time = time(NULL);
 
    strftime(out_filename, size,
          "RetroArch-%m%d-%H%M%S.", localtime(&cur_time));
    strlcat(out_filename, ext, size);
 }
 
+/**
+ * fill_str_dated_filename:
+ * @out_filename       : output filename
+ * @in_str             : input string
+ * @ext                : extension of output filename
+ * @size               : buffer size of output filename
+ *
+ * Creates a 'dated' filename prefixed by the string @in_str, and
+ * concatenates extension (@ext) to it.
+ *
+ * E.g.:
+ * out_filename = "RetroArch-{year}{month}{day}-{Hour}{Minute}{Second}.{@ext}"
+ **/
+void fill_str_dated_filename(char *out_filename,
+      const char *in_str, const char *ext, size_t size)
+{
+   char format[256];
+   time_t cur_time = time(NULL);
+
+   format[0] = '\0';
+
+   strftime(format, sizeof(format), "-%y%m%d-%H%M%S.", localtime(&cur_time));
+   strlcpy(out_filename, in_str, size);
+   strlcat(out_filename, format, size);
+   strlcat(out_filename, ext, size);
+}
+
 /**
  * path_basedir:
  * @path               : path
@@ -436,13 +458,6 @@ void path_basedir(char *path)
    if (strlen(path) < 2)
       return;
 
-#ifdef HAVE_COMPRESSION
-   /* We want to find the directory with the zipfile in basedir. */
-   last = strchr(path,'#');
-   if (last)
-      *last = '\0';
-#endif
-
    last = find_last_slash(path);
 
    if (last)
@@ -476,19 +491,17 @@ void path_parent_dir(char *path)
  **/
 const char *path_basename(const char *path)
 {
-   const char *last = find_last_slash(path);
-#ifdef HAVE_COMPRESSION
-   const char *last_hash = NULL;
+   /* We cut either at the first compression-related hash 
+    * or the last slash; whichever comes last */
+   const char *last  = find_last_slash(path);
+   const char *delim = path_get_archive_delim(path);
 
-   /* We cut either at the last hash or the last slash; whichever comes last */
-   last_hash = strchr(path,'#');
-
-   if (last_hash > last)
-      return last_hash + 1;
-#endif
+   if (delim)
+      return delim + 1;
 
    if (last)
       return last + 1;
+
    return path;
 }
 
@@ -498,7 +511,7 @@ const char *path_basename(const char *path)
  *
  * Checks if @path is an absolute path or a relative path.
  *
- * Returns: true (1) if path is absolute, false (1) if path is relative.
+ * Returns: true if path is absolute, false if path is relative.
  **/
 bool path_is_absolute(const char *path)
 {
@@ -506,8 +519,10 @@ bool path_is_absolute(const char *path)
       return true;
 #ifdef _WIN32
    /* Many roads lead to Rome ... */
-   if ((strstr(path, "\\\\") == path)
-         || strstr(path, ":/") || strstr(path, ":\\") || strstr(path, ":\\\\"))
+   if ((    strstr(path, "\\\\") == path)
+         || strstr(path, ":/") 
+         || strstr(path, ":\\") 
+         || strstr(path, ":\\\\"))
       return true;
 #endif
    return false;
@@ -524,7 +539,9 @@ bool path_is_absolute(const char *path)
 void path_resolve_realpath(char *buf, size_t size)
 {
 #ifndef RARCH_CONSOLE
-   char tmp[PATH_MAX_LENGTH] = {0};
+   char tmp[PATH_MAX_LENGTH];
+
+   tmp[0] = '\0';
 
    strlcpy(tmp, buf, sizeof(tmp));
 
@@ -612,7 +629,7 @@ void fill_pathname_join_special_ext(char *out_path,
 }
 
 void fill_pathname_join_concat(char *out_path,
-      const char *dir, const char *path, 
+      const char *dir, const char *path,
       const char *concat,
       size_t size)
 {
@@ -677,29 +694,14 @@ void fill_pathname_join_delim_concat(char *out_path, const char *dir,
 void fill_short_pathname_representation(char* out_rep,
       const char *in_path, size_t size)
 {
-   char path_short[PATH_MAX_LENGTH] = {0};
-   char *last_hash                  = NULL;
+   char path_short[PATH_MAX_LENGTH];
+
+   path_short[0] = '\0';
 
    fill_pathname(path_short, path_basename(in_path), "",
             sizeof(path_short));
 
-#ifdef HAVE_COMPRESSION
-   last_hash = (char*)strchr(path_short,'#');
-   if(last_hash != NULL)
-   {
-      /* We handle paths like:
-       * /path/to/file.7z#mygame.img
-       * short_name: mygame.img:
-       *
-       * We check whether something is actually
-       * after the hash to avoid going over the buffer.
-       */
-      retro_assert(strlen(last_hash) > 1);
-      strlcpy(out_rep, last_hash + 1, size);
-   }
-   else
-#endif
-      strlcpy(out_rep, path_short, size);
+   strlcpy(out_rep, path_short, size);
 }
 
 void fill_short_pathname_representation_noext(char* out_rep,
