@@ -1,6 +1,6 @@
 /* RetroArch - A frontend for libretro.
  * Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- * Copyright (C) 2011-2016 - Daniel De Matteis
+ * Copyright (C) 2011-2017 - Daniel De Matteis
  * Copyright (C) 2012-2015 - Michael Lelli
  *
  * RetroArch is free software: you can redistribute it and/or modify it under the terms
@@ -21,11 +21,8 @@
 #include <stdint.h>
 #include <stddef.h>
 
-#include <boolean.h>
 #include <retro_common_api.h>
 
-#include "frontend_driver.h"
-
 RETRO_BEGIN_DECLS
 
 /**
