@@ -1,6 +1,6 @@
 /* RetroArch - A frontend for libretro.
 * Copyright (C) 2010-2014 - Hans-Kristian Arntzen
-* Copyright (C) 2011-2016 - Daniel De Matteis
+* Copyright (C) 2011-2017 - Daniel De Matteis
 *
 * RetroArch is free software: you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Found-
@@ -49,21 +49,34 @@ CONSOLE EXTENSIONS
 ALGORITHMS
 ============================================================ */
 
-#include "../libretro-common/algorithms/mismatch.c"
-
 /*============================================================
 ARCHIVE FILE
 ============================================================ */
+#include "../libretro-common/file/archive_file.c"
 
 #ifdef HAVE_ZLIB
-#include "../libretro-common/file/archive_file.c"
 #include "../libretro-common/file/archive_file_zlib.c"
 #endif
 
+#ifdef HAVE_7ZIP
+#include "../libretro-common/file/archive_file_7z.c"
+#endif
+
+/*============================================================
+COMPRESSION
+============================================================ */
+#include "../libretro-common/streams/trans_stream.c"
+#include "../libretro-common/streams/trans_stream_pipe.c"
+
+#ifdef HAVE_ZLIB
+#include "../libretro-common/streams/trans_stream_zlib.c"
+#endif
+
 /*============================================================
 ENCODINGS
 ============================================================ */
 #include "../libretro-common/encodings/encoding_utf.c"
+#include "../libretro-common/encodings/encoding_crc32.c"
 
 /*============================================================
 PERFORMANCE
@@ -107,23 +120,29 @@ CONFIG FILE
 #endif
 
 #include "../libretro-common/file/config_file.c"
-#include "../config_file_userdata.c"
+#include "../libretro-common/file/config_file_userdata.c"
 #include "../managers/core_option_manager.c"
 
 /*============================================================
 ACHIEVEMENTS
 ============================================================ */
 #if defined(HAVE_CHEEVOS) && defined(HAVE_THREADS)
-#if !defined(HAVE_NETPLAY)
+#if !defined(HAVE_NETWORKING)
 #include "../libretro-common/net/net_http.c"
 #endif
 
 #include "../libretro-common/formats/json/jsonsax.c"
-#include "../libretro-common/utils/md5.c"
 #include "../network/net_http_special.c"
 #include "../cheevos.c"
 #endif
 
+/*============================================================
+MD5
+============================================================ */
+#if (defined(HAVE_CHEEVOS) && defined(HAVE_THREADS)) || (defined(HAVE_HTTPSERVER) && defined(HAVE_ZLIB))
+#include "../libretro-common/utils/md5.c"
+#endif
+
 /*============================================================
 CHEATS
 ============================================================ */
@@ -142,13 +161,14 @@ VIDEO CONTEXT
 #elif defined(ANDROID)
 #include "../gfx/drivers_context/android_ctx.c"
 #elif defined(__QNX__)
-#include "../gfx/drivers_context/bbqnx_ctx.c"
+#include "../gfx/drivers_context/qnx_ctx.c"
 #elif defined(EMSCRIPTEN)
 #include "../gfx/drivers_context/emscriptenegl_ctx.c"
 #elif defined(__APPLE__) && !defined(TARGET_IPHONE_SIMULATOR) && !defined(TARGET_OS_IPHONE)
 #include "../gfx/drivers_context/cgl_ctx.c"
 #endif
 
+
 #if defined(HAVE_VIVANTE_FBDEV)
 #include "../gfx/drivers_context/vivante_fbdev_ctx.c"
 #endif
@@ -168,7 +188,7 @@ VIDEO CONTEXT
 #ifdef HAVE_VULKAN
 #include "../gfx/common/vulkan_common.c"
 #include "../libretro-common/vulkan/vulkan_symbol_wrapper.c"
-#ifdef HAVE_VULKAN_KHR_DISPLAY
+#ifdef HAVE_VULKAN_DISPLAY
 #include "../gfx/drivers_context/khr_display_ctx.c"
 #endif
 #endif
@@ -199,7 +219,6 @@ VIDEO CONTEXT
 
 #endif
 
-
 /*============================================================
 VIDEO SHADERS
 ============================================================ */
@@ -253,6 +272,8 @@ VIDEO IMAGE
 #include "../libretro-common/formats/bmp/rbmp_encode.c"
 #endif
 
+#include "../libretro-common/formats/wav/rwav.c"
+
 /*============================================================
 VIDEO DRIVER
 ============================================================ */
@@ -261,6 +282,7 @@ VIDEO DRIVER
 #include "../libretro-common/gfx/math/matrix_3x3.c"
 #include "../libretro-common/gfx/math/vector_2.c"
 #include "../libretro-common/gfx/math/vector_3.c"
+#include "../libretro-common/gfx/math/vector_4.c"
 
 #if defined(GEKKO)
 #ifdef HW_RVL
@@ -269,6 +291,10 @@ VIDEO DRIVER
 #endif
 #endif
 
+#ifdef HAVE_SDL2
+#include "../gfx/drivers/sdl2_gfx.c"
+#endif
+
 #ifdef HAVE_VG
 #include "../gfx/drivers/vg.c"
 #endif
@@ -288,11 +314,15 @@ VIDEO DRIVER
 #ifdef HAVE_OPENGL
 #include "../gfx/common/gl_common.c"
 #include "../gfx/drivers/gl.c"
+#include "../libretro-common/gfx/gl_capabilities.c"
+#include "../gfx/drivers/gl_renderchains/render_chain_gl_legacy.c"
 
 #ifndef HAVE_PSGL
 #include "../libretro-common/glsym/rglgen.c"
-#ifdef HAVE_OPENGLES2
+#if defined(HAVE_OPENGLES2)
 #include "../libretro-common/glsym/glsym_es2.c"
+#elif defined(HAVE_OPENGLES3)
+#include "../libretro-common/glsym/glsym_es3.c"
 #else
 #include "../libretro-common/glsym/glsym_gl.c"
 #endif
@@ -305,8 +335,8 @@ VIDEO DRIVER
 #endif
 
 #if defined(HAVE_D3D)
-#include "../gfx/d3d/render_chain_driver.c"
-#include "../gfx/d3d/render_chain_null.c"
+#include "../gfx/drivers/d3d_renderchains/render_chain_driver.c"
+#include "../gfx/drivers/d3d_renderchains/render_chain_null.c"
 #endif
 
 #if defined(GEKKO)
@@ -314,14 +344,25 @@ VIDEO DRIVER
 #elif defined(PSP)
 #include "../gfx/drivers/psp1_gfx.c"
 #elif defined(HAVE_VITA2D)
+#include "../deps/libvita2d/source/vita2d.c"
+#include "../deps/libvita2d/source/vita2d_texture.c"
+#include "../deps/libvita2d/source/vita2d_draw.c"
+#include "../deps/libvita2d/source/utils.c"
+
 #include "../gfx/drivers/vita2d_gfx.c"
 #elif defined(_3DS)
 #include "../gfx/drivers/ctr_gfx.c"
 #elif defined(XENON)
 #include "../gfx/drivers/xenon360_gfx.c"
+#elif defined(DJGPP)
+#include "../gfx/drivers/vga_gfx.c"
 #endif
 #include "../gfx/drivers/nullgfx.c"
 
+#if defined(_WIN32) && !defined(_XBOX)
+#include "../gfx/drivers/gdi_gfx.c"
+#endif
+
 /*============================================================
 FONTS
 ============================================================ */
@@ -330,6 +371,7 @@ FONTS
 #include "../gfx/font_driver.c"
 
 #if defined(HAVE_STB_FONT)
+#include "../gfx/drivers_font_renderer/stb_unicode.c"
 #include "../gfx/drivers_font_renderer/stb.c"
 #endif
 
@@ -357,6 +399,22 @@ FONTS
 #include "../gfx/drivers_font/vita2d_font.c"
 #endif
 
+#if defined(_3DS)
+#include "../gfx/drivers_font/ctr_font.c"
+#endif
+
+#if defined(HAVE_CACA)
+#include "../gfx/drivers_font/caca_font.c"
+#endif
+
+#if defined(DJGPP)
+#include "../gfx/drivers_font/vga_font.c"
+#endif
+
+#if defined(_WIN32) && !defined(_XBOX)
+#include "../gfx/drivers_font/gdi_font.c"
+#endif
+
 #if defined(HAVE_VULKAN)
 #include "../gfx/drivers_font/vulkan_raster_font.c"
 #endif
@@ -364,7 +422,7 @@ FONTS
 /*============================================================
 INPUT
 ============================================================ */
-#include "../input/input_autodetect.c"
+#include "../tasks/task_autodetect.c"
 #include "../input/input_joypad_driver.c"
 #include "../input/input_config.c"
 #include "../input/input_keymaps.c"
@@ -380,30 +438,25 @@ INPUT
 #include "../input/common/x11_input_common.c"
 #endif
 
+#include "../input/input_autodetect_builtin.c"
+
 #if defined(__CELLOS_LV2__)
 #include "../input/drivers/ps3_input.c"
 #include "../input/drivers_joypad/ps3_joypad.c"
-#include "../input/autoconf/builtin_ps3.c"
 #elif defined(SN_TARGET_PSP2) || defined(PSP) || defined(VITA)
 #include "../input/drivers/psp_input.c"
 #include "../input/drivers_joypad/psp_joypad.c"
-#include "../input/autoconf/builtin_psp.c"
 #elif defined(HAVE_COCOA) || defined(HAVE_COCOATOUCH)
 #include "../input/drivers/cocoa_input.c"
 #elif defined(_3DS)
 #include "../input/drivers/ctr_input.c"
 #include "../input/drivers_joypad/ctr_joypad.c"
-#include "../input/autoconf/builtin_ctr.c"
 #elif defined(GEKKO)
 #include "../input/drivers/gx_input.c"
 #include "../input/drivers_joypad/gx_joypad.c"
-#include "../input/autoconf/builtin_gx.c"
 #elif defined(_XBOX)
 #include "../input/drivers/xdk_xinput_input.c"
 #include "../input/drivers_joypad/xdk_joypad.c"
-#include "../input/autoconf/builtin_xdk.c"
-#elif defined(_WIN32)
-#include "../input/autoconf/builtin_win.c"
 #elif defined(XENON)
 #include "../input/drivers/xenon360_input.c"
 #elif defined(ANDROID)
@@ -415,6 +468,9 @@ INPUT
 #include "../input/drivers_joypad/qnx_joypad.c"
 #elif defined(EMSCRIPTEN)
 #include "../input/drivers/rwebinput_input.c"
+#elif defined(DJGPP)
+#include "../input/drivers/dos_input.c"
+#include "../input/drivers_joypad/dos_joypad.c"
 #endif
 
 #ifdef HAVE_DINPUT
@@ -438,6 +494,7 @@ INPUT
 #endif
 
 #ifdef HAVE_UDEV
+#include "../input/common/udev_common.c"
 #include "../input/drivers/udev_input.c"
 #include "../input/drivers_keyboard/keyboard_event_udev.c"
 #include "../input/drivers_joypad/udev_joypad.c"
@@ -503,7 +560,7 @@ STATE TRACKER
 #include "../gfx/video_state_tracker.c"
 
 #ifdef HAVE_PYTHON
-#include "../gfx/video_state_python.c"
+#include "../gfx/drivers_tracker/video_state_python.c"
 #endif
 
 /*============================================================
@@ -514,11 +571,13 @@ FIFO BUFFER
 /*============================================================
 AUDIO RESAMPLER
 ============================================================ */
-#include "../audio/audio_resampler_driver.c"
-#include "../audio/drivers_resampler/sinc_resampler.c"
-#include "../audio/drivers_resampler/nearest_resampler.c"
-#include "../audio/drivers_resampler/null_resampler.c"
+#include "../libretro-common/audio/resampler/audio_resampler.c"
+#include "../libretro-common/audio/resampler/drivers/sinc_resampler.c"
+#include "../libretro-common/audio/resampler/drivers/nearest_resampler.c"
+#include "../libretro-common/audio/resampler/drivers/null_resampler.c"
+#ifdef HAVE_CC_RESAMPLER
 #include "../audio/drivers_resampler/cc_resampler.c"
+#endif
 
 /*============================================================
 CAMERA
@@ -533,6 +592,10 @@ CAMERA
 #include "../camera/drivers/video4linux2.c"
 #endif
 
+#ifdef HAVE_VIDEO_PROCESSOR
+#include "../cores/libretro-video-processor/video_processor_v4l2.c"
+#endif
+
 #include "../camera/drivers/nullcamera.c"
 
 /*============================================================
@@ -570,6 +633,10 @@ AUDIO
 #include "../audio/drivers/ctr_dsp_audio.c"
 #endif
 
+#if defined(HAVE_SDL2)
+#include "../audio/drivers/sdl_audio.c"
+#endif
+
 #ifdef HAVE_DSOUND
 #include "../audio/drivers/dsound.c"
 #endif
@@ -632,22 +699,23 @@ FILTERS
 #include "../gfx/video_filters/lq2x.c"
 #include "../gfx/video_filters/phosphor2x.c"
 
-#include "../audio/audio_filters/echo.c"
-#include "../audio/audio_filters/eq.c"
-#include "../audio/audio_filters/chorus.c"
-#include "../audio/audio_filters/iir.c"
-#include "../audio/audio_filters/panning.c"
-#include "../audio/audio_filters/phaser.c"
-#include "../audio/audio_filters/reverb.c"
-#include "../audio/audio_filters/wahwah.c"
+#include "../libretro-common/audio/dsp_filters/echo.c"
+#include "../libretro-common/audio/dsp_filters/eq.c"
+#include "../libretro-common/audio/dsp_filters/chorus.c"
+#include "../libretro-common/audio/dsp_filters/iir.c"
+#include "../libretro-common/audio/dsp_filters/panning.c"
+#include "../libretro-common/audio/dsp_filters/phaser.c"
+#include "../libretro-common/audio/dsp_filters/reverb.c"
+#include "../libretro-common/audio/dsp_filters/wahwah.c"
 #endif
+
 /*============================================================
 DYNAMIC
 ============================================================ */
 #include "../libretro-common/dynamic/dylib.c"
 #include "../dynamic.c"
 #include "../gfx/video_filter.c"
-#include "../audio/audio_dsp_filter.c"
+#include "../libretro-common/audio/dsp_filter.c"
 
 /*============================================================
 CORES
@@ -667,6 +735,7 @@ FILE
 #include "../libretro-common/lists/dir_list.c"
 #include "../libretro-common/lists/string_list.c"
 #include "../libretro-common/lists/file_list.c"
+#include "../setting_list.c"
 #include "../libretro-common/file/retro_dirent.c"
 #include "../libretro-common/streams/file_stream.c"
 #include "../libretro-common/streams/interface_stream.c"
@@ -681,11 +750,6 @@ MESSAGE
 ============================================================ */
 #include "../libretro-common/queues/message_queue.c"
 
-/*============================================================
-PATCH
-============================================================ */
-#include "../patch.c"
-
 /*============================================================
 CONFIGURATION
 ============================================================ */
@@ -724,6 +788,8 @@ FRONTEND
 #include "../frontend/drivers/platform_linux.c"
 #elif defined(BSD) && !defined(__MACH__)
 #include "../frontend/drivers/platform_bsd.c"
+#elif defined(DJGPP)
+#include "../frontend/drivers/platform_dos.c"
 #endif
 #include "../frontend/drivers/platform_null.c"
 
@@ -757,7 +823,7 @@ GIT
 ============================================================ */
 
 #ifdef HAVE_GIT_VERSION
-#include "../git_version.c"
+#include "../version_git.c"
 #endif
 
 
@@ -766,6 +832,8 @@ RETROARCH
 ============================================================ */
 #include "../core_impl.c"
 #include "../retroarch.c"
+#include "../dirs.c"
+#include "../paths.c"
 #include "../runloop.c"
 #include "../libretro-common/queues/task_queue.c"
 
@@ -776,16 +844,28 @@ RETROARCH
 #include "../intl/msg_hash_eo.c"
 #include "../intl/msg_hash_fr.c"
 #include "../intl/msg_hash_it.c"
+#include "../intl/msg_hash_ja.c"
 #include "../intl/msg_hash_nl.c"
 #include "../intl/msg_hash_pt.c"
 #include "../intl/msg_hash_pl.c"
-#ifdef HAVE_UTF8
 #include "../intl/msg_hash_ru.c"
-#endif
+#include "../intl/msg_hash_vn.c"
+#include "../intl/msg_hash_chs.c"
 #endif
 
 #include "../intl/msg_hash_us.c"
 
+/*============================================================
+WIFI
+============================================================ */
+#include "../wifi/wifi_driver.c"
+
+#include "../wifi/drivers/nullwifi.c"
+
+#ifdef HAVE_LAKKA
+#include "../wifi/drivers/connmanctl.c"
+#endif
+
 /*============================================================
 RECORDING
 ============================================================ */
@@ -813,26 +893,34 @@ THREAD
 /*============================================================
 NETPLAY
 ============================================================ */
-#ifdef HAVE_NETPLAY
-#include "../network/netplay/netplay_net.c"
-#include "../network/netplay/netplay_spectate.c"
-#include "../network/netplay/netplay_common.c"
-#include "../network/netplay/netplay.c"
+#ifdef HAVE_NETWORKING
+#include "../network/netplay/netplay_delta.c"
+#include "../network/netplay/netplay_frontend.c"
+#include "../network/netplay/netplay_handshake.c"
+#include "../network/netplay/netplay_init.c"
+#include "../network/netplay/netplay_io.c"
+#include "../network/netplay/netplay_sync.c"
+#include "../network/netplay/netplay_discovery.c"
+#include "../network/netplay/netplay_buf.c"
 #include "../libretro-common/net/net_compat.c"
 #include "../libretro-common/net/net_socket.c"
 #include "../libretro-common/net/net_http.c"
+#include "../libretro-common/net/net_natt.c"
 #ifndef HAVE_SOCKET_LEGACY
 #include "../libretro-common/net/net_ifinfo.c"
 #endif
 #include "../tasks/task_http.c"
+#include "../tasks/task_netplay_lan_scan.c"
+#include "../tasks/task_wifi.c"
+#include "../tasks/task_netplay_find_content.c"
 #endif
 
 /*============================================================
 DATA RUNLOOP
 ============================================================ */
+#include "../tasks/task_powerstate.c"
 #include "../tasks/task_content.c"
-#include "../tasks/task_save_ram.c"
-#include "../tasks/task_save_state.c"
+#include "../tasks/task_save.c"
 #include "../tasks/task_image.c"
 #include "../tasks/task_file_transfer.c"
 #ifdef HAVE_ZLIB
@@ -859,11 +947,18 @@ MENU
 #ifdef HAVE_MENU
 #include "../menu/menu_driver.c"
 #include "../menu/menu_input.c"
-#include "../menu/menu_entry.c"
+#include "../menu/menu_event.c"
 #include "../menu/menu_entries.c"
 #include "../menu/menu_setting.c"
 #include "../menu/menu_cbs.c"
 #include "../menu/menu_content.c"
+#include "../menu/widgets/menu_entry.c"
+#include "../menu/widgets/menu_filebrowser.c"
+#include "../menu/widgets/menu_dialog.c"
+#include "../menu/widgets/menu_input_dialog.c"
+#include "../menu/widgets/menu_input_bind_dialog.c"
+#include "../menu/widgets/menu_list.c"
+#include "../menu/widgets/menu_osk.c"
 #include "../menu/cbs/menu_cbs_ok.c"
 #include "../menu/cbs/menu_cbs_cancel.c"
 #include "../menu/cbs/menu_cbs_select.c"
@@ -877,6 +972,7 @@ MENU
 #include "../menu/cbs/menu_cbs_scan.c"
 #include "../menu/cbs/menu_cbs_get_value.c"
 #include "../menu/cbs/menu_cbs_label.c"
+#include "../menu/cbs/menu_cbs_sublabel.c"
 #include "../menu/cbs/menu_cbs_up.c"
 #include "../menu/cbs/menu_cbs_down.c"
 #include "../menu/cbs/menu_cbs_contentlist_switch.c"
@@ -899,6 +995,26 @@ MENU
 #include "../menu/drivers_display/menu_display_vulkan.c"
 #endif
 
+#ifdef HAVE_VITA2D
+#include "../menu/drivers_display/menu_display_vita2d.c"
+#endif
+
+#ifdef _3DS
+#include "../menu/drivers_display/menu_display_ctr.c"
+#endif
+
+#ifdef HAVE_CACA
+#include "../menu/drivers_display/menu_display_caca.c"
+#endif
+
+#ifdef DJGPP
+#include "../menu/drivers_display/menu_display_vga.c"
+#endif
+
+#if defined(_WIN32) && !defined(_XBOX)
+#include "../menu/drivers_display/menu_display_gdi.c"
+#endif
+
 #endif
 
 
@@ -906,8 +1022,7 @@ MENU
 #include "../menu/drivers/rgui.c"
 #endif
 
-#ifdef HAVE_OPENGL
-
+#if defined(HAVE_OPENGL) || defined(HAVE_VITA2D) || defined(_3DS) || defined(_MSC_VER)
 #ifdef HAVE_XMB
 #include "../menu/drivers/xmb.c"
 #endif
@@ -943,6 +1058,10 @@ MENU
 extern "C" {
 #endif
 
+#if defined(HAVE_NETWORKING)
+#include "../libretro-common/net/net_http_parse.c"
+#endif
+
 /*============================================================
 DEPENDENCIES
 ============================================================ */
@@ -993,8 +1112,9 @@ XML
 /*============================================================
  AUDIO UTILS
 ============================================================ */
-#include "../libretro-common/conversion/s16_to_float.c"
-#include "../libretro-common/conversion/float_to_s16.c"
+#include "../libretro-common/audio/conversion/s16_to_float.c"
+#include "../libretro-common/audio/conversion/float_to_s16.c"
+#include "../libretro-common/audio/audio_mix.c"
 
 /*============================================================
  LIBRETRODB
@@ -1008,6 +1128,13 @@ XML
 #include "../database_info.c"
 #endif
 
+/*============================================================
+HTTP SERVER
+============================================================ */
+#if defined(HAVE_HTTPSERVER) && defined(HAVE_ZLIB)
+#include "../deps/civetweb/civetweb.c"
+#include "network/httpserver/httpserver.c"
+#endif
 
 #ifdef __cplusplus
 }
