@@ -1,5 +1,5 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -46,9 +46,6 @@ bool input_remapping_save_file(const char *path);
 
 void input_remapping_set_defaults(void);
 
-void input_remapping_state(unsigned port,
-      unsigned *device, unsigned *idx, unsigned *id);
-
 RETRO_END_DECLS
 
 #endif
