@@ -1,5 +1,7 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *  Copyright (C) 2014-2017 - Jean-André Santoni
+ *  Copyright (C) 2016-2017 - Brad Parker
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -19,13 +21,23 @@
 #include <queues/message_queue.h>
 #include <retro_miscellaneous.h>
 #include <formats/image.h>
+#include <file/file_path.h>
 #include <string/stdstring.h>
 
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#ifdef HAVE_THREADS
+#include "../gfx/video_thread_wrapper.h"
+#endif
+
 #include "../config.def.h"
 #include "../retroarch.h"
 #include "../configuration.h"
 #include "../runloop.h"
 #include "../core.h"
+#include "../gfx/video_driver.h"
 #include "../gfx/video_thread_wrapper.h"
 #include "../verbosity.h"
 
@@ -33,13 +45,25 @@
 #include "menu_animation.h"
 #include "menu_display.h"
 
-#ifdef HAVE_THREADS
-#include "../gfx/video_thread_wrapper.h"
-#endif
-
+#define PARTICLES_COUNT            100
 
 uintptr_t menu_display_white_texture;
 
+static enum menu_toggle_reason menu_display_toggle_reason = MENU_TOGGLE_REASON_NONE;
+
+static video_coord_array_t menu_disp_ca;
+
+static unsigned menu_display_framebuf_width      = 0;
+static unsigned menu_display_framebuf_height     = 0;
+static size_t menu_display_framebuf_pitch        = 0;
+static unsigned menu_display_header_height       = 0;
+static bool menu_display_msg_force               = false;
+static bool menu_display_font_alloc_framebuf     = false;
+static bool menu_display_framebuf_dirty          = false;
+static const uint8_t *menu_display_font_framebuf = NULL;
+static msg_queue_t *menu_display_msg_queue       = NULL;
+static menu_display_ctx_driver_t *menu_disp      = NULL;
+
 static menu_display_ctx_driver_t *menu_display_ctx_drivers[] = {
 #ifdef HAVE_D3D
    &menu_display_ctx_d3d,
@@ -49,17 +73,40 @@ static menu_display_ctx_driver_t *menu_display_ctx_drivers[] = {
 #endif
 #ifdef HAVE_VULKAN
    &menu_display_ctx_vulkan,
+#endif
+#ifdef HAVE_VITA2D
+   &menu_display_ctx_vita2d,
+#endif
+#ifdef _3DS
+   &menu_display_ctx_ctr,
+#endif
+#ifdef HAVE_CACA
+   &menu_display_ctx_caca,
+#endif
+#if defined(_WIN32) && !defined(_XBOX)
+   &menu_display_ctx_gdi,
+#endif
+#ifdef DJGPP
+   &menu_display_ctx_vga,
 #endif
    &menu_display_ctx_null,
    NULL,
 };
 
+enum menu_toggle_reason menu_display_toggle_get_reason(void)
+{
+  return menu_display_toggle_reason;
+}
+
+void menu_display_toggle_set_reason(enum menu_toggle_reason reason)
+{
+  menu_display_toggle_reason = reason;
+}
+
 static const char *menu_video_get_ident(void)
 {
 #ifdef HAVE_THREADS
-   settings_t *settings = config_get_ptr();
-
-   if (settings->video.threaded)
+   if (video_driver_is_threaded())
       return video_thread_get_ident();
 #endif
 
@@ -87,6 +134,26 @@ static bool menu_display_check_compatibility(
          if (string_is_equal(video_driver, "d3d"))
             return true;
          break;
+      case MENU_VIDEO_DRIVER_VITA2D:
+         if (string_is_equal(video_driver, "vita2d"))
+            return true;
+         break;
+      case MENU_VIDEO_DRIVER_CTR:
+         if (string_is_equal(video_driver, "ctr"))
+            return true;
+         break;
+      case MENU_VIDEO_DRIVER_CACA:
+         if (string_is_equal(video_driver, "caca"))
+            return true;
+         break;
+      case MENU_VIDEO_DRIVER_GDI:
+         if (string_is_equal(video_driver, "gdi"))
+            return true;
+         break;
+      case MENU_VIDEO_DRIVER_VGA:
+         if (string_is_equal(video_driver, "vga"))
+            return true;
+         break;
    }
 
    return false;
@@ -126,20 +193,6 @@ void menu_display_timedate(menu_display_ctx_datetime_t *datetime)
    }
 }
 
-static video_coord_array_t menu_disp_ca;
-static unsigned menu_display_framebuf_width      = 0;
-static unsigned menu_display_framebuf_height     = 0;
-static size_t menu_display_framebuf_pitch        = 0;
-static int menu_display_font_size                = 0;
-static unsigned menu_display_header_height       = 0;
-static bool menu_display_msg_force               = false;
-static bool menu_display_font_alloc_framebuf     = false;
-static bool menu_display_framebuf_dirty          = false;
-static const uint8_t *menu_display_font_framebuf = NULL;
-static void *menu_display_font_buf               = NULL;
-static msg_queue_t *menu_display_msg_queue       = NULL;
-static menu_display_ctx_driver_t *menu_disp      = NULL;
-
 void menu_display_blend_begin(void)
 {
    if (!menu_disp || !menu_disp->blend_begin)
@@ -154,64 +207,51 @@ void menu_display_blend_end(void)
    menu_disp->blend_end();
 }
 
-void menu_display_font_main_deinit(void)
+void menu_display_font_free(font_data_t *font)
 {
-   if (menu_display_font_buf)
-      font_driver_free(menu_display_font_buf);
-   menu_display_font_buf  = NULL;
-   menu_display_font_size = 0;
+   font_driver_free(font);
 }
 
-bool menu_display_font(enum application_special_type type)
+font_data_t *menu_display_font(enum application_special_type type, float font_size)
 {
    menu_display_ctx_font_t font_info;
-   char fontpath[PATH_MAX_LENGTH]  = {0};
-   int                   font_size = menu_display_get_font_size();
+   char fontpath[PATH_MAX_LENGTH];
+
+   fontpath[0] = '\0';
 
    fill_pathname_application_special(fontpath, sizeof(fontpath), type);
 
    font_info.path = fontpath;
    font_info.size = font_size;
 
-   if (!menu_display_font_main_init(&font_info))
-   {
-      RARCH_WARN("Failed to load font.");
-      return false;
-   }
-
-   return true;
+   return menu_display_font_main_init(&font_info);
 }
 
-bool menu_display_font_main_init(menu_display_ctx_font_t *font)
+font_data_t *menu_display_font_main_init(menu_display_ctx_font_t *font)
 {
-   menu_display_font_main_deinit();
+   font_data_t *font_data = NULL;
+
    if (!font || !menu_disp)
-      return false;
+      return NULL;
 
-   if (!menu_disp->font_init_first)
-      return false;
-
-   if (!menu_disp->font_init_first(&menu_display_font_buf,
+   if (!menu_disp->font_init_first((void**)&font_data,
             video_driver_get_ptr(false),
             font->path, font->size))
-      return false;
+      return NULL;
 
-   menu_display_font_size = font->size;
-   return true;
+   return font_data;
 }
 
-void menu_display_font_bind_block(void *block)
+void menu_display_font_bind_block(font_data_t *font, void *block)
 {
-   font_driver_bind_block(menu_display_font_buf, block);
+   font_driver_bind_block(font, block);
 }
 
-bool menu_display_font_flush_block(void)
+bool menu_display_font_flush_block(unsigned width, unsigned height,
+      font_data_t *font)
 {
-   if (!menu_display_font_buf)
-      return false;
-
-   font_driver_flush(menu_display_font_buf);
-   font_driver_bind_block(menu_display_font_buf, NULL);
+   font_driver_flush(width, height, font);
+   font_driver_bind_block(font, NULL);
    return true;
 }
 
@@ -226,6 +266,7 @@ void menu_display_deinit(void)
 {
    if (menu_display_msg_queue)
       msg_queue_free(menu_display_msg_queue);
+
    video_coord_array_free(&menu_disp_ca);
    menu_display_msg_queue       = NULL;
    menu_display_msg_force       = false;
@@ -238,8 +279,8 @@ void menu_display_deinit(void)
 
 bool menu_display_init(void)
 {
-   retro_assert((menu_display_msg_queue = msg_queue_new(8)) != NULL);
-   menu_disp_ca.allocated              =  0;
+   menu_display_msg_queue  = msg_queue_new(8);
+   menu_disp_ca.allocated  =  0;
    return true;
 }
 
@@ -253,16 +294,6 @@ video_coord_array_t *menu_display_get_coords_array(void)
    return &menu_disp_ca;
 }
 
-void *menu_display_get_font_buffer(void)
-{
-   return menu_display_font_buf;
-}
-
-void menu_display_set_font_buffer(void *buffer)
-{
-   menu_display_font_buf = buffer;
-}
-
 const uint8_t *menu_display_get_font_framebuffer(void)
 {
    return menu_display_font_framebuf;
@@ -278,7 +309,7 @@ bool menu_display_libretro_running(void)
    settings_t *settings = config_get_ptr();
    if (!settings->menu.pause_libretro)
    {
-      if (rarch_ctl(RARCH_CTL_IS_INITED, NULL) 
+      if (rarch_ctl(RARCH_CTL_IS_INITED, NULL)
             && !rarch_ctl(RARCH_CTL_IS_DUMMY_CORE, NULL))
          return true;
    }
@@ -300,7 +331,17 @@ bool menu_display_libretro(void)
       return true;
    }
 
-   return video_driver_cached_frame_render();
+   if (runloop_ctl(RUNLOOP_CTL_IS_IDLE, NULL))
+      return true; /* Maybe return false here for indication of idleness? */
+   return video_driver_cached_frame();
+}
+
+void menu_display_get_fb_size(unsigned *fb_width,
+      unsigned *fb_height, size_t *fb_pitch)
+{
+   *fb_width  = menu_display_framebuf_width;
+   *fb_height = menu_display_framebuf_height;
+   *fb_pitch  = menu_display_framebuf_pitch;
 }
 
 void menu_display_set_width(unsigned width)
@@ -308,21 +349,11 @@ void menu_display_set_width(unsigned width)
    menu_display_framebuf_width = width;
 }
 
-unsigned menu_display_get_width(void)
-{
-   return menu_display_framebuf_width;
-}
-
 void menu_display_set_height(unsigned height)
 {
    menu_display_framebuf_height = height;
 }
 
-unsigned menu_display_get_height(void)
-{
-   return menu_display_framebuf_height;
-}
-
 void menu_display_set_header_height(unsigned height)
 {
    menu_display_header_height = height;
@@ -333,16 +364,6 @@ unsigned menu_display_get_header_height(void)
    return menu_display_header_height;
 }
 
-unsigned menu_display_get_font_size(void)
-{
-   return menu_display_font_size;
-}
-
-void menu_display_set_font_size(unsigned size)
-{
-   menu_display_font_size = size;
-}
-
 size_t menu_display_get_framebuffer_pitch(void)
 {
    return menu_display_framebuf_pitch;
@@ -375,24 +396,20 @@ void menu_display_set_font_data_init(bool state)
 
 bool menu_display_get_update_pending(void)
 {
-   if (menu_animation_ctl(MENU_ANIMATION_CTL_IS_ACTIVE, NULL))
+   if (menu_animation_is_active())
       return true;
    if (menu_display_get_framebuffer_dirty_flag())
       return true;
    return false;
 }
 
-void menu_display_set_viewport(void)
+void menu_display_set_viewport(unsigned width, unsigned height)
 {
-   unsigned width, height;
-   video_driver_get_size(&width, &height);
    video_driver_set_viewport(width, height, true, false);
 }
 
-void menu_display_unset_viewport(void)
+void menu_display_unset_viewport(unsigned width, unsigned height)
 {
-   unsigned width, height;
-   video_driver_get_size(&width, &height);
    video_driver_set_viewport(width, height, false, true);
 }
 
@@ -421,7 +438,7 @@ float menu_display_get_dpi(void)
       return true;
 
    metrics.type  = DISPLAY_METRIC_DPI;
-   metrics.value = &dpi; 
+   metrics.value = &dpi;
 
    if (settings->menu.dpi.override_enable)
       return settings->menu.dpi.override_value;
@@ -476,16 +493,6 @@ void menu_display_draw(menu_display_ctx_draw_t *draw)
    menu_disp->draw(draw);
 }
 
-bool menu_display_shader_pipeline_active(void)
-{
-   settings_t *settings          = config_get_ptr();
-   if (!string_is_equal(menu_driver_ident(), "xmb"))
-      return false;
-   if (settings->menu.xmb.shader_pipeline == XMB_SHADER_PIPELINE_WALLPAPER)
-      return false;
-   return true;
-}
-
 void menu_display_draw_pipeline(menu_display_ctx_draw_t *draw)
 {
    if (!menu_disp || !draw || !menu_disp->draw_pipeline)
@@ -493,13 +500,12 @@ void menu_display_draw_pipeline(menu_display_ctx_draw_t *draw)
    menu_disp->draw_pipeline(draw);
 }
 
-void menu_display_draw_bg(menu_display_ctx_draw_t *draw)
+void menu_display_draw_bg(menu_display_ctx_draw_t *draw, 
+      video_frame_info_t *video_info, bool add_opacity_to_wallpaper)
 {
    static struct video_coords coords;
    const float *new_vertex       = NULL;
    const float *new_tex_coord    = NULL;
-   bool add_opacity_to_wallpaper = false;
-   settings_t *settings          = config_get_ptr();
    if (!menu_disp || !draw)
       return;
 
@@ -519,14 +525,11 @@ void menu_display_draw_bg(menu_display_ctx_draw_t *draw)
 
    draw->coords      = &coords;
 
-   if (!menu_display_libretro_running() && !menu_display_shader_pipeline_active())
-      add_opacity_to_wallpaper = true;
-   if (string_is_equal(menu_driver_ident(), "xmb") 
-         && settings->menu.xmb.menu_color_theme == XMB_THEME_WALLPAPER)
+   if (!video_info->libretro_running && !draw->pipeline.active)
       add_opacity_to_wallpaper = true;
 
    if (add_opacity_to_wallpaper)
-      menu_display_set_alpha(draw->color, settings->menu.wallpaper.opacity);
+      menu_display_set_alpha(draw->color, video_info->menu_wallpaper_opacity);
 
    if (!draw->texture)
       draw->texture     = menu_display_white_texture;
@@ -534,18 +537,83 @@ void menu_display_draw_bg(menu_display_ctx_draw_t *draw)
    draw->matrix_data = (math_matrix_4x4*)menu_disp->get_default_mvp();
 }
 
-void menu_display_draw_gradient(menu_display_ctx_draw_t *draw)
+void menu_display_draw_gradient(menu_display_ctx_draw_t *draw,
+      video_frame_info_t *video_info)
 {
    draw->texture       = 0;
    draw->x             = 0;
    draw->y             = 0;
 
-   menu_display_draw_bg(draw);
+   menu_display_draw_bg(draw, video_info, false);
    menu_display_draw(draw);
 }
 
+void menu_display_draw_quad(
+      int x, int y, unsigned w, unsigned h,
+      unsigned width, unsigned height,
+      float *color)
+{
+   menu_display_ctx_draw_t draw;
+   struct video_coords coords;
+
+   coords.vertices      = 4;
+   coords.vertex        = NULL;
+   coords.tex_coord     = NULL;
+   coords.lut_tex_coord = NULL;
+   coords.color         = color;
+
+   menu_display_blend_begin();
+
+   draw.x           = x;
+   draw.y           = (int)height - y - (int)h;
+   draw.width       = w;
+   draw.height      = h;
+   draw.coords      = &coords;
+   draw.matrix_data = NULL;
+   draw.texture     = menu_display_white_texture;
+   draw.prim_type   = MENU_DISPLAY_PRIM_TRIANGLESTRIP;
+   draw.pipeline.id = 0;
+
+   menu_display_draw(&draw);
+   menu_display_blend_end();
+}
+
+void menu_display_draw_texture(
+      int x, int y, unsigned w, unsigned h,
+      unsigned width, unsigned height,
+      float *color, uintptr_t texture)
+{
+   menu_display_ctx_draw_t draw;
+   menu_display_ctx_rotate_draw_t rotate_draw;
+   struct video_coords coords;
+   math_matrix_4x4 mymat;
+   rotate_draw.matrix       = &mymat;
+   rotate_draw.rotation     = 0.0;
+   rotate_draw.scale_x      = 1.0;
+   rotate_draw.scale_y      = 1.0;
+   rotate_draw.scale_z      = 1;
+   rotate_draw.scale_enable = true;
+   coords.vertices          = 4;
+   coords.vertex            = NULL;
+   coords.tex_coord         = NULL;
+   coords.lut_tex_coord     = NULL;
+   draw.width               = w;
+   draw.height              = h;
+   draw.coords              = &coords;
+   draw.matrix_data         = &mymat;
+   draw.prim_type           = MENU_DISPLAY_PRIM_TRIANGLESTRIP;
+   draw.pipeline.id         = 0;
+   coords.color             = (const float*)color;
+   menu_display_rotate_z(&rotate_draw);
+   draw.texture             = texture;
+   draw.x                   = x;
+   draw.y                   = height - y;
+   menu_display_draw(&draw);
+}
+
 void menu_display_rotate_z(menu_display_ctx_rotate_draw_t *draw)
 {
+#if !defined(VITA)
    math_matrix_4x4 matrix_rotated, matrix_scaled;
    math_matrix_4x4 *b = NULL;
 
@@ -563,6 +631,7 @@ void menu_display_rotate_z(menu_display_ctx_rotate_draw_t *draw)
    matrix_4x4_scale(&matrix_scaled,
          draw->scale_x, draw->scale_y, draw->scale_z);
    matrix_4x4_multiply(draw->matrix, &matrix_scaled, draw->matrix);
+#endif
 }
 
 bool menu_display_get_tex_coords(menu_display_ctx_coord_draw_t *draw)
@@ -593,6 +662,22 @@ void menu_display_handle_thumbnail_upload(void *task_data,
    free(user_data);
 }
 
+void menu_display_handle_savestate_thumbnail_upload(void *task_data,
+      void *user_data, const char *err)
+{
+   menu_ctx_load_image_t load_image_info;
+   struct texture_image *img = (struct texture_image*)task_data;
+
+   load_image_info.data = img;
+   load_image_info.type = MENU_IMAGE_SAVESTATE_THUMBNAIL;
+
+   menu_driver_ctl(RARCH_MENU_CTL_LOAD_IMAGE, &load_image_info);
+
+   image_texture_free(img);
+   free(img);
+   free(user_data);
+}
+
 void menu_display_handle_wallpaper_upload(void *task_data,
       void *user_data, const char *err)
 {
@@ -676,12 +761,10 @@ void menu_display_push_quad(
       const float *colors, int x1, int y1,
       int x2, int y2)
 {
-   menu_display_ctx_coord_draw_t coord_draw;
    float vertex[8];
    video_coords_t coords;
-   video_coord_array_t *ca   = NULL;
-
-   ca = menu_display_get_coords_array();
+   menu_display_ctx_coord_draw_t coord_draw;
+   video_coord_array_t *ca = menu_display_get_coords_array();
 
    vertex[0]             = x1 / (float)width;
    vertex[1]             = y1 / (float)height;
@@ -712,8 +795,6 @@ void menu_display_push_quad(
    video_coord_array_append(ca, &coords, 3);
 }
 
-#define PARTICLES_COUNT            100
-
 void menu_display_snow(int width, int height)
 {
    struct display_particle
@@ -723,17 +804,10 @@ void menu_display_snow(int width, int height)
       float alpha;
       bool alive;
    };
-   static struct display_particle particles[PARTICLES_COUNT];
-   static bool initialized = false;
+   static struct display_particle particles[PARTICLES_COUNT] = {{0}};
    static int timeout      = 0;
    unsigned i, max_gen     = 2;
 
-   if (!initialized)
-   {
-      memset(particles, 0, sizeof(particles));
-      initialized = true;
-   }
-
    for (i = 0; i < PARTICLES_COUNT; ++i)
    {
       struct display_particle *p = (struct display_particle*)&particles[i];
@@ -746,7 +820,7 @@ void menu_display_snow(int width, int height)
          p->x            += menu_display_scalef(mouse_x, 0, width, -0.3, 0.3);
          p->x            += p->xspeed;
 
-         p->alive         = p->y >= 0 && p->y < height 
+         p->alive         = p->y >= 0 && p->y < height
             && p->x >= 0 && p->x < width;
       }
       else if (max_gen > 0 && timeout <= 0)
@@ -792,11 +866,37 @@ void menu_display_snow(int width, int height)
    }
 }
 
-void menu_display_draw_text(const char *msg, 
-      int width, int height, struct font_params *params)
+void menu_display_draw_text(
+      const font_data_t *font, const char *text,
+      float x, float y, int width, int height,
+      uint32_t color, enum text_alignment text_align,
+      float scale, bool shadows_enable, float shadow_offset)
 {
-   void *fb_buf = menu_display_get_font_buffer();
-   video_driver_set_osd_msg(msg, params, fb_buf);
+   struct font_params params;
+
+   /* Don't draw outside of the screen */
+   if (x < -64 || x > width + 64
+         || y < -64 || y > height + 64)
+      return;
+
+   params.x           = x / width;
+   params.y           = 1.0f - y / height;
+   params.scale       = scale;
+   params.drop_mod    = 0.0f;
+   params.drop_x      = 0.0f;
+   params.drop_y      = 0.0f;
+   params.color       = color;
+   params.full_screen = true;
+   params.text_align  = text_align;
+
+   if (shadows_enable)
+   {
+      params.drop_x      = shadow_offset;
+      params.drop_y      = -shadow_offset;
+      params.drop_alpha  = 0.35f;
+   }
+
+   video_driver_set_osd_msg(text, &params, (void*)font);
 }
 
 void menu_display_set_alpha(float *color, float alpha_value)
@@ -805,3 +905,30 @@ void menu_display_set_alpha(float *color, float alpha_value)
       return;
    color[3] = color[7] = color[11] = color[15] = alpha_value;
 }
+
+void menu_display_reset_textures_list(const char *texture_path, const char *iconpath,
+      uintptr_t *item)
+{
+   struct texture_image ti;
+   char path[PATH_MAX_LENGTH];
+
+   path[0]                     = '\0';
+
+   ti.width                    = 0;
+   ti.height                   = 0;
+   ti.pixels                   = NULL;
+   ti.supports_rgba            = video_driver_supports_rgba();
+
+   if (!string_is_empty(texture_path))
+      fill_pathname_join(path, iconpath, texture_path, sizeof(path));
+
+   if (string_is_empty(path) || !path_file_exists(path))
+      return;
+
+   if (!image_texture_load(&ti, path))
+      return;
+
+   video_driver_texture_load(&ti,
+         TEXTURE_FILTER_MIPMAP_LINEAR, item);
+   image_texture_free(&ti);
+}
