@@ -1,7 +1,8 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
- * 
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *  Copyright (C) 2016-2017 - Brad Parker
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -16,23 +17,29 @@
 
 #include <string.h>
 
+#include <compat/strl.h>
 #include <file/file_path.h>
 #include <string/stdstring.h>
 
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
 #include "menu_animation.h"
 #include "menu_driver.h"
 #include "menu_cbs.h"
 #include "menu_display.h"
+#include "menu_event.h"
 #include "menu_navigation.h"
+#include "widgets/menu_dialog.h"
+#include "widgets/menu_list.h"
 #include "menu_shader.h"
 
-#include "../config.def.h"
 #include "../content.h"
 #include "../configuration.h"
 #include "../dynamic.h"
-#include "../core_info.h"
+#include "../driver.h"
 #include "../retroarch.h"
-#include "../system.h"
 #include "../defaults.h"
 #include "../frontend/frontend.h"
 #include "../list_special.h"
@@ -69,15 +76,26 @@ static bool menu_driver_pending_quick_menu      = false;
 static bool menu_driver_prevent_populate        = false;
 static bool menu_driver_load_no_content         = false;
 static bool menu_driver_alive                   = false;
+static bool menu_driver_toggled                 = false;
 static bool menu_driver_data_own                = false;
 static bool menu_driver_pending_quit            = false;
 static bool menu_driver_pending_shutdown        = false;
+static bool menu_driver_is_binding              = false;
 static playlist_t *menu_driver_playlist         = NULL;
-static struct video_shader *menu_driver_shader  = NULL;
 static menu_handle_t *menu_driver_data          = NULL;
 static const menu_ctx_driver_t *menu_driver_ctx = NULL;
 static void *menu_userdata                      = NULL;
 
+bool menu_driver_is_binding_state(void)
+{
+   return menu_driver_is_binding;
+}
+
+void menu_driver_set_binding_state(bool on)
+{
+   menu_driver_is_binding = on;
+}
+
 /**
  * menu_driver_find_handle:
  * @idx              : index of driver to get handle to.
@@ -97,7 +115,7 @@ const void *menu_driver_find_handle(int idx)
  * menu_driver_find_ident:
  * @idx              : index of driver to get handle to.
  *
- * Returns: Human-readable identifier of menu driver at index. 
+ * Returns: Human-readable identifier of menu driver at index.
  * Can be NULL if nothing found.
  **/
 const char *menu_driver_find_ident(int idx)
@@ -122,7 +140,7 @@ const char *config_get_menu_driver_options(void)
    return char_list_new_special(STRING_LIST_MENU_DRIVERS, NULL);
 }
 
-#ifdef HAVE_ZLIB
+#ifdef HAVE_COMPRESSION
 static void bundle_decompressed(void *task_data,
       void *user_data, const char *err)
 {
@@ -142,7 +160,7 @@ static void bundle_decompressed(void *task_data,
       free(dec);
    }
 
-   settings->bundle_assets_extract_last_version = 
+   settings->bundle_assets_extract_last_version =
       settings->bundle_assets_extract_version_current;
    settings->bundle_finished = true;
 }
@@ -163,42 +181,35 @@ static bool menu_init(menu_handle_t *menu_data)
    if (!menu_entries_ctl(MENU_ENTRIES_CTL_INIT, NULL))
       return false;
 
-   if (!core_info_init_current_core())
-      return false;
-
-   if (!menu_driver_ctl(RARCH_MENU_CTL_SHADER_INIT, NULL))
-      return false;
-
    if (settings->menu_show_start_screen)
    {
-      menu_data->push_help_screen = true;
-      menu_data->help_screen_type = MENU_HELP_WELCOME;
+      menu_dialog_push_pending(true, MENU_DIALOG_WELCOME);
       settings->menu_show_start_screen   = false;
-      command_event(CMD_EVENT_MENU_SAVE_CURRENT_CONFIG, NULL);
+      if (settings->config_save_on_exit)
+         command_event(CMD_EVENT_MENU_SAVE_CURRENT_CONFIG, NULL);
    }
 
    if (      settings->bundle_assets_extract_enable
-         && !string_is_empty(settings->path.bundle_assets_src) 
+         && !string_is_empty(settings->path.bundle_assets_src)
          && !string_is_empty(settings->path.bundle_assets_dst)
 #ifdef IOS
-         && menu_data->push_help_screen
+         && menu_dialog_is_push_pending()
 #else
-         && (settings->bundle_assets_extract_version_current 
+         && (settings->bundle_assets_extract_version_current
             != settings->bundle_assets_extract_last_version)
 #endif
       )
    {
-      menu_data->help_screen_type           = MENU_HELP_EXTRACT;
-      menu_data->push_help_screen           = true;
-#ifdef HAVE_ZLIB
-      task_push_decompress(settings->path.bundle_assets_src, 
+      menu_dialog_push_pending(true, MENU_DIALOG_HELP_EXTRACT);
+#ifdef HAVE_COMPRESSION
+      task_push_decompress(settings->path.bundle_assets_src,
             settings->path.bundle_assets_dst,
             NULL, settings->path.bundle_assets_dst_subdir,
             NULL, bundle_decompressed, NULL);
 #endif
    }
 
-   menu_driver_ctl(RARCH_MENU_CTL_SHADER_MANAGER_INIT, NULL);
+   menu_shader_manager_init();
 
    if (!menu_display_init())
       return false;
@@ -213,27 +224,35 @@ static void menu_input_key_event(bool down, unsigned keycode,
    (void)keycode;
    (void)mod;
 
-   if (character == '/')
-      menu_entry_action(NULL, 0, MENU_ACTION_SEARCH);
+#if 0
+   RARCH_LOG("down: %d, keycode: %d, mod: %d, character: %d\n", down, keycode, mod, character);
+#endif
+
+   menu_event_kb_set(down, (enum retro_key)keycode);
 }
 
-static void menu_driver_toggle(bool latch)
+static void menu_driver_toggle(bool on)
 {
    retro_keyboard_event_t *key_event          = NULL;
    retro_keyboard_event_t *frontend_key_event = NULL;
    settings_t                 *settings       = config_get_ptr();
-   
-   menu_driver_ctl(RARCH_MENU_CTL_TOGGLE, &latch);
 
-   if (latch)
-      menu_driver_ctl(RARCH_MENU_CTL_SET_ALIVE, NULL);
+   menu_driver_toggled = on;
+
+   if (!on)
+      menu_display_toggle_set_reason(MENU_TOGGLE_REASON_NONE);
+
+   menu_driver_ctl(RARCH_MENU_CTL_TOGGLE, &on);
+
+   if (on)
+      menu_driver_alive = true;
    else
-      menu_driver_ctl(RARCH_MENU_CTL_UNSET_ALIVE, NULL);
+      menu_driver_alive = false;
 
    runloop_ctl(RUNLOOP_CTL_FRONTEND_KEY_EVENT_GET, &frontend_key_event);
    runloop_ctl(RUNLOOP_CTL_KEY_EVENT_GET,          &key_event);
 
-   if (menu_driver_ctl(RARCH_MENU_CTL_IS_ALIVE, NULL))
+   if (menu_driver_alive)
    {
       bool refresh = false;
       menu_entries_ctl(MENU_ENTRIES_CTL_SET_REFRESH, &refresh);
@@ -260,7 +279,7 @@ static void menu_driver_toggle(bool latch)
    else
    {
       if (!runloop_ctl(RUNLOOP_CTL_IS_SHUTDOWN, NULL))
-         driver_ctl(RARCH_DRIVER_CTL_SET_NONBLOCK_STATE, NULL);
+         driver_set_nonblock_state();
 
       if (settings && settings->menu.pause_libretro)
          command_event(CMD_EVENT_AUDIO_START, NULL);
@@ -283,6 +302,82 @@ const char *menu_driver_ident(void)
   return menu_driver_ctx->ident;
 }
 
+void menu_driver_frame(video_frame_info_t *video_info)
+{
+   if (menu_driver_alive && menu_driver_ctx->frame)
+      menu_driver_ctx->frame(menu_userdata, video_info);
+}
+
+/**
+ * menu_update_libretro_info:
+ *
+ * Update menu state which depends on config.
+ **/
+static void menu_update_libretro_info(void)
+{
+   struct retro_system_info *info = NULL;
+
+   menu_driver_ctl(RARCH_MENU_CTL_SYSTEM_INFO_GET,
+         &info);
+
+   if (!info)
+      return;
+
+   command_event(CMD_EVENT_CORE_INFO_INIT, NULL);
+   command_event(CMD_EVENT_LOAD_CORE_PERSIST, NULL);
+}
+
+bool menu_driver_render(bool is_idle)
+{
+   if (!menu_driver_data)
+      return false;
+
+   if (BIT64_GET(menu_driver_data->state, MENU_STATE_RENDER_FRAMEBUFFER)
+         != BIT64_GET(menu_driver_data->state, MENU_STATE_RENDER_MESSAGEBOX))
+      BIT64_SET(menu_driver_data->state, MENU_STATE_RENDER_FRAMEBUFFER);
+
+   if (BIT64_GET(menu_driver_data->state, MENU_STATE_RENDER_FRAMEBUFFER))
+      menu_display_set_framebuffer_dirty_flag();
+
+   if (BIT64_GET(menu_driver_data->state, MENU_STATE_RENDER_MESSAGEBOX)
+         && !string_is_empty(menu_driver_data->menu_state.msg))
+   {
+      if (menu_driver_ctx->render_messagebox)
+         menu_driver_ctx->render_messagebox(menu_userdata,
+               menu_driver_data->menu_state.msg);
+
+      if (ui_companion_is_on_foreground())
+      {
+         const ui_companion_driver_t *ui = ui_companion_get_ptr();
+         if (ui->render_messagebox)
+            ui->render_messagebox(menu_driver_data->menu_state.msg);
+      }
+   }
+
+   if (BIT64_GET(menu_driver_data->state, MENU_STATE_BLIT))
+   {
+      settings_t *settings = config_get_ptr();
+      menu_animation_update_time(settings->menu.timedate_enable);
+
+      if (menu_driver_ctx->render)
+         menu_driver_ctx->render(menu_userdata);
+   }
+
+   if (menu_driver_alive && !is_idle)
+      menu_display_libretro();
+
+   menu_driver_ctl(RARCH_MENU_CTL_SET_TEXTURE, NULL);
+
+   menu_driver_data->state               = 0;
+
+   return true;
+}
+
+bool menu_driver_is_alive(void)
+{
+   return menu_driver_alive;
+}
+
 bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
 {
    switch (state)
@@ -300,25 +395,16 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
       case RARCH_MENU_CTL_SET_PENDING_QUICK_MENU:
          menu_driver_pending_quick_menu = true;
          break;
-      case RARCH_MENU_CTL_UNSET_PENDING_QUICK_MENU:
-         menu_driver_pending_quick_menu = false;
-         break;
       case RARCH_MENU_CTL_IS_PENDING_QUIT:
          return menu_driver_pending_quit;
       case RARCH_MENU_CTL_SET_PENDING_QUIT:
          menu_driver_pending_quit     = true;
          break;
-      case RARCH_MENU_CTL_UNSET_PENDING_QUIT:
-         menu_driver_pending_quit     = false;
-         break;
       case RARCH_MENU_CTL_IS_PENDING_SHUTDOWN:
          return menu_driver_pending_shutdown;
       case RARCH_MENU_CTL_SET_PENDING_SHUTDOWN:
          menu_driver_pending_shutdown = true;
          break;
-      case RARCH_MENU_CTL_UNSET_PENDING_SHUTDOWN:
-         menu_driver_pending_shutdown = false;
-         break;
       case RARCH_MENU_CTL_DESTROY:
          menu_driver_pending_quick_menu = false;
          menu_driver_pending_quit       = false;
@@ -391,7 +477,7 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
          break;
       case RARCH_MENU_CTL_SYSTEM_INFO_GET:
          {
-            struct retro_system_info **system = 
+            struct retro_system_info **system =
                (struct retro_system_info**)data;
             if (!system)
                return false;
@@ -407,82 +493,6 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
             memset(&menu_driver_system, 0, sizeof(struct retro_system_info));
          }
          break;
-      case RARCH_MENU_CTL_RENDER_MESSAGEBOX:
-         if (menu_driver_ctx->render_messagebox)
-            menu_driver_ctx->render_messagebox(menu_userdata,
-                  menu_driver_data->menu_state.msg);
-         break;
-      case RARCH_MENU_CTL_BLIT_RENDER:
-         if (menu_driver_ctx->render)
-            menu_driver_ctx->render(menu_userdata);
-         break;
-      case RARCH_MENU_CTL_RENDER:
-         if (!menu_driver_data)
-            return false;
-
-         if (BIT64_GET(menu_driver_data->state, MENU_STATE_RENDER_FRAMEBUFFER) 
-               != BIT64_GET(menu_driver_data->state, MENU_STATE_RENDER_MESSAGEBOX))
-            BIT64_SET(menu_driver_data->state, MENU_STATE_RENDER_FRAMEBUFFER);
-
-         if (BIT64_GET(menu_driver_data->state, MENU_STATE_RENDER_FRAMEBUFFER))
-            menu_display_set_framebuffer_dirty_flag();
-
-         if (BIT64_GET(menu_driver_data->state, MENU_STATE_RENDER_MESSAGEBOX) 
-               && !string_is_empty(menu_driver_data->menu_state.msg))
-         {
-            menu_driver_ctl(RARCH_MENU_CTL_RENDER_MESSAGEBOX, NULL);
-
-            if (ui_companion_is_on_foreground())
-            {
-               const ui_companion_driver_t *ui = ui_companion_get_ptr();
-               if (ui->render_messagebox)
-                  ui->render_messagebox(menu_driver_data->menu_state.msg);
-            }
-         }
-
-         if (BIT64_GET(menu_driver_data->state, MENU_STATE_BLIT))
-         {
-            menu_animation_ctl(MENU_ANIMATION_CTL_UPDATE_TIME, NULL);
-            menu_driver_ctl(RARCH_MENU_CTL_BLIT_RENDER, NULL);
-         }
-
-         if (menu_driver_ctl(RARCH_MENU_CTL_IS_ALIVE, NULL) 
-               && !runloop_ctl(RUNLOOP_CTL_IS_IDLE, NULL))
-            menu_display_libretro();
-
-         menu_driver_ctl(RARCH_MENU_CTL_SET_TEXTURE, NULL);
-
-         menu_driver_data->state               = 0;
-         break;
-      case RARCH_MENU_CTL_SHADER_DEINIT:
-#ifdef HAVE_SHADER_MANAGER
-         if (menu_driver_shader)
-            free(menu_driver_shader);
-         menu_driver_shader = NULL;
-#endif
-         break;
-      case RARCH_MENU_CTL_SHADER_INIT:
-#ifdef HAVE_SHADER_MANAGER
-         menu_driver_shader = (struct video_shader*)
-            calloc(1, sizeof(struct video_shader));
-         if (!menu_driver_shader)
-            return false;
-#endif
-         break;
-      case RARCH_MENU_CTL_SHADER_GET:
-         {
-            struct video_shader **shader = (struct video_shader**)data;
-            if (!shader)
-               return false;
-            *shader = menu_driver_shader;
-         }
-         break;
-      case RARCH_MENU_CTL_FRAME:
-         if (!menu_driver_alive)
-            return false;
-         if (menu_driver_ctx->frame)
-            menu_driver_ctx->frame(menu_userdata);
-         break;
       case RARCH_MENU_CTL_SET_PREVENT_POPULATE:
          menu_driver_prevent_populate = true;
          break;
@@ -491,20 +501,14 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
          break;
       case RARCH_MENU_CTL_IS_PREVENT_POPULATE:
          return menu_driver_prevent_populate;
+      case RARCH_MENU_CTL_IS_TOGGLE:
+         return menu_driver_toggled;
       case RARCH_MENU_CTL_SET_TOGGLE:
          menu_driver_toggle(true);
          break;
       case RARCH_MENU_CTL_UNSET_TOGGLE:
          menu_driver_toggle(false);
          break;
-      case RARCH_MENU_CTL_SET_ALIVE:
-         menu_driver_alive = true;
-         break;
-      case RARCH_MENU_CTL_UNSET_ALIVE:
-         menu_driver_alive = false;
-         break;
-      case RARCH_MENU_CTL_IS_ALIVE:
-         return menu_driver_alive;
       case RARCH_MENU_CTL_SET_OWN_DRIVER:
          menu_driver_data_own = true;
          break;
@@ -526,7 +530,8 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
          if (menu_driver_ctl(RARCH_MENU_CTL_OWNS_DRIVER, NULL))
             return true;
          menu_driver_ctl(RARCH_MENU_CTL_PLAYLIST_FREE, NULL);
-         menu_driver_ctl(RARCH_MENU_CTL_SHADER_DEINIT, NULL);
+         menu_shader_manager_free();
+
          if (menu_driver_data)
          {
             menu_input_ctl(MENU_INPUT_CTL_DEINIT, NULL);
@@ -545,8 +550,7 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
 
             command_event(CMD_EVENT_HISTORY_DEINIT, NULL);
 
-            core_info_deinit_list();
-            core_info_free_current_core();
+            menu_dialog_reset();
 
             free(menu_driver_data);
          }
@@ -555,6 +559,7 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
       case RARCH_MENU_CTL_INIT:
          {
             settings_t *settings  = config_get_ptr();
+            menu_update_libretro_info();
             if (menu_driver_data)
                return true;
 
@@ -739,12 +744,12 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
          break;
       case RARCH_MENU_CTL_TOGGLE:
          {
-            bool *latch = (bool*)data;
-            if (!latch)
+            bool *on = (bool*)data;
+            if (!on)
                return false;
 
             if (menu_driver_ctx && menu_driver_ctx->toggle)
-               menu_driver_ctx->toggle(menu_userdata, *latch);
+               menu_driver_ctx->toggle(menu_userdata, *on);
          }
          break;
       case RARCH_MENU_CTL_REFRESH:
@@ -768,9 +773,6 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
             return false;
          menu_driver_ctx->context_destroy(menu_userdata);
          break;
-      case RARCH_MENU_CTL_SHADER_MANAGER_INIT:
-         menu_shader_manager_init(menu_driver_data);
-         break;
       case RARCH_MENU_CTL_LIST_SET_SELECTION:
          {
             file_list_t *list = (file_list_t*)data;
@@ -799,12 +801,13 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
             if (!list || !menu_driver_ctx || !menu_driver_ctx->list_insert)
                return false;
             menu_driver_ctx->list_insert(menu_userdata,
-                  list->list, list->path, list->label, list->idx);
+                  list->list, list->path, list->fullpath,
+                  list->label, list->idx);
          }
          break;
       case RARCH_MENU_CTL_LOAD_IMAGE:
          {
-            menu_ctx_load_image_t *load_image_info = 
+            menu_ctx_load_image_t *load_image_info =
                (menu_ctx_load_image_t*)data;
             if (!menu_driver_ctx || !menu_driver_ctx->load_image)
                return false;
@@ -817,7 +820,7 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
 
             if (menu_driver_ctl(RARCH_MENU_CTL_IS_PENDING_QUICK_MENU, NULL))
             {
-               menu_driver_ctl(RARCH_MENU_CTL_UNSET_PENDING_QUICK_MENU, NULL);
+               menu_driver_pending_quick_menu = false;
                menu_entries_flush_stack(NULL, MENU_SETTINGS);
                menu_display_set_msg_force(true);
 
@@ -826,7 +829,7 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
 
                if (menu_driver_ctl(RARCH_MENU_CTL_IS_PENDING_QUIT, NULL))
                {
-                  menu_driver_ctl(RARCH_MENU_CTL_UNSET_PENDING_QUIT, NULL);
+                  menu_driver_pending_quit     = false;
                   return false;
                }
 
@@ -835,19 +838,19 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
 
             if (menu_driver_ctl(RARCH_MENU_CTL_IS_PENDING_QUIT, NULL))
             {
-               menu_driver_ctl(RARCH_MENU_CTL_UNSET_PENDING_QUIT, NULL);
+               menu_driver_pending_quit     = false;
                return false;
             }
             if (menu_driver_ctl(RARCH_MENU_CTL_IS_PENDING_SHUTDOWN, NULL))
             {
-               menu_driver_ctl(RARCH_MENU_CTL_UNSET_PENDING_SHUTDOWN, NULL);
+               menu_driver_pending_shutdown = false;
                if (!command_event(CMD_EVENT_QUIT, NULL))
                   return false;
                return true;
             }
             if (!menu_driver_ctx || !menu_driver_ctx->iterate)
                return false;
-             
+
             if (menu_driver_ctx->iterate(menu_driver_data,
                      menu_userdata, iterate->action) == -1)
                return false;
@@ -855,7 +858,7 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
          break;
       case RARCH_MENU_CTL_ENVIRONMENT:
          {
-            menu_ctx_environment_t *menu_environ = 
+            menu_ctx_environment_t *menu_environ =
                (menu_ctx_environment_t*)data;
 
             if (menu_driver_ctx->environ_cb)
@@ -879,6 +882,21 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
                   point->cbs, point->entry, point->action);
          }
          break;
+      case RARCH_MENU_CTL_OSK_PTR_AT_POS:
+         {
+            unsigned width            = 0;
+            unsigned height           = 0;
+            menu_ctx_pointer_t *point = (menu_ctx_pointer_t*)data;
+            if (!menu_driver_ctx || !menu_driver_ctx->osk_ptr_at_pos)
+            {
+               point->retcode = 0;
+               return false;
+            }
+            video_driver_get_size(&width, &height);
+            point->retcode = menu_driver_ctx->osk_ptr_at_pos(menu_userdata,
+                  point->x, point->y, width, height);
+         }
+         break;
       case RARCH_MENU_CTL_BIND_INIT:
          {
             menu_ctx_bind_t *bind = (menu_ctx_bind_t*)data;
@@ -914,6 +932,24 @@ bool menu_driver_ctl(enum rarch_menu_ctl_state state, void *data)
             menu_driver_ctx->update_thumbnail_image(menu_userdata);
          }
          break;
+      case RARCH_MENU_CTL_UPDATE_SAVESTATE_THUMBNAIL_PATH:
+         {
+            size_t selection;
+            if (!menu_navigation_ctl(MENU_NAVIGATION_CTL_GET_SELECTION, &selection))
+               return false;
+
+            if (!menu_driver_ctx || !menu_driver_ctx->update_savestate_thumbnail_path)
+               return false;
+            menu_driver_ctx->update_savestate_thumbnail_path(menu_userdata, selection);
+         }
+         break;
+      case RARCH_MENU_CTL_UPDATE_SAVESTATE_THUMBNAIL_IMAGE:
+         {
+            if (!menu_driver_ctx || !menu_driver_ctx->update_savestate_thumbnail_image)
+               return false;
+            menu_driver_ctx->update_savestate_thumbnail_image(menu_userdata);
+         }
+         break;
       default:
       case RARCH_MENU_CTL_NONE:
          break;
