@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2014-2015 - Jay McCarthy
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -17,110 +17,19 @@
 #include <string.h>
 
 #include <retro_inline.h>
+#include <compat/strl.h>
 #include <string/stdstring.h>
 
 #include "menu_driver.h"
 #include "menu_cbs.h"
 #include "menu_navigation.h"
 
-#include "../general.h"
-#include "../system.h"
+#include "widgets/menu_list.h"
 
-struct menu_list
-{
-   file_list_t **menu_stack;
-   size_t menu_stack_size;
-   file_list_t **selection_buf;
-   size_t selection_buf_size;
-};
-
-static void menu_list_free_list(file_list_t *list)
-{
-   unsigned i;
-
-   for (i = 0; i < list->size; i++)
-   {
-      menu_ctx_list_t list_info;
-
-      list_info.list      = list;
-      list_info.idx       = i;
-      list_info.list_size = list->size;
-
-      menu_driver_ctl(RARCH_MENU_CTL_LIST_FREE, &list_info);
-   }
-
-   file_list_free(list);
-}
-
-static void menu_list_free(menu_list_t *menu_list)
-{
-   unsigned i;
-   if (!menu_list)
-      return;
-
-   for (i = 0; i < menu_list->menu_stack_size; i++)
-   {
-      if (!menu_list->menu_stack[i])
-         continue;
-
-      menu_list_free_list(menu_list->menu_stack[i]);
-      menu_list->menu_stack[i]    = NULL;
-   }
-   for (i = 0; i < menu_list->selection_buf_size; i++)
-   {
-      if (!menu_list->selection_buf[i])
-         continue;
-
-      menu_list_free_list(menu_list->selection_buf[i]);
-      menu_list->selection_buf[i] = NULL;
-   }
-
-   free(menu_list->menu_stack);
-   free(menu_list->selection_buf);
-
-   free(menu_list);
-}
-
-static menu_list_t *menu_list_new(void)
-{
-   unsigned i;
-   menu_list_t           *list = (menu_list_t*)calloc(1, sizeof(*list));
-
-   if (!list)
-      return NULL;
-
-   list->menu_stack          = (file_list_t**)calloc(1, sizeof(*list->menu_stack));
-
-   if (!list->menu_stack)
-      goto error;
-
-   list->selection_buf       = (file_list_t**)calloc(1, sizeof(*list->selection_buf));
-
-   if (!list->selection_buf)
-      goto error;
-
-   list->menu_stack_size      = 1;
-   list->selection_buf_size   = 1;
-
-   for (i = 0; i < list->menu_stack_size; i++)
-      list->menu_stack[i]    = (file_list_t*)calloc(1, sizeof(*list->menu_stack[i]));
-
-   for (i = 0; i < list->selection_buf_size; i++)
-      list->selection_buf[i] = (file_list_t*)calloc(1, sizeof(*list->selection_buf[i]));
-
-   return list;
-
-error:
-   menu_list_free(list);
-   return NULL;
-}
-
-static size_t menu_list_get_stack_size(menu_list_t *list, size_t idx)
-{
-   if (!list)
-      return 0;
-   return file_list_get_size(list->menu_stack[idx]);
-}
+#include "../core.h"
+#include "../configuration.h"
+#include "../runloop.h"
+#include "../version.h"
 
 void menu_entries_get_at_offset(const file_list_t *list, size_t idx,
       const char **path, const char **label, unsigned *file_type,
@@ -154,85 +63,6 @@ menu_file_list_cbs_t *menu_entries_get_actiondata_at_offset(
       file_list_get_actiondata_at_offset(list, idx);
 }
 
-static int menu_entries_flush_stack_type(const char *needle, const char *label,
-      unsigned type, unsigned final_type)
-{
-   return needle ? strcmp(needle, label) : (type != final_type);
-}
-
-static bool menu_list_pop_stack(menu_list_t *list,
-      size_t idx, size_t *directory_ptr, bool animate)
-{
-   menu_ctx_list_t list_info;
-   bool refresh           = false;
-   file_list_t *menu_list = NULL;
-   if (!list)
-      return false;
-
-   menu_list = list->menu_stack[idx];
-
-   if (menu_list_get_stack_size(list, idx) <= 1)
-      return false;
-
-   list_info.type   = MENU_LIST_PLAIN;
-   list_info.action = 0;
-
-   if (animate)
-      menu_driver_ctl(RARCH_MENU_CTL_LIST_CACHE, &list_info);
-
-   if (menu_list->size != 0)
-   {
-      menu_ctx_list_t list_info;
-
-      list_info.list      = menu_list;
-      list_info.idx       = menu_list->size - 1;
-      list_info.list_size = menu_list->size - 1;
-
-      menu_driver_ctl(RARCH_MENU_CTL_LIST_FREE, &list_info);
-   }
-
-   file_list_pop(menu_list, directory_ptr);
-   menu_driver_ctl(RARCH_MENU_CTL_LIST_SET_SELECTION, menu_list);
-   if (animate)
-      menu_entries_ctl(MENU_ENTRIES_CTL_SET_REFRESH, &refresh);
-
-   return true;
-}
-
-static void menu_list_flush_stack(menu_list_t *list,
-      size_t idx, const char *needle, unsigned final_type)
-{
-   bool refresh           = false;
-   const char *path       = NULL;
-   const char *label      = NULL;
-   unsigned type          = 0;
-   size_t entry_idx       = 0;
-   if (!list)
-      return;
-
-   menu_entries_ctl(MENU_ENTRIES_CTL_SET_REFRESH, &refresh);
-   menu_entries_get_last(list->menu_stack[idx],
-         &path, &label, &type, &entry_idx);
-
-   while (menu_entries_flush_stack_type(
-            needle, label, type, final_type) != 0)
-   {
-      size_t new_selection_ptr;
-
-      menu_navigation_ctl(MENU_NAVIGATION_CTL_GET_SELECTION,
-            &new_selection_ptr);
-
-      if (!menu_list_pop_stack(list, idx, &new_selection_ptr, 1))
-         break;
-
-      menu_navigation_ctl(MENU_NAVIGATION_CTL_SET_SELECTION,
-            &new_selection_ptr);
-
-      menu_entries_get_last(list->menu_stack[idx],
-            &path, &label, &type, &entry_idx);
-   }
-}
-
 static bool menu_entries_clear(file_list_t *list)
 {
    unsigned i;
@@ -382,14 +212,15 @@ size_t menu_entries_get_end(void)
 }
 
 /* Get an entry from the top of the menu stack */
-void menu_entries_get(size_t i, menu_entry_t *entry)
+void menu_entries_get(size_t i, void *entry_data)
 {
    const char *label             = NULL;
    const char *path              = NULL;
    const char *entry_label       = NULL;
    menu_file_list_cbs_t *cbs     = NULL;
    enum msg_hash_enums enum_idx  = MSG_UNKNOWN;
-   file_list_t *selection_buf = menu_entries_get_selection_buf_ptr(0);
+   menu_entry_t *entry           = (menu_entry_t*)entry_data;
+   file_list_t *selection_buf    = menu_entries_get_selection_buf_ptr(0);
 
    menu_entries_get_last_stack(NULL, &label, NULL, &enum_idx, NULL);
 
@@ -476,7 +307,7 @@ file_list_t *menu_entries_get_menu_stack_ptr(size_t idx)
    menu_entries_ctl(MENU_ENTRIES_CTL_LIST_GET, &menu_list);
    if (!menu_list)
       return NULL;
-   return menu_list->menu_stack[idx];
+   return menu_list_get(menu_list, idx);
 }
 
 file_list_t *menu_entries_get_selection_buf_ptr(size_t idx)
@@ -485,7 +316,7 @@ file_list_t *menu_entries_get_selection_buf_ptr(size_t idx)
    menu_entries_ctl(MENU_ENTRIES_CTL_LIST_GET, &menu_list);
    if (!menu_list)
       return NULL;
-   return menu_list->selection_buf[idx];
+   return menu_list_get_selection(menu_list, idx);
 }
 
 static bool menu_entries_init(void)
@@ -510,21 +341,32 @@ void menu_entries_append(file_list_t *list, const char *path, const char *label,
 {
    menu_ctx_list_t list_info;
    size_t idx;
+   const char *menu_path           = NULL;
    menu_file_list_cbs_t *cbs       = NULL;
    if (!list || !label)
       return;
 
    file_list_append(list, path, label, type, directory_ptr, entry_idx);
 
-   idx              = list->size - 1;
+   menu_entries_get_last_stack(&menu_path, NULL, NULL, NULL, NULL);
 
-   list_info.list   = list;
-   list_info.path   = path;
-   list_info.label  = label;
-   list_info.idx    = idx;
+   idx                = list->size - 1;
+
+   list_info.list     = list;
+   list_info.path     = path;
+   list_info.fullpath = NULL;
+
+   if (!string_is_empty(menu_path))
+      list_info.fullpath = strdup(menu_path);
+
+   list_info.label    = label;
+   list_info.idx      = idx;
 
    menu_driver_ctl(RARCH_MENU_CTL_LIST_INSERT, &list_info);
 
+   if (list_info.fullpath)
+      free(list_info.fullpath);
+
    file_list_free_actiondata(list, idx);
    cbs = (menu_file_list_cbs_t*)
       calloc(1, sizeof(menu_file_list_cbs_t));
@@ -546,21 +388,31 @@ void menu_entries_append_enum(file_list_t *list, const char *path, const char *l
 {
    menu_ctx_list_t list_info;
    size_t idx;
+   const char *menu_path           = NULL;
    menu_file_list_cbs_t *cbs       = NULL;
    if (!list || !label)
       return;
 
    file_list_append(list, path, label, type, directory_ptr, entry_idx);
 
-   idx              = list->size - 1;
+   menu_entries_get_last_stack(&menu_path, NULL, NULL, NULL, NULL);
 
-   list_info.list   = list;
-   list_info.path   = path;
-   list_info.label  = label;
-   list_info.idx    = idx;
+   idx                   = list->size - 1;
+
+   list_info.fullpath    = NULL;
+
+   if (!string_is_empty(menu_path))
+      list_info.fullpath = strdup(menu_path);
+   list_info.list        = list;
+   list_info.path        = path;
+   list_info.label       = label;
+   list_info.idx         = idx;
 
    menu_driver_ctl(RARCH_MENU_CTL_LIST_INSERT, &list_info);
 
+   if (list_info.fullpath)
+      free(list_info.fullpath);
+
    file_list_free_actiondata(list, idx);
    cbs = (menu_file_list_cbs_t*)
       calloc(1, sizeof(menu_file_list_cbs_t));
@@ -582,21 +434,31 @@ void menu_entries_prepend(file_list_t *list, const char *path, const char *label
 {
    menu_ctx_list_t list_info;
    size_t idx;
+   const char *menu_path           = NULL;
    menu_file_list_cbs_t *cbs       = NULL;
    if (!list || !label)
       return;
 
    file_list_prepend(list, path, label, type, directory_ptr, entry_idx);
 
+   menu_entries_get_last_stack(&menu_path, NULL, NULL, NULL, NULL);
+
    idx              = 0;
 
-   list_info.list   = list;
-   list_info.path   = path;
-   list_info.label  = label;
-   list_info.idx    = idx;
+   list_info.fullpath    = NULL;
+
+   if (!string_is_empty(menu_path))
+      list_info.fullpath = strdup(menu_path);
+   list_info.list        = list;
+   list_info.path        = path;
+   list_info.label       = label;
+   list_info.idx         = idx;
 
    menu_driver_ctl(RARCH_MENU_CTL_LIST_INSERT, &list_info);
 
+   if (list_info.fullpath)
+      free(list_info.fullpath);
+
    file_list_free_actiondata(list, idx);
    cbs = (menu_file_list_cbs_t*)
       calloc(1, sizeof(menu_file_list_cbs_t));
@@ -619,7 +481,7 @@ menu_file_list_cbs_t *menu_entries_get_last_stack_actiondata(void)
    if (!menu_list)
       return NULL;
    return (menu_file_list_cbs_t*)file_list_get_last_actiondata(
-         menu_list->menu_stack[0]);
+         menu_list_get(menu_list, 0));
 }
 
 void menu_entries_get_last_stack(const char **path, const char **label,
@@ -631,10 +493,10 @@ void menu_entries_get_last_stack(const char **path, const char **label,
    if (!menu_list)
       return;
 
-   menu_entries_get_last(menu_list->menu_stack[0],
+   menu_entries_get_last(menu_list_get(menu_list, 0),
          path, label, file_type, entry_idx);
    cbs = menu_entries_get_last_stack_actiondata();
-   if (cbs)
+   if (cbs && enum_idx)
       *enum_idx = cbs->enum_idx;
 }
 
@@ -669,7 +531,7 @@ size_t menu_entries_get_size(void)
    menu_entries_ctl(MENU_ENTRIES_CTL_LIST_GET, &menu_list);
    if (!menu_list)
       return 0;
-   return file_list_get_size(menu_list->selection_buf[0]);
+   return file_list_get_size(menu_list_get_selection(menu_list, 0));
 }
 
 rarch_setting_t *menu_entries_get_setting(uint32_t i)
