@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -564,7 +564,7 @@ extern int audioAddData(uint32_t portNum, float *data, uint32_t frames, float vo
 	NET PROTOTYPES
 ============================================================ */
 
-#if defined(HAVE_NETPLAY)
+#if defined(HAVE_NETWORKING)
 #ifdef __PSL1GHT__
 #include <net/net.h>
 
