@@ -1,5 +1,5 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -15,6 +15,8 @@
 
 #include <stdlib.h>
 
+#include <compat/strl.h>
+#include <compat/posix_string.h>
 #include <retro_miscellaneous.h>
 #include <file/file_path.h>
 #include <file/config_file.h>
@@ -30,7 +32,8 @@
 #include "../configuration.h"
 #include "../verbosity.h"
 
-typedef struct {
+typedef struct
+{
    enum overlay_status state;
    enum overlay_image_transfer_status loading_status;
    config_file_t *conf;
@@ -44,11 +47,9 @@ typedef struct {
 
 } overlay_loader_t;
 
-static void task_overlay_resolve_iterate(retro_task_t *task);
-
 static void task_overlay_image_done(struct overlay *overlay)
 {
-   overlay->pos = 0;
+   overlay->pos           = 0;
    /* Divide iteration steps by half of total descs if size is even,
     * otherwise default to 8 (arbitrary value for now to speed things up). */
    overlay->pos_increment = (overlay->size / 2) ? (overlay->size / 2) : 8;
@@ -60,10 +61,13 @@ static void task_overlay_load_desc_image(
       struct overlay *input_overlay,
       unsigned ol_idx, unsigned desc_idx)
 {
-   char overlay_desc_image_key[64]  = {0};
-   char image_path[PATH_MAX_LENGTH] = {0};
+   char overlay_desc_image_key[64];
+   char image_path[PATH_MAX_LENGTH];
    config_file_t              *conf = loader->conf;
 
+   overlay_desc_image_key[0]        = '\0';
+   image_path[0]                    = '\0';
+
    snprintf(overlay_desc_image_key, sizeof(overlay_desc_image_key),
          "overlay%u_desc%u_overlay", ol_idx, desc_idx);
 
@@ -71,10 +75,15 @@ static void task_overlay_load_desc_image(
             image_path, sizeof(image_path)))
    {
       struct texture_image image_tex;
-      char path[PATH_MAX_LENGTH] = {0};
+      char path[PATH_MAX_LENGTH];
+
+      path[0] = '\0';
+
       fill_pathname_resolve_relative(path, loader->overlay_path,
             image_path, sizeof(path));
 
+      image_tex.supports_rgba = video_driver_supports_rgba();
+
       if (image_texture_load(&image_tex, path))
       {
          input_overlay->load_images[input_overlay->load_images_size++] = image_tex;
@@ -97,13 +106,13 @@ static bool task_overlay_load_desc(
    float width_mod, height_mod;
    float tmp_float;
    uint32_t box_hash, key_hash;
+   char overlay_desc_key[64];
+   char conf_key[64];
+   char overlay_desc_normalized_key[64];
+   char overlay[256];
    bool tmp_bool                        = false;
    bool ret                             = true;
    bool by_pixel                        = false;
-   char overlay_desc_key[64]            = {0};
-   char conf_key[64]                    = {0};
-   char overlay_desc_normalized_key[64] = {0};
-   char overlay[256]                    = {0};
    char *key                            = NULL;
    struct string_list *list             = NULL;
    const char *x                        = NULL;
@@ -111,6 +120,9 @@ static bool task_overlay_load_desc(
    const char *box                      = NULL;
    config_file_t *conf                  = loader->conf;
 
+   overlay_desc_key[0] = conf_key[0] = 
+      overlay_desc_normalized_key[0] = overlay[0] = '\0';
+
    snprintf(overlay_desc_key, sizeof(overlay_desc_key),
          "overlay%u_desc%u", ol_idx, desc_idx);
 
@@ -233,7 +245,9 @@ static bool task_overlay_load_desc(
       case OVERLAY_TYPE_ANALOG_LEFT:
       case OVERLAY_TYPE_ANALOG_RIGHT:
          {
-            char overlay_analog_saturate_key[64] = {0};
+            char overlay_analog_saturate_key[64];
+
+            overlay_analog_saturate_key[0] = '\0';
 
             if (desc->hitbox != OVERLAY_HITBOX_RADIAL)
             {
@@ -298,6 +312,88 @@ end:
    return ret;
 }
 
+static ssize_t task_overlay_find_index(const struct overlay *ol,
+      const char *name, size_t size)
+{
+   size_t i;
+
+   if (!ol)
+      return -1;
+
+   for (i = 0; i < size; i++)
+   {
+      if (string_is_equal(ol[i].name, name))
+         return i;
+   }
+
+   return -1;
+}
+
+static bool task_overlay_resolve_targets(struct overlay *ol,
+      size_t idx, size_t size)
+{
+   unsigned i;
+   struct overlay *current = (struct overlay*)&ol[idx];
+
+   for (i = 0; i < current->size; i++)
+   {
+      ssize_t next_idx  = 0;
+      const char *next = current->descs[i].next_index_name;
+
+      if (!string_is_empty(next))
+      {
+         next_idx = task_overlay_find_index(ol, next, size);
+
+         if (next_idx < 0)
+         {
+            RARCH_ERR("[Overlay]: Couldn't find overlay called: \"%s\".\n",
+                  next);
+            return false;
+         }
+      }
+      else
+         next_idx = (idx + 1) & size;
+
+      current->descs[i].next_index = next_idx;
+   }
+
+   return true;
+}
+
+static void task_overlay_resolve_iterate(retro_task_t *task)
+{
+   overlay_loader_t *loader  = (overlay_loader_t*)task->state;
+   bool             not_done = loader->resolve_pos < loader->size;
+
+   if (!not_done)
+   {
+      loader->state = OVERLAY_STATUS_DEFERRED_DONE;
+      return;
+   }
+
+   if (!task_overlay_resolve_targets(loader->overlays,
+            loader->resolve_pos, loader->size))
+   {
+      RARCH_ERR("[Overlay]: Failed to resolve next targets.\n");
+      task_set_cancelled(task, true);
+      loader->state   = OVERLAY_STATUS_DEFERRED_ERROR;
+      return;
+   }
+
+   if (loader->resolve_pos == 0)
+   {
+      loader->active = &loader->overlays[0];
+
+#if 0
+      /* TODO: MOVE TO MAIN THREAD / CALLBACK */
+      input_overlay_load_active(loader->deferred.opacity);
+      input_overlay_enable(loader->deferred.enable);
+#endif
+   }
+
+   loader->resolve_pos += 1;
+}
+
 static void task_overlay_deferred_loading(retro_task_t *task)
 {
    size_t i                  = 0;
@@ -356,7 +452,7 @@ static void task_overlay_deferred_loading(retro_task_t *task)
                {
                   RARCH_ERR("[Overlay]: Failed to load overlay descs for overlay #%u.\n",
                         (unsigned)overlay->pos);
-                  task->cancelled = true;
+                  task_set_cancelled(task, true);
                   loader->state   = OVERLAY_STATUS_DEFERRED_ERROR;
                   break;
                }
@@ -377,7 +473,7 @@ static void task_overlay_deferred_loading(retro_task_t *task)
          loader->loading_status = OVERLAY_IMAGE_TRANSFER_NONE;
          break;
       case OVERLAY_IMAGE_TRANSFER_ERROR:
-         task->cancelled = true;
+         task_set_cancelled(task, true);
          loader->state   = OVERLAY_STATUS_DEFERRED_ERROR;
          break;
    }
@@ -391,14 +487,14 @@ static void task_overlay_deferred_load(retro_task_t *task)
 
    for (i = 0; i < loader->pos_increment; i++, loader->pos++)
    {
+      char tmp_str[PATH_MAX_LENGTH];
+      char conf_key[64];
+      char overlay_full_screen_key[64];
       float tmp_float                   = 0.0;
       bool tmp_bool                     = false;
-      char conf_key[64]                 = {0};
-      char overlay_full_screen_key[64]  = {0};
       struct texture_image *texture_img = NULL;
       struct overlay_desc *overlay_desc = NULL;
       struct overlay          *overlay  = NULL;
-      char tmp_str[PATH_MAX_LENGTH]     = {0};
       bool                     to_cont  = loader->pos < loader->size;
 
       if (!to_cont)
@@ -408,6 +504,8 @@ static void task_overlay_deferred_load(retro_task_t *task)
          break;
       }
 
+      tmp_str[0] = conf_key[0] = overlay_full_screen_key[0] = '\0';
+
       overlay = &loader->overlays[loader->pos];
 
       snprintf(overlay->config.descs.key,
@@ -480,12 +578,16 @@ static void task_overlay_deferred_load(retro_task_t *task)
       if (!string_is_empty(overlay->config.paths.path))
       {
          struct texture_image image_tex;
-         char overlay_resolved_path[PATH_MAX_LENGTH] = {0};
+         char overlay_resolved_path[PATH_MAX_LENGTH];
+
+         overlay_resolved_path[0] = '\0';
 
          fill_pathname_resolve_relative(overlay_resolved_path,
                loader->overlay_path,
                overlay->config.paths.path, sizeof(overlay_resolved_path));
 
+         image_tex.supports_rgba = video_driver_supports_rgba();
+
          if (!image_texture_load(&image_tex, overlay_resolved_path))
          {
             RARCH_ERR("[Overlay]: Failed to load image: %s.\n",
@@ -540,93 +642,11 @@ static void task_overlay_deferred_load(retro_task_t *task)
    return;
 
 error:
-   task->cancelled = true;
+   task_set_cancelled(task, true);
    loader->pos     = 0;
    loader->state   = OVERLAY_STATUS_DEFERRED_ERROR;
 }
 
-static ssize_t task_overlay_find_index(const struct overlay *ol,
-      const char *name, size_t size)
-{
-   size_t i;
-
-   if (!ol)
-      return -1;
-
-   for (i = 0; i < size; i++)
-   {
-      if (string_is_equal(ol[i].name, name))
-         return i;
-   }
-
-   return -1;
-}
-
-static bool task_overlay_resolve_targets(struct overlay *ol,
-      size_t idx, size_t size)
-{
-   unsigned i;
-   struct overlay *current = (struct overlay*)&ol[idx];
-
-   for (i = 0; i < current->size; i++)
-   {
-      ssize_t next_idx  = 0;
-      const char *next = current->descs[i].next_index_name;
-
-      if (!string_is_empty(next))
-      {
-         next_idx = task_overlay_find_index(ol, next, size);
-
-         if (next_idx < 0)
-         {
-            RARCH_ERR("[Overlay]: Couldn't find overlay called: \"%s\".\n",
-                  next);
-            return false;
-         }
-      }
-      else
-         next_idx = (idx + 1) & size;
-
-      current->descs[i].next_index = next_idx;
-   }
-
-   return true;
-}
-
-static void task_overlay_resolve_iterate(retro_task_t *task)
-{
-   overlay_loader_t *loader  = (overlay_loader_t*)task->state;
-   bool             not_done = loader->resolve_pos < loader->size;
-
-   if (!not_done)
-   {
-      loader->state = OVERLAY_STATUS_DEFERRED_DONE;
-      return;
-   }
-
-   if (!task_overlay_resolve_targets(loader->overlays,
-            loader->resolve_pos, loader->size))
-   {
-      RARCH_ERR("[Overlay]: Failed to resolve next targets.\n");
-      task->cancelled = true;
-      loader->state   = OVERLAY_STATUS_DEFERRED_ERROR;
-      return;
-   }
-
-   if (loader->resolve_pos == 0)
-   {
-      loader->active = &loader->overlays[0];
-
-#if 0
-      /* TODO: MOVE TO MAIN THREAD / CALLBACK */
-      input_overlay_load_active(loader->deferred.opacity);
-      input_overlay_enable(loader->deferred.enable);
-#endif
-   }
-
-   loader->resolve_pos += 1;
-}
-
 static void task_overlay_free(retro_task_t *task)
 {
    unsigned i;
@@ -636,8 +656,7 @@ static void task_overlay_free(retro_task_t *task)
    if (loader->overlay_path)
       free(loader->overlay_path);
 
-
-   if (task->cancelled)
+   if (task_get_cancelled(task))
    {
       for (i = 0; i < overlay->load_images_size; i++)
       {
@@ -673,25 +692,30 @@ static void task_overlay_handler(retro_task_t *task)
          task_overlay_resolve_iterate(task);
          break;
       case OVERLAY_STATUS_DEFERRED_ERROR:
-         task->cancelled = true;
+         task_set_cancelled(task, true);
          break;
       case OVERLAY_STATUS_DEFERRED_DONE:
       default:
       case OVERLAY_STATUS_NONE:
-         task->finished = true;
+         task_set_finished(task, true);
          break;
    }
 
-   if (task->finished && !task->cancelled)
+   if (task_get_finished(task) && !task_get_cancelled(task))
    {
+      settings_t *settings      = config_get_ptr();
       overlay_task_data_t *data = (overlay_task_data_t*)
          calloc(1, sizeof(*data));
 
-      data->overlays = loader->overlays;
-      data->size     = loader->size;
-      data->active   = loader->active;
+      data->overlays        = loader->overlays;
+      data->size            = loader->size;
+      data->active          = loader->active;
+      data->hide_in_menu    = settings->input.overlay_hide_in_menu;
+      data->overlay_enable  = settings->input.overlay_enable;
+      data->overlay_opacity = settings->input.overlay_opacity;
+      data->overlay_scale   = settings->input.overlay_scale;
 
-      task->task_data = data;
+      task_set_data(task, data);
    }
 }
 
@@ -715,10 +739,10 @@ static bool task_overlay_finder(retro_task_t *task, void *user_data)
 static bool task_push_overlay_load(const char *overlay_path,
       retro_task_callback_t cb, void *user_data)
 {
+   task_finder_data_t find_data;
    retro_task_t *t          = NULL;
    config_file_t *conf      = NULL;
    overlay_loader_t *loader = (overlay_loader_t*)calloc(1, sizeof(*loader));
-   task_finder_data_t find_data;
 
    if (!loader)
       goto error;
@@ -785,20 +809,10 @@ bool task_push_overlay_load_default(
       retro_task_callback_t cb, void *user_data)
 {
    settings_t *settings = config_get_ptr();
-   bool osk_enable      = input_driver_is_onscreen_keyboard_enabled();
+   const char *path     = settings->path.overlay;
 
-   if (osk_enable)
-   {
-      if (!*settings->path.osk_overlay)
-         return false;
-   }
-   else
-   {
-      if (!*settings->path.overlay)
-         return false;
-   }
+   if (string_is_empty(path))
+      return false;
 
-   return task_push_overlay_load(
-            osk_enable ? settings->path.osk_overlay : settings->path.overlay,
-            cb, user_data);
+   return task_push_overlay_load(path, cb, user_data);
 }
