@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -23,9 +23,16 @@
 
 #include <boolean.h>
 #include <retro_common_api.h>
-#include <libretro.h>
 
-#include "font_driver.h"
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#ifdef HAVE_OVERLAY
+#include "../input/input_overlay.h"
+#endif
+
+#include "video_defines.h"
 #include "video_filter.h"
 #include "video_shader_parse.h"
 
@@ -33,92 +40,127 @@
 
 RETRO_BEGIN_DECLS
 
-enum texture_filter_type
-{
-   TEXTURE_FILTER_LINEAR = 0,
-   TEXTURE_FILTER_NEAREST,
-   TEXTURE_FILTER_MIPMAP_LINEAR,
-   TEXTURE_FILTER_MIPMAP_NEAREST
-};
-
-enum aspect_ratio
-{
-   ASPECT_RATIO_4_3 = 0,
-   ASPECT_RATIO_16_9,
-   ASPECT_RATIO_16_10,
-   ASPECT_RATIO_16_15,
-   ASPECT_RATIO_1_1,
-   ASPECT_RATIO_2_1,
-   ASPECT_RATIO_3_2,
-   ASPECT_RATIO_3_4,
-   ASPECT_RATIO_4_1,
-   ASPECT_RATIO_4_4,
-   ASPECT_RATIO_5_4,
-   ASPECT_RATIO_6_5,
-   ASPECT_RATIO_7_9,
-   ASPECT_RATIO_8_3,
-   ASPECT_RATIO_8_7,
-   ASPECT_RATIO_19_12,
-   ASPECT_RATIO_19_14,
-   ASPECT_RATIO_30_17,
-   ASPECT_RATIO_32_9,
-   ASPECT_RATIO_CONFIG,
-   ASPECT_RATIO_SQUARE,
-   ASPECT_RATIO_CORE,
-   ASPECT_RATIO_CUSTOM,
-
-   ASPECT_RATIO_END
-};
-
-enum rotation
-{
-   ORIENTATION_NORMAL = 0,
-   ORIENTATION_VERTICAL,
-   ORIENTATION_FLIPPED,
-   ORIENTATION_FLIPPED_ROTATED,
-   ORIENTATION_END
-};
-
-enum rarch_display_type
-{
-   /* Non-bindable types like consoles, KMS, VideoCore, etc. */
-   RARCH_DISPLAY_NONE = 0,
-   /* video_display => Display*, video_window => Window */
-   RARCH_DISPLAY_X11,
-   /* video_display => N/A, video_window => HWND */
-   RARCH_DISPLAY_WIN32,
-   RARCH_DISPLAY_OSX
-};
-
 typedef struct video_info
 {
+   /* Width of window. 
+    * If fullscreen mode is requested, 
+    * a width of 0 means the resolution of the 
+    * desktop should be used. */
    unsigned width;
+
+   /* Height of window. 
+    * If fullscreen mode is requested, 
+    * a height of 0 means the resolutiof the desktop should be used.
+    */
    unsigned height;
+
+   /* Launch in fullscreen mode instead of windowed mode. */
    bool fullscreen;
+
+   /* Start with V-Sync enabled. */
    bool vsync;
+
+   /* If true, the output image should have the aspect ratio 
+    * as set in aspect_ratio. */
    bool force_aspect;
+
+   unsigned swap_interval;
+
+   bool font_enable;
+
 #ifdef GEKKO
    /* TODO - we can't really have driver system-specific
     * variables in here. There should be some
     * kind of publicly accessible driver implementation
     * video struct for specific things like this.
     */
+
+   /* Wii-specific settings. Ignored for everything else. */
    unsigned viwidth;
-#endif
    bool vfilter;
+#endif
+
+   /* If true, applies bilinear filtering to the image,
+    * otherwise nearest filtering. */
    bool smooth;
-   /* Maximum input size: RARCH_SCALE_BASE * input_scale */
+
+   /* 
+    * input_scale defines the maximum size of the picture that will
+    * ever be used with the frame callback.
+    *
+    * The maximum resolution is a multiple of 256x256 size (RARCH_SCALE_BASE),
+    * so an input scale of 2 means you should allocate a texture or of 512x512.
+    *
+    * Maximum input size: RARCH_SCALE_BASE * input_scale 
+    */
    unsigned input_scale;
-   /* Use 32bit RGBA rather than native RGB565/XBGR1555. */
+
+   /* Use 32bit RGBA rather than native RGB565/XBGR1555. 
+    *
+    * XRGB1555 format is 16-bit and has byte ordering: 0RRRRRGGGGGBBBBB,
+    * in native endian.
+    *
+    * ARGB8888 is AAAAAAAARRRRRRRRGGGGGGGGBBBBBBBB, native endian.
+    * Alpha channel should be disregarded.
+    * */
    bool rgb32;
+   
+#ifndef RARCH_INTERNAL
+   uintptr_t parent;
+#endif
 } video_info_t;
 
-#define FONT_COLOR_RGBA(r, g, b, a) (((r) << 24) | ((g) << 16) | ((b) << 8) | ((a) << 0))
-#define FONT_COLOR_GET_RED(col)   (((col) >> 24) & 0xff)
-#define FONT_COLOR_GET_GREEN(col) (((col) >> 16) & 0xff)
-#define FONT_COLOR_GET_BLUE(col)  (((col) >>  8) & 0xff)
-#define FONT_COLOR_GET_ALPHA(col) (((col) >>  0) & 0xff)
-#define FONT_COLOR_ARGB_TO_RGBA(col) ( (((col) >> 24) & 0xff) | (((col) << 8) & 0xffffff00) )
+typedef struct video_frame_info
+{
+   float menu_wallpaper_opacity;
+   float refresh_rate;
+   bool shared_context;
+   bool black_frame_insertion;
+   bool hard_sync;
+   unsigned hard_sync_frames;
+   bool fps_show;
+   bool scale_integer;
+   unsigned aspect_ratio_idx;
+   bool post_filter_record;
+   unsigned max_swapchain_images;
+   bool windowed_fullscreen;
+   bool fullscreen;
+   unsigned monitor_index;
+   bool font_enable;
+   char fps_text[128];
+   uint64_t frame_count;
+
+   unsigned width;
+   unsigned height;
+
+   float font_msg_pos_x;
+   float font_msg_pos_y;
+   float font_msg_color_r;
+   float font_msg_color_g;
+   float font_msg_color_b;
+   bool use_rgba;
+   bool libretro_running;
+
+   float menu_header_opacity;
+   float menu_footer_opacity;
+
+   bool xmb_shadows_enable;
+   float xmb_alpha_factor;
+   unsigned xmb_theme;
+   unsigned xmb_color_theme;
+   unsigned menu_shader_pipeline;
+
+   unsigned materialui_color_theme;
+
+   bool battery_level_enable;
+   bool timedate_enable;
+   bool runloop_is_slowmotion;
+   bool runloop_is_idle;
+   bool runloop_is_paused;
+   bool is_perfcnt_enable;
+
+   bool menu_is_alive;
+} video_frame_info_t;
 
 /* Optionally implemented interface to poke more
  * deeply into video driver. */
@@ -128,12 +170,18 @@ typedef struct video_poke_interface
    uintptr_t (*load_texture)(void *video_data, void *data,
          bool threaded, enum texture_filter_type filter_type);
    void (*unload_texture)(void *data, uintptr_t id);
-   void (*set_video_mode)(void *data, unsigned width, unsigned height, bool fullscreen);
+   void (*set_video_mode)(void *data, unsigned width,
+         unsigned height, bool fullscreen);
    void (*set_filtering)(void *data, unsigned index, bool smooth);
    void (*get_video_output_size)(void *data,
          unsigned *width, unsigned *height);
+
+   /* Move index to previous resolution */
    void (*get_video_output_prev)(void *data);
+
+   /* Move index to next resolution */
    void (*get_video_output_next)(void *data);
+
    uintptr_t (*get_current_framebuffer)(void *data);
    retro_proc_address_t (*get_proc_address)(void *data, const char *sym);
    void (*set_aspect_ratio)(void *data, unsigned aspectratio_index);
@@ -147,7 +195,7 @@ typedef struct video_poke_interface
    /* Enable or disable rendering. */
    void (*set_texture_enable)(void *data, bool enable, bool full_screen);
    void (*set_osd_msg)(void *data, const char *msg,
-         const struct font_params *params, void *font);
+         const void *params, void *font);
 
    void (*show_mouse)(void *data, bool state);
    void (*grab_mouse_toggle)(void *data);
@@ -174,7 +222,7 @@ typedef struct video_viewport
 typedef bool (*video_driver_frame_t)(void *data,
       const void *frame, unsigned width,
       unsigned height, uint64_t frame_count,
-      unsigned pitch, const char *msg);
+      unsigned pitch, const char *msg, video_frame_info_t *video_info);
 
 typedef struct video_driver
 {
@@ -186,9 +234,22 @@ typedef struct video_driver
          const input_driver_t **input,
          void **input_data);
 
+   /* Updates frame on the screen. 
+    * Frame can be either XRGB1555, RGB565 or ARGB32 format
+    * depending on rgb32 setting in video_info_t. 
+    * Pitch is the distance in bytes between two scanlines in memory. 
+    * 
+    * When msg is non-NULL, 
+    * it's a message that should be displayed to the user. */
    video_driver_frame_t frame;
 
-   /* Should we care about syncing to vblank? Fast forwarding. */
+   /* Should we care about syncing to vblank? Fast forwarding. 
+    *
+    * Requests nonblocking operation. 
+    *
+    * True = VSync is turned off. 
+    * False = VSync is turned on.
+    * */
    void (*set_nonblock_state)(void *data, bool toggle);
 
    /* Is the window still active? */
@@ -221,7 +282,7 @@ typedef struct video_driver
    void (*viewport_info)(void *data, struct video_viewport *vp);
 
    /* Reads out in BGR byte order (24bpp). */
-   bool (*read_viewport)(void *data, uint8_t *buffer);
+   bool (*read_viewport)(void *data, uint8_t *buffer, bool is_idle);
 
    /* Returns a pointer to a newly allocated buffer that can
     * (and must) be passed to free() by the caller, containing a
@@ -238,24 +299,6 @@ typedef struct video_driver
    unsigned (*wrap_type_to_enum)(enum gfx_wrap_type type);
 } video_driver_t;
 
-#define LAST_ASPECT_RATIO ASPECT_RATIO_CUSTOM
-
-extern char rotation_lut[4][32];
-
-/* ABGR color format defines */
-
-#define WHITE		  0xffffffffu
-#define RED         0xff0000ffu
-#define GREEN		  0xff00ff00u
-#define BLUE        0xffff0000u
-#define YELLOW      0xff00ffffu
-#define PURPLE      0xffff00ffu
-#define CYAN        0xffffff00u
-#define ORANGE      0xff0063ffu
-#define SILVER      0xff8c848cu
-#define LIGHTBLUE   0xFFFFE0E0U
-#define LIGHTORANGE 0xFFE0EEFFu
-
 struct aspect_ratio_elem
 {
    char name[64];
@@ -264,10 +307,18 @@ struct aspect_ratio_elem
 
 extern struct aspect_ratio_elem aspectratio_lut[ASPECT_RATIO_END];
 
-void video_driver_lock(void);
-void video_driver_unlock(void);
-void video_driver_lock_free(void);
-void video_driver_lock_new(void);
+#define video_driver_is_alive()   ((current_video) ? current_video->alive(video_driver_data) : true)
+
+#define video_driver_is_focused() (current_video->focus(video_driver_data))
+
+#if defined(RARCH_CONSOLE) || defined(RARCH_MOBILE)
+#define video_driver_has_windowed() (false)
+#else
+#define video_driver_has_windowed() (current_video->has_windowed && current_video->has_windowed(video_driver_data))
+#endif
+
+#define video_driver_cached_frame_has_valid_framebuffer() (frame_cache_data ? (frame_cache_data == RETRO_HW_FRAME_BUFFER_VALID) : false)
+
 void video_driver_destroy(void);
 void video_driver_set_cached_frame_ptr(const void *data);
 void video_driver_set_stub_frame(void);
@@ -286,7 +337,8 @@ bool video_driver_get_next_video_out(void);
 bool video_driver_get_prev_video_out(void);
 bool video_driver_init(void);
 void video_driver_destroy_data(void);
-void video_driver_deinit(void);
+void video_driver_free(void);
+void video_driver_free_hw_context(void);
 void video_driver_monitor_reset(void);
 void video_driver_set_aspect_ratio(void);
 void video_driver_show_mouse(void);
@@ -294,13 +346,9 @@ void video_driver_hide_mouse(void);
 void video_driver_set_nonblock_state(bool toggle);
 bool video_driver_find_driver(void);
 void video_driver_apply_state_changes(void);
-bool video_driver_read_viewport(uint8_t *buffer);
-bool video_driver_cached_frame_has_valid_framebuffer(void);
-bool video_driver_cached_frame_render(void);
-bool video_driver_is_alive(void);
-bool video_driver_is_focused(void);
-bool video_driver_has_windowed(void);
-uint64_t *video_driver_get_frame_count_ptr(void);
+bool video_driver_read_viewport(uint8_t *buffer, bool is_idle);
+bool video_driver_cached_frame(void);
+uint64_t video_driver_get_frame_count(void);
 bool video_driver_frame_filter_alive(void);
 bool video_driver_frame_filter_is_32bit(void);
 void video_driver_default_settings(void);
@@ -310,28 +358,26 @@ void video_driver_set_own_driver(void);
 void video_driver_unset_own_driver(void);
 bool video_driver_owns_driver(void);
 bool video_driver_is_hw_context(void);
-void video_driver_deinit_hw_context(void);
 struct retro_hw_render_callback *video_driver_get_hw_context(void);
-const struct retro_hw_render_context_negotiation_interface *video_driver_get_context_negotiation_interface(void);
-void video_driver_set_context_negotiation_interface(const struct retro_hw_render_context_negotiation_interface *iface);
-void video_driver_set_video_cache_context(void);
-void video_driver_unset_video_cache_context(void);
+const struct retro_hw_render_context_negotiation_interface 
+*video_driver_get_context_negotiation_interface(void);
+void video_driver_set_context_negotiation_interface(const struct 
+      retro_hw_render_context_negotiation_interface *iface);
 bool video_driver_is_video_cache_context(void);
 void video_driver_set_video_cache_context_ack(void);
-void video_driver_unset_video_cache_context_ack(void);
 bool video_driver_is_video_cache_context_ack(void);
 void video_driver_set_active(void);
-void video_driver_unset_active(void);
 bool video_driver_is_active(void);
-bool video_driver_has_gpu_record(void);
-uint8_t *video_driver_get_gpu_record(void);
 bool video_driver_gpu_record_init(unsigned size);
 void video_driver_gpu_record_deinit(void);
-bool video_driver_get_current_software_framebuffer(struct retro_framebuffer *fb);
-bool video_driver_get_hw_render_interface(const struct retro_hw_render_interface **iface);
+bool video_driver_get_current_software_framebuffer(struct 
+      retro_framebuffer *fb);
+bool video_driver_get_hw_render_interface(const struct 
+      retro_hw_render_interface **iface);
 bool video_driver_get_viewport_info(struct video_viewport *viewport);
 void video_driver_set_title_buf(void);
 void video_driver_monitor_adjust_system_rates(void);
+bool video_driver_is_threaded(void);
 
 /**
  * video_driver_find_handle:
@@ -394,7 +440,7 @@ bool video_driver_get_video_output_size(
       unsigned *width, unsigned *height);
 
 void video_driver_set_osd_msg(const char *msg,
-      const struct font_params *params, void *font);
+      const void *params, void *font);
 
 void video_driver_set_texture_enable(bool enable, bool full_screen);
 
@@ -420,6 +466,8 @@ void video_driver_get_size(unsigned *width, unsigned *height);
 
 void video_driver_set_size(unsigned *width, unsigned *height);
 
+void video_driver_unset_video_cache_context_ack(void);
+
 float video_driver_get_aspect_ratio(void);
 
 void video_driver_set_aspect_ratio_value(float value);
@@ -484,22 +532,6 @@ void video_monitor_set_refresh_rate(float hz);
 bool video_monitor_fps_statistics(double *refresh_rate,
       double *deviation, unsigned *sample_points);
 
-/**
- * video_monitor_get_fps:
- * @buf           : string suitable for Window title
- * @size          : size of buffer.
- * @buf_fps       : string of raw FPS only (optional).
- * @size_fps      : size of raw FPS buffer.
- *
- * Get the amount of frames per seconds.
- *
- * Returns: true if framerate per seconds could be obtained,
- * otherwise false.
- *
- **/
-bool video_monitor_get_fps(char *buf, size_t size,
-      char *buf_fps, size_t size_fps);
-
 unsigned video_pixel_get_alignment(unsigned pitch);
 
 const video_poke_interface_t *video_driver_get_poke(void);
@@ -516,6 +548,30 @@ const video_poke_interface_t *video_driver_get_poke(void);
 void video_driver_frame(const void *data, unsigned width,
       unsigned height, size_t pitch);
 
+#define video_driver_translate_coord_viewport_wrap(vp, mouse_x, mouse_y, res_x, res_y, res_screen_x, res_screen_y) \
+   (video_driver_get_viewport_info(vp) ? video_driver_translate_coord_viewport(vp, mouse_x, mouse_y, res_x, res_y, res_screen_x, res_screen_y) : false)
+
+/**
+ * video_driver_translate_coord_viewport:
+ * @mouse_x                        : Pointer X coordinate.
+ * @mouse_y                        : Pointer Y coordinate.
+ * @res_x                          : Scaled  X coordinate.
+ * @res_y                          : Scaled  Y coordinate.
+ * @res_screen_x                   : Scaled screen X coordinate.
+ * @res_screen_y                   : Scaled screen Y coordinate.
+ *
+ * Translates pointer [X,Y] coordinates into scaled screen
+ * coordinates based on viewport info.
+ *
+ * Returns: true (1) if successful, false if video driver doesn't support
+ * viewport info.
+ **/
+bool video_driver_translate_coord_viewport(
+      void *data,
+      int mouse_x, int mouse_y,
+      int16_t *res_x, int16_t *res_y, int16_t *res_screen_x,
+      int16_t *res_screen_y);
+
 uintptr_t video_driver_display_get(void);
 
 enum rarch_display_type video_driver_display_type_get(void);
@@ -534,6 +590,19 @@ bool video_driver_texture_load(void *data,
 
 bool video_driver_texture_unload(uintptr_t *id);
 
+void video_driver_build_info(video_frame_info_t *video_info);
+
+void video_driver_reinit(void);
+
+void video_driver_get_window_title(char *buf, unsigned len);
+
+void video_driver_get_record_status(
+      bool *has_gpu_record, 
+      uint8_t **gpu_buf);
+
+void video_driver_get_status(uint64_t *frame_count, bool * is_alive,
+      bool *is_focused);
+
 extern video_driver_t video_gl;
 extern video_driver_t video_vulkan;
 extern video_driver_t video_psp1;
@@ -541,6 +610,7 @@ extern video_driver_t video_vita2d;
 extern video_driver_t video_ctr;
 extern video_driver_t video_d3d;
 extern video_driver_t video_gx;
+extern video_driver_t video_wiiu;
 extern video_driver_t video_xenon360;
 extern video_driver_t video_xvideo;
 extern video_driver_t video_xdk_d3d;
@@ -553,8 +623,16 @@ extern video_driver_t video_dispmanx;
 extern video_driver_t video_sunxi;
 extern video_driver_t video_drm;
 extern video_driver_t video_xshm;
+extern video_driver_t video_caca;
+extern video_driver_t video_gdi;
+extern video_driver_t video_vga;
 extern video_driver_t video_null;
 
+extern const void *frame_cache_data;
+
+extern void *video_driver_data;
+extern video_driver_t *current_video;
+
 RETRO_END_DECLS
 
 #endif
