@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -25,11 +25,14 @@
 #include <string/stdstring.h>
 #include <streams/file_stream.h>
 
-#include "../general.h"
 #include "../msg_hash.h"
 #include "../verbosity.h"
 #include "video_shader_parse.h"
 
+#ifdef HAVE_SLANG
+#include "drivers_shader/slang_preprocess.h"
+#endif
+
 #define WRAP_MODE_CLAMP_TO_BORDER      0x3676ed11U
 #define WRAP_MODE_CLAMP_TO_EDGE        0x9427a608U
 #define WRAP_MODE_REPEAT               0x192dec66U
@@ -116,27 +119,35 @@ static enum gfx_wrap_type wrap_str_to_mode(const char *wrap_mode)
 static bool video_shader_parse_pass(config_file_t *conf,
       struct video_shader_pass *pass, unsigned i)
 {
-   char shader_name[64]         = {0};
-   char filter_name_buf[64]     = {0};
-   char wrap_name_buf[64]       = {0};
-   char wrap_mode[64]           = {0};
-   char frame_count_mod_buf[64] = {0};
-   char srgb_output_buf[64]     = {0};
-   char fp_fbo_buf[64]          = {0};
-   char mipmap_buf[64]          = {0};
-   char alias_buf[64]           = {0};
-   char scale_name_buf[64]      = {0};
-   char attr_name_buf[64]       = {0};
-   char scale_type[64]          = {0};
-   char scale_type_x[64]        = {0};
-   char scale_type_y[64]        = {0};
-   char frame_count_mod[64]     = {0};
+   char tmp_str[PATH_MAX_LENGTH];
+   char tmp_path[PATH_MAX_LENGTH];
+   char shader_name[64];
+   char filter_name_buf[64];
+   char wrap_name_buf[64];
+   char wrap_mode[64];
+   char frame_count_mod_buf[64];
+   char srgb_output_buf[64];
+   char fp_fbo_buf[64];
+   char mipmap_buf[64];
+   char alias_buf[64];
+   char scale_name_buf[64];
+   char attr_name_buf[64];
+   char scale_type[64];
+   char scale_type_x[64];
+   char scale_type_y[64];
+   char frame_count_mod[64];
    struct gfx_fbo_scale *scale  = NULL;
    bool tmp_bool                = false;
-   char tmp_str[PATH_MAX_LENGTH]= {0};
    float fattr                  = 0.0f;
    int iattr                    = 0;
 
+   fp_fbo_buf[0]     = mipmap_buf[0]    = alias_buf[0]           = 
+   scale_name_buf[0] = attr_name_buf[0] = scale_type[0]          =
+   scale_type_x[0]   = scale_type_y[0]  = frame_count_mod[0]     =
+   tmp_str[0]        = shader_name[0]   = filter_name_buf[0]     = 
+   wrap_name_buf[0]  = wrap_mode[0]     = frame_count_mod_buf[0] = '\0';
+   srgb_output_buf[0] = '\0';
+
    /* Source */
    snprintf(shader_name, sizeof(shader_name), "shader%u", i);
    if (!config_get_path(conf, shader_name, tmp_str, sizeof(tmp_str)))
@@ -145,8 +156,14 @@ static bool video_shader_parse_pass(config_file_t *conf,
       return false;
    }
 
-   strlcpy(pass->source.path, tmp_str, sizeof(pass->source.path));
-   
+   strlcpy(tmp_path, tmp_str, sizeof(tmp_path));
+   path_resolve_realpath(tmp_path, sizeof(tmp_path));
+
+   if (!path_file_exists(tmp_path))
+      strlcpy(pass->source.path, tmp_str, sizeof(pass->source.path));
+   else
+      strlcpy(pass->source.path, tmp_path, sizeof(pass->source.path));
+
    /* Smooth */
    snprintf(filter_name_buf, sizeof(filter_name_buf), "filter_linear%u", i);
 
@@ -317,9 +334,12 @@ static bool video_shader_parse_pass(config_file_t *conf,
 static bool video_shader_parse_textures(config_file_t *conf,
       struct video_shader *shader)
 {
+   char textures[1024];
    const char *id       = NULL;
    char *save           = NULL;
-   char textures[1024]  = {0};
+   char tmp_path[PATH_MAX_LENGTH];
+
+   textures[0] = '\0';
 
    if (!config_get_array(conf, "textures", textures, sizeof(textures)))
       return true;
@@ -328,13 +348,15 @@ static bool video_shader_parse_textures(config_file_t *conf,
          id && shader->luts < GFX_MAX_TEXTURES;
          shader->luts++, id = strtok_r(NULL, ";", &save))
    {
-      char id_filter[64]  = {0};
-      char id_wrap[64]    = {0};
-      char wrap_mode[64]  = {0};
-      char id_mipmap[64]  = {0};
+      char id_filter[64];
+      char id_wrap[64];
+      char wrap_mode[64];
+      char id_mipmap[64];
       bool mipmap         = false;
       bool smooth         = false;
 
+      id_filter[0] = id_wrap[0] = wrap_mode[0] = id_mipmap[0] = '\0';
+
       if (!config_get_array(conf, id, shader->lut[shader->luts].path,
                sizeof(shader->lut[shader->luts].path)))
       {
@@ -342,6 +364,15 @@ static bool video_shader_parse_textures(config_file_t *conf,
          return false;
       }
 
+      strlcpy(tmp_path, shader->lut[shader->luts].path, sizeof(tmp_path));
+      path_resolve_realpath(tmp_path, sizeof(tmp_path));
+
+      if (path_file_exists(tmp_path))
+      {
+         strlcpy(shader->lut[shader->luts].path, 
+            tmp_path, sizeof(shader->lut[shader->luts].path));
+      }
+
       strlcpy(shader->lut[shader->luts].id, id,
             sizeof(shader->lut[shader->luts].id));
 
@@ -391,6 +422,50 @@ static struct video_shader_parameter *video_shader_parse_find_parameter(
    return NULL;
 }
 
+/** 
+ * video_shader_set_current_parameters:
+ * @conf              : Preset file to read from.
+ * @shader            : Shader passes handle.
+ *
+ * Reads the current value for all parameters from config file.
+ *
+ * Returns: true (1) if successful, otherwise false (0).
+ **/
+bool video_shader_resolve_current_parameters(config_file_t *conf,
+      struct video_shader *shader)
+{
+   char parameters[4096];
+   const char *id        = NULL;
+   char *save            = NULL;
+
+   if (!conf)
+      return false;
+
+   parameters[0] = '\0';
+
+   /* Read in parameters which override the defaults. */
+   if (!config_get_array(conf, "parameters",
+            parameters, sizeof(parameters)))
+      return true;
+
+   for (id = strtok_r(parameters, ";", &save); id; 
+         id = strtok_r(NULL, ";", &save))
+   {
+      struct video_shader_parameter *parameter = (struct video_shader_parameter*)
+         video_shader_parse_find_parameter(shader->parameters, shader->num_parameters, id);
+
+      if (!parameter)
+      {
+         RARCH_WARN("[CGP/GLSLP]: Parameter %s is set in the preset, but no shader uses this parameter, ignoring.\n", id);
+         continue;
+      }
+
+      if (!config_get_float(conf, id, &parameter->current))
+         RARCH_WARN("[CGP/GLSLP]: Parameter %s is not set in preset.\n", id);
+   }
+   return true;
+}
+
 /** 
  * video_shader_resolve_parameters:
  * @conf              : Preset file to read from.
@@ -404,8 +479,7 @@ bool video_shader_resolve_parameters(config_file_t *conf,
       struct video_shader *shader)
 {
    unsigned i;
-   struct video_shader_parameter *param = 
-      &shader->parameters[shader->num_parameters];
+   struct video_shader_parameter *param = &shader->parameters[0];
 
    shader->num_parameters = 0;
 
@@ -413,12 +487,25 @@ bool video_shader_resolve_parameters(config_file_t *conf,
 
    for (i = 0; i < shader->passes; i++)
    {
-      char line[4096] = {0};
+#ifdef HAVE_SLANG
+      /* First try to use the more robust slang implementation to support #includes. */
+      /* FIXME: The check for slang can be removed if it's sufficiently tested for
+       * GLSL/Cg as well, it should be the same implementation. */
+      if (string_is_equal(path_get_extension(shader->pass[i].source.path), "slang") &&
+            slang_preprocess_parse_parameters(shader->pass[i].source.path, shader))
+         continue;
+      /* If that doesn't work, fallback to the old path.
+       * Ideally, we'd get rid of this path sooner or later. */
+#endif
+
+      char line[4096];
       RFILE *file = filestream_open(shader->pass[i].source.path, RFILE_MODE_READ_TEXT, -1);
 
       if (!file)
          continue;
 
+      line[0] = '\0';
+
       while (shader->num_parameters < ARRAY_SIZE(shader->parameters)
             && filestream_gets(file, line, sizeof(line)))
       {
@@ -448,33 +535,8 @@ bool video_shader_resolve_parameters(config_file_t *conf,
       filestream_close(file);
    }
 
-   if (conf)
-   {
-      /* Read in parameters which override the defaults. */
-      char parameters[4096] = {0};
-      const char *id        = NULL;
-      char *save            = NULL;
-
-      if (!config_get_array(conf, "parameters",
-               parameters, sizeof(parameters)))
-         return true;
-
-      for (id = strtok_r(parameters, ";", &save); id; 
-            id = strtok_r(NULL, ";", &save))
-      {
-         struct video_shader_parameter *parameter = (struct video_shader_parameter*)
-            video_shader_parse_find_parameter(shader->parameters, shader->num_parameters, id);
-
-         if (!parameter)
-         {
-            RARCH_WARN("[CGP/GLSLP]: Parameter %s is set in the preset, but no shader uses this parameter, ignoring.\n", id);
-            continue;
-         }
-
-         if (!config_get_float(conf, id, &parameter->current))
-            RARCH_WARN("[CGP/GLSLP]: Parameter %s is not set in preset.\n", id);
-      }
-   }
+   if (conf && !video_shader_resolve_current_parameters(conf, shader))
+      return false;
 
    return true;
 }
@@ -491,10 +553,12 @@ bool video_shader_resolve_parameters(config_file_t *conf,
 static bool video_shader_parse_imports(config_file_t *conf,
       struct video_shader *shader)
 {
+   char imports[1024];
+   char tmp_str[PATH_MAX_LENGTH];
    const char *id     = NULL;
    char *save         = NULL;
-   char imports[1024] = {0};
-   char tmp_str[PATH_MAX_LENGTH]= {0};
+
+   imports[0] = tmp_str[0] = '\0';
 
    if (!config_get_array(conf, "imports", imports, sizeof(imports)))
       return true;
@@ -504,18 +568,21 @@ static bool video_shader_parse_imports(config_file_t *conf,
          shader->variables++, id = strtok_r(NULL, ";", &save))
    {
       uint32_t semantic_hash;
-      char semantic_buf[64]   = {0};
-      char wram_buf[64]       = {0};
-      char input_slot_buf[64] = {0};
-      char mask_buf[64]       = {0};
-      char equal_buf[64]      = {0};
-      char semantic[64]       = {0};
+      char semantic_buf[64];
+      char wram_buf[64];
+      char input_slot_buf[64];
+      char mask_buf[64];
+      char equal_buf[64];
+      char semantic[64];
       unsigned addr           = 0;
       unsigned mask           = 0;
       unsigned equal          = 0;
       struct state_tracker_uniform_info *var = 
          &shader->variable[shader->variables];
 
+      semantic_buf[0] = wram_buf[0] = input_slot_buf[0] = 
+         mask_buf[0] = equal_buf[0] = semantic[0] = '\0';
+
       strlcpy(var->id, id, sizeof(var->id));
 
       snprintf(semantic_buf, sizeof(semantic_buf), "%s_semantic", id);
@@ -676,7 +743,9 @@ static const char *scale_type_to_str(enum gfx_scale_type type)
 static void shader_write_scale_dim(config_file_t *conf, const char *dim,
       enum gfx_scale_type type, float scale, unsigned absolute, unsigned i)
 {
-   char key[64] = {0};
+   char key[64];
+
+   key[0] = '\0';
 
    snprintf(key, sizeof(key), "scale_type_%s%u", dim, i);
    config_set_string(conf, key, scale_type_to_str(type));
@@ -691,7 +760,9 @@ static void shader_write_scale_dim(config_file_t *conf, const char *dim,
 static void shader_write_fbo(config_file_t *conf,
       const struct gfx_fbo_scale *fbo, unsigned i)
 {
-   char key[64] = {0};
+   char key[64];
+
+   key[0] = '\0';
 
    snprintf(key, sizeof(key), "float_framebuffer%u", i);
    config_set_bool(conf, key, fbo->fp_fbo);
@@ -746,13 +817,16 @@ static const char *import_semantic_to_str(enum state_tracker_type type)
 static void shader_write_variable(config_file_t *conf,
       const struct state_tracker_uniform_info *info)
 {
-   char semantic_buf[64]   = {0};
-   char wram_buf[64]       = {0};
-   char input_slot_buf[64] = {0};
-   char mask_buf[64]       = {0};
-   char equal_buf[64]      = {0};
+   char semantic_buf[64];
+   char wram_buf[64];
+   char input_slot_buf[64];
+   char mask_buf[64];
+   char equal_buf[64];
    const char *id          = info->id;
 
+   semantic_buf[0] = wram_buf[0] = input_slot_buf[0] =
+      mask_buf[0] = equal_buf[0] = '\0';
+
    snprintf(semantic_buf, sizeof(semantic_buf), "%s_semantic", id);
    snprintf(wram_buf, sizeof(wram_buf), "%s_wram", id);
    snprintf(input_slot_buf, sizeof(input_slot_buf), "%s_input_slot", id);
@@ -802,11 +876,18 @@ void video_shader_write_conf_cgp(config_file_t *conf,
 
    for (i = 0; i < shader->passes; i++)
    {
-      char key[64] = {0};
+      char key[64];
+      char tmp[PATH_MAX_LENGTH];
       const struct video_shader_pass *pass = &shader->pass[i];
 
+      key[0] = '\0';
+
       snprintf(key, sizeof(key), "shader%u", i);
-      config_set_string(conf, key, pass->source.path);
+      strlcpy(tmp, pass->source.path, sizeof(tmp));
+
+      if (!path_is_absolute(tmp))
+         path_resolve_realpath(tmp, sizeof(tmp));
+      config_set_string(conf, key, tmp);
 
       if (pass->filter != RARCH_FILTER_UNSPEC)
       {
@@ -834,7 +915,9 @@ void video_shader_write_conf_cgp(config_file_t *conf,
 
    if (shader->num_parameters)
    {
-      char parameters[4096] = {0};
+      char parameters[4096];
+
+      parameters[0] = '\0';
 
       strlcpy(parameters, shader->parameters[0].id, sizeof(parameters));
 
@@ -854,7 +937,9 @@ void video_shader_write_conf_cgp(config_file_t *conf,
 
    if (shader->luts)
    {
-      char textures[4096] = {0};
+      char textures[4096];
+
+      textures[0] = '\0';
 
       strlcpy(textures, shader->lut[0].id, sizeof(textures));
       for (i = 1; i < shader->luts; i++)
@@ -868,7 +953,9 @@ void video_shader_write_conf_cgp(config_file_t *conf,
 
       for (i = 0; i < shader->luts; i++)
       {
-         char key[64] = {0};
+         char key[64];
+
+         key[0] = '\0';
 
          config_set_string(conf, shader->lut[i].id, shader->lut[i].path);
 
@@ -894,7 +981,9 @@ void video_shader_write_conf_cgp(config_file_t *conf,
 
    if (shader->variables)
    {
-      char variables[4096] = {0};
+      char variables[4096];
+
+      variables[0] = '\0';
 
       strlcpy(variables, shader->variable[0].id, sizeof(variables));
 
@@ -958,7 +1047,9 @@ void video_shader_resolve_relative(struct video_shader *shader,
       const char *ref_path)
 {
    unsigned i;
-   char tmp_path[4096] = {0};
+   char tmp_path[4096];
+   
+   tmp_path[0] = '\0';
 
    for (i = 0; i < shader->passes; i++)
    {
