@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  * 
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -18,16 +18,52 @@
 #include <string.h>
 #include <limits.h>
 
+#include <compat/strl.h>
 #include <features/features_cpu.h>
 #include <rthreads/rthreads.h>
 
 #include "video_thread_wrapper.h"
 #include "font_driver.h"
 #include "video_shader_driver.h"
+
 #include "../performance_counters.h"
 #include "../runloop.h"
 #include "../verbosity.h"
 
+enum thread_cmd
+{
+   CMD_VIDEO_NONE = 0,
+   CMD_INIT,
+   CMD_SET_SHADER,
+   CMD_FREE,
+   CMD_ALIVE, /* Blocking alive check. Used when paused. */
+   CMD_SET_VIEWPORT,
+   CMD_SET_ROTATION,
+   CMD_READ_VIEWPORT,
+
+   CMD_OVERLAY_ENABLE,
+   CMD_OVERLAY_LOAD,
+   CMD_OVERLAY_TEX_GEOM,
+   CMD_OVERLAY_VERTEX_GEOM,
+   CMD_OVERLAY_FULL_SCREEN,
+
+   CMD_POKE_SET_VIDEO_MODE,
+   CMD_POKE_SET_FILTERING,
+   CMD_POKE_GET_VIDEO_OUTPUT_SIZE,
+   CMD_POKE_GET_VIDEO_OUTPUT_PREV,
+   CMD_POKE_GET_VIDEO_OUTPUT_NEXT,
+
+   CMD_POKE_SET_FBO_STATE,
+   CMD_POKE_GET_FBO_STATE,
+
+   CMD_POKE_SET_ASPECT_RATIO,
+   CMD_POKE_SET_OSD_MSG,
+   CMD_FONT_INIT,
+   CMD_CUSTOM_COMMAND,
+
+   CMD_DUMMY = INT_MAX
+};
+
 struct thread_packet
 {
    enum thread_cmd type;
@@ -86,7 +122,7 @@ struct thread_packet
 
       struct
       {
-         char msg[PATH_MAX_LENGTH];
+         char msg[255];
          struct font_params params;
       } osd_message;
 
@@ -106,6 +142,7 @@ struct thread_packet
          const char *font_path;
          float font_size;
          bool return_value;
+         bool is_threaded;
          enum font_driver_render_api api;
       } font_init;
    } data;
@@ -151,6 +188,7 @@ struct thread_video
    bool suppress_screensaver;
    bool has_windowed;
    bool nonblock;
+   bool is_idle;
 
    retro_time_t last_time;
    unsigned hit_count;
@@ -179,7 +217,7 @@ struct thread_video
       bool updated;
       bool within_thread;
       uint64_t count;
-      char msg[PATH_MAX_LENGTH];
+      char msg[255];
    } frame;
 
    video_driver_t video_thread;
@@ -202,10 +240,10 @@ static void video_thread_reply(thread_video_t *thr, const thread_packet_t *pkt)
 {
    slock_lock(thr->lock);
 
-   thr->cmd_data = *pkt;
+   thr->cmd_data  = *pkt;
 
    thr->reply_cmd = pkt->type;
-   thr->send_cmd  = CMD_NONE;
+   thr->send_cmd  = CMD_VIDEO_NONE;
 
    scond_signal(thr->cond_cmd);
    slock_unlock(thr->lock);
@@ -217,10 +255,10 @@ static void video_thread_send_packet(thread_video_t *thr,
 {
    slock_lock(thr->lock);
 
-   thr->cmd_data = *pkt;
+   thr->cmd_data  = *pkt;
 
    thr->send_cmd  = pkt->type;
-   thr->reply_cmd = CMD_NONE;
+   thr->reply_cmd = CMD_VIDEO_NONE;
 
    scond_signal(thr->cond_thread);
    slock_unlock(thr->lock);
@@ -235,8 +273,8 @@ static void video_thread_wait_reply(thread_video_t *thr, thread_packet_t *pkt)
    while (pkt->type != thr->reply_cmd)
       scond_wait(thr->cond_cmd, thr->lock);
 
-   *pkt = thr->cmd_data;
-   thr->cmd_data.type = CMD_NONE;
+   *pkt               = thr->cmd_data;
+   thr->cmd_data.type = CMD_VIDEO_NONE;
 
    slock_unlock(thr->lock);
 }
@@ -328,7 +366,14 @@ static bool video_thread_handle_packet(
 
       case CMD_READ_VIEWPORT:
       {
-         struct video_viewport vp = {0};
+         struct video_viewport vp;
+
+         vp.x                     = 0;
+         vp.y                     = 0;
+         vp.width                 = 0;
+         vp.height                = 0;
+         vp.full_width            = 0;
+         vp.full_height           = 0;
 
          thr->driver->viewport_info(thr->driver_data, &vp);
 
@@ -350,7 +395,7 @@ static bool video_thread_handle_packet(
 
             if (thr->driver->read_viewport)
                ret = thr->driver->read_viewport(thr->driver_data,
-                     (uint8_t*)pkt.data.v);
+                     (uint8_t*)pkt.data.v, thr->is_idle);
 
             pkt.data.b = ret;
             thr->frame.within_thread = false;
@@ -500,7 +545,8 @@ static bool video_thread_handle_packet(
                      pkt.data.font_init.video_data,
                      pkt.data.font_init.font_path,
                      pkt.data.font_init.font_size,
-                     pkt.data.font_init.api);
+                     pkt.data.font_init.api,
+                     pkt.data.font_init.is_threaded);
          video_thread_reply(thr, &pkt);
          break;
 
@@ -512,7 +558,7 @@ static bool video_thread_handle_packet(
          video_thread_reply(thr, &pkt);
          break;
 
-      case CMD_NONE:
+      case CMD_VIDEO_NONE:
          /* Never reply on no command. Possible deadlock if
           * thread sends command right after frame update. */
          break;
@@ -534,7 +580,7 @@ static void video_thread_loop(void *data)
       bool updated = false;
 
       slock_lock(thr->lock);
-      while (thr->send_cmd == CMD_NONE && !thr->frame.updated)
+      while (thr->send_cmd == CMD_VIDEO_NONE && !thr->frame.updated)
          scond_wait(thr->cond_thread, thr->lock);
       if (thr->frame.updated)
          updated = true;
@@ -550,21 +596,34 @@ static void video_thread_loop(void *data)
 
       if (updated)
       {
+         struct video_viewport vp;
          bool                 ret = false;
          bool               alive = false;
          bool               focus = false;
          bool        has_windowed = true;
-         struct video_viewport vp = {0};
+
+         vp.x                     = 0;
+         vp.y                     = 0;
+         vp.width                 = 0;
+         vp.height                = 0;
+         vp.full_width            = 0;
+         vp.full_height           = 0;
 
          slock_lock(thr->frame.lock);
 
          thread_update_driver_state(thr);
 
          if (thr->driver && thr->driver->frame)
+         {
+            video_frame_info_t video_info;
+            video_driver_build_info(&video_info);
+
             ret = thr->driver->frame(thr->driver_data,
-               thr->frame.buffer, thr->frame.width, thr->frame.height,
-               thr->frame.count,
-               thr->frame.pitch, *thr->frame.msg ? thr->frame.msg : NULL);
+                  thr->frame.buffer, thr->frame.width, thr->frame.height,
+                  thr->frame.count,
+                  thr->frame.pitch, *thr->frame.msg ? thr->frame.msg : NULL,
+                  &video_info);
+         }
 
          slock_unlock(thr->frame.lock);
 
@@ -650,7 +709,7 @@ static bool video_thread_has_windowed(void *data)
 
 static bool video_thread_frame(void *data, const void *frame_,
       unsigned width, unsigned height, uint64_t frame_count,
-      unsigned pitch, const char *msg)
+      unsigned pitch, const char *msg, video_frame_info_t *video_info)
 {
    unsigned copy_stride;
    static struct retro_perf_counter thr_frame = {0};
@@ -666,12 +725,12 @@ static bool video_thread_frame(void *data, const void *frame_,
 
       if (thr->driver && thr->driver->frame)
          return thr->driver->frame(thr->driver_data, frame_,
-               width, height, frame_count, pitch, msg);
+               width, height, frame_count, pitch, msg, video_info);
       return false;
    }
 
-   performance_counter_init(&thr_frame, "thr_frame");
-   performance_counter_start(&thr_frame);
+   performance_counter_init(thr_frame, "thr_frame");
+   performance_counter_start_plus(video_info->is_perfcnt_enable, thr_frame);
 
    copy_stride = width * (thr->info.rgb32 
          ? sizeof(uint32_t) : sizeof(uint16_t));
@@ -683,10 +742,9 @@ static bool video_thread_frame(void *data, const void *frame_,
 
    if (!thr->nonblock)
    {
-      settings_t *settings = config_get_ptr();
 
       retro_time_t target_frame_time = (retro_time_t)
-         roundf(1000000 / settings->video.refresh_rate);
+         roundf(1000000 / video_info->refresh_rate);
       retro_time_t target = thr->last_time + target_frame_time;
 
       /* Ideally, use absolute time, but that is only a good idea on POSIX. */
@@ -741,7 +799,7 @@ static bool video_thread_frame(void *data, const void *frame_,
 
    slock_unlock(thr->lock);
 
-   performance_counter_stop(&thr_frame);
+   performance_counter_stop_plus(video_info->is_perfcnt_enable, thr_frame);
 
    thr->last_time = cpu_features_get_time_usec();
    return true;
@@ -754,7 +812,8 @@ static void video_thread_set_nonblock_state(void *data, bool state)
       thr->nonblock = state;
 }
 
-static bool video_thread_init(thread_video_t *thr, const video_info_t *info,
+static bool video_thread_init(thread_video_t *thr,
+      const video_info_t info,
       const input_driver_t **input, void **input_data)
 {
    size_t max_size;
@@ -767,15 +826,15 @@ static bool video_thread_init(thread_video_t *thr, const video_info_t *info,
    thr->cond_thread          = scond_new();
    thr->input                = input;
    thr->input_data           = input_data;
-   thr->info                 = *info;
+   thr->info                 = info;
    thr->alive                = true;
    thr->focus                = true;
    thr->has_windowed         = true;
    thr->suppress_screensaver = true;
 
-   max_size                  = info->input_scale * RARCH_SCALE_BASE;
+   max_size                  = info.input_scale * RARCH_SCALE_BASE;
    max_size                 *= max_size;
-   max_size                 *= info->rgb32 ? sizeof(uint32_t) : sizeof(uint16_t);
+   max_size                 *= info.rgb32 ? sizeof(uint32_t) : sizeof(uint16_t);
    thr->frame.buffer         = (uint8_t*)malloc(max_size);
 
    if (!thr->frame.buffer)
@@ -860,7 +919,7 @@ static void video_thread_viewport_info(void *data, struct video_viewport *vp)
    slock_unlock(thr->lock);
 }
 
-static bool video_thread_read_viewport(void *data, uint8_t *buffer)
+static bool video_thread_read_viewport(void *data, uint8_t *buffer, bool is_idle)
 {
    thread_video_t *thr = (thread_video_t*)data;
    thread_packet_t pkt = { CMD_READ_VIEWPORT };
@@ -868,7 +927,8 @@ static bool video_thread_read_viewport(void *data, uint8_t *buffer)
    if (!thr)
       return false;
 
-   pkt.data.v = buffer;
+   pkt.data.v   = buffer;
+   thr->is_idle = is_idle;
 
    video_thread_send_and_wait_user_to_thread(thr, &pkt);
 
@@ -1137,7 +1197,7 @@ static void thread_set_texture_enable(void *data, bool state, bool full_screen)
 }
 
 static void thread_set_osd_msg(void *data, const char *msg,
-      const struct font_params *params, void *font)
+      const void *params, void *font)
 {
    thread_video_t *thr = (thread_video_t*)data;
 
@@ -1300,7 +1360,7 @@ static void video_thread_set_callbacks(
  **/
 bool video_init_thread(const video_driver_t **out_driver,
       void **out_data,  const input_driver_t **input, void **input_data,
-      const video_driver_t *drv, const video_info_t *info)
+      const video_driver_t *drv, const video_info_t info)
 {
    thread_video_t *thr = (thread_video_t*)calloc(1, sizeof(*thr));
    if (!thr)
@@ -1359,7 +1419,8 @@ static void video_thread_send_and_wait(thread_video_t *thr,
 
 bool video_thread_font_init(const void **font_driver, void **font_handle,
       void *data, const char *font_path, float font_size,
-      enum font_driver_render_api api, custom_font_command_method_t func)
+      enum font_driver_render_api api, custom_font_command_method_t func,
+      bool is_threaded)
 {
    thread_packet_t pkt;
    thread_video_t *thr = (thread_video_t*)video_driver_get_ptr(true);
@@ -1374,6 +1435,7 @@ bool video_thread_font_init(const void **font_driver, void **font_handle,
    pkt.data.font_init.video_data  = data;
    pkt.data.font_init.font_path   = font_path;
    pkt.data.font_init.font_size   = font_size;
+   pkt.data.font_init.is_threaded = is_threaded;
    pkt.data.font_init.api         = api;
 
    video_thread_send_and_wait(thr, &pkt);
